// conses.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

Cons* NIL = NULL;

boolean Cons::emptyP() {
  // Return `true' iff `self' equals `nil'.
  { Cons* self = this;

    return (self == NIL);
  }
}

boolean Cons::non_emptyP() {
  // Return `true' iff `self' is not equal to `nil'.
  { Cons* self = this;

    return (!(self == NIL));
  }
}

boolean nilP(Object* x) {
  // Return `true' iff `x' equals `nil'.
  return (x == NIL);
}

int Cons::length() {
  // Return the length of the CONS list `self'.
  { Cons* self = this;

    { Cons* cons = self;
      int i = 0;

      while (!(cons == NIL)) {
        i = i + 1;
        cons = cons->rest;
      }
      return (i);
    }
  }
}

int Cons::position(Object* object, int start) {
  // Return the position of `object' within the cons-list
  // `self' (counting from zero); or return `null' if `object' does not occur within 
  // `self' (uses an `eql?' test).  If `start' was supplied as non-`null', only 
  // consider the sublist starting at `start', however, the returned position 
  // will always be relative to the entire list.
  { Cons* self = this;

    if (start == NULL_INTEGER) {
      start = 0;
    }
    { Object* element = NULL;
      int position = NULL_INTEGER;
      Cons* iter_000 = self->nth_rest(start);
      int iter_001 = start;

      while (!(iter_000 == NIL)) {
        element = iter_000->value;
        iter_000 = iter_000->rest;
        position = iter_001;
        iter_001 = iter_001 + 1;
        if (eqlP(element, object)) {
          return (position);
        }
      }
    }
    return (NULL_INTEGER);
  }
}

boolean Cons::memberP(Object* object) {
  // Return `true' iff `object' is a member of the cons list
  // `self' (uses an `eql?' test).
  { Cons* self = this;

    if ((!((boolean)(object))) ||
        object->standard_objectP()) {
      { Object* i = NULL;
        Cons* iter_000 = self;

        while (!(iter_000 == NIL)) {
          i = iter_000->value;
          iter_000 = iter_000->rest;
          if (i == object) {
            return (TRUE);
          }
        }
      }
    }
    else {
      { Object* i = NULL;
        Cons* iter_001 = self;

        while (!(iter_001 == NIL)) {
          i = iter_001->value;
          iter_001 = iter_001->rest;
          if (eqlP(i, object)) {
            return (TRUE);
          }
        }
      }
    }
    return (FALSE);
  }
}

boolean Cons::membP(Object* object) {
  // Return `true' iff `object' is a member of the cons list
  // `self' (uses an `eq?' test).
  { Cons* self = this;

    { Object* i = NULL;
      Cons* iter_000 = self;

      while (!(iter_000 == NIL)) {
        i = iter_000->value;
        iter_000 = iter_000->rest;
        if (i == object) {
          return (TRUE);
        }
      }
    }
    return (FALSE);
  }
}

Cons* cons(Object* value, Cons* rest) {
  // Return a cons record that points to `value' and `rest'.
  { Cons* cons = new Cons();

    cons->value = value;
    cons->rest = rest;
    return (cons);
  }
}

Cons* local_cons(Object* value, Cons* rest) {
  // Return local cons record that points to `value' and `rest'.
  return (cons(value, rest));
}

Cons* Cons::remove(Object* value) {
  // Destructively remove all entries in the cons list `self' that
  // match `value'.  Unless the remaining list is `nil', insure that the cons that
  // heads the list is unchanged.
  { Cons* self = this;

    { Cons* cons = self;
      Cons* previouscons = NULL;
      Cons* matchingcons = NULL;

      if (!(cons == NIL)) {
        if (!(cons->rest == NIL)) {
          previouscons = cons;
          cons = cons->rest;
          while (!(cons == NIL)) {
            if (eqlP(cons->value, value)) {
              {
                previouscons->rest = cons->rest;
                matchingcons = cons;
                cons = cons->rest;
              }
            }
            else {
              {
                previouscons = cons;
                cons = cons->rest;
              }
            }
          }
        }
        if (eqlP(self->value, value)) {
          if (self->rest == NIL) {
            return (NIL);
          }
          cons = self->rest;
          self->value = cons->value;
          self->rest = cons->rest;
        }
      }
      return (self);
    }
  }
}

Cons* Cons::remove_if(cpp_function_code testP) {
  // Destructively removes all members of the cons list
  // `self' for which `test?' evaluates to `true'.  `test' takes a single 
  // argument of type OBJECT and returns `true' or `false'.  Returns a cons list.
  // In case the first element is removed, the return result should be
  // assigned to a variable.
  { Cons* self = this;

    { Cons* cursor = self;
      Cons* trailer = NULL;

      while (!(cursor == NIL)) {
        if (((boolean  (*) (Object*))testP)(cursor->value) &&
            ((boolean)(trailer))) {
          {
            trailer->rest = cursor->rest;
            cursor = trailer->rest;
          }
        }
        else {
          {
            trailer = cursor;
            cursor = cursor->rest;
          }
        }
      }
      cursor = self;
      if ((!(cursor == NIL)) &&
          ((boolean  (*) (Object*))testP)(cursor->value)) {
        trailer = cursor->rest;
        return (trailer);
      }
      return (self);
    }
  }
}

// Point where we switch from using a quadratic remove
// duplicates algorithm to a linear one using a hash table.  For
// an unoptimized Common Lisp, 20 is a good crossover point.
int oREMOVE_DUPLICATES_CROSSOVER_POINTo = 20;

Cons* Cons::remove_duplicates() {
  // Destructively remove duplicates from `self' and return the
  // result.  Removes all but the first occurrence of items in the list.
  // Preserves the original order of the remaining members.  Runs in linear time.
  { Cons* self = this;

    if (self->length() > oREMOVE_DUPLICATES_CROSSOVER_POINTo) {
      return (remove_duplicates_from_long_list(self));
    }
    else {
      { Cons* cursor = self;
        Cons* rest = NULL;

        while (!(cursor == NIL)) {
          rest = cursor->rest;
          if (rest->memberP(cursor->value)) {
            rest = rest->remove(cursor->value);
            cursor->rest = rest;
          }
          cursor = rest;
        }
        return (self);
      }
    }
  }
}

Cons* remove_duplicates_from_long_list(Cons* self) {
  { int tablesize = ceiling(self->length() * 0.3);
    Cons** table = new (GC)(Cons*[tablesize]);
    Cons* cursor = self;
    Object* item = NULL;
    Cons* bucket = NULL;
    int bucketindex = 0;

    { int i = NULL_INTEGER;
      int iter_000 = 0;
      int upper_bound_000 = tablesize - 1;

      while (iter_000 <= upper_bound_000) {
        i = iter_000;
        iter_000 = iter_000 + 1;
        table[i] = NIL;
      }
    }
    while (!(cursor == NIL)) {
      item = cursor->value;
      bucketindex = ((item->hash_code()) % tablesize);
      bucket = table[bucketindex];
      { boolean foundP_000 = FALSE;

        { Object* it = NULL;
          Cons* iter_001 = bucket;

          while (!(iter_001 == NIL)) {
            it = iter_001->value;
            iter_001 = iter_001->rest;
            if (eqlP(it, item)) {
              foundP_000 = TRUE;
              break;
            }
          }
        }
        if (foundP_000) {
          cursor->value = NULL;
        }
        else {
          table[bucketindex] = cons(item, bucket);
        }
      }
      cursor = cursor->rest;
    }
    return (self->remove(NULL));
  }
}

Cons* Cons::concatenate(Cons* list2, int otherlists, ...) {
  // Return a cons list consisting of the concatenation of 
  // `list1', `list2', and `otherLists'.  The operation is destructive wrt all
  // but the last list argument which is left intact.  The two mandatory
  // parameters allow us to optimize the common binary case by not relying on
  // the somewhat less efficient variable arguments mechanism.
  { Cons* list1 = this;

    if (list1 == NIL) {
      list1 = list2;
    }
    else {
      if (list2 == NIL) {
        list2 = list1;
      }
      else {
        last_cons(list1)->rest = list2;
      }
    }
    if (otherlists > 0) {
      { Cons* list = NULL;
        va_list iter_000;
        int iter_000_count = otherlists;

        va_start(iter_000, otherlists);
        while ((iter_000_count--) > 0) {
          list = va_arg(iter_000, Cons*);
          if (!(list == NIL)) {
            if (!(list2 == NIL)) {
              last_cons(list2)->rest = list;
            }
            else {
              list1 = list;
            }
            list2 = list;
          }
        }
        va_end(iter_000);
      }
    }
    return (list1);
  }
}

Cons* Cons::prepend(Cons* list1) {
  // Return a cons list consisting of the concatenation of
  // `list1' and `self'.  A copy of `list1' is prepended to `self'.  This
  // operation results in structure sharing of `self'; to avoid this, `self'
  // should not be pointed to by anything other than the tail of the prepended
  // copy.
  { Cons* self = this;

    if (list1 == NIL) {
      return (self);
    }
    { Cons* copy1 = copy_cons_list(list1);

      copy1->concatenate(self, 0);
      return (copy1);
    }
  }
}

Cons* Cons::reverse() {
  // Destructively reverse the members of the cons list `self'.
  { Cons* self = this;

    if (self == NIL) {
      return (self);
    }
    { Cons* reversedconslist = self;
      Cons* remainingconslist = reversedconslist->rest;
      Cons* detachedcons = NULL;

      if (remainingconslist == NIL) {
        return (reversedconslist);
      }
      reversedconslist->rest = NIL;
      while (!(remainingconslist == NIL)) {
        detachedcons = remainingconslist;
        remainingconslist = remainingconslist->rest;
        detachedcons->rest = reversedconslist;
        reversedconslist = detachedcons;
      }
      return (reversedconslist);
    }
  }
}

Cons* Cons::substitute(Object* invalue, Object* outvalue) {
  // Destructively replace each appearance of `outValue' by
  // `inValue' in the cons list `self'.
  { Cons* self = this;

    { Cons* cursor = self;

      while (!(cursor == NIL)) {
        if (eqlP(cursor->value, outvalue)) {
          cursor->value = invalue;
        }
        cursor = cursor->rest;
      }
      return (self);
    }
  }
}

Cons* map_null_to_nil(Cons* self) {
  // Return `nil' iff `self' is `null' or `self' otherwise.
  return (((!((boolean)(self))) ? NIL : self));
}

Object* Cons::first() {
  // Return the first element of `self'.  The first element
  // of `self' can be set with `setf'.  Note that '(first NIL)' = `null'.
  { Cons* self = this;

    return (self->value);
  }
}

Object* Cons::second() {
  // Return the second element of `self'.  The second element
  // of `self' can be set with `setf'.  Note that '(second NIL)' = `null'.
  { Cons* self = this;

    return (self->rest->value);
  }
}

Object* Cons::third() {
  // Return the third element of `self'.  The third element
  // of `self' can be set with `setf'.  Note that '(third NIL)' = `null'.
  { Cons* self = this;

    return (self->rest->rest->value);
  }
}

Object* Cons::fourth() {
  // Return the fourth element of `self'.  The fourth element
  // of `self' can be set with `setf'.  Note that '(fourth NIL)' = `null'.
  { Cons* self = this;

    return (self->rest->rest->rest->value);
  }
}

Object* Cons::fifth() {
  // Return the fifth element of `self'.  The fifth element
  // of `self' can be set with `setf'.  Note, that '(fifth NIL)' = `null'.
  { Cons* self = this;

    return (self->rest->rest->rest->rest->value);
  }
}

Object* Cons::nth(int position) {
  // Return the element of `self' at `position'.  The nth element
  // of `self' can be set with `setf'.  Note, that '(nth NIL <pos>)' = `null'.
  { Cons* self = this;

    { int i = -1;

      { Object* value = NULL;
        Cons* iter_000 = self;

        while (!(iter_000 == NIL)) {
          value = iter_000->value;
          iter_000 = iter_000->rest;
          if ((i = i + 1) == position) {
            return (value);
          }
        }
      }
      return (NULL);
    }
  }
}

Cons* Cons::nth_rest(int position) {
  // Apply `rest' `position' times to `self'.
  { Cons* self = this;

    if (position < 0) {
      return (NULL);
    }
    { Cons* cursor = self;

      { int i = NULL_INTEGER;
        int iter_000 = 1;
        int upper_bound_000 = position;
        boolean unboundedP_000 = upper_bound_000 == NULL_INTEGER;

        while (unboundedP_000 ||
            (iter_000 <= upper_bound_000)) {
          i = iter_000;
          iter_000 = iter_000 + 1;
          cursor = cursor->rest;
          if (cursor == NIL) {
            return (NIL);
          }
        }
      }
      return (cursor);
    }
  }
}

Cons* last_cons(Cons* self) {
  // Return the last cons of `self'.
  while (!(self->rest == NIL)) {
    self = self->rest;
  }
  return (self);
}

Object* Cons::last() {
  // Return the last element of `self'.
  { Cons* self = this;

    return (last_cons(self)->value);
  }
}

Object* Cons::first_setter(Object* value) {
  { Cons* self = this;

    if (self == NIL) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't apply first setter to empty list." << endl;
      return (NULL);
    }
    return (self->value = value);
  }
}

Object* Cons::second_setter(Object* value) {
  { Cons* self = this;

    if (self->rest == NIL) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "'second setter' applied to too short cons list." << endl;
      return (NULL);
    }
    return (self->rest->value = value);
  }
}

Object* Cons::third_setter(Object* value) {
  { Cons* self = this;

    { Cons* thirdcons = self->rest->rest;

      if (!(thirdcons == NIL)) {
        return (thirdcons->value = value);
      }
      else {
        {
          *(STANDARD_WARNING->native_stream) << "Warning: " << "'third setter' applied to too short cons list." << endl;
          return (NULL);
        }
      }
    }
  }
}

Object* Cons::nth_setter(Object* value, int position) {
  { Cons* self = this;

    if (position < 0) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't apply nth setter to negative position " << "`" << position << "'" << "." << endl;
      return (NULL);
    }
    { Cons* cursor = self;

      { int i = NULL_INTEGER;
        int iter_000 = 1;
        int upper_bound_000 = position;
        boolean unboundedP_000 = upper_bound_000 == NULL_INTEGER;

        while (unboundedP_000 ||
            (iter_000 <= upper_bound_000)) {
          i = iter_000;
          iter_000 = iter_000 + 1;
          i = i;
          cursor = cursor->rest;
          if (cursor == NIL) {
            *(STANDARD_WARNING->native_stream) << "Warning: " << "'nth setter' ran off end of cons list." << endl;
            return (NULL);
          }
        }
      }
      cursor->value = value;
      return (value);
    }
  }
}

Object* Cons::fourth_setter(Object* value) {
  { Cons* self = this;

    return (self->nth_setter(value, 3));
  }
}

Object* Cons::fifth_setter(Object* value) {
  { Cons* self = this;

    return (self->nth_setter(value, 4));
  }
}

Object* Cons::nth_rest_setter(Object* value, int position) {
  { Cons* self = this;

    if (position <= 0) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't apply nth-rest setter to negative position " << "`" << position << "'" << "." << endl;
    }
    { Cons* cursor = self;

      { int i = NULL_INTEGER;
        int iter_000 = 2;
        int upper_bound_000 = position;
        boolean unboundedP_000 = upper_bound_000 == NULL_INTEGER;

        while (unboundedP_000 ||
            (iter_000 <= upper_bound_000)) {
          i = iter_000;
          iter_000 = iter_000 + 1;
          cursor = cursor->rest;
          if (cursor == NIL) {
            *(STANDARD_WARNING->native_stream) << "Warning: " << "'nth-rest setter' ran off end of cons list." << endl;
            return (NULL);
          }
        }
      }
      cursor->rest = ((Cons*)(value));
      return (value);
    }
  }
}

Cons_Iterator* Cons::allocate_iterator() {
  { Cons* self = this;

    { Cons_Iterator* iterator = new_cons_iterator();

      iterator->cons_iterator_cursor = self;
      iterator->first_iterationP = TRUE;
      return (iterator);
    }
  }
}

boolean terminate_cons_iteratorP(Cons_Iterator* self) {
  self->cons_iterator_cursor = NULL;
  return (TRUE);
}

boolean Cons_Iterator::nextP() {
  { Cons_Iterator* self = this;

    if (self->first_iterationP) {
      self->first_iterationP = FALSE;
    }
    else {
      self->cons_iterator_cursor = self->cons_iterator_cursor->rest;
    }
    self->value = self->cons_iterator_cursor->value;
    return (!(self->cons_iterator_cursor == NIL));
  }
}

boolean Cons_Iterator::emptyP() {
  { Cons_Iterator* self = this;

    return ((self->first_iterationP ? (self->cons_iterator_cursor == NIL) : (self->cons_iterator_cursor->rest == NIL)));
  }
}

Object* Cons_Iterator::value_setter(Object* value) {
  { Cons_Iterator* self = this;

    self->cons_iterator_cursor->value = value;
    self->value = value;
    return (value);
  }
}

void add_cons_to_end_of_cons_list(Cons* self, Cons* lastcons) {
  while (!(self->rest == NIL)) {
    self = self->rest;
  }
  self->rest = lastcons;
}

boolean but_last_nextP(All_Purpose_Iterator* self) {
  if (self->first_iterationP) {
    self->first_iterationP = FALSE;
  }
  else {
    self->iterator_cons_list = self->iterator_cons_list->rest;
  }
  if (self->iterator_cons_list->rest == NIL) {
    return (FALSE);
  }
  self->value = self->iterator_cons_list->value;
  return (TRUE);
}

Iterator* Cons::but_last() {
  // Generate all but the last element of the cons list `self'.
  { Cons* self = this;

    { All_Purpose_Iterator* iterator = new_all_purpose_iterator();

      iterator->iterator_cons_list = self;
      iterator->first_iterationP = TRUE;
      iterator->iterator_next_code = ((cpp_function_code)(&but_last_nextP));
      return (iterator);
    }
  }
}

Cons* Cons::consify() {
  // Return `self'.
  { Cons* self = this;

    return (self);
  }
}

Cons* List::consify() {
  // Return a list of elements in `self'.
  { List* self = this;

    return (self->the_cons_list);
  }
}

Cons* Key_Value_List::consify() {
  // Return a list of key-value pairs in `self'.
  { Key_Value_List* self = this;

    { Cons* result = NIL;

      { Object* k = NULL;
        Object* v = NULL;
        Kv_Cons* iter_000 = self->the_kv_list;
        Cons* collect_000 = NULL;

        while (((boolean)(iter_000))) {
          k = iter_000->key;
          v = iter_000->value;
          iter_000 = iter_000->rest;
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(cons(k, cons(v, NIL)), NIL);
              if (result == NIL) {
                result = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(result, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(cons(k, cons(v, NIL)), NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
      return (result);
    }
  }
}

Cons* Vector::consify() {
  // Return a list of elements in `self'.
  { Vector* self = this;

    { Cons* result = NIL;

      { Object* v = NULL;
        Vector* vector_000 = self;
        int index_000 = 0;
        int length_000 = vector_000->length();
        Cons* collect_000 = NULL;

        while (index_000 < length_000) {
          v = (vector_000->the_array)[index_000];
          index_000 = index_000 + 1;
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(v, NIL);
              if (result == NIL) {
                result = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(result, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(v, NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
      return (result);
    }
  }
}

Cons* Iterator::consify() {
  // Return a list of elements generated by `self'.
  { Iterator* self = this;

    { Cons* list = NIL;

      { Object* value = NULL;
        Iterator* iter_000 = self;

        while (iter_000->nextP()) {
          value = iter_000->value;
          list = cons(value, list);
        }
      }
      return (list->reverse());
    }
  }
}

Cons* Object::consify() {
  // If `object' is a CONS, return it.  Otherwise, return
  // a singleton cons list containing it.
  { Object* self = this;

    return ((consP(self) ? ((Cons*)(self)) : cons(self, NIL)));
  }
}

Cons* Cons::remove_deleted_members() {
  { Cons* self = this;

    { Cons* cursor = self;
      Cons* trailer = NULL;

      while (!(cursor == NIL)) {
        if (cursor->value->deletedP() &&
            ((boolean)(trailer))) {
          {
            trailer->rest = cursor->rest;
            cursor = trailer->rest;
          }
        }
        else {
          {
            trailer = cursor;
            cursor = cursor->rest;
          }
        }
      }
      if ((!(self == NIL)) &&
          self->value->deletedP()) {
        return (self->rest);
      }
      return (self);
    }
  }
}

Cons* copy_cons_list(Cons* self) {
  // Return a copy of the cons list `self'.
  if (self == NIL) {
    return (NIL);
  }
  { Cons* newconslist = cons(self->value, NIL);
    Cons* nextcons = newconslist;
    Cons* copyfromcons = NULL;
    Cons* previouscons = NULL;

    copyfromcons = self->rest;
    while (!(copyfromcons == NIL)) {
      previouscons = nextcons;
      nextcons = cons(copyfromcons->value, NIL);
      previouscons->rest = nextcons;
      copyfromcons = copyfromcons->rest;
    }
    return (newconslist);
  }
}

Cons* cons_list(int values, ...) {
  // Return a cons list containing `values', in order.
  { Cons* list = NIL;

    { Object* v = NULL;
      va_list iter_000;
      int iter_000_count = values;
      Cons* collect_000 = NULL;

      va_start(iter_000, values);
      while ((iter_000_count--) > 0) {
        v = va_arg(iter_000, Object*);
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(v, NIL);
            if (list == NIL) {
              list = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(list, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(v, NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
      va_end(iter_000);
    }
    return (list);
  }
}

Cons* listO(int values, ...) {
  // Return a list of conses that make up the list `values',
  // terminated by the last value rather than by `nil'.  Assumes that
  // at least one value is passed in.
  { Cons* headcons = NIL;
    Object* answer = NULL;
    Cons* lastcons = NULL;

    { Object* v = NULL;
      va_list iter_000;
      int iter_000_count = values;

      va_start(iter_000, values);
      while ((iter_000_count--) > 0) {
        v = va_arg(iter_000, Object*);
        headcons = cons(v, headcons);
      }
      va_end(iter_000);
    }
    answer = headcons->rest->reverse();
    if (answer == NIL) {
      answer = headcons->value;
      return (((Cons*)(answer)));
    }
    lastcons = ((Cons*)(answer));
    while (!(lastcons->rest == NIL)) {
      lastcons = lastcons->rest;
    }
    lastcons->rest = ((Cons*)(headcons->value));
    return (((Cons*)(answer)));
  }
}

Cons* append(Cons* conslist1, Cons* conslist2) {
  // Return a cons list representing the concatenation
  // of `consList1' and `consList2'.  The concatenation is NOT destructive.
  if (conslist1 == NIL) {
    return (conslist2);
  }
  else if (conslist2 == NIL) {
    return (conslist1);
  }
  else {
    return (copy_cons_list(conslist1)->concatenate(conslist2, 0));
  }
}

Object* Kv_Cons::lookup(Object* key) {
  { Kv_Cons* self = this;

    while (((boolean)(self))) {
      if (eqlP(self->key, key)) {
        return (self->value);
      }
      self = self->rest;
    }
    return (NULL);
  }
}

boolean Cons::equivalent_setsP(Cons* otherlist) {
  { Cons* self = this;

    { boolean test_value_000 = FALSE;

      { boolean alwaysP_000 = TRUE;

        { Object* item = NULL;
          Cons* iter_000 = self;

          while (!(iter_000 == NIL)) {
            item = iter_000->value;
            iter_000 = iter_000->rest;
            if (!otherlist->memberP(item)) {
              alwaysP_000 = FALSE;
              break;
            }
          }
        }
        test_value_000 = alwaysP_000;
      }
      if (test_value_000) {
        test_value_000 = self->length() == otherlist->length();
      }
      { boolean value_000 = test_value_000;

        return (value_000);
      }
    }
  }
}

Cons* Cons::intersection(Cons* otherlist) {
  { Cons* self = this;

    { Cons* list = NIL;

      if (!((boolean)(otherlist))) {
        return (list);
      }
      { Object* i = NULL;
        Cons* iter_000 = self;
        Cons* collect_000 = NULL;

        while (!(iter_000 == NIL)) {
          i = iter_000->value;
          iter_000 = iter_000->rest;
          if (otherlist->memberP(i)) {
            if (!((boolean)(collect_000))) {
              {
                collect_000 = cons(i, NIL);
                if (list == NIL) {
                  list = collect_000;
                }
                else {
                  add_cons_to_end_of_cons_list(list, collect_000);
                }
              }
            }
            else {
              {
                collect_000->rest = cons(i, NIL);
                collect_000 = collect_000->rest;
              }
            }
          }
        }
      }
      return (list);
    }
  }
}

Cons* Cons::renamed_Union(Cons* otherlist) {
  { Cons* self = this;

    { Cons* list = copy_cons_list(self);
      Cons* othersurvivors = NIL;

      if (!((boolean)(otherlist))) {
        return (list);
      }
      { Object* i = NULL;
        Cons* iter_000 = otherlist;
        Cons* collect_000 = NULL;

        while (!(iter_000 == NIL)) {
          i = iter_000->value;
          iter_000 = iter_000->rest;
          if (!list->memberP(i)) {
            if (!((boolean)(collect_000))) {
              {
                collect_000 = cons(i, NIL);
                if (othersurvivors == NIL) {
                  othersurvivors = collect_000;
                }
                else {
                  add_cons_to_end_of_cons_list(othersurvivors, collect_000);
                }
              }
            }
            else {
              {
                collect_000->rest = cons(i, NIL);
                collect_000 = collect_000->rest;
              }
            }
          }
        }
      }
      return (list->concatenate(othersurvivors, 0));
    }
  }
}

Cons* Cons::difference(Cons* otherlist) {
  { Cons* self = this;

    { Cons* list = copy_cons_list(self);

      if (!((boolean)(otherlist))) {
        return (list);
      }
      { Object* i = NULL;
        Cons* iter_000 = otherlist;

        while (!(iter_000 == NIL)) {
          i = iter_000->value;
          iter_000 = iter_000->rest;
          if (list->memberP(i)) {
            list = list->remove(i);
          }
        }
      }
      return (list);
    }
  }
}

Cons* Cons::subtract(Cons* otherlist) {
  { Cons* self = this;

    if (!((boolean)(otherlist))) {
      return (self);
    }
    { Object* i = NULL;
      Cons* iter_000 = otherlist;

      while (!(iter_000 == NIL)) {
        i = iter_000->value;
        iter_000 = iter_000->rest;
        if (self->memberP(i)) {
          self = self->remove(i);
        }
      }
    }
    return (self);
  }
}

Cons* Cons::sort(cpp_function_code predicate) {
  // Perform a stable, destructive sort of `self' according to
  // `predicate', and return the result.  If `predicate' has a '<' semantics, the
  // result will be in ascending order.  It is not guaranteed that `self' will
  // point to the beginning of the sorted result.  If `predicate' is `null', a
  // suitable '<' predicate is chosen depending on the first element of `self',
  // and it is assumed that all elements of `self' have the same type (supported
  // element types are GENERALIZED-SYMBOL, STRING, INTEGER, and FLOAT).
  { Cons* self = this;

    if (predicate == NULL) {
      predicate = choose_sort_predicate(self);
    }
    return (help_sort_cons_list(self, self->length(), predicate));
  }
}

Cons* help_sort_cons_list(Cons* list, int length, cpp_function_code predicate) {
  if (length < 2) {
    return (list);
  }
  { int frontlength = floor_(length / 2.0);
    Cons* temp = list;
    Cons* back = NIL;

    { int i = NULL_INTEGER;
      int iter_000 = 2;
      int upper_bound_000 = frontlength;
      boolean unboundedP_000 = upper_bound_000 == NULL_INTEGER;

      while (unboundedP_000 ||
          (iter_000 <= upper_bound_000)) {
        i = iter_000;
        iter_000 = iter_000 + 1;
        temp = temp->rest;
      }
    }
    back = temp->rest;
    temp->rest = NIL;
    return (merge_cons_lists(help_sort_cons_list(list, frontlength, predicate), help_sort_cons_list(back, length - frontlength, predicate), predicate));
  }
}

Cons* merge_cons_lists(Cons* list1, Cons* list2, cpp_function_code predicate) {
  { Cons* cursor1 = list1;
    Cons* cursor2 = list2;
    Cons* result = NIL;
    Cons* tail = NIL;
    Cons* temp = NIL;

    for (;;) {
      if (cursor1 == NIL) {
        if (tail == NIL) {
          return (cursor2);
        }
        tail->rest = cursor2;
        return (result);
      }
      if (cursor2 == NIL) {
        if (tail == NIL) {
          return (cursor1);
        }
        tail->rest = cursor1;
        return (result);
      }
      if (((boolean  (*) (Object*, Object*))predicate)(cursor2->value, cursor1->value)) {
        temp = cursor2;
        cursor2 = cursor2->rest;
        list2 = cursor2;
      }
      else {
        temp = cursor1;
        cursor1 = cursor1->rest;
        list1 = cursor1;
      }
      if (tail == NIL) {
        result = temp;
      }
      else {
        tail->rest = temp;
      }
      tail = temp;
    }
  }
}

boolean generalized_symbol_less_thanP(Generalized_Symbol* x, Generalized_Symbol* y) {
  return (stringL(x->symbol_name, y->symbol_name));
}

boolean wrapped_integer_less_thanP(Integer_Wrapper* x, Integer_Wrapper* y) {
  return (x->wrapper_value < y->wrapper_value);
}

boolean wrapped_float_less_thanP(Float_Wrapper* x, Float_Wrapper* y) {
  return (x->wrapper_value < y->wrapper_value);
}

boolean wrapped_string_less_thanP(String_Wrapper* x, String_Wrapper* y) {
  return (stringL(x->wrapper_value, y->wrapper_value));
}

boolean wrapped_mutable_string_less_thanP(Mutable_String_Wrapper* x, Mutable_String_Wrapper* y) {
  return (stringL((x->wrapper_value), (y->wrapper_value)));
}

cpp_function_code choose_sort_predicate(Cons* self) {
  { Object* firstelement = self->value;

    if (!((boolean)(firstelement))) {
      return (NULL);
    }
    { Surrogate* test_value_000 = safe_primary_type(firstelement);

      if (subtype_ofP(test_value_000, SGT_CONSES_STELLA_GENERALIZED_SYMBOL)) {
        { Object* firstelement_000 = firstelement;
          Generalized_Symbol* firstelement = ((Generalized_Symbol*)(firstelement_000));

          return (((cpp_function_code)(&generalized_symbol_less_thanP)));
        }
      }
      else if (subtype_of_integerP(test_value_000)) {
        { Object* firstelement_001 = firstelement;
          Integer_Wrapper* firstelement = ((Integer_Wrapper*)(firstelement_001));

          return (((cpp_function_code)(&wrapped_integer_less_thanP)));
        }
      }
      else if (subtype_of_floatP(test_value_000)) {
        { Object* firstelement_002 = firstelement;
          Float_Wrapper* firstelement = ((Float_Wrapper*)(firstelement_002));

          return (((cpp_function_code)(&wrapped_float_less_thanP)));
        }
      }
      else if (subtype_of_stringP(test_value_000)) {
        { Object* firstelement_003 = firstelement;
          String_Wrapper* firstelement = ((String_Wrapper*)(firstelement_003));

          return (((cpp_function_code)(&wrapped_string_less_thanP)));
        }
      }
      else if (subtype_ofP(test_value_000, SGT_CONSES_STELLA_MUTABLE_STRING_WRAPPER)) {
        { Object* firstelement_004 = firstelement;
          Mutable_String_Wrapper* firstelement = ((Mutable_String_Wrapper*)(firstelement_004));

          return (((cpp_function_code)(&wrapped_mutable_string_less_thanP)));
        }
      }
      else {
        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "choose-sort-predicate: Don't know how to sort " << "`" << firstelement->primary_type() << "'" << "s";
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      }
    }
  }
}

boolean search_cons_treeP(Object* tree, Object* value) {
  // Return `true' iff the value `value' is embedded within
  // the cons tree `tree'.  Uses an `eql?' test.
  if (safe_primary_type(tree) == SGT_CONSES_STELLA_CONS) {
    { Object* tree_000 = tree;
      Cons* tree = ((Cons*)(tree_000));

      if (tree == NIL) {
        return (value == NIL);
      }
      else {
        return (search_cons_treeP(tree->value, value) ||
            search_cons_treeP(tree->rest, value));
      }
    }
  }
  else {
    return (eqlP(tree, value));
  }
}

Object* copy_cons_tree(Object* self) {
  // Return a copy of the cons tree `self'.
  if (safe_primary_type(self) == SGT_CONSES_STELLA_CONS) {
    { Object* self_000 = self;
      Cons* self = ((Cons*)(self_000));

      if (self == NIL) {
        return (NIL);
      }
      else {
        return (cons(copy_cons_tree(self->value), ((Cons*)(copy_cons_tree(self->rest)))));
      }
    }
  }
  else {
    return (self);
  }
}

boolean equal_cons_treesP(Object* tree1, Object* tree2) {
  // Return `true' iff the cons trees `tree1' and `tree2' are
  // structurally equivalent.  Uses an `eql?' test.
  if (eqlP(tree1, tree2)) {
    return (TRUE);
  }
  else {
    if (safe_primary_type(tree1) == SGT_CONSES_STELLA_CONS) {
      { Object* tree1_000 = tree1;
        Cons* tree1 = ((Cons*)(tree1_000));

        if (safe_primary_type(tree2) == SGT_CONSES_STELLA_CONS) {
          { Object* tree2_000 = tree2;
            Cons* tree2 = ((Cons*)(tree2_000));

            return (equal_cons_treesP(tree1->value, tree2->value) &&
                equal_cons_treesP(tree1->rest, tree2->rest));
          }
        }
        else {
          return (FALSE);
        }
      }
    }
    else {
      return (FALSE);
    }
  }
}

boolean Cons::object_equalP(Object* tree2) {
  // Return `true' iff the cons trees `tree1' and `tree2' are
  // structurally equivalent.  Uses `equal?' to test equality of subtrees.
  { Cons* tree1 = this;

    if (safe_primary_type(tree2) == SGT_CONSES_STELLA_CONS) {
      { Object* tree2_000 = tree2;
        Cons* tree2 = ((Cons*)(tree2_000));

        while (!(tree1 == NIL)) {
          if (!equalP(tree1->value, tree2->value)) {
            return (FALSE);
          }
          tree1 = tree1->rest;
          tree2 = tree2->rest;
        }
        return (tree2 == NIL);
      }
    }
    else {
      return (FALSE);
    }
  }
}

Object* substitute_cons_tree(Object* tree, Object* newvalue, Object* oldvalue) {
  // Destructively replace each appearance of `oldValue' by
  // `newValue' in the cons tree `tree'.  Return the tree.  Uses an `eql?' test.
  if (safe_primary_type(tree) == SGT_CONSES_STELLA_CONS) {
    { Object* tree_000 = tree;
      Cons* tree = ((Cons*)(tree_000));

      if (!(tree == NIL)) {
        if (eqlP(tree->value, oldvalue)) {
          tree->value = newvalue;
        }
        else {
          substitute_cons_tree(tree->value, newvalue, oldvalue);
        }
        substitute_cons_tree(tree->rest, newvalue, oldvalue);
      }
    }
  }
  else {
  }
  return (tree);
}

// If `true' conses will be pretty printed.
DEFINE_STELLA_SPECIAL(oPRINTPRETTYpo, boolean , TRUE);

// If `true' conses will be printed as readable Stella code.
DEFINE_STELLA_SPECIAL(oPRINTREADABLYpo, boolean , FALSE);

// Generates an elipsis '...' when found in a
// pretty-printed parse tree.
Symbol* ELIPSIS = NULL;

void print_cons(Cons* tree, ostream* stream, char* lparen, char* rparen) {
  if (oPRINTREADABLYpo.get()) {
    lparen = "(";
    rparen = ")";
  }
  if (oPRINTPRETTYpo.get()) {
    pprint_cons(tree, stream, lparen, rparen);
    return;
  }
  *(stream) << lparen;
  if (!(tree == NIL)) {
    *(stream) << tree->value;
    { Object* element = NULL;
      Cons* iter_000 = tree->rest;

      while (!(iter_000 == NIL)) {
        element = iter_000->value;
        iter_000 = iter_000->rest;
        *(stream) << " ";
        *(stream) << element;
      }
    }
  }
  *(stream) << rparen;
}

void pprint_cons(Cons* tree, ostream* stream, char* lparen, char* rparen) {
  { 
    BIND_STELLA_SPECIAL(oPRINTPRETTYpo, boolean, FALSE);
    print_cons(tree, stream, lparen, rparen);
  }
}

// When `true' pretty-print Stella and translated code.
// Since (Lisp) pretty-printing is somewhat slow, turning this off speeds up
// file translation, but it also makes translated output very unreadable.
DEFINE_STELLA_SPECIAL(oPRINTPRETTYCODEpo, boolean , TRUE);

void print_stella_code(Object* tree, ostream* stream) {
  { 
    BIND_STELLA_SPECIAL(oPRINTPRETTYpo, boolean, oPRINTPRETTYCODEpo.get());
    BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
    *(stream) << tree << endl;
  }
}

void print_stella_definition(Object* tree, ostream* stream) {
  { 
    BIND_STELLA_SPECIAL(oPRINTPRETTYpo, boolean, oPRINTPRETTYCODEpo.get());
    BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
    if (safe_primary_type(tree) == SGT_CONSES_STELLA_CONS) {
      { Object* tree_000 = tree;
        Cons* tree = ((Cons*)(tree_000));

        if (tree->length() >= 3) {
          oPRINTPRETTYpo.set(FALSE);
          *(stream) << "(" << tree->value << " " << tree->rest->value << " " << tree->rest->rest->value;
          oPRINTPRETTYpo.set(oPRINTPRETTYCODEpo.get());
          { Object* form = NULL;
            Cons* iter_000 = tree->nth_rest(3);

            while (!(iter_000 == NIL)) {
              form = iter_000->value;
              iter_000 = iter_000->rest;
              *(stream) << endl << "  " << form;
            }
          }
          *(stream) << ")";
        }
        else {
          *(stream) << tree;
        }
      }
    }
    else {
      *(stream) << tree;
    }
    *(stream) << endl;
  }
}

int tree_size(Object* self) {
  if (safe_primary_type(self) == SGT_CONSES_STELLA_CONS) {
    { Object* self_000 = self;
      Cons* self = ((Cons*)(self_000));

      if (self == NIL) {
        return (0);
      }
      else {
        return (1 + tree_size(self->value) + tree_size(self->rest));
      }
    }
  }
  else {
    return (0);
  }
}

DEFINE_STELLA_SPECIAL(oDEPTHEXCEEDEDpo, boolean , FALSE);

int oDEPTHCUTOFFo = 33;

int safely_compute_tree_size(Cons* tree, int depthcount, int cutoff) {
  { int count = 1;
    Object* value = NULL;

    if (depthcount > oDEPTHCUTOFFo) {
      oDEPTHEXCEEDEDpo.set(TRUE);
      return (1);
    }
    for (;;) {
      value = tree->value;
      if (consP(value)) {
        count = count + safely_compute_tree_size(((Cons*)(tree->value)), depthcount + 1, cutoff);
      }
      if ((!((boolean)(tree->rest))) ||
          ((tree->rest == NIL) ||
           (count > cutoff))) {
        return (count);
      }
      tree = tree->rest;
    }
  }
}

int safe_tree_size(Cons* tree, char*& _return1) {
  { 
    BIND_STELLA_SPECIAL(oDEPTHEXCEEDEDpo, boolean, FALSE);
    { int cutoff = 99;
      int size = safely_compute_tree_size(tree, 0, cutoff);

      { int _return_temp = size;

        _return1 = ((oDEPTHEXCEEDEDpo.get() ||
            (size >= cutoff)) ? (char*)"CIRCULAR" : (char*)"OK");
        return (_return_temp);
      }
    }
  }
}

void help_startup_conses1() {
  {
    define_method_object("(DEFMETHOD (EMPTY? BOOLEAN) ((SELF CONS)) :DOCUMENTATION \"Return `true' iff `self' equals `nil'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (EQ? SELF NIL)))", ((cpp_method_code)(&Cons::emptyP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NON-EMPTY? BOOLEAN) ((SELF CONS)) :DOCUMENTATION \"Return `true' iff `self' is not equal to `nil'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (NOT (EQ? SELF NIL))))", ((cpp_method_code)(&Cons::non_emptyP)), ((cpp_method_code)(NULL)));
    define_function_object("NIL?", "(DEFUN (NIL? BOOLEAN) ((X OBJECT)) :DOCUMENTATION \"Return `true' iff `x' equals `nil'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (EQ? X NIL)))", ((cpp_function_code)(&nilP)), NULL);
    define_method_object("(DEFMETHOD (LENGTH INTEGER) ((SELF CONS)) :DOCUMENTATION \"Return the length of the CONS list `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::length)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (POSITION INTEGER) ((SELF CONS) (OBJECT OBJECT) (START INTEGER)) :DOCUMENTATION \"Return the position of `object' within the cons-list\n`self' (counting from zero); or return `null' if `object' does not occur within \n`self' (uses an `eql?' test).  If `start' was supplied as non-`null', only \nconsider the sublist starting at `start', however, the returned position \nwill always be relative to the entire list.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::position)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (MEMBER? BOOLEAN) ((SELF CONS) (OBJECT OBJECT)) :DOCUMENTATION \"Return `true' iff `object' is a member of the cons list\n`self' (uses an `eql?' test).\" :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::memberP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (MEMB? BOOLEAN) ((SELF CONS) (OBJECT OBJECT)) :DOCUMENTATION \"Return `true' iff `object' is a member of the cons list\n`self' (uses an `eq?' test).\" :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::membP)), ((cpp_method_code)(NULL)));
    define_function_object("CONS", "(DEFUN (CONS CONS) ((VALUE OBJECT) (REST CONS)) :DOCUMENTATION \"Return a cons record that points to `value' and `rest'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&cons)), NULL);
    define_function_object("LOCAL-CONS", "(DEFUN (LOCAL-CONS CONS) ((VALUE OBJECT) (REST CONS)) :DOCUMENTATION \"Return local cons record that points to `value' and `rest'.\")", ((cpp_function_code)(&local_cons)), NULL);
    define_method_object("(DEFMETHOD (REMOVE (LIKE SELF)) ((SELF CONS) (VALUE OBJECT)) :PUBLIC? TRUE :DOCUMENTATION \"Destructively remove all entries in the cons list `self' that\nmatch `value'.  Unless the remaining list is `nil', insure that the cons that\nheads the list is unchanged.\")", ((cpp_method_code)(&Cons::remove)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (REMOVE-IF (LIKE SELF)) ((SELF CONS) (TEST? FUNCTION-CODE)) :PUBLIC? TRUE :DOCUMENTATION \"Destructively removes all members of the cons list\n`self' for which `test?' evaluates to `true'.  `test' takes a single \nargument of type OBJECT and returns `true' or `false'.  Returns a cons list.\nIn case the first element is removed, the return result should be\nassigned to a variable.\")", ((cpp_method_code)(&Cons::remove_if)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (REMOVE-DUPLICATES (LIKE SELF)) ((SELF CONS)) :PUBLIC? TRUE :DOCUMENTATION \"Destructively remove duplicates from `self' and return the\nresult.  Removes all but the first occurrence of items in the list.\nPreserves the original order of the remaining members.  Runs in linear time.\")", ((cpp_method_code)(&Cons::remove_duplicates)), ((cpp_method_code)(NULL)));
    define_function_object("REMOVE-DUPLICATES-FROM-LONG-LIST", "(DEFUN (REMOVE-DUPLICATES-FROM-LONG-LIST (LIKE SELF)) ((SELF CONS)))", ((cpp_function_code)(&remove_duplicates_from_long_list)), NULL);
    define_method_object("(DEFMETHOD (CONCATENATE CONS) ((LIST1 CONS) (LIST2 CONS) |&REST| (OTHERLISTS CONS)) :PUBLIC? TRUE :DOCUMENTATION \"Return a cons list consisting of the concatenation of \n`list1', `list2', and `otherLists'.  The operation is destructive wrt all\nbut the last list argument which is left intact.  The two mandatory\nparameters allow us to optimize the common binary case by not relying on\nthe somewhat less efficient variable arguments mechanism.\")", ((cpp_method_code)(&Cons::concatenate)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (PREPEND CONS) ((SELF CONS) (LIST1 CONS)) :DOCUMENTATION \"Return a cons list consisting of the concatenation of\n`list1' and `self'.  A copy of `list1' is prepended to `self'.  This\noperation results in structure sharing of `self'; to avoid this, `self'\nshould not be pointed to by anything other than the tail of the prepended\ncopy.\")", ((cpp_method_code)(&Cons::prepend)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (REVERSE (LIKE SELF)) ((SELF CONS)) :DOCUMENTATION \"Destructively reverse the members of the cons list `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::reverse)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SUBSTITUTE CONS) ((SELF CONS) (INVALUE OBJECT) (OUTVALUE OBJECT)) :DOCUMENTATION \"Destructively replace each appearance of `outValue' by\n`inValue' in the cons list `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::substitute)), ((cpp_method_code)(NULL)));
    define_function_object("MAP-NULL-TO-NIL", "(DEFUN (MAP-NULL-TO-NIL CONS) ((SELF CONS)) :DOCUMENTATION \"Return `nil' iff `self' is `null' or `self' otherwise.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (CHOOSE (NULL? SELF) NIL SELF)))", ((cpp_function_code)(&map_null_to_nil)), NULL);
    define_method_object("(DEFMETHOD (FIRST (LIKE (ANY-VALUE SELF))) ((SELF CONS)) :DOCUMENTATION \"Return the first element of `self'.  The first element\nof `self' can be set with `setf'.  Note that '(first NIL)' = `null'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VALUE SELF)))", ((cpp_method_code)(&Cons::first)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SECOND (LIKE (ANY-VALUE SELF))) ((SELF CONS)) :DOCUMENTATION \"Return the second element of `self'.  The second element\nof `self' can be set with `setf'.  Note that '(second NIL)' = `null'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VALUE (REST SELF))))", ((cpp_method_code)(&Cons::second)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (THIRD (LIKE (ANY-VALUE SELF))) ((SELF CONS)) :DOCUMENTATION \"Return the third element of `self'.  The third element\nof `self' can be set with `setf'.  Note that '(third NIL)' = `null'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VALUE (REST (REST SELF)))))", ((cpp_method_code)(&Cons::third)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (FOURTH (LIKE (ANY-VALUE SELF))) ((SELF CONS)) :DOCUMENTATION \"Return the fourth element of `self'.  The fourth element\nof `self' can be set with `setf'.  Note that '(fourth NIL)' = `null'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::fourth)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (FIFTH (LIKE (ANY-VALUE SELF))) ((SELF CONS)) :DOCUMENTATION \"Return the fifth element of `self'.  The fifth element\nof `self' can be set with `setf'.  Note, that '(fifth NIL)' = `null'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::fifth)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NTH (LIKE (ANY-VALUE SELF))) ((SELF CONS) (POSITION INTEGER)) :DOCUMENTATION \"Return the element of `self' at `position'.  The nth element\nof `self' can be set with `setf'.  Note, that '(nth NIL <pos>)' = `null'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::nth)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NTH-REST (LIKE SELF)) ((SELF CONS) (POSITION INTEGER)) :DOCUMENTATION \"Apply `rest' `position' times to `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::nth_rest)), ((cpp_method_code)(NULL)));
    define_function_object("LAST-CONS", "(DEFUN (LAST-CONS (CONS OF (LIKE (ANY-VALUE SELF)))) ((SELF CONS)) :DOCUMENTATION \"Return the last cons of `self'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&last_cons)), NULL);
    define_method_object("(DEFMETHOD (LAST (LIKE (ANY-VALUE SELF))) ((SELF CONS)) :DOCUMENTATION \"Return the last element of `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::last)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (FIRST-SETTER (LIKE (ANY-VALUE SELF))) ((SELF CONS) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::first_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SECOND-SETTER (LIKE (ANY-VALUE SELF))) ((SELF CONS) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::second_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (THIRD-SETTER (LIKE (ANY-VALUE SELF))) ((SELF CONS) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::third_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NTH-SETTER (LIKE (ANY-VALUE SELF))) ((SELF CONS) (VALUE (LIKE (ANY-VALUE SELF))) (POSITION INTEGER)) :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::nth_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (FOURTH-SETTER (LIKE (ANY-VALUE SELF))) ((SELF CONS) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::fourth_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (FIFTH-SETTER (LIKE (ANY-VALUE SELF))) ((SELF CONS) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::fifth_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NTH-REST-SETTER OBJECT) ((SELF CONS) (VALUE OBJECT) (POSITION INTEGER)) :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::nth_rest_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (ALLOCATE-ITERATOR (CONS-ITERATOR OF (LIKE (ANY-VALUE SELF)))) ((SELF CONS)) :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::allocate_iterator)), ((cpp_method_code)(NULL)));
    define_function_object("TERMINATE-CONS-ITERATOR?", "(DEFUN (TERMINATE-CONS-ITERATOR? BOOLEAN) ((SELF CONS-ITERATOR)) :PUBLIC? TRUE)", ((cpp_function_code)(&terminate_cons_iteratorP)), NULL);
    define_method_object("(DEFMETHOD (NEXT? BOOLEAN) ((SELF CONS-ITERATOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&Cons_Iterator::nextP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (EMPTY? BOOLEAN) ((SELF CONS-ITERATOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&Cons_Iterator::emptyP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (VALUE-SETTER OBJECT) ((SELF CONS-ITERATOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&Cons_Iterator::value_setter)), ((cpp_method_code)(NULL)));
    define_function_object("ADD-CONS-TO-END-OF-CONS-LIST", "(DEFUN ADD-CONS-TO-END-OF-CONS-LIST ((SELF CONS) (LASTCONS CONS)) :PUBLIC? TRUE)", ((cpp_function_code)(&add_cons_to_end_of_cons_list)), NULL);
    define_function_object("BUT-LAST-NEXT?", "(DEFUN (BUT-LAST-NEXT? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)) :PUBLIC? TRUE)", ((cpp_function_code)(&but_last_nextP)), NULL);
    define_method_object("(DEFMETHOD (BUT-LAST (ITERATOR OF (LIKE (ANY-VALUE SELF)))) ((SELF CONS)) :DOCUMENTATION \"Generate all but the last element of the cons list `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::but_last)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (CONSIFY (CONS OF (LIKE (ANY-VALUE SELF)))) ((SELF CONS)) :DOCUMENTATION \"Return `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::consify)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (CONSIFY (CONS OF (LIKE (ANY-VALUE SELF)))) ((SELF LIST)) :DOCUMENTATION \"Return a list of elements in `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::consify)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (CONSIFY (CONS OF (LIKE (ANY-VALUE SELF)))) ((SELF KEY-VALUE-LIST)) :DOCUMENTATION \"Return a list of key-value pairs in `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Key_Value_List::consify)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (CONSIFY (CONS OF (LIKE (ANY-VALUE SELF)))) ((SELF VECTOR)) :DOCUMENTATION \"Return a list of elements in `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Vector::consify)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (CONSIFY (CONS OF (LIKE (ANY-VALUE SELF)))) ((SELF ITERATOR)) :DOCUMENTATION \"Return a list of elements generated by `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Iterator::consify)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (CONSIFY CONS) ((SELF OBJECT)) :DOCUMENTATION \"If `object' is a CONS, return it.  Otherwise, return\na singleton cons list containing it.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Object::consify)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (REMOVE-DELETED-MEMBERS (LIKE SELF)) ((SELF CONS)) :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::remove_deleted_members)), ((cpp_method_code)(NULL)));
    define_function_object("COPY-CONS-LIST", "(DEFUN (COPY-CONS-LIST (LIKE SELF)) ((SELF CONS)) :DOCUMENTATION \"Return a copy of the cons list `self'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&copy_cons_list)), NULL);
    define_function_object("CONS-LIST", "(DEFUN (CONS-LIST CONS) (|&REST| (VALUES OBJECT)) :DOCUMENTATION \"Return a cons list containing `values', in order.\" :PUBLIC? TRUE)", ((cpp_function_code)(&cons_list)), NULL);
    define_function_object("LIST*", "(DEFUN (LIST* CONS) (|&REST| (VALUES OBJECT)) :DOCUMENTATION \"Return a list of conses that make up the list `values',\nterminated by the last value rather than by `nil'.  Assumes that\nat least one value is passed in.\" :PUBLIC? TRUE)", ((cpp_function_code)(&listO)), NULL);
    define_function_object("APPEND", "(DEFUN (APPEND CONS) ((CONSLIST1 CONS) (CONSLIST2 CONS)) :DOCUMENTATION \"Return a cons list representing the concatenation\nof `consList1' and `consList2'.  The concatenation is NOT destructive.\" :PUBLIC? TRUE)", ((cpp_function_code)(&append)), NULL);
    define_method_object("(DEFMETHOD (LOOKUP OBJECT) ((SELF KV-CONS) (KEY OBJECT)))", ((cpp_method_code)(&Kv_Cons::lookup)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (EQUIVALENT-SETS? BOOLEAN) ((SELF CONS) (OTHERLIST CONS)))", ((cpp_method_code)(&Cons::equivalent_setsP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (INTERSECTION CONS) ((SELF CONS) (OTHERLIST CONS)))", ((cpp_method_code)(&Cons::intersection)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (UNION CONS) ((SELF CONS) (OTHERLIST CONS)))", ((cpp_method_code)(&Cons::renamed_Union)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (DIFFERENCE CONS) ((SELF CONS) (OTHERLIST CONS)))", ((cpp_method_code)(&Cons::difference)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SUBTRACT CONS) ((SELF CONS) (OTHERLIST CONS)))", ((cpp_method_code)(&Cons::subtract)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SORT (CONS OF (LIKE (ANY-VALUE SELF)))) ((SELF CONS) (PREDICATE FUNCTION-CODE)) :PUBLIC? TRUE :DOCUMENTATION \"Perform a stable, destructive sort of `self' according to\n`predicate', and return the result.  If `predicate' has a '<' semantics, the\nresult will be in ascending order.  It is not guaranteed that `self' will\npoint to the beginning of the sorted result.  If `predicate' is `null', a\nsuitable '<' predicate is chosen depending on the first element of `self',\nand it is assumed that all elements of `self' have the same type (supported\nelement types are GENERALIZED-SYMBOL, STRING, INTEGER, and FLOAT).\")", ((cpp_method_code)(&Cons::sort)), ((cpp_method_code)(NULL)));
  }
}

void startup_conses() {
  if (current_startup_time_phaseP(0)) {
    if (!(((boolean)(NIL)))) {
      NIL = new Cons();
      NIL->value = NULL;
      NIL->rest = NIL;
    }
  }
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      SGT_CONSES_STELLA_GENERALIZED_SYMBOL = ((Surrogate*)(intern_rigid_symbol_wrt_module("GENERALIZED-SYMBOL", NULL, 1)));
      SGT_CONSES_STELLA_MUTABLE_STRING_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("MUTABLE-STRING-WRAPPER", NULL, 1)));
      SGT_CONSES_STELLA_CONS = ((Surrogate*)(intern_rigid_symbol_wrt_module("CONS", NULL, 1)));
      SYM_CONSES_STELLA_STARTUP_CONSES = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-CONSES", NULL, 0)));
      SYM_CONSES_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      help_startup_conses1();
      define_function_object("HELP-SORT-CONS-LIST", "(DEFUN (HELP-SORT-CONS-LIST CONS) ((LIST CONS) (LENGTH INTEGER) (PREDICATE FUNCTION-CODE)))", ((cpp_function_code)(&help_sort_cons_list)), NULL);
      define_function_object("MERGE-CONS-LISTS", "(DEFUN (MERGE-CONS-LISTS CONS) ((LIST1 CONS) (LIST2 CONS) (PREDICATE FUNCTION-CODE)))", ((cpp_function_code)(&merge_cons_lists)), NULL);
      define_function_object("GENERALIZED-SYMBOL-LESS-THAN?", "(DEFUN (GENERALIZED-SYMBOL-LESS-THAN? BOOLEAN) ((X GENERALIZED-SYMBOL) (Y GENERALIZED-SYMBOL)) :PUBLIC? TRUE)", ((cpp_function_code)(&generalized_symbol_less_thanP)), NULL);
      define_function_object("WRAPPED-INTEGER-LESS-THAN?", "(DEFUN (WRAPPED-INTEGER-LESS-THAN? BOOLEAN) ((X INTEGER-WRAPPER) (Y INTEGER-WRAPPER)) :PUBLIC? TRUE)", ((cpp_function_code)(&wrapped_integer_less_thanP)), NULL);
      define_function_object("WRAPPED-FLOAT-LESS-THAN?", "(DEFUN (WRAPPED-FLOAT-LESS-THAN? BOOLEAN) ((X FLOAT-WRAPPER) (Y FLOAT-WRAPPER)) :PUBLIC? TRUE)", ((cpp_function_code)(&wrapped_float_less_thanP)), NULL);
      define_function_object("WRAPPED-STRING-LESS-THAN?", "(DEFUN (WRAPPED-STRING-LESS-THAN? BOOLEAN) ((X STRING-WRAPPER) (Y STRING-WRAPPER)) :PUBLIC? TRUE)", ((cpp_function_code)(&wrapped_string_less_thanP)), NULL);
      define_function_object("WRAPPED-MUTABLE-STRING-LESS-THAN?", "(DEFUN (WRAPPED-MUTABLE-STRING-LESS-THAN? BOOLEAN) ((X MUTABLE-STRING-WRAPPER) (Y MUTABLE-STRING-WRAPPER)) :PUBLIC? TRUE)", ((cpp_function_code)(&wrapped_mutable_string_less_thanP)), NULL);
      define_function_object("CHOOSE-SORT-PREDICATE", "(DEFUN (CHOOSE-SORT-PREDICATE FUNCTION-CODE) ((SELF CONS)))", ((cpp_function_code)(&choose_sort_predicate)), NULL);
      define_function_object("SEARCH-CONS-TREE?", "(DEFUN (SEARCH-CONS-TREE? BOOLEAN) ((TREE OBJECT) (VALUE OBJECT)) :DOCUMENTATION \"Return `true' iff the value `value' is embedded within\nthe cons tree `tree'.  Uses an `eql?' test.\" :PUBLIC? TRUE)", ((cpp_function_code)(&search_cons_treeP)), NULL);
      define_function_object("COPY-CONS-TREE", "(DEFUN (COPY-CONS-TREE (LIKE SELF)) ((SELF OBJECT)) :DOCUMENTATION \"Return a copy of the cons tree `self'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&copy_cons_tree)), NULL);
      define_function_object("EQUAL-CONS-TREES?", "(DEFUN (EQUAL-CONS-TREES? BOOLEAN) ((TREE1 OBJECT) (TREE2 OBJECT)) :DOCUMENTATION \"Return `true' iff the cons trees `tree1' and `tree2' are\nstructurally equivalent.  Uses an `eql?' test.\" :PUBLIC? TRUE)", ((cpp_function_code)(&equal_cons_treesP)), NULL);
      define_method_object("(DEFMETHOD (OBJECT-EQUAL? BOOLEAN) ((TREE1 CONS) (TREE2 OBJECT)) :DOCUMENTATION \"Return `true' iff the cons trees `tree1' and `tree2' are\nstructurally equivalent.  Uses `equal?' to test equality of subtrees.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::object_equalP)), ((cpp_method_code)(NULL)));
      define_function_object("SUBSTITUTE-CONS-TREE", "(DEFUN (SUBSTITUTE-CONS-TREE OBJECT) ((TREE OBJECT) (NEWVALUE OBJECT) (OLDVALUE OBJECT)) :DOCUMENTATION \"Destructively replace each appearance of `oldValue' by\n`newValue' in the cons tree `tree'.  Return the tree.  Uses an `eql?' test.\" :PUBLIC? TRUE)", ((cpp_function_code)(&substitute_cons_tree)), NULL);
      define_function_object("PRINT-CONS", "(DEFUN PRINT-CONS ((TREE CONS) (STREAM NATIVE-OUTPUT-STREAM) (LPAREN STRING) (RPAREN STRING)))", ((cpp_function_code)(&print_cons)), NULL);
      define_function_object("PPRINT-CONS", "(DEFUN PPRINT-CONS ((TREE CONS) (STREAM NATIVE-OUTPUT-STREAM) (LPAREN STRING) (RPAREN STRING)))", ((cpp_function_code)(&pprint_cons)), NULL);
      define_function_object("PRINT-STELLA-CODE", "(DEFUN PRINT-STELLA-CODE ((TREE OBJECT) (STREAM NATIVE-OUTPUT-STREAM)))", ((cpp_function_code)(&print_stella_code)), NULL);
      define_function_object("PRINT-STELLA-DEFINITION", "(DEFUN PRINT-STELLA-DEFINITION ((TREE OBJECT) (STREAM NATIVE-OUTPUT-STREAM)) :PUBLIC? TRUE)", ((cpp_function_code)(&print_stella_definition)), NULL);
      define_function_object("TREE-SIZE", "(DEFUN (TREE-SIZE INTEGER) ((SELF OBJECT)))", ((cpp_function_code)(&tree_size)), NULL);
      define_function_object("SAFELY-COMPUTE-TREE-SIZE", "(DEFUN (SAFELY-COMPUTE-TREE-SIZE INTEGER) ((TREE CONS) (DEPTHCOUNT INTEGER) (CUTOFF INTEGER)))", ((cpp_function_code)(&safely_compute_tree_size)), NULL);
      define_function_object("SAFE-TREE-SIZE", "(DEFUN (SAFE-TREE-SIZE INTEGER STRING) ((TREE CONS)))", ((cpp_function_code)(&safe_tree_size)), NULL);
      define_function_object("STARTUP-CONSES", "(DEFUN STARTUP-CONSES () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_conses)), NULL);
      { Method_Slot* function = lookup_function(SYM_CONSES_STELLA_STARTUP_CONSES);

        set_dynamic_slot_value(function->dynamic_slots, SYM_CONSES_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Conses"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL NIL CONS NULL :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *REMOVE-DUPLICATES-CROSSOVER-POINT* INTEGER 20 :DOCUMENTATION \"Point where we switch from using a quadratic remove\nduplicates algorithm to a linear one using a hash table.  For\nan unoptimized Common Lisp, 20 is a good crossover point.\")");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *PRINTPRETTY?* BOOLEAN TRUE :DOCUMENTATION \"If `true' conses will be pretty printed.\" :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *PRINTREADABLY?* BOOLEAN FALSE :DOCUMENTATION \"If `true' conses will be printed as readable Stella code.\" :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL ELIPSIS SYMBOL NULL :DOCUMENTATION \"Generates an elipsis '...' when found in a\npretty-printed parse tree.\" :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *PRINTPRETTYCODE?* BOOLEAN TRUE :DOCUMENTATION \"When `true' pretty-print Stella and translated code.\nSince (Lisp) pretty-printing is somewhat slow, turning this off speeds up\nfile translation, but it also makes translated output very unreadable.\" :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *DEPTHEXCEEDED?* BOOLEAN FALSE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *DEPTHCUTOFF* INTEGER 33)");
    }
  }
}

Surrogate* SGT_CONSES_STELLA_GENERALIZED_SYMBOL = NULL;

Surrogate* SGT_CONSES_STELLA_MUTABLE_STRING_WRAPPER = NULL;

Surrogate* SGT_CONSES_STELLA_CONS = NULL;

Symbol* SYM_CONSES_STELLA_STARTUP_CONSES = NULL;

Symbol* SYM_CONSES_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
