// cpp-class-out.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

Cons* cpp_yield_initial_null_value() {
  return (((Cons*)(cpp_translate_a_tree(NULL))));
}

Cons* cpp_yield_member_var_tree(Storage_Slot* slot, Class* renamed_Class) {
  { Symbol* slotname = true_slot_name(slot->slot_name, renamed_Class->class_type);
    Standard_Object* slottype = (slot->slot_context_sensitiveP ? SGT_CPP_CLASS_OUT_STELLA_OBJECT : ((Standard_Object*)(slot->compute_return_type_spec(renamed_Class->class_type))));
    Keyword* allocation = slot->allocation();
    Cons* allocationform = NIL;

    if (allocation == KWD_CPP_CLASS_OUT_EMBEDDED) {
      allocationform = NIL;
    }
    else if (allocation == KWD_CPP_CLASS_OUT_INSTANCE) {
      allocationform = NIL;
    }
    else if (allocation == KWD_CPP_CLASS_OUT_CLASS) {
      allocationform = cons_list(1, wrap_string("static"));
    }
    else {
      allocationform = NIL;
    }
    return (listO(3, allocationform->concatenate(cons(cpp_translate_and_pointerize_type_spec(slottype), NIL), 0), cpp_translate_name(slotname), cons(SYM_CPP_CLASS_OUT_STELLA_NULL, NIL)));
  }
}

Cons* cpp_yield_parameterized_member_var_tree(Storage_Slot* slot, Class* renamed_Class) {
  { Symbol* slotname = true_slot_name(slot->slot_name, renamed_Class->class_type);
    Keyword* allocation = slot->allocation();
    Cons* allocationform = NIL;

    if (allocation == KWD_CPP_CLASS_OUT_EMBEDDED) {
      allocationform = NIL;
    }
    else if (allocation == KWD_CPP_CLASS_OUT_INSTANCE) {
      allocationform = NIL;
    }
    else if (allocation == KWD_CPP_CLASS_OUT_CLASS) {
      allocationform = cons_list(1, wrap_string("static"));
    }
    else {
      allocationform = NIL;
    }
    return (listO(3, allocationform->concatenate(cons(cpp_translate_class_parameter_name(get_slot_class_parameter_name(slot)), NIL), 0), cpp_translate_name(slotname), cons(SYM_CPP_CLASS_OUT_STELLA_NULL, NIL)));
  }
}

Cons* cpp_yield_static_member_allocation_tree(Storage_Slot* slot, Class* renamed_Class) {
  { Symbol* slotname = slot->slot_name;
    Cons* allocationform = NIL;
    Slot* mostgeneralslot = NULL;

    mostgeneralslot = slot;
    for (;;) {
      if (((boolean)(mostgeneralslot->slot_direct_equivalent))) {
        mostgeneralslot = mostgeneralslot->slot_direct_equivalent;
      }
      else {
        break;
      }
    }
    return (listO(3, allocationform->concatenate(cons(cpp_translate_and_pointerize_type_spec(mostgeneralslot->compute_return_type_spec(renamed_Class->class_type)), NIL), 0), cpp_translate_name(slotname), cons(SYM_CPP_CLASS_OUT_STELLA_NULL, NIL)));
  }
}

Cons* cpp_yield_member_func_signature_tree(Method_Slot* method) {
  oMETHODBEINGWALKEDo.set(method);
  return (cons((method->method_functionP ? NULL : cpp_translate_class_name(intern_symbol_in_module(method->slot_owner->symbol_name, ((Module*)(method->slot_owner->home_context)), TRUE))), listO(3, cons(wrap_string("virtual"), cons(cpp_translate_and_pointerize_type_spec(compute_most_general_return_type(method, method->compute_return_type_spec(method->slot_owner))), NIL)), cpp_translate_function_name(method), cons(cpp_translate_function_parameters(method), NIL))));
}

Cons* cpp_yield_constructor_signature_tree(Method_Slot* method) {
  oMETHODBEINGWALKEDo.set(method);
  return (cons(cpp_translate_class_name(intern_symbol_in_module(method->slot_owner->symbol_name, ((Module*)(method->slot_owner->home_context)), TRUE)), listO(3, NIL, cpp_translate_constructor_name(method), cons(cpp_translate_function_parameters(method), NIL))));
}

Cons* cpp_yield_exception_constructor_definition(Class* renamed_Class) {
  { Surrogate* value_000 = NULL;

    { Surrogate* super = NULL;
      Cons* iter_000 = class_native_supers(renamed_Class)->the_cons_list;

      while (!(iter_000 == NIL)) {
        super = ((Surrogate*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (!((Class*)(super->surrogate_value))->mixinP) {
          value_000 = super;
          break;
        }
      }
    }
    { Surrogate* super = value_000;

      return (listO(4, SYM_CPP_CLASS_OUT_STELLA_CPP_FUNCTION, NIL, cpp_translate_class_name(renamed_Class->class_type), listO(4, cons(listO(3, wrap_string("const string&"), wrap_string("msg"), NIL), NIL), NIL, cons(listO(4, SYM_CPP_CLASS_OUT_STELLA_CPP_FUNCTION_CALL, listO(3, SYM_CPP_CLASS_OUT_STELLA_CPP_IDENT, cpp_translate_class_name(super), NIL), listO(3, SYM_CPP_CLASS_OUT_STELLA_CPP_ACTUALS, listO(3, SYM_CPP_CLASS_OUT_STELLA_CPP_IDENT, wrap_string("msg"), NIL), NIL), NIL), NIL), NIL)));
    }
  }
}

boolean slot_has_class_parameter_typeP(Storage_Slot* slot, Class* renamed_Class) {
  return (((boolean)(((Compound_Type_Specifier*)(dynamic_slot_value(slot->dynamic_slots, SYM_CPP_CLASS_OUT_STELLA_SLOT_TYPE_SPECIFIER, NULL))))) &&
      (isaP(((Compound_Type_Specifier*)(dynamic_slot_value(slot->dynamic_slots, SYM_CPP_CLASS_OUT_STELLA_SLOT_TYPE_SPECIFIER, NULL))), SGT_CPP_CLASS_OUT_STELLA_ANCHORED_TYPE_SPECIFIER) &&
       (slot->compute_return_type_spec(renamed_Class->class_type) == SGT_CPP_CLASS_OUT_STELLA_UNKNOWN)));
}

Symbol* get_slot_class_parameter_name(Storage_Slot* slot) {
  return (((Anchored_Type_Specifier*)(((Compound_Type_Specifier*)(dynamic_slot_value(slot->dynamic_slots, SYM_CPP_CLASS_OUT_STELLA_SLOT_TYPE_SPECIFIER, NULL)))))->specifier_parameter_name);
}

boolean cpp_native_methodP(Method_Slot* methodslot) {
  { Symbol* test_value_000 = methodslot->slot_name;

    if ((test_value_000 == SYM_CPP_CLASS_OUT_STELLA_DEFINEDp) ||
        (test_value_000 == SYM_CPP_CLASS_OUT_STELLA_NULLp)) {
      return (TRUE);
    }
    else {
      return (FALSE);
    }
  }
}

Cons* cpp_get_parameterized_member_variable_definitions(Class* renamed_Class) {
  { Cons* membervardefs = NIL;

    { Slot* slot = NULL;
      Cons* iter_000 = renamed_Class->class_local_slots->the_cons_list;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        slot = ((Slot*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (storage_slotP(slot) &&
            (native_slotP(((Storage_Slot*)(slot))) &&
             slot_has_class_parameter_typeP(((Storage_Slot*)(slot)), renamed_Class))) {
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(cpp_yield_parameterized_member_var_tree(((Storage_Slot*)(slot)), renamed_Class), NIL);
              if (membervardefs == NIL) {
                membervardefs = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(membervardefs, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(cpp_yield_parameterized_member_var_tree(((Storage_Slot*)(slot)), renamed_Class), NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
    }
    return (membervardefs);
  }
}

Cons* cpp_get_member_variable_definitions(Class* renamed_Class, Cons*& _return1) {
  { Cons* publicmembervardefs = NIL;
    Cons* protectedmembervardefs = NIL;

    { Slot* slot = NULL;
      Iterator* iter_000 = renamed_Class->class_slots();

      while (iter_000->nextP()) {
        slot = ((Slot*)(iter_000->value));
        if (storage_slotP(slot) &&
            (native_slotP(((Storage_Slot*)(slot))) &&
             (native_slot_home(slot, renamed_Class) == renamed_Class))) {
          if (slot->slot_publicP) {
            publicmembervardefs = cons(cpp_yield_member_var_tree(((Storage_Slot*)(slot)), renamed_Class), publicmembervardefs);
          }
          else {
            protectedmembervardefs = cons(cpp_yield_member_var_tree(((Storage_Slot*)(slot)), renamed_Class), protectedmembervardefs);
          }
        }
      }
    }
    { Cons* _return_temp = publicmembervardefs->reverse();

      _return1 = protectedmembervardefs->reverse();
      return (_return_temp);
    }
  }
}

Cons* cpp_get_static_variable_definitions(Class* renamed_Class) {
  { Cons* staticmembervardefs = NIL;

    { Slot* slot = NULL;
      Iterator* iter_000 = renamed_Class->class_slots();

      while (iter_000->nextP()) {
        slot = ((Slot*)(iter_000->value));
        if (storage_slotP(slot) &&
            (native_slotP(((Storage_Slot*)(slot))) &&
             ((((Storage_Slot*)(slot))->allocation() == KWD_CPP_CLASS_OUT_CLASS) &&
              ((!((Storage_Slot*)(slot))->slot_hardwiredP) &&
               ((!slot->slot_markedP) &&
                (native_slot_home(slot, renamed_Class) == renamed_Class)))))) {
          staticmembervardefs = cons(cpp_yield_static_member_allocation_tree(((Storage_Slot*)(slot)), renamed_Class), staticmembervardefs);
        }
      }
    }
    return (staticmembervardefs);
  }
}

Cons* cpp_get_constructor_definitions(Class* renamed_Class) {
  { Cons* constructordefs = NIL;

    { Slot* slot = NULL;
      Iterator* iter_000 = renamed_Class->class_slots();
      Cons* collect_000 = NULL;

      while (iter_000->nextP()) {
        slot = ((Slot*)(iter_000->value));
        if ((!slot->slot_markedP) &&
            ((slot->primary_type() == SGT_CPP_CLASS_OUT_STELLA_METHOD_SLOT) &&
             (local_slotP(slot, renamed_Class) &&
              cpp_constructorP(((Method_Slot*)(slot)))))) {
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(cpp_yield_constructor_signature_tree(((Method_Slot*)(slot))), NIL);
              if (constructordefs == NIL) {
                constructordefs = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(constructordefs, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(cpp_yield_constructor_signature_tree(((Method_Slot*)(slot))), NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
    }
    return (constructordefs);
  }
}

Cons* cpp_get_method_definitions(Class* renamed_Class, Cons*& _return1) {
  { Cons* publicmemberfuncdefs = NIL;
    Cons* protectedmemberfuncdefs = NIL;

    { Slot* slot = NULL;
      Cons* iter_000 = renamed_Class->class_local_slots->the_cons_list;

      while (!(iter_000 == NIL)) {
        slot = ((Slot*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (subtype_of_method_slotP(safe_primary_type(slot))) {
          { Slot* slot_000 = slot;
            Method_Slot* slot = ((Method_Slot*)(slot_000));

            if ((!slot->slot_markedP) &&
                (native_methodP(slot) &&
                 ((!cpp_constructorP(slot)) &&
                  (!cpp_native_methodP(slot))))) {
              if (slot->slot_publicP) {
                publicmemberfuncdefs = cons(cpp_yield_member_func_signature_tree(slot), publicmemberfuncdefs);
              }
              else {
                protectedmemberfuncdefs = cons(cpp_yield_member_func_signature_tree(slot), protectedmemberfuncdefs);
              }
            }
          }
        }
        else {
        }
      }
    }
    { Method_Slot* method = NULL;
      Cons* iter_001 = renamed_Class->class_auxiliary_methods_reader()->the_cons_list;

      while (!(iter_001 == NIL)) {
        method = ((Method_Slot*)(iter_001->value));
        iter_001 = iter_001->rest;
        if (method->slot_publicP) {
          publicmemberfuncdefs = cons(cpp_yield_member_func_signature_tree(method), publicmemberfuncdefs);
        }
        else {
          protectedmemberfuncdefs = cons(cpp_yield_member_func_signature_tree(method), protectedmemberfuncdefs);
        }
      }
    }
    { Cons* _return_temp = publicmemberfuncdefs->reverse();

      _return1 = protectedmemberfuncdefs->reverse();
      return (_return_temp);
    }
  }
}

void mark_renaming_slots(Class* renamed_Class) {
  { Slot* slot = NULL;
    Iterator* iter_000 = renamed_Class->class_slots();

    while (iter_000->nextP()) {
      slot = ((Slot*)(iter_000->value));
      slot->slot_markedP = FALSE;
    }
  }
  { Slot* slot = NULL;
    Iterator* iter_001 = renamed_Class->class_slots();

    while (iter_001->nextP()) {
      slot = ((Slot*)(iter_001->value));
      if (!slot->slot_markedP) {
        if (((boolean)(slot->slot_direct_equivalent)) &&
            (!slot->slot_direct_equivalent->slot_renamedP)) {
          mark_direct_equivalent_slot(slot);
          slot->slot_markedP = FALSE;
        }
      }
    }
  }
}

boolean cpp_class_has_only_native_parentsP(Class* renamed_Class) {
  { Class* super = NULL;
    Cons* iter_000 = renamed_Class->class_all_super_classes;

    while (!(iter_000 == NIL)) {
      super = ((Class*)(iter_000->value));
      iter_000 = iter_000->rest;
      if (!(super->mixinP)) {
        if (create_native_classP(super) &&
            (((String_Wrapper*)(dynamic_slot_value(super->dynamic_slots, SYM_CPP_CLASS_OUT_STELLA_CLASS_CPP_NATIVE_TYPE, NULL_STRING_WRAPPER)))->wrapper_value == NULL)) {
          return (FALSE);
        }
      }
    }
  }
  return (TRUE);
}

Cons* cpp_translate_define_native_class_unit(Translation_Unit* unit) {
  { Class* renamed_Class = ((Class*)(unit->the_object));
    Cons* baseclasses = NIL;
    Cons* parameterizedmembervardefs = NIL;
    Cons* publicmembervardefs = NIL;
    Cons* protectedmembervardefs = NIL;
    Cons* publicmemberfuncdefs = NIL;
    Cons* protectedmemberfuncdefs = NIL;
    Cons* constructordefs = NIL;
    Cons* parameters = NIL;
    Cons* staticmembervardefs = NIL;

    { Surrogate* super = NULL;
      Cons* iter_000 = class_native_supers(renamed_Class)->the_cons_list;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        super = ((Surrogate*)(iter_000->value));
        iter_000 = iter_000->rest;
        if ((!((Class*)(super->surrogate_value))->mixinP) ||
            (!translate_to_single_inheritance_languageP())) {
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(cpp_translate_class_name(super), NIL);
              if (baseclasses == NIL) {
                baseclasses = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(baseclasses, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(cpp_translate_class_name(super), NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
    }
    if (((baseclasses == NIL) ||
        cpp_class_has_only_native_parentsP(renamed_Class)) &&
        ((!renamed_Class->mixinP) &&
         ((oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_CPP_CLASS_OUT_CPP) &&
          cpp_use_garbage_collectorP()))) {
      baseclasses = cons(cpp_translate_name(SYM_CPP_CLASS_OUT_STELLA_GC), baseclasses);
    }
    { Symbol* parameter = NULL;
      Cons* iter_001 = renamed_Class->class_parameters_reader()->the_cons_list;
      Cons* collect_001 = NULL;

      while (!(iter_001 == NIL)) {
        parameter = ((Symbol*)(iter_001->value));
        iter_001 = iter_001->rest;
        if (!((boolean)(collect_001))) {
          {
            collect_001 = cons(cpp_translate_class_parameter_name(parameter), NIL);
            if (parameters == NIL) {
              parameters = collect_001;
            }
            else {
              add_cons_to_end_of_cons_list(parameters, collect_001);
            }
          }
        }
        else {
          {
            collect_001->rest = cons(cpp_translate_class_parameter_name(parameter), NIL);
            collect_001 = collect_001->rest;
          }
        }
      }
    }
    mark_renaming_slots(renamed_Class);
    if (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_CPP_CLASS_OUT_CPP_STANDALONE) {
      constructordefs = cpp_get_constructor_definitions(renamed_Class);
    }
    if (cpp_output_template_classesP()) {
      parameterizedmembervardefs = cpp_get_parameterized_member_variable_definitions(renamed_Class);
    }
    publicmembervardefs = cpp_get_member_variable_definitions(renamed_Class, protectedmembervardefs);
    publicmemberfuncdefs = cpp_get_method_definitions(renamed_Class, protectedmemberfuncdefs);
    staticmembervardefs = cpp_get_static_variable_definitions(renamed_Class);
    return (listO(3, SYM_CPP_CLASS_OUT_STELLA_CPP_CLASS, cpp_translate_class_name(intern_symbol_in_module(renamed_Class->class_type->symbol_name, ((Module*)(renamed_Class->class_type->home_context)), TRUE)), cons(parameters, cons(baseclasses, cons(staticmembervardefs, (((((String_Wrapper*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CPP_CLASS_OUT_STELLA_DOCUMENTATION, NULL_STRING_WRAPPER)))->wrapper_value != NULL) ? cons(listO(3, SYM_CPP_CLASS_OUT_STELLA_CPP_COMMENT, wrap_string(((String_Wrapper*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_CPP_CLASS_OUT_STELLA_DOCUMENTATION, NULL_STRING_WRAPPER)))->wrapper_value), NIL), NIL) : NIL))->concatenate((((!(parameterizedmembervardefs == NIL)) ? cons(listO(4, SYM_CPP_CLASS_OUT_STELLA_CPP_DECLARATIONS, KWD_CPP_CLASS_OUT_PUBLIC, parameterizedmembervardefs, NIL), NIL) : NIL))->concatenate((((!(publicmembervardefs == NIL)) ? cons(listO(4, SYM_CPP_CLASS_OUT_STELLA_CPP_DECLARATIONS, KWD_CPP_CLASS_OUT_PUBLIC, publicmembervardefs, NIL), NIL) : NIL))->concatenate((((!(protectedmembervardefs == NIL)) ? cons(listO(4, SYM_CPP_CLASS_OUT_STELLA_CPP_DECLARATIONS, KWD_CPP_CLASS_OUT_PUBLIC, protectedmembervardefs, NIL), NIL) : NIL))->concatenate((((!(constructordefs == NIL)) ? cons(listO(4, SYM_CPP_CLASS_OUT_STELLA_CPP_SIGNATURES, KWD_CPP_CLASS_OUT_PUBLIC, constructordefs, NIL), NIL) : NIL))->concatenate(((exception_classP(renamed_Class) ? cons(listO(4, SYM_CPP_CLASS_OUT_STELLA_CPP_DEFINITIONS, KWD_CPP_CLASS_OUT_PUBLIC, cons(cpp_yield_exception_constructor_definition(renamed_Class), NIL), NIL), NIL) : NIL))->concatenate((((!(publicmemberfuncdefs == NIL)) ? cons(listO(4, SYM_CPP_CLASS_OUT_STELLA_CPP_SIGNATURES, KWD_CPP_CLASS_OUT_PUBLIC, publicmemberfuncdefs, NIL), NIL) : NIL))->concatenate((((!(protectedmemberfuncdefs == NIL)) ? cons(listO(4, SYM_CPP_CLASS_OUT_STELLA_CPP_SIGNATURES, KWD_CPP_CLASS_OUT_PUBLIC, protectedmemberfuncdefs, NIL), NIL) : NIL))->concatenate(NIL, 0), 0), 0), 0), 0), 0), 0), 0))))));
  }
}

void startup_cpp_class_out() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      SGT_CPP_CLASS_OUT_STELLA_OBJECT = ((Surrogate*)(intern_rigid_symbol_wrt_module("OBJECT", NULL, 1)));
      KWD_CPP_CLASS_OUT_EMBEDDED = ((Keyword*)(intern_rigid_symbol_wrt_module("EMBEDDED", NULL, 2)));
      KWD_CPP_CLASS_OUT_INSTANCE = ((Keyword*)(intern_rigid_symbol_wrt_module("INSTANCE", NULL, 2)));
      KWD_CPP_CLASS_OUT_CLASS = ((Keyword*)(intern_rigid_symbol_wrt_module("CLASS", NULL, 2)));
      SYM_CPP_CLASS_OUT_STELLA_NULL = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL", NULL, 0)));
      SYM_CPP_CLASS_OUT_STELLA_CPP_FUNCTION = ((Symbol*)(intern_rigid_symbol_wrt_module("CPP_FUNCTION", NULL, 0)));
      SYM_CPP_CLASS_OUT_STELLA_CPP_FUNCTION_CALL = ((Symbol*)(intern_rigid_symbol_wrt_module("CPP_FUNCTION_CALL", NULL, 0)));
      SYM_CPP_CLASS_OUT_STELLA_CPP_IDENT = ((Symbol*)(intern_rigid_symbol_wrt_module("CPP_IDENT", NULL, 0)));
      SYM_CPP_CLASS_OUT_STELLA_CPP_ACTUALS = ((Symbol*)(intern_rigid_symbol_wrt_module("CPP_ACTUALS", NULL, 0)));
      SYM_CPP_CLASS_OUT_STELLA_SLOT_TYPE_SPECIFIER = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-TYPE-SPECIFIER", NULL, 0)));
      SGT_CPP_CLASS_OUT_STELLA_ANCHORED_TYPE_SPECIFIER = ((Surrogate*)(intern_rigid_symbol_wrt_module("ANCHORED-TYPE-SPECIFIER", NULL, 1)));
      SGT_CPP_CLASS_OUT_STELLA_UNKNOWN = ((Surrogate*)(intern_rigid_symbol_wrt_module("UNKNOWN", NULL, 1)));
      SYM_CPP_CLASS_OUT_STELLA_DEFINEDp = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFINED?", NULL, 0)));
      SYM_CPP_CLASS_OUT_STELLA_NULLp = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL?", NULL, 0)));
      SGT_CPP_CLASS_OUT_STELLA_METHOD_SLOT = ((Surrogate*)(intern_rigid_symbol_wrt_module("METHOD-SLOT", NULL, 1)));
      SYM_CPP_CLASS_OUT_STELLA_CLASS_CPP_NATIVE_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-CPP-NATIVE-TYPE", NULL, 0)));
      KWD_CPP_CLASS_OUT_CPP = ((Keyword*)(intern_rigid_symbol_wrt_module("CPP", NULL, 2)));
      SYM_CPP_CLASS_OUT_STELLA_GC = ((Symbol*)(intern_rigid_symbol_wrt_module("GC", NULL, 0)));
      KWD_CPP_CLASS_OUT_CPP_STANDALONE = ((Keyword*)(intern_rigid_symbol_wrt_module("CPP-STANDALONE", NULL, 2)));
      SYM_CPP_CLASS_OUT_STELLA_CPP_CLASS = ((Symbol*)(intern_rigid_symbol_wrt_module("CPP_CLASS", NULL, 0)));
      SYM_CPP_CLASS_OUT_STELLA_DOCUMENTATION = ((Symbol*)(intern_rigid_symbol_wrt_module("DOCUMENTATION", NULL, 0)));
      SYM_CPP_CLASS_OUT_STELLA_CPP_COMMENT = ((Symbol*)(intern_rigid_symbol_wrt_module("CPP_COMMENT", NULL, 0)));
      SYM_CPP_CLASS_OUT_STELLA_CPP_DECLARATIONS = ((Symbol*)(intern_rigid_symbol_wrt_module("CPP_DECLARATIONS", NULL, 0)));
      KWD_CPP_CLASS_OUT_PUBLIC = ((Keyword*)(intern_rigid_symbol_wrt_module("PUBLIC", NULL, 2)));
      SYM_CPP_CLASS_OUT_STELLA_CPP_SIGNATURES = ((Symbol*)(intern_rigid_symbol_wrt_module("CPP_SIGNATURES", NULL, 0)));
      SYM_CPP_CLASS_OUT_STELLA_CPP_DEFINITIONS = ((Symbol*)(intern_rigid_symbol_wrt_module("CPP_DEFINITIONS", NULL, 0)));
      SYM_CPP_CLASS_OUT_STELLA_STARTUP_CPP_CLASS_OUT = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-CPP-CLASS-OUT", NULL, 0)));
      SYM_CPP_CLASS_OUT_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      define_function_object("CPP-YIELD-INITIAL-NULL-VALUE", "(DEFUN (CPP-YIELD-INITIAL-NULL-VALUE CONS) ())", ((cpp_function_code)(&cpp_yield_initial_null_value)), NULL);
      define_function_object("CPP-YIELD-MEMBER-VAR-TREE", "(DEFUN (CPP-YIELD-MEMBER-VAR-TREE CONS) ((SLOT STORAGE-SLOT) (CLASS CLASS)))", ((cpp_function_code)(&cpp_yield_member_var_tree)), NULL);
      define_function_object("CPP-YIELD-PARAMETERIZED-MEMBER-VAR-TREE", "(DEFUN (CPP-YIELD-PARAMETERIZED-MEMBER-VAR-TREE CONS) ((SLOT STORAGE-SLOT) (CLASS CLASS)))", ((cpp_function_code)(&cpp_yield_parameterized_member_var_tree)), NULL);
      define_function_object("CPP-YIELD-STATIC-MEMBER-ALLOCATION-TREE", "(DEFUN (CPP-YIELD-STATIC-MEMBER-ALLOCATION-TREE CONS) ((SLOT STORAGE-SLOT) (CLASS CLASS)))", ((cpp_function_code)(&cpp_yield_static_member_allocation_tree)), NULL);
      define_function_object("CPP-YIELD-MEMBER-FUNC-SIGNATURE-TREE", "(DEFUN (CPP-YIELD-MEMBER-FUNC-SIGNATURE-TREE CONS) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&cpp_yield_member_func_signature_tree)), NULL);
      define_function_object("CPP-YIELD-CONSTRUCTOR-SIGNATURE-TREE", "(DEFUN (CPP-YIELD-CONSTRUCTOR-SIGNATURE-TREE CONS) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&cpp_yield_constructor_signature_tree)), NULL);
      define_function_object("CPP-YIELD-EXCEPTION-CONSTRUCTOR-DEFINITION", "(DEFUN (CPP-YIELD-EXCEPTION-CONSTRUCTOR-DEFINITION CONS) ((CLASS CLASS)))", ((cpp_function_code)(&cpp_yield_exception_constructor_definition)), NULL);
      define_function_object("SLOT-HAS-CLASS-PARAMETER-TYPE?", "(DEFUN (SLOT-HAS-CLASS-PARAMETER-TYPE? BOOLEAN) ((SLOT STORAGE-SLOT) (CLASS CLASS)))", ((cpp_function_code)(&slot_has_class_parameter_typeP)), NULL);
      define_function_object("GET-SLOT-CLASS-PARAMETER-NAME", "(DEFUN (GET-SLOT-CLASS-PARAMETER-NAME SYMBOL) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&get_slot_class_parameter_name)), NULL);
      define_function_object("CPP-NATIVE-METHOD?", "(DEFUN (CPP-NATIVE-METHOD? BOOLEAN) ((METHODSLOT METHOD-SLOT)))", ((cpp_function_code)(&cpp_native_methodP)), NULL);
      define_function_object("CPP-GET-PARAMETERIZED-MEMBER-VARIABLE-DEFINITIONS", "(DEFUN (CPP-GET-PARAMETERIZED-MEMBER-VARIABLE-DEFINITIONS CONS) ((CLASS CLASS)))", ((cpp_function_code)(&cpp_get_parameterized_member_variable_definitions)), NULL);
      define_function_object("CPP-GET-MEMBER-VARIABLE-DEFINITIONS", "(DEFUN (CPP-GET-MEMBER-VARIABLE-DEFINITIONS CONS CONS) ((CLASS CLASS)))", ((cpp_function_code)(&cpp_get_member_variable_definitions)), NULL);
      define_function_object("CPP-GET-STATIC-VARIABLE-DEFINITIONS", "(DEFUN (CPP-GET-STATIC-VARIABLE-DEFINITIONS CONS) ((CLASS CLASS)))", ((cpp_function_code)(&cpp_get_static_variable_definitions)), NULL);
      define_function_object("CPP-GET-CONSTRUCTOR-DEFINITIONS", "(DEFUN (CPP-GET-CONSTRUCTOR-DEFINITIONS CONS) ((CLASS CLASS)))", ((cpp_function_code)(&cpp_get_constructor_definitions)), NULL);
      define_function_object("CPP-GET-METHOD-DEFINITIONS", "(DEFUN (CPP-GET-METHOD-DEFINITIONS CONS CONS) ((CLASS CLASS)))", ((cpp_function_code)(&cpp_get_method_definitions)), NULL);
      define_function_object("MARK-RENAMING-SLOTS", "(DEFUN (MARK-RENAMING-SLOTS VOID) ((CLASS CLASS)))", ((cpp_function_code)(&mark_renaming_slots)), NULL);
      define_function_object("CPP-CLASS-HAS-ONLY-NATIVE-PARENTS?", "(DEFUN (CPP-CLASS-HAS-ONLY-NATIVE-PARENTS? BOOLEAN) ((CLASS CLASS)))", ((cpp_function_code)(&cpp_class_has_only_native_parentsP)), NULL);
      define_function_object("CPP-TRANSLATE-DEFINE-NATIVE-CLASS-UNIT", "(DEFUN (CPP-TRANSLATE-DEFINE-NATIVE-CLASS-UNIT CONS) ((UNIT TRANSLATION-UNIT)))", ((cpp_function_code)(&cpp_translate_define_native_class_unit)), NULL);
      define_function_object("STARTUP-CPP-CLASS-OUT", "(DEFUN STARTUP-CPP-CLASS-OUT () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_cpp_class_out)), NULL);
      { Method_Slot* function = lookup_function(SYM_CPP_CLASS_OUT_STELLA_STARTUP_CPP_CLASS_OUT);

        set_dynamic_slot_value(function->dynamic_slots, SYM_CPP_CLASS_OUT_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Cpp-Class-Out"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
  }
}

Surrogate* SGT_CPP_CLASS_OUT_STELLA_OBJECT = NULL;

Keyword* KWD_CPP_CLASS_OUT_EMBEDDED = NULL;

Keyword* KWD_CPP_CLASS_OUT_INSTANCE = NULL;

Keyword* KWD_CPP_CLASS_OUT_CLASS = NULL;

Symbol* SYM_CPP_CLASS_OUT_STELLA_NULL = NULL;

Symbol* SYM_CPP_CLASS_OUT_STELLA_CPP_FUNCTION = NULL;

Symbol* SYM_CPP_CLASS_OUT_STELLA_CPP_FUNCTION_CALL = NULL;

Symbol* SYM_CPP_CLASS_OUT_STELLA_CPP_IDENT = NULL;

Symbol* SYM_CPP_CLASS_OUT_STELLA_CPP_ACTUALS = NULL;

Symbol* SYM_CPP_CLASS_OUT_STELLA_SLOT_TYPE_SPECIFIER = NULL;

Surrogate* SGT_CPP_CLASS_OUT_STELLA_ANCHORED_TYPE_SPECIFIER = NULL;

Surrogate* SGT_CPP_CLASS_OUT_STELLA_UNKNOWN = NULL;

Symbol* SYM_CPP_CLASS_OUT_STELLA_DEFINEDp = NULL;

Symbol* SYM_CPP_CLASS_OUT_STELLA_NULLp = NULL;

Surrogate* SGT_CPP_CLASS_OUT_STELLA_METHOD_SLOT = NULL;

Symbol* SYM_CPP_CLASS_OUT_STELLA_CLASS_CPP_NATIVE_TYPE = NULL;

Keyword* KWD_CPP_CLASS_OUT_CPP = NULL;

Symbol* SYM_CPP_CLASS_OUT_STELLA_GC = NULL;

Keyword* KWD_CPP_CLASS_OUT_CPP_STANDALONE = NULL;

Symbol* SYM_CPP_CLASS_OUT_STELLA_CPP_CLASS = NULL;

Symbol* SYM_CPP_CLASS_OUT_STELLA_DOCUMENTATION = NULL;

Symbol* SYM_CPP_CLASS_OUT_STELLA_CPP_COMMENT = NULL;

Symbol* SYM_CPP_CLASS_OUT_STELLA_CPP_DECLARATIONS = NULL;

Keyword* KWD_CPP_CLASS_OUT_PUBLIC = NULL;

Symbol* SYM_CPP_CLASS_OUT_STELLA_CPP_SIGNATURES = NULL;

Symbol* SYM_CPP_CLASS_OUT_STELLA_CPP_DEFINITIONS = NULL;

Symbol* SYM_CPP_CLASS_OUT_STELLA_STARTUP_CPP_CLASS_OUT = NULL;

Symbol* SYM_CPP_CLASS_OUT_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
