// defclass.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

Object* type_to_null_value_tree(Surrogate* type) {
  { Class* renamed_Class = type_to_class(type);

    if (((boolean)(renamed_Class)) &&
        (subtype_ofP(type, SGT_DEFCLASS_STELLA_LITERAL) ||
         subtype_ofP(type, SGT_DEFCLASS_STELLA_SECOND_CLASS_OBJECT))) {
      { Object* nullvalue = renamed_Class->initial_value();

        if (((boolean)(nullvalue))) {
          return (copy_cons_tree(nullvalue));
        }
      }
    }
    return (SYM_DEFCLASS_STELLA_NULL);
  }
}

Object* type_to_walked_null_value_tree(Surrogate* type) {
  { Standard_Object* dummy1;

    { Object* nullvalue = type_to_null_value_tree(type);

      if (nullvalue == SYM_DEFCLASS_STELLA_NULL) {
        return (nullvalue);
      }
      else {
        return (walk_expression_tree(nullvalue, type, SYM_DEFCLASS_STELLA_NULL, FALSE, dummy1));
      }
    }
  }
}

Cons* yield_cons_list_from_sequence(Sequence* sequence) {
  { Cons* list = NIL;

    { Object* item = NULL;
      Iterator* iter_000 = ((Iterator*)(sequence->allocate_iterator()));
      Cons* collect_000 = NULL;

      while (iter_000->nextP()) {
        item = iter_000->value;
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(item, NIL);
            if (list == NIL) {
              list = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(list, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(item, NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
    }
    return (list);
  }
}

Cons* yield_define_stella_class(Class* renamed_Class) {
  // Return a cons tree that (when evaluated) constructs
  // a Stella class object.
  return (listO(3, SYM_DEFCLASS_STELLA_DEFINE_CLASS_FROM_STRINGIFIED_SOURCE, wrap_string(renamed_Class->name()), cons(yield_string_constant_tree(renamed_Class->class_stringified_source), NIL)));
}

boolean create_native_classP(Class* renamed_Class) {
  { char* classnativetype = NULL;

    if (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_DEFCLASS_COMMON_LISP) {
      classnativetype = ((String_Wrapper*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_DEFCLASS_STELLA_CLASS_CL_NATIVE_TYPE, NULL_STRING_WRAPPER)))->wrapper_value;
    }
    else if (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_DEFCLASS_IDL) {
      classnativetype = ((String_Wrapper*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_DEFCLASS_STELLA_CLASS_IDL_NATIVE_TYPE, NULL_STRING_WRAPPER)))->wrapper_value;
    }
    else if (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_DEFCLASS_JAVA) {
      classnativetype = ((String_Wrapper*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_DEFCLASS_STELLA_CLASS_JAVA_NATIVE_TYPE, NULL_STRING_WRAPPER)))->wrapper_value;
    }
    else if ((oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_DEFCLASS_CPP) ||
        (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_DEFCLASS_CPP_STANDALONE)) {
      classnativetype = ((String_Wrapper*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_DEFCLASS_STELLA_CLASS_CPP_NATIVE_TYPE, NULL_STRING_WRAPPER)))->wrapper_value;
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "`" << oTRANSLATOROUTPUTLANGUAGEo.get() << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
    return (!(subtype_ofP(renamed_Class->class_type, SGT_DEFCLASS_STELLA_SECOND_CLASS_OBJECT) ||
        (subtype_ofP(renamed_Class->class_type, SGT_DEFCLASS_STELLA_NON_OBJECT) ||
         ((classnativetype != NULL) ||
          (!renamed_Class->primitiveP())))));
  }
}

boolean exception_classP(Class* renamed_Class) {
  return (subtype_ofP(renamed_Class->class_type, SGT_DEFCLASS_STELLA_NATIVE_EXCEPTION));
}

void warn_about_multiple_parents(Class* renamed_Class) {
  if (renamed_Class->multiple_parentsP() &&
      create_native_classP(renamed_Class)) {
    if ((oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_DEFCLASS_CPP_STANDALONE) ||
        (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_DEFCLASS_IDL)) {
    }
    else if ((oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_DEFCLASS_COMMON_LISP) ||
        ((oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_DEFCLASS_CPP) ||
         (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_DEFCLASS_JAVA))) {
      { int nofsupers = 0;
        int nofmixinsupers = 0;

        { Surrogate* super = NULL;
          Cons* iter_000 = class_native_supers(renamed_Class)->the_cons_list;

          while (!(iter_000 == NIL)) {
            super = ((Surrogate*)(iter_000->value));
            iter_000 = iter_000->rest;
            nofsupers = nofsupers + 1;
            if (((Class*)(super->surrogate_value))->mixinP) {
              nofmixinsupers = nofmixinsupers + 1;
            }
          }
        }
        if ((nofsupers - nofmixinsupers) > 1) {
          { 
            BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
            signal_translation_error();
            if (!(suppress_warningsP())) {
              print_error_context(">> ERROR: ", STANDARD_ERROR);
              *(STANDARD_ERROR->native_stream) << endl << " " << "Class " << "`" << class_name(renamed_Class) << "'" << " illegally inherits " << "`" << (nofsupers - nofmixinsupers) << "'" << " native non-mixin classes" << "." << endl;
            }
          }
        }
      }
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "`" << oTRANSLATOROUTPUTLANGUAGEo.get() << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
}

void warn_about_non_direct_supers(Class* renamed_Class) {
  if (renamed_Class->multiple_parentsP() &&
      create_native_classP(renamed_Class)) {
    { List* directsupers = renamed_Class->class_direct_supers;

      { Surrogate* super = NULL;
        Cons* iter_000 = directsupers->the_cons_list;

        while (!(iter_000 == NIL)) {
          super = ((Surrogate*)(iter_000->value));
          iter_000 = iter_000->rest;
          { boolean foundP_000 = FALSE;

            { Surrogate* othersuper = NULL;
              Cons* iter_001 = directsupers->the_cons_list;

              while (!(iter_001 == NIL)) {
                othersuper = ((Surrogate*)(iter_001->value));
                iter_001 = iter_001->rest;
                if ((!(super == othersuper)) &&
                    subtype_ofP(othersuper, super)) {
                  foundP_000 = TRUE;
                  break;
                }
              }
            }
            if (foundP_000) {
              { 
                BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
                signal_translation_error();
                if (!(suppress_warningsP())) {
                  print_error_context(">> ERROR: ", STANDARD_ERROR);
                  *(STANDARD_ERROR->native_stream) << endl << " " << "Bad list of supers defined for class " << "`" << class_name(renamed_Class) << "'" << "." << endl << " The class " << "`" << super->symbol_name << "'" << " is not a direct superclass" << "." << endl;
                }
              }
              return;
            }
          }
        }
      }
    }
  }
}

boolean space_saver_slot_allocationP(Storage_Slot* slot) {
  if (slot->allocation() == KWD_DEFCLASS_DYNAMIC) {
    if (((boolean)(dynamic_slot_value(slot->dynamic_slots, SYM_DEFCLASS_STELLA_SLOT_INITIAL_VALUE, NULL)))) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "Dynamic slot " << "`" << slot->slot_name << "'" << " should not have an" << endl << "   initial value (or it shouldn't be dynamic)." << endl;
    }
    return (TRUE);
  }
  else if ((slot->allocation() == KWD_DEFCLASS_BIT) ||
      ((slot->type() == SGT_DEFCLASS_STELLA_BOOLEAN) &&
       (!((boolean)(slot->slot_allocation_reader()))))) {
    return (TRUE);
  }
  else {
    return (FALSE);
  }
}

boolean dynamic_slotP(Storage_Slot* slot) {
  return (slot->allocation() == KWD_DEFCLASS_DYNAMIC);
}

boolean class_slotP(Storage_Slot* slot) {
  return (slot->allocation() == KWD_DEFCLASS_CLASS);
}

boolean redundant_slotP(Storage_Slot* slot) {
  { Slot* equivslot = slot->slot_direct_equivalent;

    return (((boolean)(equivslot)) &&
        (((Symbol*)(dynamic_slot_value(slot->dynamic_slots, SYM_DEFCLASS_STELLA_SLOT_RENAMES, NULL))) == equivslot->slot_name));
  }
}

boolean native_slotP(Storage_Slot* slot) {
  return ((!slot->abstractP) &&
      ((!slot->slot_hardwiredP) &&
       ((!space_saver_slot_allocationP(slot)) &&
        (!redundant_slotP(slot)))));
}

boolean native_methodP(Method_Slot* slot) {
  return ((!slot->abstractP) &&
      (!method_contains_unknown_typeP(slot)));
}

boolean mixin_slotP(Slot* self) {
  { Surrogate* owner = self->slot_owner;

    return (storage_slotP(self) &&
        (((boolean)(owner)) &&
         (((boolean)(((Class*)(owner->surrogate_value)))) &&
          ((Class*)(owner->surrogate_value))->mixinP)));
  }
}

boolean mixin_methodP(Slot* self) {
  { Surrogate* owner = self->slot_owner;

    return (method_slotP(self) &&
        ((!((Method_Slot*)(self))->method_functionP) &&
         (((boolean)(owner)) &&
          (((boolean)(((Class*)(owner->surrogate_value)))) &&
           ((Class*)(owner->surrogate_value))->mixinP))));
  }
}

Storage_Slot* canonical_slot(Storage_Slot* slot) {
  { Storage_Slot* canonicalslot = slot;

    while (((boolean)(canonicalslot->slot_direct_equivalent)) &&
        (canonicalslot->slot_direct_equivalent->slot_name == ((Symbol*)(dynamic_slot_value(canonicalslot->dynamic_slots, SYM_DEFCLASS_STELLA_SLOT_RENAMES, NULL))))) {
      canonicalslot = ((Storage_Slot*)(canonicalslot->slot_direct_equivalent));
    }
    return (canonicalslot);
  }
}

boolean unknown_typeP(Surrogate* type) {
  return ((type == SGT_DEFCLASS_STELLA_UNKNOWN) ||
      ((!((boolean)(type_to_class(type)))) ||
       ((type_to_class(type) == type_to_class(SGT_DEFCLASS_STELLA_UNKNOWN)) ||
        (subtype_ofP(type, SGT_DEFCLASS_STELLA_NON_OBJECT) &&
         type_to_class(type)->abstractP))));
}

boolean slot_has_unknown_typeP(Storage_Slot* slot, Class* renamed_Class) {
  return (unknown_typeP(type_spec_to_base_type(slot->compute_return_type_spec(renamed_Class->class_type))));
}

Class* native_storage_slot_home(Storage_Slot* slot, Class* renamed_Class) {
  { Storage_Slot* slotwithknowntype = NULL;

    for (;;) {
      if (unknown_typeP(slot->type())) {
        break;
      }
      slotwithknowntype = slot;
      renamed_Class = type_to_class(slot->slot_owner);
      if (((boolean)(slot->slot_direct_equivalent)) &&
          native_slotP(((Storage_Slot*)(slot->slot_direct_equivalent)))) {
        slot = ((Storage_Slot*)(slot->slot_direct_equivalent));
      }
      else {
        break;
      }
    }
    if (((boolean)(slotwithknowntype))) {
      return (renamed_Class);
    }
    if (slot_has_unknown_typeP(slot, renamed_Class)) {
      return (NULL);
    }
    { Class* super = NULL;
      Cons* iter_000 = renamed_Class->class_all_super_classes;

      while (!(iter_000 == NIL)) {
        super = ((Class*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (slot_has_unknown_typeP(slot, super)) {
          return (renamed_Class);
        }
        else {
          renamed_Class = super;
        }
      }
    }
    return (renamed_Class);
  }
}

Class* native_slot_home(Slot* slot, Class* renamed_Class) {
  { Class* nativeslothome = (storage_slotP(slot) ? native_storage_slot_home(((Storage_Slot*)(slot)), renamed_Class) : ((Class*)(slot->slot_owner->surrogate_value)));
    Class* candidate = renamed_Class;
    Slot* visibleslot = NULL;

    if (!(((boolean)(nativeslothome)) &&
        (nativeslothome->mixinP &&
         translate_to_single_inheritance_languageP()))) {
      return (nativeslothome);
    }
    if (candidate->mixinP) {
      return (NULL);
    }
    visibleslot = lookup_slot(renamed_Class, slot->slot_name);
    if ((!(slot == visibleslot)) &&
        subtype_ofP(visibleslot->slot_owner, slot->slot_owner)) {
      return (NULL);
    }
    { Class* superclass = NULL;
      Cons* iter_000 = renamed_Class->class_all_super_classes;

      while (!(iter_000 == NIL)) {
        superclass = ((Class*)(iter_000->value));
        iter_000 = iter_000->rest;
        if ((!superclass->mixinP) &&
            superclass->class_all_super_classes->memberP(nativeslothome)) {
          candidate = superclass;
        }
      }
    }
    return (candidate);
  }
}

boolean method_contains_unknown_typeP(Method_Slot* method) {
  { boolean foundP_000 = FALSE;

    { Standard_Object* rtype = NULL;
      Cons* iter_000 = method->method_return_type_specifiers_reader()->the_cons_list;

      while (!(iter_000 == NIL)) {
        rtype = ((Standard_Object*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (unknown_typeP(type_spec_to_base_type(compute_relative_type_spec(rtype, method->slot_owner)))) {
          foundP_000 = TRUE;
          break;
        }
      }
    }
    if (foundP_000) {
      return (TRUE);
    }
  }
  { Standard_Object* ptype = NULL;
    Cons* iter_001 = method->method_parameter_type_specifiers_reader()->the_cons_list;

    while (!(iter_001 == NIL)) {
      ptype = ((Standard_Object*)(iter_001->value));
      iter_001 = iter_001->rest;
      if (unknown_typeP(type_spec_to_base_type(compute_relative_type_spec(ptype, method->slot_owner)))) {
        return (TRUE);
      }
    }
  }
  return (FALSE);
}

boolean method_contains_anchored_typeP(Method_Slot* method) {
  { Object* type = NULL;

    if (((Class*)(method->slot_owner->surrogate_value))->abstractP) {
      if (((boolean)(((Compound_Type_Specifier*)(dynamic_slot_value(method->dynamic_slots, SYM_DEFCLASS_STELLA_SLOT_TYPE_SPECIFIER, NULL))))) &&
          isaP(((Compound_Type_Specifier*)(dynamic_slot_value(method->dynamic_slots, SYM_DEFCLASS_STELLA_SLOT_TYPE_SPECIFIER, NULL))), SGT_DEFCLASS_STELLA_ANCHORED_TYPE_SPECIFIER)) {
        return (TRUE);
      }
      { Standard_Object* ptype = NULL;
        Cons* iter_000 = method->method_parameter_type_specifiers_reader()->the_cons_list;

        while (!(iter_000 == NIL)) {
          ptype = ((Standard_Object*)(iter_000->value));
          iter_000 = iter_000->rest;
          if (isaP(ptype, SGT_DEFCLASS_STELLA_ANCHORED_TYPE_SPECIFIER)) {
            return (TRUE);
          }
        }
      }
    }
    return (FALSE);
  }
}

Symbol* yield_recycle_list_name(Class* renamed_Class) {
  return (intern_derived_symbol(renamed_Class->class_type, string_concatenate("*RECYCLE-LIST-FOR-", class_name(renamed_Class), 1, "-S*")));
}

Storage_Slot* recycle_slot(Class* renamed_Class) {
  { Surrogate* instancetype = renamed_Class->class_type;
    Storage_Slot* recycleslot = NULL;

    { Slot* slot = NULL;
      Iterator* iter_000 = renamed_Class->class_slots();

      while (iter_000->nextP()) {
        slot = ((Slot*)(iter_000->value));
        if (subtype_of_storage_slotP(safe_primary_type(slot))) {
          { Slot* slot_000 = slot;
            Storage_Slot* slot = ((Storage_Slot*)(slot_000));

            if ((!(slot->slot_name == SYM_DEFCLASS_STELLA_NEXT_SWEEP_LIST_OBJECT)) &&
                native_slotP(slot)) {
              if (instancetype == slot->slot_base_type) {
                return (slot);
              }
              if (((boolean)(((Class*)(slot->slot_base_type->surrogate_value)))) &&
                  subtype_ofP(instancetype, slot->slot_base_type)) {
                recycleslot = slot;
              }
            }
          }
        }
        else {
        }
      }
    }
    return (recycleslot);
  }
}

boolean use_free_listP(Class* renamed_Class) {
  return ((renamed_Class->class_recycle_method_reader() == KWD_DEFCLASS_FREE_LIST) ||
      (renamed_Class->class_recycle_method_reader() == KWD_DEFCLASS_FREE_AND_SWEEP_LIST));
}

boolean use_sweep_listP(Class* renamed_Class) {
  return ((renamed_Class->class_recycle_method_reader() == KWD_DEFCLASS_SWEEP_LIST) ||
      ((renamed_Class->class_recycle_method_reader() == KWD_DEFCLASS_FREE_AND_SWEEP_LIST) &&
       (!(oTRANSLATE_WITH_RECYCLE_LIST_DEBUGGINGpo ||
      ((boolean)(oCURRENTLY_RECYCLED_ITEMSo))))));
}

boolean decided_to_recycleP(Class* renamed_Class) {
  return ((!(renamed_Class->abstractP ||
      ((boolean)(((Symbol*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_DEFCLASS_STELLA_CLASS_CREATOR, NULL))))))) &&
      (use_free_listP(renamed_Class) ||
       use_sweep_listP(renamed_Class)));
}

int compute_recycled_item_size(Class* renamed_Class) {
  { int size = NULL_INTEGER;

    if (((boolean)(renamed_Class))) {
      if (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_DEFCLASS_COMMON_LISP) {
        if (oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_DEFCLASS_USE_COMMON_LISP_STRUCTS) ||
            oCURRENT_ENVIRONMENT_FEATURESo->membP(KWD_DEFCLASS_USE_COMMON_LISP_VECTOR_STRUCTS)) {
          size = 4;
        }
        else {
          size = 10;
        }
      }
      else if (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_DEFCLASS_CPP) {
        size = 1;
      }
      else if (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_DEFCLASS_JAVA) {
        size = 2;
      }
      else {
        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "`" << oTRANSLATOROUTPUTLANGUAGEo.get() << "'" << " is not a valid case option";
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      }
      { Slot* slot = NULL;
        Iterator* iter_000 = renamed_Class->class_slots();

        while (iter_000->nextP()) {
          slot = ((Slot*)(iter_000->value));
          if (storage_slotP(slot) &&
              (native_slotP(((Storage_Slot*)(slot))) &&
               (!(((boolean)(slot->slot_direct_equivalent)) &&
              native_slotP(((Storage_Slot*)(slot->slot_direct_equivalent))))))) {
            size = size + 1;
          }
        }
      }
      if (use_free_listP(renamed_Class) &&
          (!((boolean)(recycle_slot(renamed_Class))))) {
        size = size + 2;
      }
    }
    return (size);
  }
}

Cons* yield_recycle_list_definitions(Class* renamed_Class) {
  if (decided_to_recycleP(renamed_Class)) {
    return (cons(listO(3, SYM_DEFCLASS_STELLA_DEFGLOBAL, yield_recycle_list_name(renamed_Class), listO(3, listO(4, SYM_DEFCLASS_STELLA_RECYCLE_LIST, SYM_DEFCLASS_STELLA_OF, class_symbol(renamed_Class), NIL), listO(3, SYM_DEFCLASS_STELLA_CREATE_RECYCLE_LIST, renamed_Class->class_type, cons(wrap_integer(compute_recycled_item_size(renamed_Class)), NIL)), NIL)), NIL));
  }
  else {
    return (NIL);
  }
}

Cons* yield_make_trees(Class* renamed_Class) {
  return (cons(listO(4, SYM_DEFCLASS_STELLA_SETQ, SYM_DEFCLASS_STELLA_SELF, listO(3, SYM_DEFCLASS_STELLA_MAKE, class_symbol(renamed_Class), NIL), NIL), NIL));
}

Symbol* yield_constructor_name(Class* renamed_Class) {
  return (intern_derived_symbol(renamed_Class->class_type, string_concatenate("NEW-", class_name(renamed_Class), 0)));
}

Cons* yield_creation_trees(Class* renamed_Class) {
  if (((boolean)(((Symbol*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_DEFCLASS_STELLA_CLASS_CREATOR, NULL)))))) {
    if (((Symbol*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_DEFCLASS_STELLA_CLASS_CREATOR, NULL))) == yield_constructor_name(renamed_Class)) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "Illegal creator name " << "`" << ((Symbol*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_DEFCLASS_STELLA_CLASS_CREATOR, NULL))) << "'" << " for the class " << endl << "`" << class_name(renamed_Class) << "'" << ".  Using the default creator." << endl;
      return (yield_make_trees(renamed_Class));
    }
    return (cons(listO(4, SYM_DEFCLASS_STELLA_SETQ, SYM_DEFCLASS_STELLA_SELF, cons(((Symbol*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_DEFCLASS_STELLA_CLASS_CREATOR, NULL))), NIL), NIL), NIL));
  }
  else if (decided_to_recycleP(renamed_Class)) {
    return (yield_creation_or_reuse_trees(renamed_Class));
  }
  else {
    return (yield_make_trees(renamed_Class));
  }
}

Cons* yield_terminate_object_trees(Class* renamed_Class) {
  { Cons* trees = NIL;
    Symbol* terminator = renamed_Class->terminator();

    { Slot* slot = NULL;
      Iterator* iter_000 = renamed_Class->class_slots();

      while (iter_000->nextP()) {
        slot = ((Slot*)(iter_000->value));
        if (subtype_of_storage_slotP(safe_primary_type(slot))) {
          { Slot* slot_000 = slot;
            Storage_Slot* slot = ((Storage_Slot*)(slot_000));

            if (slot->componentP()) {
              trees = cons(listO(4, SYM_DEFCLASS_STELLA_WHEN, listO(3, SYM_DEFCLASS_STELLA_DEFINEDp, cons(slot->slot_name, cons(SYM_DEFCLASS_STELLA_SELF, NIL)), NIL), listO(3, SYM_DEFCLASS_STELLA_FREE, cons(slot->slot_name, cons(SYM_DEFCLASS_STELLA_SELF, NIL)), NIL), NIL), trees);
            }
          }
        }
        else {
        }
      }
    }
    if (((boolean)(lookup_slot(renamed_Class, SYM_DEFCLASS_STELLA_DYNAMIC_SLOTS)))) {
      trees = cons(listO(3, SYM_DEFCLASS_STELLA_CLEAR, listO(3, SYM_DEFCLASS_STELLA_DYNAMIC_SLOTS, SYM_DEFCLASS_STELLA_SELF, NIL), NIL), trees);
    }
    if (((boolean)(terminator))) {
      if (use_sweep_listP(renamed_Class)) {
        trees = cons(cons(terminator, cons(SYM_DEFCLASS_STELLA_SELF, NIL)), trees);
      }
      else {
        trees = cons(listO(4, SYM_DEFCLASS_STELLA_UNLESS, cons(terminator, cons(SYM_DEFCLASS_STELLA_SELF, NIL)), cons(SYM_DEFCLASS_STELLA_RETURN, NIL), NIL), trees);
      }
    }
    return (trees->reverse()->concatenate(yield_initial_value_assignments(renamed_Class, KWD_DEFCLASS_FORCE_NULL_VALUE), 0));
  }
}

Cons* yield_creation_or_reuse_trees(Class* renamed_Class) {
  { boolean usefreelistP = use_free_listP(renamed_Class);
    boolean usesweeplistP = use_sweep_listP(renamed_Class);

    if (usefreelistP &&
        usesweeplistP) {
      return (yield_creation_or_reuse_trees_from_free_or_sweep_list(renamed_Class));
    }
    else if (usefreelistP) {
      return (yield_creation_or_reuse_trees_from_free_list(renamed_Class));
    }
    else if (usesweeplistP) {
      return (yield_creation_or_reuse_trees_from_sweep_list(renamed_Class));
    }
    else {
      throw *new_stella_exception("Assuming CLASS uses at least one of Free-list or sweep-list");
    }
  }
}

Cons* yield_creation_or_reuse_trees_from_free_list(Class* renamed_Class) {
  { Storage_Slot* recycleslot = recycle_slot(renamed_Class);
    Cons* recycleslotresettree = (((boolean)(recycleslot)) ? yield_initial_value_assignment(recycleslot, KWD_DEFCLASS_NULL_VALUE_ONLY) : ((Cons*)(NULL)));
    Symbol* recyclelist = yield_recycle_list_name(renamed_Class);

    recyclelist = yield_recycle_list_name(renamed_Class);
    return (listO(3, listO(4, SYM_DEFCLASS_STELLA_WHEN, listO(4, SYM_DEFCLASS_STELLA_AND, SYM_DEFCLASS_STELLA_oRECYCLING_ENABLEDpo, listO(3, SYM_DEFCLASS_STELLA_DEFINEDp, recyclelist, NIL), NIL), listO(4, SYM_DEFCLASS_STELLA_SETQ, SYM_DEFCLASS_STELLA_SELF, (((boolean)(recycleslot)) ? listO(3, SYM_DEFCLASS_STELLA_RECYCLED_ITEMS, recyclelist, NIL) : listO(3, SYM_DEFCLASS_STELLA_POP, listO(3, SYM_DEFCLASS_STELLA_LIST_OF_RECYCLED_ITEMS, recyclelist, NIL), NIL)), NIL), NIL), listO(4, SYM_DEFCLASS_STELLA_COND, cons(listO(3, SYM_DEFCLASS_STELLA_DEFINEDp, SYM_DEFCLASS_STELLA_SELF, NIL), ((((boolean)(recycleslot)) ? cons(listO(4, SYM_DEFCLASS_STELLA_SETF, listO(3, SYM_DEFCLASS_STELLA_RECYCLED_ITEMS, recyclelist, NIL), listO(4, SYM_DEFCLASS_STELLA_SLOT_VALUE, SYM_DEFCLASS_STELLA_SELF, recycleslot->slot_name, NIL), NIL), NIL) : NIL))->concatenate(((((boolean)(recycleslotresettree)) ? cons(recycleslotresettree, NIL) : NIL))->concatenate(cons(listO(3, SYM_DEFCLASS_STELLA___, listO(3, SYM_DEFCLASS_STELLA_CURRENT_LENGTH, recyclelist, NIL), NIL), yield_unregister_recycled_item_trees()->concatenate(NIL, 0)), 0), 0)), cons(SYM_DEFCLASS_STELLA_OTHERWISE, yield_make_trees(renamed_Class)->concatenate(yield_initial_value_assignments(renamed_Class, KWD_DEFCLASS_NULL_VALUE_ONLY)->concatenate(NIL, 0), 0)), NIL), NIL));
  }
}

Cons* yield_creation_or_reuse_trees_from_sweep_list(Class* renamed_Class) {
  { Symbol* recyclelist = yield_recycle_list_name(renamed_Class);

    recyclelist = yield_recycle_list_name(renamed_Class);
    return (cons(listO(4, SYM_DEFCLASS_STELLA_COND, listO(4, listO(4, SYM_DEFCLASS_STELLA_AND, SYM_DEFCLASS_STELLA_oRECYCLING_ENABLEDpo, listO(3, SYM_DEFCLASS_STELLA_DEFINEDp, recyclelist, NIL), NIL), listO(4, SYM_DEFCLASS_STELLA_SETQ, SYM_DEFCLASS_STELLA_SELF, listO(3, SYM_DEFCLASS_STELLA_UNUSED_ITEMS, recyclelist, NIL), NIL), listO(4, SYM_DEFCLASS_STELLA_COND, listO(3, listO(3, SYM_DEFCLASS_STELLA_DEFINEDp, SYM_DEFCLASS_STELLA_SELF, NIL), listO(4, SYM_DEFCLASS_STELLA_SETF, listO(3, SYM_DEFCLASS_STELLA_UNUSED_ITEMS, recyclelist, NIL), listO(3, SYM_DEFCLASS_STELLA_NEXT_SWEEP_LIST_OBJECT, listO(3, SYM_DEFCLASS_STELLA_UNUSED_ITEMS, recyclelist, NIL), NIL), NIL), NIL), cons(SYM_DEFCLASS_STELLA_OTHERWISE, yield_make_trees(renamed_Class)->concatenate(yield_initial_value_assignments(renamed_Class, KWD_DEFCLASS_NULL_VALUE_ONLY)->concatenate(listO(5, listO(4, SYM_DEFCLASS_STELLA_SETF, listO(3, SYM_DEFCLASS_STELLA_NEXT_SWEEP_LIST_OBJECT, SYM_DEFCLASS_STELLA_SELF, NIL), listO(3, SYM_DEFCLASS_STELLA_ALL_ITEMS, recyclelist, NIL), NIL), listO(4, SYM_DEFCLASS_STELLA_SETF, listO(3, SYM_DEFCLASS_STELLA_ALL_ITEMS, recyclelist, NIL), SYM_DEFCLASS_STELLA_SELF, NIL), listO(3, SYM_DEFCLASS_STELLA_ii, listO(3, SYM_DEFCLASS_STELLA_CURRENT_LENGTH, recyclelist, NIL), NIL), listO(4, SYM_DEFCLASS_STELLA_WHEN, listO(4, SYM_DEFCLASS_STELLA_e, listO(3, SYM_DEFCLASS_STELLA___, SYM_DEFCLASS_STELLA_oRECYCLE_LISTS_MAINTENANCE_TIMERo, NIL), wrap_integer(0), NIL), cons(SYM_DEFCLASS_STELLA_MAINTAIN_RECYCLE_LISTS, NIL), NIL), NIL), 0), 0)), NIL), NIL), cons(SYM_DEFCLASS_STELLA_OTHERWISE, yield_make_trees(renamed_Class)->concatenate(yield_initial_value_assignments(renamed_Class, KWD_DEFCLASS_NULL_VALUE_ONLY)->concatenate(NIL, 0), 0)), NIL), NIL));
  }
}

Cons* yield_creation_or_reuse_trees_from_free_or_sweep_list(Class* renamed_Class) {
  { Storage_Slot* recycleslot = recycle_slot(renamed_Class);
    Cons* recycleslotresettree = (((boolean)(recycleslot)) ? yield_initial_value_assignment(recycleslot, KWD_DEFCLASS_NULL_VALUE_ONLY) : ((Cons*)(NULL)));
    Symbol* recyclelist = yield_recycle_list_name(renamed_Class);

    recyclelist = yield_recycle_list_name(renamed_Class);
    return (cons(listO(4, SYM_DEFCLASS_STELLA_COND, listO(4, listO(4, SYM_DEFCLASS_STELLA_AND, SYM_DEFCLASS_STELLA_oRECYCLING_ENABLEDpo, listO(3, SYM_DEFCLASS_STELLA_DEFINEDp, recyclelist, NIL), NIL), listO(4, SYM_DEFCLASS_STELLA_SETQ, SYM_DEFCLASS_STELLA_SELF, (((boolean)(recycleslot)) ? listO(3, SYM_DEFCLASS_STELLA_RECYCLED_ITEMS, recyclelist, NIL) : listO(3, SYM_DEFCLASS_STELLA_POP, listO(3, SYM_DEFCLASS_STELLA_LIST_OF_RECYCLED_ITEMS, recyclelist, NIL), NIL)), NIL), listO(4, SYM_DEFCLASS_STELLA_COND, cons(listO(3, SYM_DEFCLASS_STELLA_DEFINEDp, SYM_DEFCLASS_STELLA_SELF, NIL), ((((boolean)(recycleslot)) ? cons(listO(4, SYM_DEFCLASS_STELLA_SETF, listO(3, SYM_DEFCLASS_STELLA_RECYCLED_ITEMS, recyclelist, NIL), listO(4, SYM_DEFCLASS_STELLA_SLOT_VALUE, SYM_DEFCLASS_STELLA_SELF, recycleslot->slot_name, NIL), NIL), NIL) : NIL))->concatenate(((((boolean)(recycleslotresettree)) ? cons(recycleslotresettree, NIL) : NIL))->concatenate(NIL, 0), 0)), listO(4, SYM_DEFCLASS_STELLA_OTHERWISE, listO(4, SYM_DEFCLASS_STELLA_SETQ, SYM_DEFCLASS_STELLA_SELF, listO(3, SYM_DEFCLASS_STELLA_UNUSED_ITEMS, recyclelist, NIL), NIL), listO(4, SYM_DEFCLASS_STELLA_COND, listO(3, listO(3, SYM_DEFCLASS_STELLA_DEFINEDp, SYM_DEFCLASS_STELLA_SELF, NIL), listO(4, SYM_DEFCLASS_STELLA_SETF, listO(3, SYM_DEFCLASS_STELLA_UNUSED_ITEMS, recyclelist, NIL), listO(3, SYM_DEFCLASS_STELLA_NEXT_SWEEP_LIST_OBJECT, listO(3, SYM_DEFCLASS_STELLA_UNUSED_ITEMS, recyclelist, NIL), NIL), NIL), NIL), cons(SYM_DEFCLASS_STELLA_OTHERWISE, yield_make_trees(renamed_Class)->concatenate(yield_initial_value_assignments(renamed_Class, KWD_DEFCLASS_NULL_VALUE_ONLY)->concatenate(listO(5, listO(4, SYM_DEFCLASS_STELLA_SETF, listO(3, SYM_DEFCLASS_STELLA_NEXT_SWEEP_LIST_OBJECT, SYM_DEFCLASS_STELLA_SELF, NIL), listO(3, SYM_DEFCLASS_STELLA_ALL_ITEMS, recyclelist, NIL), NIL), listO(4, SYM_DEFCLASS_STELLA_SETF, listO(3, SYM_DEFCLASS_STELLA_ALL_ITEMS, recyclelist, NIL), SYM_DEFCLASS_STELLA_SELF, NIL), listO(3, SYM_DEFCLASS_STELLA_ii, listO(3, SYM_DEFCLASS_STELLA_CURRENT_LENGTH, recyclelist, NIL), NIL), listO(4, SYM_DEFCLASS_STELLA_WHEN, listO(4, SYM_DEFCLASS_STELLA_e, listO(3, SYM_DEFCLASS_STELLA___, SYM_DEFCLASS_STELLA_oRECYCLE_LISTS_MAINTENANCE_TIMERo, NIL), wrap_integer(0), NIL), cons(SYM_DEFCLASS_STELLA_MAINTAIN_RECYCLE_LISTS, NIL), NIL), NIL), 0), 0)), NIL), NIL), NIL), NIL), cons(SYM_DEFCLASS_STELLA_OTHERWISE, yield_make_trees(renamed_Class)->concatenate(yield_initial_value_assignments(renamed_Class, KWD_DEFCLASS_NULL_VALUE_ONLY)->concatenate(NIL, 0), 0)), NIL), NIL));
  }
}

void mark_direct_equivalent_slot(Slot* slot) {
  { Slot* directequivalent = slot->slot_direct_equivalent;

    if (((boolean)(directequivalent))) {
      if (directequivalent->slot_markedP) {
        slot->slot_markedP = TRUE;
      }
      else {
        directequivalent->slot_markedP = TRUE;
      }
    }
  }
}

boolean is_context_sensitive_slotP(Storage_Slot* slot) {
  return (slot->slot_context_sensitiveP ||
      (((boolean)(slot->slot_direct_equivalent)) &&
       is_context_sensitive_slotP(((Storage_Slot*)(slot->slot_direct_equivalent)))));
}

Cons* yield_initial_value_assignment(Storage_Slot* slot, Keyword* mode) {
  { Object* initialvalueexpression = yield_initial_value_expression(slot);
    boolean requiredslotP = slot->requiredP();

    if (((boolean)(initialvalueexpression)) &&
        (!(slot->slot_name == SYM_DEFCLASS_STELLA_NEXT_SWEEP_LIST_OBJECT))) {
      if (mode == KWD_DEFCLASS_NULL_VALUE_ONLY) {
        if (requiredslotP ||
            (!equal_cons_treesP(type_to_null_value_tree(slot->type()), initialvalueexpression))) {
          initialvalueexpression = NULL;
        }
      }
      else if (mode == KWD_DEFCLASS_INITIAL_VALUE_ONLY) {
        if (requiredslotP ||
            equal_cons_treesP(type_to_null_value_tree(slot->type()), initialvalueexpression)) {
          initialvalueexpression = NULL;
        }
      }
      else if (mode == KWD_DEFCLASS_FORCE_NULL_VALUE) {
        initialvalueexpression = type_to_null_value_tree(slot->type());
      }
      else if (mode == KWD_DEFCLASS_ANY_INITIAL_VALUE) {
        if (requiredslotP) {
          initialvalueexpression = NULL;
        }
      }
      else {
        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "`" << mode << "'" << " is not a valid case option";
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      }
      if (((boolean)(initialvalueexpression))) {
        return (listO(4, SYM_DEFCLASS_STELLA_SETF, listO(4, SYM_DEFCLASS_STELLA_SLOT_VALUE, SYM_DEFCLASS_STELLA_SELF, canonical_slot(slot)->slot_name, ((is_context_sensitive_slotP(slot) ? listO(3, KWD_DEFCLASS_CONTEXT_SENSITIVEp, SYM_DEFCLASS_STELLA_FALSE, NIL) : NIL))->concatenate(NIL, 0)), initialvalueexpression, NIL));
      }
    }
    return (NULL);
  }
}

Cons* yield_initial_value_assignments(Class* renamed_Class, Keyword* mode) {
  { Cons* assignments = NIL;
    Object* initialvalueassignment = NULL;

    { Slot* slot = NULL;
      Iterator* iter_000 = renamed_Class->class_slots();

      while (iter_000->nextP()) {
        slot = ((Slot*)(iter_000->value));
        slot->slot_markedP = FALSE;
      }
    }
    { Slot* slot = NULL;
      Iterator* iter_001 = renamed_Class->class_slots();

      while (iter_001->nextP()) {
        slot = ((Slot*)(iter_001->value));
        mark_direct_equivalent_slot(slot);
      }
    }
    { Slot* slot = NULL;
      Iterator* iter_002 = renamed_Class->class_slots();

      while (iter_002->nextP()) {
        slot = ((Slot*)(iter_002->value));
        if ((!slot->slot_markedP) &&
            storage_slotP(slot)) {
          initialvalueassignment = yield_initial_value_assignment(((Storage_Slot*)(slot)), mode);
          if (((boolean)(initialvalueassignment))) {
            assignments = cons(initialvalueassignment, assignments);
          }
        }
      }
    }
    return (assignments);
  }
}

Cons* yield_local_initial_value_assignments(Class* renamed_Class) {
  { Cons* assignments = NIL;
    Object* initialvalueassignment = NULL;

    { Slot* slot = NULL;
      Cons* iter_000 = renamed_Class->class_local_slots->the_cons_list;

      while (!(iter_000 == NIL)) {
        slot = ((Slot*)(iter_000->value));
        iter_000 = iter_000->rest;
        slot->slot_markedP = FALSE;
      }
    }
    { Slot* slot = NULL;
      Cons* iter_001 = renamed_Class->class_local_slots->the_cons_list;

      while (!(iter_001 == NIL)) {
        slot = ((Slot*)(iter_001->value));
        iter_001 = iter_001->rest;
        mark_direct_equivalent_slot(slot);
      }
    }
    { Slot* slot = NULL;
      Cons* iter_002 = renamed_Class->class_local_slots->the_cons_list;

      while (!(iter_002 == NIL)) {
        slot = ((Slot*)(iter_002->value));
        iter_002 = iter_002->rest;
        if ((!slot->slot_markedP) &&
            storage_slotP(slot)) {
          initialvalueassignment = yield_initial_value_assignment(((Storage_Slot*)(slot)), KWD_DEFCLASS_ANY_INITIAL_VALUE);
          if (((boolean)(initialvalueassignment))) {
            assignments = cons(initialvalueassignment, assignments);
          }
        }
      }
    }
    return (assignments->reverse());
  }
}

Cons* yield_initializer_trees(Class* renamed_Class) {
  { Cons* parentinitializers = (((boolean)(((Symbol*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_DEFCLASS_STELLA_CLASS_INITIALIZER, NULL))))) ? cons(cons(renamed_Class->initializer(), cons(SYM_DEFCLASS_STELLA_SELF, NIL)), NIL) : NIL);

    { Class* super = NULL;
      Cons* iter_000 = renamed_Class->class_all_super_classes;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        super = ((Class*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (((boolean)(((Symbol*)(dynamic_slot_value(super->dynamic_slots, SYM_DEFCLASS_STELLA_CLASS_INITIALIZER, NULL)))))) {
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(cons(super->initializer(), cons(SYM_DEFCLASS_STELLA_SELF, NIL)), NIL);
              if (parentinitializers == NIL) {
                parentinitializers = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(parentinitializers, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(cons(super->initializer(), cons(SYM_DEFCLASS_STELLA_SELF, NIL)), NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
    }
    return (parentinitializers->reverse());
  }
}

Cons* yield_constructor_body(Class* renamed_Class, Cons* requiredassignments) {
  { Cons* initialvalueassignments = yield_initial_value_assignments(renamed_Class, (decided_to_recycleP(renamed_Class) ? KWD_DEFCLASS_INITIAL_VALUE_ONLY : KWD_DEFCLASS_ANY_INITIAL_VALUE));

    if ((initialvalueassignments == NIL) &&
        ((requiredassignments == NIL) &&
         ((!((boolean)(renamed_Class->initializer()))) &&
          ((!renamed_Class->activeP()) &&
           (!decided_to_recycleP(renamed_Class)))))) {
      return (yield_creation_trees(renamed_Class));
    }
    return (yield_creation_trees(renamed_Class)->concatenate(requiredassignments->concatenate(initialvalueassignments->concatenate(yield_initializer_trees(renamed_Class)->concatenate(((renamed_Class->activeP() ? cons(listO(3, SYM_DEFCLASS_STELLA_RUN_CONSTRUCTOR_DEMONS, SYM_DEFCLASS_STELLA_SELF, NIL), NIL) : NIL))->concatenate(NIL, 0), 0), 0), 0), 0));
  }
}

Cons* yield_constructor_definition(Class* renamed_Class) {
  if (exception_classP(renamed_Class)) {
    return (yield_exception_constructor_definition(renamed_Class));
  }
  { Cons* requiredparameters = NIL;
    Cons* assignments = NIL;
    Slot* slot = NULL;

    { Symbol* slotname = NULL;
      Cons* iter_000 = renamed_Class->class_required_slot_names_reader()->the_cons_list;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        slotname = ((Symbol*)(iter_000->value));
        iter_000 = iter_000->rest;
        assignments = cons(listO(4, SYM_DEFCLASS_STELLA_SETF, cons(slotname, cons(SYM_DEFCLASS_STELLA_SELF, NIL)), slotname, NIL), assignments);
        slot = lookup_slot(renamed_Class, slotname);
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(cons(slotname, cons(type_to_symbol(slot->type()), NIL)), NIL);
            if (requiredparameters == NIL) {
              requiredparameters = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(requiredparameters, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(cons(slotname, cons(type_to_symbol(slot->type()), NIL)), NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
    }
    return (listO(8, SYM_DEFCLASS_STELLA_DEFUN, cons(yield_constructor_name(renamed_Class), cons(class_symbol(renamed_Class), NIL)), requiredparameters->concatenate(NIL, 0), KWD_DEFCLASS_CONSTRUCTORp, SYM_DEFCLASS_STELLA_TRUE, KWD_DEFCLASS_PUBLICp, (renamed_Class->publicP() ? TRUE_WRAPPER : FALSE_WRAPPER), cons(listO(3, SYM_DEFCLASS_STELLA_LET, cons(listO(3, SYM_DEFCLASS_STELLA_SELF, class_symbol(renamed_Class), cons(SYM_DEFCLASS_STELLA_NULL, NIL)), NIL), yield_constructor_body(renamed_Class, assignments->reverse())->concatenate(cons(listO(3, SYM_DEFCLASS_STELLA_RETURN, SYM_DEFCLASS_STELLA_SELF, NIL), NIL), 0)), NIL)));
  }
}

Cons* yield_exception_constructor_body(Class* renamed_Class, Symbol* messageparameter, Cons* requiredassignments) {
  { Cons* initialvalueassignments = yield_initial_value_assignments(renamed_Class, (decided_to_recycleP(renamed_Class) ? KWD_DEFCLASS_INITIAL_VALUE_ONLY : KWD_DEFCLASS_ANY_INITIAL_VALUE));

    if ((initialvalueassignments == NIL) &&
        ((requiredassignments == NIL) &&
         ((!((boolean)(renamed_Class->initializer()))) &&
          ((!renamed_Class->activeP()) &&
           (!decided_to_recycleP(renamed_Class)))))) {
      return (cons(listO(4, SYM_DEFCLASS_STELLA_SETQ, SYM_DEFCLASS_STELLA_SELF, listO(3, SYM_DEFCLASS_STELLA_MAKE, class_symbol(renamed_Class), cons(messageparameter, NIL)), NIL), NIL));
    }
    return (cons(listO(4, SYM_DEFCLASS_STELLA_SETQ, SYM_DEFCLASS_STELLA_SELF, listO(3, SYM_DEFCLASS_STELLA_MAKE, class_symbol(renamed_Class), cons(messageparameter, NIL)), NIL), requiredassignments->concatenate(initialvalueassignments->concatenate(yield_initializer_trees(renamed_Class)->concatenate(((renamed_Class->activeP() ? cons(listO(3, SYM_DEFCLASS_STELLA_RUN_CONSTRUCTOR_DEMONS, SYM_DEFCLASS_STELLA_SELF, NIL), NIL) : NIL))->concatenate(NIL, 0), 0), 0), 0)));
  }
}

Cons* yield_exception_constructor_definition(Class* renamed_Class) {
  { Cons* requiredparameters = NIL;
    Cons* assignments = NIL;
    Slot* slot = NULL;
    Symbol* messageparameter = ((Symbol*)(renamed_Class->class_required_slot_names_reader()->first()));

    { Symbol* slotname = NULL;
      Cons* iter_000 = renamed_Class->class_required_slot_names_reader()->rest();
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        slotname = ((Symbol*)(iter_000->value));
        iter_000 = iter_000->rest;
        assignments = cons(listO(4, SYM_DEFCLASS_STELLA_SETF, cons(slotname, cons(SYM_DEFCLASS_STELLA_SELF, NIL)), slotname, NIL), assignments);
        slot = lookup_slot(renamed_Class, slotname);
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(cons(slotname, cons(type_to_symbol(slot->type()), NIL)), NIL);
            if (requiredparameters == NIL) {
              requiredparameters = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(requiredparameters, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(cons(slotname, cons(type_to_symbol(slot->type()), NIL)), NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
    }
    return (listO(8, SYM_DEFCLASS_STELLA_DEFUN, cons(yield_constructor_name(renamed_Class), cons(class_symbol(renamed_Class), NIL)), cons(cons(messageparameter, cons(type_to_symbol(lookup_slot(renamed_Class, messageparameter)->type()), NIL)), requiredparameters->concatenate(NIL, 0)), KWD_DEFCLASS_CONSTRUCTORp, SYM_DEFCLASS_STELLA_TRUE, KWD_DEFCLASS_PUBLICp, (renamed_Class->publicP() ? TRUE_WRAPPER : FALSE_WRAPPER), cons(listO(3, SYM_DEFCLASS_STELLA_LET, cons(listO(3, SYM_DEFCLASS_STELLA_SELF, class_symbol(renamed_Class), cons(SYM_DEFCLASS_STELLA_NULL, NIL)), NIL), yield_exception_constructor_body(renamed_Class, messageparameter, assignments->reverse())->concatenate(cons(listO(3, SYM_DEFCLASS_STELLA_RETURN, SYM_DEFCLASS_STELLA_SELF, NIL), NIL), 0)), NIL)));
  }
}

Cons* yield_destructor_definitions(Class* renamed_Class) {
  { Symbol* classname = class_symbol(renamed_Class);
    Symbol* destructor = ((Symbol*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_DEFCLASS_STELLA_CLASS_DESTRUCTOR, NULL)));
    Symbol* terminator = renamed_Class->terminator();
    Cons* definitions = NIL;

    if (decided_to_recycleP(renamed_Class)) {
      { Storage_Slot* recycleslot = recycle_slot(renamed_Class);
        Symbol* recyclelist = yield_recycle_list_name(renamed_Class);

        if (use_free_listP(renamed_Class)) {
          definitions = cons(listO(5, SYM_DEFCLASS_STELLA_DEFMETHOD, SYM_DEFCLASS_STELLA_FREE, cons(listO(3, SYM_DEFCLASS_STELLA_SELF, classname, NIL), NIL), listO(3, SYM_DEFCLASS_STELLA_WHEN, listO(4, SYM_DEFCLASS_STELLA_AND, SYM_DEFCLASS_STELLA_oRECYCLING_ENABLEDpo, listO(3, SYM_DEFCLASS_STELLA_DEFINEDp, recyclelist, NIL), (((use_sweep_listP(renamed_Class) &&
              (!subtype_ofP(renamed_Class->class_type, SGT_DEFCLASS_STELLA_STANDARD_OBJECT))) ? cons(listO(3, SYM_DEFCLASS_STELLA_DEFINEDp, listO(3, SYM_DEFCLASS_STELLA_ALL_ITEMS, recyclelist, NIL), NIL), NIL) : NIL))->concatenate(NIL, 0)), yield_terminate_object_trees(renamed_Class)->concatenate(yield_register_recycled_item_trees()->concatenate(((((boolean)(recycleslot)) ? listO(3, listO(4, SYM_DEFCLASS_STELLA_SETF, listO(4, SYM_DEFCLASS_STELLA_SLOT_VALUE, SYM_DEFCLASS_STELLA_SELF, recycleslot->slot_name, NIL), listO(3, SYM_DEFCLASS_STELLA_RECYCLED_ITEMS, recyclelist, NIL), NIL), listO(4, SYM_DEFCLASS_STELLA_SETF, listO(3, SYM_DEFCLASS_STELLA_RECYCLED_ITEMS, recyclelist, NIL), SYM_DEFCLASS_STELLA_SELF, NIL), NIL) : cons(listO(4, SYM_DEFCLASS_STELLA_PUSH, listO(3, SYM_DEFCLASS_STELLA_LIST_OF_RECYCLED_ITEMS, recyclelist, NIL), SYM_DEFCLASS_STELLA_SELF, NIL), NIL)))->concatenate((((!use_sweep_listP(renamed_Class)) ? listO(3, listO(3, SYM_DEFCLASS_STELLA_ii, listO(3, SYM_DEFCLASS_STELLA_CURRENT_LENGTH, recyclelist, NIL), NIL), listO(4, SYM_DEFCLASS_STELLA_WHEN, listO(4, SYM_DEFCLASS_STELLA_e, listO(3, SYM_DEFCLASS_STELLA___, SYM_DEFCLASS_STELLA_oRECYCLE_LISTS_MAINTENANCE_TIMERo, NIL), wrap_integer(0), NIL), cons(SYM_DEFCLASS_STELLA_MAINTAIN_RECYCLE_LISTS, NIL), NIL), NIL) : NIL))->concatenate(NIL, 0), 0), 0), 0)), NIL), definitions);
        }
        if (use_sweep_listP(renamed_Class)) {
          if (!use_free_listP(renamed_Class)) {
            definitions = cons(listO(5, SYM_DEFCLASS_STELLA_DEFMETHOD, SYM_DEFCLASS_STELLA_FREE, cons(listO(3, SYM_DEFCLASS_STELLA_SELF, classname, NIL), NIL), SYM_DEFCLASS_STELLA_NULL, NIL), definitions);
          }
          definitions = cons(listO(5, SYM_DEFCLASS_STELLA_DEFMETHOD, SYM_DEFCLASS_STELLA_SWEEP, cons(listO(3, SYM_DEFCLASS_STELLA_SELF, classname, NIL), NIL), listO(3, SYM_DEFCLASS_STELLA_WHEN, listO(4, SYM_DEFCLASS_STELLA_AND, SYM_DEFCLASS_STELLA_oRECYCLING_ENABLEDpo, listO(3, SYM_DEFCLASS_STELLA_DEFINEDp, recyclelist, NIL), NIL), ((use_free_listP(renamed_Class) ? ((((boolean)(recycle_slot(renamed_Class))) ? cons(listO(4, SYM_DEFCLASS_STELLA_SETF, listO(3, SYM_DEFCLASS_STELLA_RECYCLED_ITEMS, recyclelist, NIL), SYM_DEFCLASS_STELLA_NULL, NIL), NIL) : cons(listO(3, SYM_DEFCLASS_STELLA_CLEAR, listO(3, SYM_DEFCLASS_STELLA_LIST_OF_RECYCLED_ITEMS, recyclelist, NIL), NIL), NIL))) : NIL))->concatenate(cons(listO(6, SYM_DEFCLASS_STELLA_LET, cons(listO(3, SYM_DEFCLASS_STELLA_UNUSED, listO(3, SYM_DEFCLASS_STELLA_UNUSED_ITEMS, recyclelist, NIL), NIL), NIL), listO(4, SYM_DEFCLASS_STELLA_SETQ, SYM_DEFCLASS_STELLA_SELF, listO(3, SYM_DEFCLASS_STELLA_ALL_ITEMS, recyclelist, NIL), NIL), listO(3, SYM_DEFCLASS_STELLA_WHILE, listO(3, SYM_DEFCLASS_STELLA_NOT, listO(4, SYM_DEFCLASS_STELLA_EQLp, SYM_DEFCLASS_STELLA_SELF, SYM_DEFCLASS_STELLA_UNUSED, NIL), NIL), yield_terminate_object_trees(renamed_Class)->concatenate(cons(listO(4, SYM_DEFCLASS_STELLA_SETQ, SYM_DEFCLASS_STELLA_SELF, listO(3, SYM_DEFCLASS_STELLA_NEXT_SWEEP_LIST_OBJECT, SYM_DEFCLASS_STELLA_SELF, NIL), NIL), NIL), 0)), listO(4, SYM_DEFCLASS_STELLA_SETF, listO(3, SYM_DEFCLASS_STELLA_UNUSED_ITEMS, recyclelist, NIL), listO(3, SYM_DEFCLASS_STELLA_ALL_ITEMS, recyclelist, NIL), NIL), NIL), NIL), 0)), NIL), definitions);
        }
      }
    }
    else if (((boolean)(terminator)) ||
        ((boolean)(destructor))) {
      if (!(((boolean)(destructor)))) {
        destructor = SYM_DEFCLASS_STELLA_UNMAKE;
      }
      if (!(destructor == SYM_DEFCLASS_STELLA_FREE)) {
        definitions = cons(listO(5, SYM_DEFCLASS_STELLA_DEFMETHOD, SYM_DEFCLASS_STELLA_FREE, cons(listO(3, SYM_DEFCLASS_STELLA_SELF, classname, NIL), NIL), (((boolean)(terminator)) ? listO(4, SYM_DEFCLASS_STELLA_WHEN, cons(terminator, cons(SYM_DEFCLASS_STELLA_SELF, NIL)), cons(destructor, cons(SYM_DEFCLASS_STELLA_SELF, NIL)), NIL) : cons(destructor, cons(SYM_DEFCLASS_STELLA_SELF, NIL))), NIL), definitions);
      }
    }
    return (definitions->reverse());
  }
}

void create_constructor_and_destructor_units(Class* renamed_Class) {
  if (oTRANSLATOROUTPUTLANGUAGEo.get() == KWD_DEFCLASS_CPP_STANDALONE) {
    create_cpp_constructor_unit(renamed_Class);
    return;
  }
  if (!(renamed_Class->abstractP ||
      subtype_ofP(renamed_Class->class_type, SGT_DEFCLASS_STELLA_LITERAL))) {
    { Object* definition = NULL;
      Cons* iter_000 = yield_recycle_list_definitions(renamed_Class);

      while (!(iter_000 == NIL)) {
        definition = iter_000->value;
        iter_000 = iter_000->rest;
        walk_auxiliary_tree(((Cons*)(definition)));
      }
    }
    walk_auxiliary_tree(yield_constructor_definition(renamed_Class));
    { Object* destructor = NULL;
      Cons* iter_001 = yield_destructor_definitions(renamed_Class);

      while (!(iter_001 == NIL)) {
        destructor = iter_001->value;
        iter_001 = iter_001->rest;
        walk_auxiliary_tree(((Cons*)(destructor)));
      }
    }
  }
}

boolean parameterized_classP(Class* renamed_Class) {
  { boolean foundP_000 = FALSE;

    { Symbol* parameters = NULL;
      Cons* iter_000 = renamed_Class->parameters()->the_cons_list;

      while (!(iter_000 == NIL)) {
        parameters = ((Symbol*)(iter_000->value));
        iter_000 = iter_000->rest;
        foundP_000 = TRUE;
        break;
      }
    }
    { boolean value_000 = foundP_000;

      return (value_000);
    }
  }
}

boolean class_with_initializerP(Class* renamed_Class) {
  { boolean foundP_000 = FALSE;

    { Slot* slot = NULL;
      Iterator* iter_000 = renamed_Class->class_slots();

      while (iter_000->nextP()) {
        slot = ((Slot*)(iter_000->value));
        if (storage_slotP(slot)) {
          foundP_000 = TRUE;
          break;
        }
      }
    }
    { boolean value_000 = foundP_000;

      return (value_000);
    }
  }
}

Symbol* yield_initializer_name(Class* renamed_Class) {
  return (intern_derived_symbol(renamed_Class->class_type, string_concatenate("initialize-", class_name(renamed_Class), 0)));
}

Cons* yield_initializer_definition(Class* renamed_Class) {
  { Cons* parentinitforms = NIL;

    { Surrogate* super = NULL;
      Cons* iter_000 = renamed_Class->class_direct_supers->the_cons_list;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        super = ((Surrogate*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (class_with_initializerP(((Class*)(super->surrogate_value)))) {
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(cons(yield_initializer_name(((Class*)(super->surrogate_value))), cons(SYM_DEFCLASS_STELLA_SELF, NIL)), NIL);
              if (parentinitforms == NIL) {
                parentinitforms = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(parentinitforms, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(cons(yield_initializer_name(((Class*)(super->surrogate_value))), cons(SYM_DEFCLASS_STELLA_SELF, NIL)), NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
    }
    return (listO(3, SYM_DEFCLASS_STELLA_DEFUN, yield_initializer_name(renamed_Class), cons(cons(listO(3, SYM_DEFCLASS_STELLA_SELF, yield_type_spec_tree(renamed_Class->class_type), NIL), NIL), parentinitforms->concatenate(yield_local_initial_value_assignments(renamed_Class)->concatenate(NIL, 0), 0))));
  }
}

void create_cpp_constructor_unit(Class* renamed_Class) {
  if (!(parameterized_classP(renamed_Class))) {
    { Object* classname = yield_type_spec_tree(renamed_Class->class_type);
      boolean hasinitializerP = class_with_initializerP(renamed_Class);

      if (hasinitializerP) {
        walk_auxiliary_tree(yield_initializer_definition(renamed_Class));
      }
      help_walk_auxiliary_tree(listO(3, SYM_DEFCLASS_STELLA_DEFMETHOD, classname, listO(4, cons(listO(3, SYM_DEFCLASS_STELLA_SELF, classname, NIL), NIL), KWD_DEFCLASS_PUBLICp, (renamed_Class->publicP() ? TRUE_WRAPPER : FALSE_WRAPPER), ((hasinitializerP ? cons(cons(yield_initializer_name(renamed_Class), cons(SYM_DEFCLASS_STELLA_SELF, NIL)), NIL) : cons(SYM_DEFCLASS_STELLA_NULL, NIL)))->concatenate(NIL, 0))), FALSE);
    }
  }
}

boolean cpp_constructorP(Method_Slot* method) {
  return ((method->type() == SGT_DEFCLASS_STELLA_VOID) &&
      ((method->method_parameter_names_reader()->length() == 1) &&
       (string_eqlP(method->slot_owner->symbol_name, method->slot_name->symbol_name) &&
        (((Module*)(method->slot_owner->home_context)) == ((Module*)(method->slot_name->home_context))))));
}

// If `TRUE' calls to `new' or `free' on classes with :recycle-method;
// specifications will actually operate on recycle lists.  Otherwise, all
// recycle list operations will be no-ops.
DEFINE_STELLA_SPECIAL(oRECYCLING_ENABLEDpo, boolean , TRUE);

List* oALL_RECYCLE_LISTSo = NULL;

Recycle_List* create_recycle_list(Surrogate* classtype, int itemsize) {
  { Recycle_List* list = new_recycle_list();

    { Recycle_List* value_000 = NULL;

      { Recycle_List* list = NULL;
        Cons* iter_000 = oALL_RECYCLE_LISTSo->the_cons_list;

        while (!(iter_000 == NIL)) {
          list = ((Recycle_List*)(iter_000->value));
          iter_000 = iter_000->rest;
          if (list->recycle_list_of == classtype) {
            value_000 = list;
            break;
          }
        }
      }
      { Recycle_List* oldlist = value_000;

        if (((boolean)(oldlist))) {
          clear_recycle_list(oldlist);
          oALL_RECYCLE_LISTSo->remove(oldlist);
        }
        oALL_RECYCLE_LISTSo->push(list);
        list->recycle_list_of = classtype;
        list->item_size = itemsize;
        return (list);
      }
    }
  }
}

void clear_recycle_list(Recycle_List* list) {
  // Reset `list' to its empty state.
  list->all_items = NULL;
  list->unused_items = NULL;
  list->recycled_items = NULL;
  list->list_of_recycled_items->the_cons_list = NIL;
  list->current_length = 0;
}

void clear_recycle_lists() {
  // Reset all currently active recycle lists to their empty state.
  oRECYCLE_LISTS_MAINTENANCE_TIMERo = oRECYCLE_LISTS_MAINTENANCE_INTERVALo;
  { Recycle_List* list = NULL;
    Cons* iter_000 = oALL_RECYCLE_LISTSo->the_cons_list;

    while (!(iter_000 == NIL)) {
      list = ((Recycle_List*)(iter_000->value));
      iter_000 = iter_000->rest;
      clear_recycle_list(list);
    }
  }
}

void print_recycle_lists() {
  // Print the current state of all recycle lists.
  cout << "Approximate space occupied by recycle lists: " << (total_recycle_lists_size() * 4) << " bytes" << endl;
  { Recycle_List* list = NULL;
    Cons* iter_000 = oALL_RECYCLE_LISTSo->the_cons_list;

    while (!(iter_000 == NIL)) {
      list = ((Recycle_List*)(iter_000->value));
      iter_000 = iter_000->rest;
      cout << list << endl;
    }
  }
}

// Maximum number of bytes to be occupied by recycle lists.
int oMAX_RECYCLE_LIST_BYTESo = 3000000;

// Total number of recycle list extensions after which a
// size maintenance operation is performed to ensure that
// the `*MAX-RECYCLE-LIST-BYTES*' limit is still met.
int oRECYCLE_LISTS_MAINTENANCE_INTERVALo = 5000;

int oRECYCLE_LISTS_MAINTENANCE_TIMERo = NULL_INTEGER;

void maintain_recycle_lists() {
  if (((boolean)(oCURRENTLY_RECYCLED_ITEMSo))) {
    return;
  }
  { int totalsize = total_recycle_lists_size();
    int maxsize = floor_(oMAX_RECYCLE_LIST_BYTESo / 4.0);
    int listsize = NULL_INTEGER;
    boolean classredefinitionP = FALSE;

    oRECYCLE_LISTS_MAINTENANCE_TIMERo = oRECYCLE_LISTS_MAINTENANCE_INTERVALo;
    { Recycle_List* list = NULL;
      Cons* iter_000 = oALL_RECYCLE_LISTSo->the_cons_list;

      while (!(iter_000 == NIL)) {
        list = ((Recycle_List*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (((boolean)(((Class*)(list->recycle_list_of->surrogate_value)))) &&
            (!decided_to_recycleP(((Class*)(list->recycle_list_of->surrogate_value))))) {
          classredefinitionP = TRUE;
          clear_recycle_list(list);
          list->deletedP_setter(TRUE);
        }
      }
    }
    if (classredefinitionP) {
      oALL_RECYCLE_LISTSo->remove_deleted_members();
    }
    if (totalsize > maxsize) {
      oALL_RECYCLE_LISTSo->sort(((cpp_function_code)(&recycle_list_sizeL)));
      oALL_RECYCLE_LISTSo->reverse();
      while (totalsize > maxsize) {
        { Recycle_List* list = NULL;
          Cons* iter_001 = oALL_RECYCLE_LISTSo->the_cons_list;

          while (!(iter_001 == NIL)) {
            list = ((Recycle_List*)(iter_001->value));
            iter_001 = iter_001->rest;
            if (totalsize <= maxsize) {
              break;
            }
            listsize = list->item_size * list->current_length;
            if (listsize > 0) {
              totalsize = totalsize - listsize;
            }
            else {
              continue;
            }
            reduce_recycle_list_size(list, 0.5);
            totalsize = totalsize + (list->item_size * list->current_length);
          }
        }
      }
    }
  }
}

boolean Recycle_List::deletedP() {
  { Recycle_List* self = this;

    return (!((boolean)(self->recycle_list_of)));
  }
}

boolean Recycle_List::deletedP_setter(boolean value) {
  { Recycle_List* self = this;

    if (value) {
      self->recycle_list_of = NULL;
    }
    else {
      if (!((boolean)(self->recycle_list_of))) {
        self->recycle_list_of = SGT_DEFCLASS_STELLA_UNKNOWN;
      }
    }
    return (value);
  }
}

int recycle_list_size(Recycle_List* list) {
  return (list->item_size * list->current_length);
}

int total_recycle_lists_size() {
  { int size = 0;

    { Recycle_List* list = NULL;
      Cons* iter_000 = oALL_RECYCLE_LISTSo->the_cons_list;

      while (!(iter_000 == NIL)) {
        list = ((Recycle_List*)(iter_000->value));
        iter_000 = iter_000->rest;
        size = size + (list->item_size * list->current_length);
      }
    }
    return (size);
  }
}

boolean recycle_list_sizeL(Recycle_List* list1, Recycle_List* list2) {
  return ((list1->item_size * list1->current_length) < (list2->item_size * list2->current_length));
}

void reduce_recycle_list_size(Recycle_List* list, double fraction) {
  { int length = list->current_length;
    int reducedlength = floor_(length * fraction);
    Surrogate* itemtype = list->recycle_list_of;
    Object* cursor = NULL;
    Object* trailingcursor = NULL;
    Object* unused = NULL;
    Storage_Slot* nextslot = NULL;

    if ((((boolean)(list->all_items)) ||
        ((boolean)(list->recycled_items))) &&
        (!subtype_ofP(itemtype, SGT_DEFCLASS_STELLA_STANDARD_OBJECT))) {
      clear_recycle_list(list);
      return;
    }
    oRECYCLING_ENABLEDpo.set(FALSE);
    if (((boolean)(list->all_items))) {
      cursor = list->all_items;
      unused = list->unused_items;
      nextslot = ((Storage_Slot*)(lookup_slot(((Class*)(itemtype->surrogate_value)), SYM_DEFCLASS_STELLA_NEXT_SWEEP_LIST_OBJECT)));
      { int i = NULL_INTEGER;
        int iter_000 = 1;
        int upper_bound_000 = length - reducedlength;

        while (iter_000 <= upper_bound_000) {
          i = iter_000;
          iter_000 = iter_000 + 1;
          if (cursor == unused) {
            list->unused_items = list->all_items;
          }
          trailingcursor = cursor;
          cursor = read_slot_value(((Standard_Object*)(cursor)), nextslot);
          put_slot_value(((Standard_Object*)(trailingcursor)), nextslot, NULL);
        }
      }
      if (eqlP(list->unused_items, list->all_items)) {
        list->unused_items = cursor;
      }
      list->all_items = cursor;
    }
    if (list->list_of_recycled_items->non_emptyP()) {
      cursor = list->list_of_recycled_items->the_cons_list->nth_rest(reducedlength);
      if (((boolean)(cursor))) {
        ((Cons*)(cursor))->rest = NIL;
      }
    }
    else if (((boolean)(list->recycled_items))) {
      if (((boolean)(list->all_items))) {
        list->recycled_items = NULL;
      }
      else {
        cursor = list->recycled_items;
        nextslot = recycle_slot(((Class*)(itemtype->surrogate_value)));
        { int i = NULL_INTEGER;
          int iter_001 = 2;
          int upper_bound_001 = reducedlength;
          boolean unboundedP_000 = upper_bound_001 == NULL_INTEGER;

          while (unboundedP_000 ||
              (iter_001 <= upper_bound_001)) {
            i = iter_001;
            iter_001 = iter_001 + 1;
            cursor = read_slot_value(((Standard_Object*)(cursor)), nextslot);
            if (!((boolean)(cursor))) {
              break;
            }
          }
        }
        if (((boolean)(cursor))) {
          put_slot_value(((Standard_Object*)(cursor)), nextslot, NULL);
        }
      }
    }
    list->current_length = reducedlength;
    oRECYCLING_ENABLEDpo.set(TRUE);
  }
}

void Object::sweep() {
  // Default method.  Sweep up all `self'-type objects.
  { Object* self = this;

  }
}

void sweep_transients() {
  { Recycle_List* recyclelist = NULL;
    Cons* iter_000 = oALL_RECYCLE_LISTSo->the_cons_list;

    while (!(iter_000 == NIL)) {
      recyclelist = ((Recycle_List*)(iter_000->value));
      iter_000 = iter_000->rest;
      if (((boolean)(recyclelist->all_items)) &&
          isaP(recyclelist->all_items, SGT_DEFCLASS_STELLA_TRANSIENT_MIXIN)) {
        recyclelist->all_items->sweep();
      }
    }
  }
  free_transient_symbols();
}

List* make_non_recycled_list() {
  { List* list = new List();

    list->the_cons_list = NIL;
    return (list);
  }
}

// During recycle list debugging holds all items that live in
// some free list.  Depending on program behavior, this table can get quite large!
Hash_Table* oCURRENTLY_RECYCLED_ITEMSo = NULL;

void register_recycled_item(Object* item) {
  if (((boolean)(oCURRENTLY_RECYCLED_ITEMSo->lookup(item)))) {
    break_program("Trying to recyle an already recycled item!");
  }
  else {
    oCURRENTLY_RECYCLED_ITEMSo->insert_at(item, item);
  }
}

void unregister_recycled_item(Object* item) {
  oCURRENTLY_RECYCLED_ITEMSo->remove_at(item);
}

void start_recycle_list_debugging() {
  oCURRENTLY_RECYCLED_ITEMSo = NULL;
  clear_recycle_lists();
  oCURRENTLY_RECYCLED_ITEMSo = new_hash_table();
}

void stop_recycle_list_debugging() {
  oCURRENTLY_RECYCLED_ITEMSo = NULL;
}

boolean recycle_list_debugging_enabledP() {
  return (((boolean)(oCURRENTLY_RECYCLED_ITEMSo)));
}

boolean oTRANSLATE_WITH_RECYCLE_LIST_DEBUGGINGpo = FALSE;

boolean translate_with_recycle_list_debuggingP() {
  return (oTRANSLATE_WITH_RECYCLE_LIST_DEBUGGINGpo);
}

Cons* yield_register_recycled_item_trees() {
  if (oTRANSLATE_WITH_RECYCLE_LIST_DEBUGGINGpo) {
    return (cons(listO(4, SYM_DEFCLASS_STELLA_WHEN, cons(SYM_DEFCLASS_STELLA_RECYCLE_LIST_DEBUGGING_ENABLEDp, NIL), listO(3, SYM_DEFCLASS_STELLA_REGISTER_RECYCLED_ITEM, SYM_DEFCLASS_STELLA_SELF, NIL), NIL), NIL));
  }
  else {
    return (NIL);
  }
}

Cons* yield_unregister_recycled_item_trees() {
  if (oTRANSLATE_WITH_RECYCLE_LIST_DEBUGGINGpo) {
    return (cons(listO(4, SYM_DEFCLASS_STELLA_WHEN, cons(SYM_DEFCLASS_STELLA_RECYCLE_LIST_DEBUGGING_ENABLEDp, NIL), listO(3, SYM_DEFCLASS_STELLA_UNREGISTER_RECYCLED_ITEM, SYM_DEFCLASS_STELLA_SELF, NIL), NIL), NIL));
  }
  else {
    return (NIL);
  }
}

boolean has_non_null_default_valueP(Storage_Slot* slot) {
  { Object* defaultform = slot->system_default_value();

    return (((boolean)(defaultform)) &&
        (!equal_cons_treesP(type_to_null_value_tree(slot->type()), defaultform)));
  }
}

boolean system_defined_slot_readerP(Storage_Slot* slot) {
  return ((!((boolean)(slot->reader()))) &&
      (has_non_null_default_valueP(slot) ||
       (slot->activeP() ||
        slot->slot_hardwiredP)));
}

boolean system_defined_slot_writerP(Storage_Slot* slot) {
  return ((!((boolean)(slot->writer()))) &&
      slot->activeP());
}

Object* yield_slot_type_tree(Storage_Slot* slot) {
  return (yield_type_spec_tree((((boolean)(((Compound_Type_Specifier*)(dynamic_slot_value(slot->dynamic_slots, SYM_DEFCLASS_STELLA_SLOT_TYPE_SPECIFIER, NULL))))) ? ((Standard_Object*)(((Compound_Type_Specifier*)(dynamic_slot_value(slot->dynamic_slots, SYM_DEFCLASS_STELLA_SLOT_TYPE_SPECIFIER, NULL))))) : ((Standard_Object*)(slot->slot_base_type)))));
}

Symbol* yield_hardwired_slot_variable(Storage_Slot* slot) {
  return (intern_derived_symbol(slot->slot_name, string_concatenate("*HARDWIRED-", string_concatenate(slot->slot_name->symbol_name, string_concatenate("-ON-", string_concatenate(slot->slot_owner->symbol_name, "*", 0), 0), 0), 0)));
}

Cons* yield_hardwired_slot_reader_body(Storage_Slot* slot, Cons*& _return1) {
  { Symbol* slotvariable = yield_hardwired_slot_variable(slot);
    Object* slottypedesc = yield_type_spec_tree(slot->compute_return_type_spec(slot->slot_owner));

    { Cons* _return_temp = listO(3, SYM_DEFCLASS_STELLA_RETURN, slotvariable, NIL);

      _return1 = listO(3, SYM_DEFCLASS_STELLA_DEFGLOBAL, slotvariable, cons(slottypedesc, cons(SYM_DEFCLASS_STELLA_NULL, NIL)));
      return (_return_temp);
    }
  }
}

void finalize_auxiliary_method(Method_Slot* method) {
  if ((!((boolean)(method->slot_base_type))) &&
      ((boolean)(((Compound_Type_Specifier*)(dynamic_slot_value(method->dynamic_slots, SYM_DEFCLASS_STELLA_SLOT_TYPE_SPECIFIER, NULL)))))) {
    method->slot_base_type = validate_type_specifier(((Compound_Type_Specifier*)(dynamic_slot_value(method->dynamic_slots, SYM_DEFCLASS_STELLA_SLOT_TYPE_SPECIFIER, NULL))), ((Class*)(method->slot_owner->surrogate_value)), FALSE);
  }
}

Cons* wrap_with_default_value_and_return_code(Storage_Slot* slot, Object* defaultexpression) {
  { Standard_Object* dummy1;
    Standard_Object* dummy2;

    { Cons* readcode = listO(4, SYM_DEFCLASS_STELLA_SLOT_VALUE, SYM_DEFCLASS_STELLA_SELF, slot->slot_name, NIL);

      if (((boolean)(defaultexpression))) {
        if (slot->type() == SGT_DEFCLASS_STELLA_BOOLEAN) {
          if (!(slot->allocation() == KWD_DEFCLASS_DYNAMIC)) {
            { 
              BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
              signal_translation_warning();
              if (!(suppress_warningsP())) {
                print_error_context(">> WARNING: ", STANDARD_WARNING);
                *(STANDARD_WARNING->native_stream) << endl << " " << "Ignoring :default annotation on non-dynamic BOOLEAN slot " << "`" << de_uglify_parse_tree(slot) << "'" << "." << endl << " Limited :default support is available for dynamic BOOLEAN " << "slots," << endl << " and full support for THREE-VALUED-BOOLEAN slots" << "." << endl;
              }
            }
          }
          else {
            {
              set_literal_type_info(SGT_DEFCLASS_STELLA_BOOLEAN, KWD_DEFCLASS_NULL_WRAPPER, listO(3, SYM_DEFCLASS_STELLA_INLINE_WRAP_BOOLEAN, defaultexpression, NIL));
              push_variable_binding(SYM_DEFCLASS_STELLA_SELF, slot->slot_owner);
              readcode = sys_tree(walk_expression_tree(readcode, SGT_DEFCLASS_STELLA_BOOLEAN, slot->slot_name, FALSE, dummy2), SGT_DEFCLASS_STELLA_BOOLEAN, dummy1);
              pop_variable_binding();
              set_literal_type_info(SGT_DEFCLASS_STELLA_BOOLEAN, KWD_DEFCLASS_NULL_WRAPPER, SYM_DEFCLASS_STELLA_FALSE_WRAPPER);
            }
          }
          return (listO(3, SYM_DEFCLASS_STELLA_RETURN, readcode, NIL));
        }
        else {
          return (listO(4, SYM_DEFCLASS_STELLA_LET, cons(listO(3, SYM_DEFCLASS_STELLA_ANSWER, readcode, NIL), NIL), listO(5, SYM_DEFCLASS_STELLA_IF, listO(3, SYM_DEFCLASS_STELLA_NULLp, SYM_DEFCLASS_STELLA_ANSWER, NIL), listO(3, SYM_DEFCLASS_STELLA_RETURN, copy_cons_tree(defaultexpression), NIL), listO(3, SYM_DEFCLASS_STELLA_RETURN, SYM_DEFCLASS_STELLA_ANSWER, NIL), NIL), NIL));
        }
      }
      else {
        return (listO(3, SYM_DEFCLASS_STELLA_RETURN, readcode, NIL));
      }
    }
  }
}

Cons* yield_slot_reader_tree(Storage_Slot* slot, Class* renamed_Class) {
  { Cons* parameters = cons(listO(3, SYM_DEFCLASS_STELLA_SELF, intern_symbol_in_module(renamed_Class->class_type->symbol_name, ((Module*)(renamed_Class->class_type->home_context)), TRUE), NIL), NIL);
    Object* typetree = yield_slot_type_tree(slot);
    Object* defaultexpression = (has_non_null_default_valueP(slot) ? slot->system_default_value() : ((Object*)(NULL)));
    boolean slotvisibleP = subtype_ofP(renamed_Class->class_type, slot->slot_owner);
    Cons* readcode = NULL;
    Cons* auxiliarycode = NULL;

    if (!slotvisibleP) {
      readcode = listO(4, SYM_DEFCLASS_STELLA_PROGN, listO(4, SYM_DEFCLASS_STELLA_ERROR, wrap_string("Slot '"), wrap_string(slot->slot_name->symbol_name), listO(3, wrap_string("' does not exist on "), SYM_DEFCLASS_STELLA_SELF, NIL)), listO(3, SYM_DEFCLASS_STELLA_RETURN, SYM_DEFCLASS_STELLA_NULL, NIL), NIL);
    }
    else if (slot->activeP()) {
      readcode = yield_active_slot_reader_body(slot, defaultexpression);
    }
    else if (slot->slot_hardwiredP) {
      readcode = yield_hardwired_slot_reader_body(slot, auxiliarycode);
      if (renamed_Class->class_type == slot->slot_owner) {
        walk_auxiliary_tree(auxiliarycode);
      }
    }
    else {
      readcode = wrap_with_default_value_and_return_code(slot, defaultexpression);
    }
    return (listO(4, SYM_DEFCLASS_STELLA_DEFMETHOD, cons(slot->slot_name, cons(typetree, NIL)), parameters, listO(4, KWD_DEFCLASS_AUXILIARYp, SYM_DEFCLASS_STELLA_TRUE, readcode, NIL)));
  }
}

Cons* yield_slot_writer_tree(Storage_Slot* slot, Class* renamed_Class) {
  { Object* typetree = yield_slot_type_tree(slot);
    Cons* parameters = listO(3, listO(3, SYM_DEFCLASS_STELLA_SELF, intern_symbol_in_module(renamed_Class->class_type->symbol_name, ((Module*)(renamed_Class->class_type->home_context)), TRUE), NIL), listO(3, SYM_DEFCLASS_STELLA_VALUE, typetree, NIL), NIL);
    boolean slotvisibleP = subtype_ofP(renamed_Class->class_type, slot->slot_owner);
    Cons* writecode = NULL;

    if (!slotvisibleP) {
      writecode = listO(4, SYM_DEFCLASS_STELLA_PROGN, listO(4, SYM_DEFCLASS_STELLA_ERROR, wrap_string("Slot '"), wrap_string(slot->slot_name->symbol_name), listO(3, wrap_string("' does not exist on "), SYM_DEFCLASS_STELLA_SELF, NIL)), listO(3, SYM_DEFCLASS_STELLA_RETURN, SYM_DEFCLASS_STELLA_NULL, NIL), NIL);
    }
    else if (slot->activeP()) {
      writecode = yield_active_slot_writer_body(slot);
    }
    else {
      writecode = listO(3, SYM_DEFCLASS_STELLA_RETURN, listO(4, SYM_DEFCLASS_STELLA_SETF, listO(4, SYM_DEFCLASS_STELLA_SLOT_VALUE, SYM_DEFCLASS_STELLA_SELF, slot->slot_name, NIL), SYM_DEFCLASS_STELLA_VALUE, NIL), NIL);
    }
    return (listO(4, SYM_DEFCLASS_STELLA_DEFMETHOD, cons(yield_setter_method_name(slot->slot_name), cons(copy_cons_tree(typetree), NIL)), parameters, listO(3, KWD_DEFCLASS_AUXILIARYp, SYM_DEFCLASS_STELLA_TRUE, ((slotvisibleP ? NIL : cons(listO(3, SYM_DEFCLASS_STELLA_IGNORE, SYM_DEFCLASS_STELLA_VALUE, NIL), NIL)))->concatenate(cons(writecode, NIL), 0))));
  }
}

boolean slot_value_is_bare_literalP(Slot* slot) {
  { Surrogate* type = slot->type();

    return (((boolean)(type)) &&
        (((boolean)(((Class*)(type->surrogate_value)))) &&
         subtype_ofP(slot->type(), SGT_DEFCLASS_STELLA_LITERAL)));
  }
}

boolean generically_accessible_typeP(Surrogate* type) {
  return (subtype_ofP(type, SGT_DEFCLASS_STELLA_OBJECT) ||
      (subtype_ofP(type, SGT_DEFCLASS_STELLA_LITERAL) &&
       ((boolean)(lookup_literal_type_info(type, KWD_DEFCLASS_WRAP_FUNCTION)))));
}

boolean generic_slot_readerP(Storage_Slot* slot) {
  { Surrogate* type = canonical_type(slot->type());

    return ((!slot->abstractP) &&
        (subtype_ofP(slot->slot_owner, SGT_DEFCLASS_STELLA_OBJECT) &&
         (((boolean)(((Class*)(type->surrogate_value)))) &&
          generically_accessible_typeP(type))));
  }
}

boolean generic_slot_writerP(Storage_Slot* slot) {
  return ((!slot->slot_hardwiredP) &&
      generic_slot_readerP(slot));
}

boolean slot_access_methodP(Storage_Slot* slot) {
  return (((boolean)(slot->reader())) ||
      (system_defined_slot_readerP(slot) ||
       (((boolean)(slot->writer())) ||
        system_defined_slot_writerP(slot))));
}

Symbol* yield_generic_slot_accessor_name(Class* renamed_Class) {
  return (intern_derived_symbol(renamed_Class->class_type, string_concatenate("ACCESS-", string_concatenate(renamed_Class->class_type->symbol_name, "-SLOT-VALUE", 0), 0)));
}

Cons* yield_generic_slot_accessor_definition(Class* renamed_Class) {
  { Cons* clauses = NIL;
    Cons* accesstree = NIL;
    Object* valuetree = NULL;
    Surrogate* slottype = NULL;

    if (!subclass_ofP(renamed_Class, ((Class*)(SGT_DEFCLASS_STELLA_OBJECT->surrogate_value)))) {
      return (NULL);
    }
    { Slot* slot = NULL;
      Cons* iter_000 = renamed_Class->class_local_slots->the_cons_list;

      while (!(iter_000 == NIL)) {
        slot = ((Slot*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (storage_slotP(slot) &&
            generic_slot_readerP(((Storage_Slot*)(slot)))) {
          if (slot->slot_externalP ||
              (space_saver_slot_allocationP(((Storage_Slot*)(slot))) &&
               (!slot_access_methodP(((Storage_Slot*)(slot)))))) {
            continue;
          }
          accesstree = listO(4, SYM_DEFCLASS_STELLA_SETQ, SYM_DEFCLASS_STELLA_VALUE, cons(slot->slot_name, cons(SYM_DEFCLASS_STELLA_SELF, NIL)), NIL);
          if (!((Storage_Slot*)(slot))->slot_hardwiredP) {
            slottype = canonical_type(slot->type());
            valuetree = SYM_DEFCLASS_STELLA_VALUE;
            if (subtype_ofP(slottype, SGT_DEFCLASS_STELLA_LITERAL) &&
                ((boolean)(lookup_literal_type_info(slottype, KWD_DEFCLASS_WRAP_FUNCTION)))) {
              valuetree = listO(4, SYM_DEFCLASS_STELLA_CAST, SYM_DEFCLASS_STELLA_VALUE, slottype->type_to_wrapped_type(), NIL);
            }
            accesstree = listO(5, SYM_DEFCLASS_STELLA_IF, SYM_DEFCLASS_STELLA_SETVALUEp, listO(4, SYM_DEFCLASS_STELLA_SETF, cons(slot->slot_name, cons(SYM_DEFCLASS_STELLA_SELF, NIL)), valuetree, NIL), accesstree, NIL);
          }
          clauses = cons(cons(slot->slot_name, cons(accesstree, NIL)), clauses);
        }
      }
    }
    if (subclass_ofP(renamed_Class, ((Class*)(SGT_DEFCLASS_STELLA_DYNAMIC_SLOTS_MIXIN->surrogate_value)))) {
      { Cons* renameclauses = NIL;

        { Slot* slot = NULL;
          Cons* iter_001 = renamed_Class->class_local_slots->the_cons_list;
          Cons* collect_000 = NULL;

          while (!(iter_001 == NIL)) {
            slot = ((Slot*)(iter_001->value));
            iter_001 = iter_001->rest;
            if (storage_slotP(slot) &&
                ((!slot->slot_externalP) &&
                 (space_saver_slot_allocationP(((Storage_Slot*)(slot))) &&
                  ((!slot_access_methodP(((Storage_Slot*)(slot)))) &&
                   (!(slot == canonical_slot(((Storage_Slot*)(slot))))))))) {
              if (!((boolean)(collect_000))) {
                {
                  collect_000 = cons(cons(slot->slot_name, cons(listO(4, SYM_DEFCLASS_STELLA_SETQ, SYM_DEFCLASS_STELLA_SLOTNAME, listO(3, SYM_DEFCLASS_STELLA_QUOTE, canonical_slot(((Storage_Slot*)(slot)))->slot_name, NIL), NIL), NIL)), NIL);
                  if (renameclauses == NIL) {
                    renameclauses = collect_000;
                  }
                  else {
                    add_cons_to_end_of_cons_list(renameclauses, collect_000);
                  }
                }
              }
              else {
                {
                  collect_000->rest = cons(cons(slot->slot_name, cons(listO(4, SYM_DEFCLASS_STELLA_SETQ, SYM_DEFCLASS_STELLA_SLOTNAME, listO(3, SYM_DEFCLASS_STELLA_QUOTE, canonical_slot(((Storage_Slot*)(slot)))->slot_name, NIL), NIL), NIL)), NIL);
                  collect_000 = collect_000->rest;
                }
              }
            }
          }
        }
        if (!(renameclauses == NIL)) {
          renameclauses = cons(listO(3, SYM_DEFCLASS_STELLA_CASE, SYM_DEFCLASS_STELLA_SLOTNAME, renameclauses->concatenate(cons(listO(3, SYM_DEFCLASS_STELLA_OTHERWISE, SYM_DEFCLASS_STELLA_NULL, NIL), NIL), 0)), NIL);
        }
        clauses = cons(cons(SYM_DEFCLASS_STELLA_OTHERWISE, renameclauses->concatenate(cons(listO(5, SYM_DEFCLASS_STELLA_IF, SYM_DEFCLASS_STELLA_SETVALUEp, listO(6, SYM_DEFCLASS_STELLA_SET_DYNAMIC_SLOT_VALUE, listO(3, SYM_DEFCLASS_STELLA_DYNAMIC_SLOTS, SYM_DEFCLASS_STELLA_SELF, NIL), SYM_DEFCLASS_STELLA_SLOTNAME, SYM_DEFCLASS_STELLA_VALUE, SYM_DEFCLASS_STELLA_NULL, NIL), listO(4, SYM_DEFCLASS_STELLA_SETQ, SYM_DEFCLASS_STELLA_VALUE, listO(4, SYM_DEFCLASS_STELLA_LOOKUP, listO(3, SYM_DEFCLASS_STELLA_DYNAMIC_SLOTS, SYM_DEFCLASS_STELLA_SELF, NIL), SYM_DEFCLASS_STELLA_SLOTNAME, NIL), NIL), NIL), NIL), 0)), clauses);
      }
    }
    if (clauses == NIL) {
      return (NULL);
    }
    else {
      return (listO(6, SYM_DEFCLASS_STELLA_DEFUN, cons(yield_generic_slot_accessor_name(renamed_Class), cons(SYM_DEFCLASS_STELLA_OBJECT, NIL)), listO(5, listO(3, SYM_DEFCLASS_STELLA_SELF, renamed_Class->class_type, NIL), listO(3, SYM_DEFCLASS_STELLA_SLOTNAME, SYM_DEFCLASS_STELLA_SYMBOL, NIL), listO(3, SYM_DEFCLASS_STELLA_VALUE, SYM_DEFCLASS_STELLA_OBJECT, NIL), listO(3, SYM_DEFCLASS_STELLA_SETVALUEp, SYM_DEFCLASS_STELLA_BOOLEAN, NIL), NIL), listO(3, SYM_DEFCLASS_STELLA_CASE, SYM_DEFCLASS_STELLA_SLOTNAME, clauses->reverse()->concatenate(NIL, 0)), listO(3, SYM_DEFCLASS_STELLA_RETURN, SYM_DEFCLASS_STELLA_VALUE, NIL), NIL));
    }
  }
}

void create_generic_slot_accessor_unit(Class* renamed_Class) {
  { Cons* tree = yield_generic_slot_accessor_definition(renamed_Class);

    if (((boolean)(tree))) {
      walk_auxiliary_tree(tree);
    }
  }
}

Cons* yield_generic_slot_accessor_attachment(Class* renamed_Class, Symbol* classref) {
  { boolean foundP_000 = FALSE;

    { Slot* slot = NULL;
      Cons* iter_000 = renamed_Class->class_local_slots->the_cons_list;

      while (!(iter_000 == NIL)) {
        slot = ((Slot*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (storage_slotP(slot) &&
            generic_slot_readerP(((Storage_Slot*)(slot)))) {
          foundP_000 = TRUE;
          break;
        }
      }
    }
    if (foundP_000) {
      return (cons(listO(4, SYM_DEFCLASS_STELLA_SETF, listO(3, SYM_DEFCLASS_STELLA_CLASS_SLOT_ACCESSOR_CODE, classref, NIL), listO(4, SYM_DEFCLASS_STELLA_THE_CODE, KWD_DEFCLASS_FUNCTION, yield_generic_slot_accessor_name(renamed_Class), NIL), NIL), NIL));
    }
    else {
      return (NIL);
    }
  }
}

Cons* yield_mixin_class_users_tree(Class* renamed_Class) {
  { Cons* result = NIL;

    { Surrogate* sub = NULL;
      Cons* iter_000 = renamed_Class->class_direct_subs->the_cons_list;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        sub = ((Surrogate*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (!type_to_class(sub)->mixinP) {
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(intern_symbol_in_module(sub->symbol_name, ((Module*)(sub->home_context)), TRUE), NIL);
              if (result == NIL) {
                result = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(result, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(intern_symbol_in_module(sub->symbol_name, ((Module*)(sub->home_context)), TRUE), NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
    }
    return (result);
  }
}

Symbol* yield_mixin_slot_reader_name(Storage_Slot* slot) {
  return (intern_derived_symbol(slot->slot_name, string_concatenate(slot->slot_name->symbol_name, string_concatenate("-ON-", slot->slot_owner->symbol_name, 0), 0)));
}

Symbol* yield_mixin_slot_writer_name(Storage_Slot* slot) {
  return (yield_setter_method_name(yield_mixin_slot_reader_name(slot)));
}

Cons* yield_mixin_slot_reader_tree(Storage_Slot* slot) {
  { Symbol* readername = yield_mixin_slot_reader_name(slot);
    Object* typetree = yield_slot_type_tree(slot);

    return (listO(7, SYM_DEFCLASS_STELLA_DEFUN, cons(readername, cons(typetree, NIL)), cons(listO(3, SYM_DEFCLASS_STELLA_SELF, SYM_DEFCLASS_STELLA_OBJECT, NIL), NIL), KWD_DEFCLASS_AUXILIARYp, SYM_DEFCLASS_STELLA_TRUE, listO(5, SYM_DEFCLASS_STELLA_TYPECASE, SYM_DEFCLASS_STELLA_SELF, cons(yield_mixin_class_users_tree(((Class*)(slot->slot_owner->surrogate_value))), cons(listO(3, SYM_DEFCLASS_STELLA_RETURN, cons(slot->slot_name, cons(SYM_DEFCLASS_STELLA_SELF, NIL)), NIL), NIL)), listO(3, SYM_DEFCLASS_STELLA_OTHERWISE, listO(3, SYM_DEFCLASS_STELLA_RETURN, SYM_DEFCLASS_STELLA_NULL, NIL), NIL), NIL), NIL));
  }
}

Cons* yield_mixin_slot_writer_tree(Storage_Slot* slot) {
  { Symbol* writername = yield_mixin_slot_writer_name(slot);
    Object* typetree = yield_slot_type_tree(slot);

    return (listO(7, SYM_DEFCLASS_STELLA_DEFUN, cons(writername, cons(typetree, NIL)), listO(3, listO(3, SYM_DEFCLASS_STELLA_SELF, SYM_DEFCLASS_STELLA_OBJECT, NIL), listO(3, SYM_DEFCLASS_STELLA_VALUE, copy_cons_tree(typetree), NIL), NIL), KWD_DEFCLASS_AUXILIARYp, SYM_DEFCLASS_STELLA_TRUE, listO(5, SYM_DEFCLASS_STELLA_TYPECASE, SYM_DEFCLASS_STELLA_SELF, cons(yield_mixin_class_users_tree(((Class*)(slot->slot_owner->surrogate_value))), cons(listO(3, SYM_DEFCLASS_STELLA_RETURN, listO(4, SYM_DEFCLASS_STELLA_SETF, cons(slot->slot_name, cons(SYM_DEFCLASS_STELLA_SELF, NIL)), SYM_DEFCLASS_STELLA_VALUE, NIL), NIL), NIL)), listO(3, SYM_DEFCLASS_STELLA_OTHERWISE, listO(3, SYM_DEFCLASS_STELLA_RETURN, SYM_DEFCLASS_STELLA_NULL, NIL), NIL), NIL), NIL));
  }
}

Symbol* yield_native_slot_reader_name(Storage_Slot* slot) {
  return (intern_derived_symbol(slot->slot_name, string_concatenate(slot->slot_name->symbol_name, "-NATIVE-VALUE", 0)));
}

Symbol* yield_native_slot_writer_name(Storage_Slot* slot) {
  return (yield_setter_method_name(yield_native_slot_reader_name(slot)));
}

Cons* yield_native_slot_reader_tree(Storage_Slot* slot, Class* renamed_Class) {
  { Symbol* readername = yield_native_slot_reader_name(slot);
    Object* typetree = yield_slot_type_tree(slot);
    boolean slotvisibleP = subtype_ofP(renamed_Class->class_type, slot->slot_owner);

    return (listO(6, SYM_DEFCLASS_STELLA_DEFMETHOD, cons(readername, cons(typetree, NIL)), cons(listO(3, SYM_DEFCLASS_STELLA_SELF, class_symbol(renamed_Class), NIL), NIL), KWD_DEFCLASS_AUXILIARYp, SYM_DEFCLASS_STELLA_TRUE, ((slotvisibleP ? cons(listO(3, SYM_DEFCLASS_STELLA_RETURN, listO(4, SYM_DEFCLASS_STELLA_SLOT_VALUE, SYM_DEFCLASS_STELLA_SELF, slot->slot_name, listO(3, KWD_DEFCLASS_CONTEXT_SENSITIVEp, SYM_DEFCLASS_STELLA_FALSE, NIL)), NIL), NIL) : listO(3, listO(4, SYM_DEFCLASS_STELLA_ERROR, wrap_string("Slot '"), wrap_string(slot->slot_name->symbol_name), listO(3, wrap_string("' does not exist on "), SYM_DEFCLASS_STELLA_SELF, NIL)), listO(3, SYM_DEFCLASS_STELLA_RETURN, SYM_DEFCLASS_STELLA_NULL, NIL), NIL)))->concatenate(NIL, 0)));
  }
}

Cons* yield_native_slot_writer_tree(Storage_Slot* slot, Class* renamed_Class) {
  { Symbol* writername = yield_native_slot_writer_name(slot);
    Object* typetree = yield_slot_type_tree(slot);
    boolean slotvisibleP = subtype_ofP(renamed_Class->class_type, slot->slot_owner);

    return (listO(6, SYM_DEFCLASS_STELLA_DEFMETHOD, cons(writername, cons(typetree, NIL)), listO(3, listO(3, SYM_DEFCLASS_STELLA_SELF, class_symbol(renamed_Class), NIL), listO(3, SYM_DEFCLASS_STELLA_VALUE, copy_cons_tree(typetree), NIL), NIL), KWD_DEFCLASS_AUXILIARYp, SYM_DEFCLASS_STELLA_TRUE, ((slotvisibleP ? cons(listO(3, SYM_DEFCLASS_STELLA_RETURN, listO(4, SYM_DEFCLASS_STELLA_SETF, listO(4, SYM_DEFCLASS_STELLA_SLOT_VALUE, SYM_DEFCLASS_STELLA_SELF, slot->slot_name, listO(3, KWD_DEFCLASS_CONTEXT_SENSITIVEp, SYM_DEFCLASS_STELLA_FALSE, NIL)), SYM_DEFCLASS_STELLA_VALUE, NIL), NIL), NIL) : listO(4, listO(3, SYM_DEFCLASS_STELLA_IGNORE, SYM_DEFCLASS_STELLA_VALUE, NIL), listO(4, SYM_DEFCLASS_STELLA_ERROR, wrap_string("Slot '"), wrap_string(slot->slot_name->symbol_name), listO(3, wrap_string("' does not exist on "), SYM_DEFCLASS_STELLA_SELF, NIL)), listO(3, SYM_DEFCLASS_STELLA_RETURN, SYM_DEFCLASS_STELLA_NULL, NIL), NIL)))->concatenate(NIL, 0)));
  }
}

void create_keyed_equality_method_unit(Class* renamed_Class) {
  { Cons* testtrees = NIL;

    { Slot* slotname = NULL;
      Cons* iter_000 = renamed_Class->class_key_reader()->the_cons_list;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        slotname = ((Slot*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(listO(4, SYM_DEFCLASS_STELLA_EQp, cons(slotname, cons(SYM_DEFCLASS_STELLA_SELF, NIL)), cons(slotname, cons(SYM_DEFCLASS_STELLA_OTHER, NIL)), NIL), NIL);
            if (testtrees == NIL) {
              testtrees = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(testtrees, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(listO(4, SYM_DEFCLASS_STELLA_EQp, cons(slotname, cons(SYM_DEFCLASS_STELLA_SELF, NIL)), cons(slotname, cons(SYM_DEFCLASS_STELLA_OTHER, NIL)), NIL), NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
    }
    walk_auxiliary_tree(listO(5, SYM_DEFCLASS_STELLA_DEFMETHOD, listO(3, SYM_DEFCLASS_STELLA_EQUAL, SYM_DEFCLASS_STELLA_BOOLEAN, NIL), listO(3, listO(3, SYM_DEFCLASS_STELLA_SELF, class_symbol(renamed_Class), NIL), SYM_DEFCLASS_STELLA_OTHER, NIL), listO(3, SYM_DEFCLASS_STELLA_RETURN, cons(SYM_DEFCLASS_STELLA_AND, testtrees->concatenate(NIL, 0)), NIL), NIL));
  }
}

boolean transient_objectP(Object* object) {
  { Surrogate* type = object->primary_type();

    return (subtype_ofP(type, SGT_DEFCLASS_STELLA_TRANSIENT_MIXIN));
  }
}

void help_startup_defclass1() {
  {
    SGT_DEFCLASS_STELLA_LITERAL = ((Surrogate*)(intern_rigid_symbol_wrt_module("LITERAL", NULL, 1)));
    SGT_DEFCLASS_STELLA_SECOND_CLASS_OBJECT = ((Surrogate*)(intern_rigid_symbol_wrt_module("SECOND-CLASS-OBJECT", NULL, 1)));
    SYM_DEFCLASS_STELLA_NULL = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL", NULL, 0)));
    SYM_DEFCLASS_STELLA_DEFINE_CLASS_FROM_STRINGIFIED_SOURCE = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFINE-CLASS-FROM-STRINGIFIED-SOURCE", NULL, 0)));
    KWD_DEFCLASS_COMMON_LISP = ((Keyword*)(intern_rigid_symbol_wrt_module("COMMON-LISP", NULL, 2)));
    SYM_DEFCLASS_STELLA_CLASS_CL_NATIVE_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-CL-NATIVE-TYPE", NULL, 0)));
    KWD_DEFCLASS_IDL = ((Keyword*)(intern_rigid_symbol_wrt_module("IDL", NULL, 2)));
    SYM_DEFCLASS_STELLA_CLASS_IDL_NATIVE_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-IDL-NATIVE-TYPE", NULL, 0)));
    KWD_DEFCLASS_JAVA = ((Keyword*)(intern_rigid_symbol_wrt_module("JAVA", NULL, 2)));
    SYM_DEFCLASS_STELLA_CLASS_JAVA_NATIVE_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-JAVA-NATIVE-TYPE", NULL, 0)));
    KWD_DEFCLASS_CPP = ((Keyword*)(intern_rigid_symbol_wrt_module("CPP", NULL, 2)));
    KWD_DEFCLASS_CPP_STANDALONE = ((Keyword*)(intern_rigid_symbol_wrt_module("CPP-STANDALONE", NULL, 2)));
    SYM_DEFCLASS_STELLA_CLASS_CPP_NATIVE_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-CPP-NATIVE-TYPE", NULL, 0)));
    SGT_DEFCLASS_STELLA_NON_OBJECT = ((Surrogate*)(intern_rigid_symbol_wrt_module("NON-OBJECT", NULL, 1)));
    SGT_DEFCLASS_STELLA_NATIVE_EXCEPTION = ((Surrogate*)(intern_rigid_symbol_wrt_module("NATIVE-EXCEPTION", NULL, 1)));
    KWD_DEFCLASS_DYNAMIC = ((Keyword*)(intern_rigid_symbol_wrt_module("DYNAMIC", NULL, 2)));
    SYM_DEFCLASS_STELLA_SLOT_INITIAL_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-INITIAL-VALUE", NULL, 0)));
    KWD_DEFCLASS_BIT = ((Keyword*)(intern_rigid_symbol_wrt_module("BIT", NULL, 2)));
    SGT_DEFCLASS_STELLA_BOOLEAN = ((Surrogate*)(intern_rigid_symbol_wrt_module("BOOLEAN", NULL, 1)));
    KWD_DEFCLASS_CLASS = ((Keyword*)(intern_rigid_symbol_wrt_module("CLASS", NULL, 2)));
    SYM_DEFCLASS_STELLA_SLOT_RENAMES = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-RENAMES", NULL, 0)));
    SGT_DEFCLASS_STELLA_UNKNOWN = ((Surrogate*)(intern_rigid_symbol_wrt_module("UNKNOWN", NULL, 1)));
    SYM_DEFCLASS_STELLA_SLOT_TYPE_SPECIFIER = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-TYPE-SPECIFIER", NULL, 0)));
    SGT_DEFCLASS_STELLA_ANCHORED_TYPE_SPECIFIER = ((Surrogate*)(intern_rigid_symbol_wrt_module("ANCHORED-TYPE-SPECIFIER", NULL, 1)));
    SYM_DEFCLASS_STELLA_NEXT_SWEEP_LIST_OBJECT = ((Symbol*)(intern_rigid_symbol_wrt_module("NEXT-SWEEP-LIST-OBJECT", NULL, 0)));
    KWD_DEFCLASS_FREE_LIST = ((Keyword*)(intern_rigid_symbol_wrt_module("FREE-LIST", NULL, 2)));
    KWD_DEFCLASS_FREE_AND_SWEEP_LIST = ((Keyword*)(intern_rigid_symbol_wrt_module("FREE-AND-SWEEP-LIST", NULL, 2)));
    KWD_DEFCLASS_SWEEP_LIST = ((Keyword*)(intern_rigid_symbol_wrt_module("SWEEP-LIST", NULL, 2)));
    SYM_DEFCLASS_STELLA_CLASS_CREATOR = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-CREATOR", NULL, 0)));
    KWD_DEFCLASS_USE_COMMON_LISP_STRUCTS = ((Keyword*)(intern_rigid_symbol_wrt_module("USE-COMMON-LISP-STRUCTS", NULL, 2)));
    KWD_DEFCLASS_USE_COMMON_LISP_VECTOR_STRUCTS = ((Keyword*)(intern_rigid_symbol_wrt_module("USE-COMMON-LISP-VECTOR-STRUCTS", NULL, 2)));
    SYM_DEFCLASS_STELLA_DEFGLOBAL = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFGLOBAL", NULL, 0)));
    SYM_DEFCLASS_STELLA_RECYCLE_LIST = ((Symbol*)(intern_rigid_symbol_wrt_module("RECYCLE-LIST", NULL, 0)));
    SYM_DEFCLASS_STELLA_OF = ((Symbol*)(intern_rigid_symbol_wrt_module("OF", NULL, 0)));
    SYM_DEFCLASS_STELLA_CREATE_RECYCLE_LIST = ((Symbol*)(intern_rigid_symbol_wrt_module("CREATE-RECYCLE-LIST", NULL, 0)));
    SYM_DEFCLASS_STELLA_SETQ = ((Symbol*)(intern_rigid_symbol_wrt_module("SETQ", NULL, 0)));
    SYM_DEFCLASS_STELLA_SELF = ((Symbol*)(intern_rigid_symbol_wrt_module("SELF", NULL, 0)));
    SYM_DEFCLASS_STELLA_MAKE = ((Symbol*)(intern_rigid_symbol_wrt_module("MAKE", NULL, 0)));
    SYM_DEFCLASS_STELLA_WHEN = ((Symbol*)(intern_rigid_symbol_wrt_module("WHEN", NULL, 0)));
    SYM_DEFCLASS_STELLA_DEFINEDp = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFINED?", NULL, 0)));
    SYM_DEFCLASS_STELLA_FREE = ((Symbol*)(intern_rigid_symbol_wrt_module("FREE", NULL, 0)));
    SYM_DEFCLASS_STELLA_DYNAMIC_SLOTS = ((Symbol*)(intern_rigid_symbol_wrt_module("DYNAMIC-SLOTS", NULL, 0)));
    SYM_DEFCLASS_STELLA_CLEAR = ((Symbol*)(intern_rigid_symbol_wrt_module("CLEAR", NULL, 0)));
    SYM_DEFCLASS_STELLA_UNLESS = ((Symbol*)(intern_rigid_symbol_wrt_module("UNLESS", NULL, 0)));
    SYM_DEFCLASS_STELLA_RETURN = ((Symbol*)(intern_rigid_symbol_wrt_module("RETURN", NULL, 0)));
    KWD_DEFCLASS_FORCE_NULL_VALUE = ((Keyword*)(intern_rigid_symbol_wrt_module("FORCE-NULL-VALUE", NULL, 2)));
    KWD_DEFCLASS_NULL_VALUE_ONLY = ((Keyword*)(intern_rigid_symbol_wrt_module("NULL-VALUE-ONLY", NULL, 2)));
    SYM_DEFCLASS_STELLA_AND = ((Symbol*)(intern_rigid_symbol_wrt_module("AND", NULL, 0)));
    SYM_DEFCLASS_STELLA_oRECYCLING_ENABLEDpo = ((Symbol*)(intern_rigid_symbol_wrt_module("*RECYCLING-ENABLED?*", NULL, 0)));
    SYM_DEFCLASS_STELLA_COND = ((Symbol*)(intern_rigid_symbol_wrt_module("COND", NULL, 0)));
    SYM_DEFCLASS_STELLA___ = ((Symbol*)(intern_rigid_symbol_wrt_module("--", NULL, 0)));
    SYM_DEFCLASS_STELLA_CURRENT_LENGTH = ((Symbol*)(intern_rigid_symbol_wrt_module("CURRENT-LENGTH", NULL, 0)));
    SYM_DEFCLASS_STELLA_OTHERWISE = ((Symbol*)(intern_rigid_symbol_wrt_module("OTHERWISE", NULL, 0)));
    SYM_DEFCLASS_STELLA_RECYCLED_ITEMS = ((Symbol*)(intern_rigid_symbol_wrt_module("RECYCLED-ITEMS", NULL, 0)));
    SYM_DEFCLASS_STELLA_POP = ((Symbol*)(intern_rigid_symbol_wrt_module("POP", NULL, 0)));
    SYM_DEFCLASS_STELLA_LIST_OF_RECYCLED_ITEMS = ((Symbol*)(intern_rigid_symbol_wrt_module("LIST-OF-RECYCLED-ITEMS", NULL, 0)));
    SYM_DEFCLASS_STELLA_SETF = ((Symbol*)(intern_rigid_symbol_wrt_module("SETF", NULL, 0)));
    SYM_DEFCLASS_STELLA_SLOT_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-VALUE", NULL, 0)));
    SYM_DEFCLASS_STELLA_UNUSED_ITEMS = ((Symbol*)(intern_rigid_symbol_wrt_module("UNUSED-ITEMS", NULL, 0)));
    SYM_DEFCLASS_STELLA_ALL_ITEMS = ((Symbol*)(intern_rigid_symbol_wrt_module("ALL-ITEMS", NULL, 0)));
  }
}

void help_startup_defclass2() {
  {
    SYM_DEFCLASS_STELLA_ii = ((Symbol*)(intern_rigid_symbol_wrt_module("++", NULL, 0)));
    SYM_DEFCLASS_STELLA_e = ((Symbol*)(intern_rigid_symbol_wrt_module("=", NULL, 0)));
    SYM_DEFCLASS_STELLA_oRECYCLE_LISTS_MAINTENANCE_TIMERo = ((Symbol*)(intern_rigid_symbol_wrt_module("*RECYCLE-LISTS-MAINTENANCE-TIMER*", NULL, 0)));
    SYM_DEFCLASS_STELLA_MAINTAIN_RECYCLE_LISTS = ((Symbol*)(intern_rigid_symbol_wrt_module("MAINTAIN-RECYCLE-LISTS", NULL, 0)));
    KWD_DEFCLASS_INITIAL_VALUE_ONLY = ((Keyword*)(intern_rigid_symbol_wrt_module("INITIAL-VALUE-ONLY", NULL, 2)));
    KWD_DEFCLASS_ANY_INITIAL_VALUE = ((Keyword*)(intern_rigid_symbol_wrt_module("ANY-INITIAL-VALUE", NULL, 2)));
    KWD_DEFCLASS_CONTEXT_SENSITIVEp = ((Keyword*)(intern_rigid_symbol_wrt_module("CONTEXT-SENSITIVE?", NULL, 2)));
    SYM_DEFCLASS_STELLA_FALSE = ((Symbol*)(intern_rigid_symbol_wrt_module("FALSE", NULL, 0)));
    SYM_DEFCLASS_STELLA_CLASS_INITIALIZER = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-INITIALIZER", NULL, 0)));
    SYM_DEFCLASS_STELLA_RUN_CONSTRUCTOR_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("RUN-CONSTRUCTOR-DEMONS", NULL, 0)));
    SYM_DEFCLASS_STELLA_DEFUN = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFUN", NULL, 0)));
    KWD_DEFCLASS_CONSTRUCTORp = ((Keyword*)(intern_rigid_symbol_wrt_module("CONSTRUCTOR?", NULL, 2)));
    SYM_DEFCLASS_STELLA_TRUE = ((Symbol*)(intern_rigid_symbol_wrt_module("TRUE", NULL, 0)));
    KWD_DEFCLASS_PUBLICp = ((Keyword*)(intern_rigid_symbol_wrt_module("PUBLIC?", NULL, 2)));
    SYM_DEFCLASS_STELLA_LET = ((Symbol*)(intern_rigid_symbol_wrt_module("LET", NULL, 0)));
    SYM_DEFCLASS_STELLA_CLASS_DESTRUCTOR = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-DESTRUCTOR", NULL, 0)));
    SYM_DEFCLASS_STELLA_DEFMETHOD = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFMETHOD", NULL, 0)));
    SGT_DEFCLASS_STELLA_STANDARD_OBJECT = ((Surrogate*)(intern_rigid_symbol_wrt_module("STANDARD-OBJECT", NULL, 1)));
    SYM_DEFCLASS_STELLA_PUSH = ((Symbol*)(intern_rigid_symbol_wrt_module("PUSH", NULL, 0)));
    SYM_DEFCLASS_STELLA_SWEEP = ((Symbol*)(intern_rigid_symbol_wrt_module("SWEEP", NULL, 0)));
    SYM_DEFCLASS_STELLA_UNUSED = ((Symbol*)(intern_rigid_symbol_wrt_module("UNUSED", NULL, 0)));
    SYM_DEFCLASS_STELLA_WHILE = ((Symbol*)(intern_rigid_symbol_wrt_module("WHILE", NULL, 0)));
    SYM_DEFCLASS_STELLA_NOT = ((Symbol*)(intern_rigid_symbol_wrt_module("NOT", NULL, 0)));
    SYM_DEFCLASS_STELLA_EQLp = ((Symbol*)(intern_rigid_symbol_wrt_module("EQL?", NULL, 0)));
    SYM_DEFCLASS_STELLA_UNMAKE = ((Symbol*)(intern_rigid_symbol_wrt_module("UNMAKE", NULL, 0)));
    SGT_DEFCLASS_STELLA_VOID = ((Surrogate*)(intern_rigid_symbol_wrt_module("VOID", NULL, 1)));
    SGT_DEFCLASS_STELLA_TRANSIENT_MIXIN = ((Surrogate*)(intern_rigid_symbol_wrt_module("TRANSIENT-MIXIN", NULL, 1)));
    SYM_DEFCLASS_STELLA_RECYCLE_LIST_DEBUGGING_ENABLEDp = ((Symbol*)(intern_rigid_symbol_wrt_module("RECYCLE-LIST-DEBUGGING-ENABLED?", NULL, 0)));
    SYM_DEFCLASS_STELLA_REGISTER_RECYCLED_ITEM = ((Symbol*)(intern_rigid_symbol_wrt_module("REGISTER-RECYCLED-ITEM", NULL, 0)));
    SYM_DEFCLASS_STELLA_UNREGISTER_RECYCLED_ITEM = ((Symbol*)(intern_rigid_symbol_wrt_module("UNREGISTER-RECYCLED-ITEM", NULL, 0)));
    KWD_DEFCLASS_NULL_WRAPPER = ((Keyword*)(intern_rigid_symbol_wrt_module("NULL-WRAPPER", NULL, 2)));
    SYM_DEFCLASS_STELLA_INLINE_WRAP_BOOLEAN = ((Symbol*)(intern_rigid_symbol_wrt_module("INLINE-WRAP-BOOLEAN", NULL, 0)));
    SYM_DEFCLASS_STELLA_FALSE_WRAPPER = ((Symbol*)(intern_rigid_symbol_wrt_module("FALSE-WRAPPER", NULL, 0)));
    SYM_DEFCLASS_STELLA_ANSWER = ((Symbol*)(intern_rigid_symbol_wrt_module("ANSWER", NULL, 0)));
    SYM_DEFCLASS_STELLA_IF = ((Symbol*)(intern_rigid_symbol_wrt_module("IF", NULL, 0)));
    SYM_DEFCLASS_STELLA_NULLp = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL?", NULL, 0)));
    SYM_DEFCLASS_STELLA_PROGN = ((Symbol*)(intern_rigid_symbol_wrt_module("PROGN", NULL, 0)));
    SYM_DEFCLASS_STELLA_ERROR = ((Symbol*)(intern_rigid_symbol_wrt_module("ERROR", NULL, 0)));
    KWD_DEFCLASS_AUXILIARYp = ((Keyword*)(intern_rigid_symbol_wrt_module("AUXILIARY?", NULL, 2)));
    SYM_DEFCLASS_STELLA_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("VALUE", NULL, 0)));
    SYM_DEFCLASS_STELLA_IGNORE = ((Symbol*)(intern_rigid_symbol_wrt_module("IGNORE", NULL, 0)));
    SGT_DEFCLASS_STELLA_OBJECT = ((Surrogate*)(intern_rigid_symbol_wrt_module("OBJECT", NULL, 1)));
    KWD_DEFCLASS_WRAP_FUNCTION = ((Keyword*)(intern_rigid_symbol_wrt_module("WRAP-FUNCTION", NULL, 2)));
    SYM_DEFCLASS_STELLA_CAST = ((Symbol*)(intern_rigid_symbol_wrt_module("CAST", NULL, 0)));
    SYM_DEFCLASS_STELLA_SETVALUEp = ((Symbol*)(intern_rigid_symbol_wrt_module("SETVALUE?", NULL, 0)));
    SGT_DEFCLASS_STELLA_DYNAMIC_SLOTS_MIXIN = ((Surrogate*)(intern_rigid_symbol_wrt_module("DYNAMIC-SLOTS-MIXIN", NULL, 1)));
    SYM_DEFCLASS_STELLA_SLOTNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOTNAME", NULL, 0)));
    SYM_DEFCLASS_STELLA_QUOTE = ((Symbol*)(intern_rigid_symbol_wrt_module("QUOTE", NULL, 0)));
    SYM_DEFCLASS_STELLA_CASE = ((Symbol*)(intern_rigid_symbol_wrt_module("CASE", NULL, 0)));
    SYM_DEFCLASS_STELLA_SET_DYNAMIC_SLOT_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("SET-DYNAMIC-SLOT-VALUE", NULL, 0)));
    SYM_DEFCLASS_STELLA_LOOKUP = ((Symbol*)(intern_rigid_symbol_wrt_module("LOOKUP", NULL, 0)));
    SYM_DEFCLASS_STELLA_OBJECT = ((Symbol*)(intern_rigid_symbol_wrt_module("OBJECT", NULL, 0)));
    SYM_DEFCLASS_STELLA_SYMBOL = ((Symbol*)(intern_rigid_symbol_wrt_module("SYMBOL", NULL, 0)));
    SYM_DEFCLASS_STELLA_BOOLEAN = ((Symbol*)(intern_rigid_symbol_wrt_module("BOOLEAN", NULL, 0)));
    SYM_DEFCLASS_STELLA_CLASS_SLOT_ACCESSOR_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-SLOT-ACCESSOR-CODE", NULL, 0)));
    SYM_DEFCLASS_STELLA_THE_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("THE-CODE", NULL, 0)));
    KWD_DEFCLASS_FUNCTION = ((Keyword*)(intern_rigid_symbol_wrt_module("FUNCTION", NULL, 2)));
    SYM_DEFCLASS_STELLA_TYPECASE = ((Symbol*)(intern_rigid_symbol_wrt_module("TYPECASE", NULL, 0)));
    SYM_DEFCLASS_STELLA_EQp = ((Symbol*)(intern_rigid_symbol_wrt_module("EQ?", NULL, 0)));
    SYM_DEFCLASS_STELLA_OTHER = ((Symbol*)(intern_rigid_symbol_wrt_module("OTHER", NULL, 0)));
  }
}

void help_startup_defclass3() {
  {
    define_function_object("TYPE-TO-NULL-VALUE-TREE", "(DEFUN (TYPE-TO-NULL-VALUE-TREE OBJECT) ((TYPE TYPE)))", ((cpp_function_code)(&type_to_null_value_tree)), NULL);
    define_function_object("TYPE-TO-WALKED-NULL-VALUE-TREE", "(DEFUN (TYPE-TO-WALKED-NULL-VALUE-TREE OBJECT) ((TYPE TYPE)))", ((cpp_function_code)(&type_to_walked_null_value_tree)), NULL);
    define_function_object("YIELD-CONS-LIST-FROM-SEQUENCE", "(DEFUN (YIELD-CONS-LIST-FROM-SEQUENCE CONS) ((SEQUENCE SEQUENCE)))", ((cpp_function_code)(&yield_cons_list_from_sequence)), NULL);
    define_function_object("YIELD-DEFINE-STELLA-CLASS", "(DEFUN (YIELD-DEFINE-STELLA-CLASS CONS) ((CLASS CLASS)) :DOCUMENTATION \"Return a cons tree that (when evaluated) constructs\na Stella class object.\")", ((cpp_function_code)(&yield_define_stella_class)), NULL);
    define_function_object("CREATE-NATIVE-CLASS?", "(DEFUN (CREATE-NATIVE-CLASS? BOOLEAN) ((CLASS CLASS)))", ((cpp_function_code)(&create_native_classP)), NULL);
    define_function_object("EXCEPTION-CLASS?", "(DEFUN (EXCEPTION-CLASS? BOOLEAN) ((CLASS CLASS)))", ((cpp_function_code)(&exception_classP)), NULL);
    define_function_object("WARN-ABOUT-MULTIPLE-PARENTS", "(DEFUN WARN-ABOUT-MULTIPLE-PARENTS ((CLASS CLASS)))", ((cpp_function_code)(&warn_about_multiple_parents)), NULL);
    define_function_object("WARN-ABOUT-NON-DIRECT-SUPERS", "(DEFUN WARN-ABOUT-NON-DIRECT-SUPERS ((CLASS CLASS)))", ((cpp_function_code)(&warn_about_non_direct_supers)), NULL);
    define_function_object("SPACE-SAVER-SLOT-ALLOCATION?", "(DEFUN (SPACE-SAVER-SLOT-ALLOCATION? BOOLEAN) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&space_saver_slot_allocationP)), NULL);
    define_function_object("DYNAMIC-SLOT?", "(DEFUN (DYNAMIC-SLOT? BOOLEAN) ((SLOT STORAGE-SLOT)) :GLOBALLY-INLINE? TRUE (RETURN (EQL? (ALLOCATION SLOT) :DYNAMIC)))", ((cpp_function_code)(&dynamic_slotP)), NULL);
    define_function_object("CLASS-SLOT?", "(DEFUN (CLASS-SLOT? BOOLEAN) ((SLOT STORAGE-SLOT)) :GLOBALLY-INLINE? TRUE (RETURN (EQL? (ALLOCATION SLOT) :CLASS)))", ((cpp_function_code)(&class_slotP)), NULL);
    define_function_object("REDUNDANT-SLOT?", "(DEFUN (REDUNDANT-SLOT? BOOLEAN) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&redundant_slotP)), NULL);
    define_function_object("NATIVE-SLOT?", "(DEFUN (NATIVE-SLOT? BOOLEAN) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&native_slotP)), NULL);
    define_function_object("NATIVE-METHOD?", "(DEFUN (NATIVE-METHOD? BOOLEAN) ((SLOT METHOD-SLOT)))", ((cpp_function_code)(&native_methodP)), NULL);
    define_function_object("MIXIN-SLOT?", "(DEFUN (MIXIN-SLOT? BOOLEAN) ((SELF SLOT)))", ((cpp_function_code)(&mixin_slotP)), NULL);
    define_function_object("MIXIN-METHOD?", "(DEFUN (MIXIN-METHOD? BOOLEAN) ((SELF SLOT)))", ((cpp_function_code)(&mixin_methodP)), NULL);
    define_function_object("CANONICAL-SLOT", "(DEFUN (CANONICAL-SLOT STORAGE-SLOT) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&canonical_slot)), NULL);
    define_function_object("UNKNOWN-TYPE?", "(DEFUN (UNKNOWN-TYPE? BOOLEAN) ((TYPE TYPE)))", ((cpp_function_code)(&unknown_typeP)), NULL);
    define_function_object("SLOT-HAS-UNKNOWN-TYPE?", "(DEFUN (SLOT-HAS-UNKNOWN-TYPE? BOOLEAN) ((SLOT STORAGE-SLOT) (CLASS CLASS)))", ((cpp_function_code)(&slot_has_unknown_typeP)), NULL);
    define_function_object("NATIVE-STORAGE-SLOT-HOME", "(DEFUN (NATIVE-STORAGE-SLOT-HOME CLASS) ((SLOT STORAGE-SLOT) (CLASS CLASS)))", ((cpp_function_code)(&native_storage_slot_home)), NULL);
    define_function_object("NATIVE-SLOT-HOME", "(DEFUN (NATIVE-SLOT-HOME CLASS) ((SLOT SLOT) (CLASS CLASS)))", ((cpp_function_code)(&native_slot_home)), NULL);
    define_function_object("METHOD-CONTAINS-UNKNOWN-TYPE?", "(DEFUN (METHOD-CONTAINS-UNKNOWN-TYPE? BOOLEAN) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&method_contains_unknown_typeP)), NULL);
    define_function_object("METHOD-CONTAINS-ANCHORED-TYPE?", "(DEFUN (METHOD-CONTAINS-ANCHORED-TYPE? BOOLEAN) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&method_contains_anchored_typeP)), NULL);
    define_function_object("YIELD-RECYCLE-LIST-NAME", "(DEFUN (YIELD-RECYCLE-LIST-NAME SYMBOL) ((CLASS CLASS)))", ((cpp_function_code)(&yield_recycle_list_name)), NULL);
    define_function_object("RECYCLE-SLOT", "(DEFUN (RECYCLE-SLOT STORAGE-SLOT) ((CLASS CLASS)))", ((cpp_function_code)(&recycle_slot)), NULL);
    define_function_object("USE-FREE-LIST?", "(DEFUN (USE-FREE-LIST? BOOLEAN) ((CLASS CLASS)))", ((cpp_function_code)(&use_free_listP)), NULL);
    define_function_object("USE-SWEEP-LIST?", "(DEFUN (USE-SWEEP-LIST? BOOLEAN) ((CLASS CLASS)))", ((cpp_function_code)(&use_sweep_listP)), NULL);
    define_function_object("DECIDED-TO-RECYCLE?", "(DEFUN (DECIDED-TO-RECYCLE? BOOLEAN) ((CLASS CLASS)))", ((cpp_function_code)(&decided_to_recycleP)), NULL);
    define_function_object("COMPUTE-RECYCLED-ITEM-SIZE", "(DEFUN (COMPUTE-RECYCLED-ITEM-SIZE INTEGER) ((CLASS CLASS)))", ((cpp_function_code)(&compute_recycled_item_size)), NULL);
    define_function_object("YIELD-RECYCLE-LIST-DEFINITIONS", "(DEFUN (YIELD-RECYCLE-LIST-DEFINITIONS CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_recycle_list_definitions)), NULL);
    define_function_object("YIELD-MAKE-TREES", "(DEFUN (YIELD-MAKE-TREES CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_make_trees)), NULL);
    define_function_object("YIELD-CONSTRUCTOR-NAME", "(DEFUN (YIELD-CONSTRUCTOR-NAME SYMBOL) ((CLASS CLASS)))", ((cpp_function_code)(&yield_constructor_name)), NULL);
    define_function_object("YIELD-CREATION-TREES", "(DEFUN (YIELD-CREATION-TREES CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_creation_trees)), NULL);
    define_function_object("YIELD-TERMINATE-OBJECT-TREES", "(DEFUN (YIELD-TERMINATE-OBJECT-TREES CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_terminate_object_trees)), NULL);
    define_function_object("YIELD-CREATION-OR-REUSE-TREES", "(DEFUN (YIELD-CREATION-OR-REUSE-TREES CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_creation_or_reuse_trees)), NULL);
    define_function_object("YIELD-CREATION-OR-REUSE-TREES-FROM-FREE-LIST", "(DEFUN (YIELD-CREATION-OR-REUSE-TREES-FROM-FREE-LIST CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_creation_or_reuse_trees_from_free_list)), NULL);
    define_function_object("YIELD-CREATION-OR-REUSE-TREES-FROM-SWEEP-LIST", "(DEFUN (YIELD-CREATION-OR-REUSE-TREES-FROM-SWEEP-LIST CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_creation_or_reuse_trees_from_sweep_list)), NULL);
    define_function_object("YIELD-CREATION-OR-REUSE-TREES-FROM-FREE-OR-SWEEP-LIST", "(DEFUN (YIELD-CREATION-OR-REUSE-TREES-FROM-FREE-OR-SWEEP-LIST CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_creation_or_reuse_trees_from_free_or_sweep_list)), NULL);
    define_function_object("MARK-DIRECT-EQUIVALENT-SLOT", "(DEFUN MARK-DIRECT-EQUIVALENT-SLOT ((SLOT SLOT)))", ((cpp_function_code)(&mark_direct_equivalent_slot)), NULL);
    define_function_object("IS-CONTEXT-SENSITIVE-SLOT?", "(DEFUN (IS-CONTEXT-SENSITIVE-SLOT? BOOLEAN) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&is_context_sensitive_slotP)), NULL);
    define_function_object("YIELD-INITIAL-VALUE-ASSIGNMENT", "(DEFUN (YIELD-INITIAL-VALUE-ASSIGNMENT CONS) ((SLOT STORAGE-SLOT) (MODE KEYWORD)))", ((cpp_function_code)(&yield_initial_value_assignment)), NULL);
    define_function_object("YIELD-INITIAL-VALUE-ASSIGNMENTS", "(DEFUN (YIELD-INITIAL-VALUE-ASSIGNMENTS CONS) ((CLASS CLASS) (MODE KEYWORD)))", ((cpp_function_code)(&yield_initial_value_assignments)), NULL);
    define_function_object("YIELD-LOCAL-INITIAL-VALUE-ASSIGNMENTS", "(DEFUN (YIELD-LOCAL-INITIAL-VALUE-ASSIGNMENTS CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_local_initial_value_assignments)), NULL);
    define_function_object("YIELD-INITIALIZER-TREES", "(DEFUN (YIELD-INITIALIZER-TREES CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_initializer_trees)), NULL);
    define_function_object("YIELD-CONSTRUCTOR-BODY", "(DEFUN (YIELD-CONSTRUCTOR-BODY CONS) ((CLASS CLASS) (REQUIREDASSIGNMENTS CONS)))", ((cpp_function_code)(&yield_constructor_body)), NULL);
    define_function_object("YIELD-CONSTRUCTOR-DEFINITION", "(DEFUN (YIELD-CONSTRUCTOR-DEFINITION CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_constructor_definition)), NULL);
    define_function_object("YIELD-EXCEPTION-CONSTRUCTOR-BODY", "(DEFUN (YIELD-EXCEPTION-CONSTRUCTOR-BODY CONS) ((CLASS CLASS) (MESSAGEPARAMETER SYMBOL) (REQUIREDASSIGNMENTS CONS)))", ((cpp_function_code)(&yield_exception_constructor_body)), NULL);
    define_function_object("YIELD-EXCEPTION-CONSTRUCTOR-DEFINITION", "(DEFUN (YIELD-EXCEPTION-CONSTRUCTOR-DEFINITION CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_exception_constructor_definition)), NULL);
    define_function_object("YIELD-DESTRUCTOR-DEFINITIONS", "(DEFUN (YIELD-DESTRUCTOR-DEFINITIONS CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_destructor_definitions)), NULL);
    define_function_object("CREATE-CONSTRUCTOR-AND-DESTRUCTOR-UNITS", "(DEFUN CREATE-CONSTRUCTOR-AND-DESTRUCTOR-UNITS ((CLASS CLASS)))", ((cpp_function_code)(&create_constructor_and_destructor_units)), NULL);
    define_function_object("PARAMETERIZED-CLASS?", "(DEFUN (PARAMETERIZED-CLASS? BOOLEAN) ((CLASS CLASS)))", ((cpp_function_code)(&parameterized_classP)), NULL);
    define_function_object("CLASS-WITH-INITIALIZER?", "(DEFUN (CLASS-WITH-INITIALIZER? BOOLEAN) ((CLASS CLASS)))", ((cpp_function_code)(&class_with_initializerP)), NULL);
    define_function_object("YIELD-INITIALIZER-NAME", "(DEFUN (YIELD-INITIALIZER-NAME SYMBOL) ((CLASS CLASS)))", ((cpp_function_code)(&yield_initializer_name)), NULL);
    define_function_object("YIELD-INITIALIZER-DEFINITION", "(DEFUN (YIELD-INITIALIZER-DEFINITION CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_initializer_definition)), NULL);
    define_function_object("CREATE-CPP-CONSTRUCTOR-UNIT", "(DEFUN CREATE-CPP-CONSTRUCTOR-UNIT ((CLASS CLASS)))", ((cpp_function_code)(&create_cpp_constructor_unit)), NULL);
    define_function_object("CPP-CONSTRUCTOR?", "(DEFUN (CPP-CONSTRUCTOR? BOOLEAN) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&cpp_constructorP)), NULL);
    define_function_object("CREATE-RECYCLE-LIST", "(DEFUN (CREATE-RECYCLE-LIST RECYCLE-LIST) ((CLASSTYPE TYPE) (ITEMSIZE INTEGER)))", ((cpp_function_code)(&create_recycle_list)), NULL);
    define_function_object("CLEAR-RECYCLE-LIST", "(DEFUN CLEAR-RECYCLE-LIST ((LIST RECYCLE-LIST)) :DOCUMENTATION \"Reset `list' to its empty state.\")", ((cpp_function_code)(&clear_recycle_list)), NULL);
    define_function_object("CLEAR-RECYCLE-LISTS", "(DEFUN CLEAR-RECYCLE-LISTS () :DOCUMENTATION \"Reset all currently active recycle lists to their empty state.\" :PUBLIC? TRUE)", ((cpp_function_code)(&clear_recycle_lists)), NULL);
    define_function_object("PRINT-RECYCLE-LISTS", "(DEFUN PRINT-RECYCLE-LISTS () :DOCUMENTATION \"Print the current state of all recycle lists.\" :PUBLIC? TRUE)", ((cpp_function_code)(&print_recycle_lists)), NULL);
  }
}

void help_startup_defclass4() {
  {
    define_function_object("MAINTAIN-RECYCLE-LISTS", "(DEFUN MAINTAIN-RECYCLE-LISTS () :PUBLIC? TRUE)", ((cpp_function_code)(&maintain_recycle_lists)), NULL);
    define_method_object("(DEFMETHOD (DELETED? BOOLEAN) ((SELF RECYCLE-LIST)))", ((cpp_method_code)(&Recycle_List::deletedP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (DELETED?-SETTER BOOLEAN) ((SELF RECYCLE-LIST) (VALUE BOOLEAN)))", ((cpp_method_code)(&Recycle_List::deletedP_setter)), ((cpp_method_code)(NULL)));
    define_function_object("RECYCLE-LIST-SIZE", "(DEFUN (RECYCLE-LIST-SIZE INTEGER) ((LIST RECYCLE-LIST)) :GLOBALLY-INLINE? TRUE (RETURN (* (ITEM-SIZE LIST) (CURRENT-LENGTH LIST))))", ((cpp_function_code)(&recycle_list_size)), NULL);
    define_function_object("TOTAL-RECYCLE-LISTS-SIZE", "(DEFUN (TOTAL-RECYCLE-LISTS-SIZE INTEGER) ())", ((cpp_function_code)(&total_recycle_lists_size)), NULL);
    define_function_object("RECYCLE-LIST-SIZE<", "(DEFUN (RECYCLE-LIST-SIZE< BOOLEAN) ((LIST1 RECYCLE-LIST) (LIST2 RECYCLE-LIST)))", ((cpp_function_code)(&recycle_list_sizeL)), NULL);
    define_function_object("REDUCE-RECYCLE-LIST-SIZE", "(DEFUN REDUCE-RECYCLE-LIST-SIZE ((LIST RECYCLE-LIST) (FRACTION FLOAT)))", ((cpp_function_code)(&reduce_recycle_list_size)), NULL);
    define_method_object("(DEFMETHOD SWEEP ((SELF OBJECT)) :DOCUMENTATION \"Default method.  Sweep up all `self'-type objects.\")", ((cpp_method_code)(&Object::sweep)), ((cpp_method_code)(NULL)));
    define_function_object("SWEEP-TRANSIENTS", "(DEFUN SWEEP-TRANSIENTS () :PUBLIC? TRUE)", ((cpp_function_code)(&sweep_transients)), NULL);
    define_function_object("MAKE-NON-RECYCLED-LIST", "(DEFUN (MAKE-NON-RECYCLED-LIST LIST) ())", ((cpp_function_code)(&make_non_recycled_list)), NULL);
    define_function_object("REGISTER-RECYCLED-ITEM", "(DEFUN REGISTER-RECYCLED-ITEM ((ITEM OBJECT)))", ((cpp_function_code)(&register_recycled_item)), NULL);
    define_function_object("UNREGISTER-RECYCLED-ITEM", "(DEFUN UNREGISTER-RECYCLED-ITEM ((ITEM OBJECT)))", ((cpp_function_code)(&unregister_recycled_item)), NULL);
    define_function_object("START-RECYCLE-LIST-DEBUGGING", "(DEFUN START-RECYCLE-LIST-DEBUGGING ())", ((cpp_function_code)(&start_recycle_list_debugging)), NULL);
    define_function_object("STOP-RECYCLE-LIST-DEBUGGING", "(DEFUN STOP-RECYCLE-LIST-DEBUGGING ())", ((cpp_function_code)(&stop_recycle_list_debugging)), NULL);
    define_function_object("RECYCLE-LIST-DEBUGGING-ENABLED?", "(DEFUN (RECYCLE-LIST-DEBUGGING-ENABLED? BOOLEAN) () :GLOBALLY-INLINE? TRUE (RETURN (DEFINED? *CURRENTLY-RECYCLED-ITEMS*)))", ((cpp_function_code)(&recycle_list_debugging_enabledP)), NULL);
    define_function_object("TRANSLATE-WITH-RECYCLE-LIST-DEBUGGING?", "(DEFUN (TRANSLATE-WITH-RECYCLE-LIST-DEBUGGING? BOOLEAN) () :GLOBALLY-INLINE? TRUE (RETURN *TRANSLATE-WITH-RECYCLE-LIST-DEBUGGING?*))", ((cpp_function_code)(&translate_with_recycle_list_debuggingP)), NULL);
    define_function_object("YIELD-REGISTER-RECYCLED-ITEM-TREES", "(DEFUN (YIELD-REGISTER-RECYCLED-ITEM-TREES CONS) ())", ((cpp_function_code)(&yield_register_recycled_item_trees)), NULL);
    define_function_object("YIELD-UNREGISTER-RECYCLED-ITEM-TREES", "(DEFUN (YIELD-UNREGISTER-RECYCLED-ITEM-TREES CONS) ())", ((cpp_function_code)(&yield_unregister_recycled_item_trees)), NULL);
    define_function_object("HAS-NON-NULL-DEFAULT-VALUE?", "(DEFUN (HAS-NON-NULL-DEFAULT-VALUE? BOOLEAN) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&has_non_null_default_valueP)), NULL);
    define_function_object("SYSTEM-DEFINED-SLOT-READER?", "(DEFUN (SYSTEM-DEFINED-SLOT-READER? BOOLEAN) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&system_defined_slot_readerP)), NULL);
    define_function_object("SYSTEM-DEFINED-SLOT-WRITER?", "(DEFUN (SYSTEM-DEFINED-SLOT-WRITER? BOOLEAN) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&system_defined_slot_writerP)), NULL);
    define_function_object("YIELD-SLOT-TYPE-TREE", "(DEFUN (YIELD-SLOT-TYPE-TREE OBJECT) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&yield_slot_type_tree)), NULL);
    define_function_object("YIELD-HARDWIRED-SLOT-VARIABLE", "(DEFUN (YIELD-HARDWIRED-SLOT-VARIABLE SYMBOL) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&yield_hardwired_slot_variable)), NULL);
    define_function_object("YIELD-HARDWIRED-SLOT-READER-BODY", "(DEFUN (YIELD-HARDWIRED-SLOT-READER-BODY CONS CONS) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&yield_hardwired_slot_reader_body)), NULL);
    define_function_object("FINALIZE-AUXILIARY-METHOD", "(DEFUN FINALIZE-AUXILIARY-METHOD ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&finalize_auxiliary_method)), NULL);
    define_function_object("WRAP-WITH-DEFAULT-VALUE-AND-RETURN-CODE", "(DEFUN (WRAP-WITH-DEFAULT-VALUE-AND-RETURN-CODE CONS) ((SLOT STORAGE-SLOT) (DEFAULTEXPRESSION OBJECT)))", ((cpp_function_code)(&wrap_with_default_value_and_return_code)), NULL);
    define_function_object("YIELD-SLOT-READER-TREE", "(DEFUN (YIELD-SLOT-READER-TREE CONS) ((SLOT STORAGE-SLOT) (CLASS CLASS)))", ((cpp_function_code)(&yield_slot_reader_tree)), NULL);
    define_function_object("YIELD-SLOT-WRITER-TREE", "(DEFUN (YIELD-SLOT-WRITER-TREE CONS) ((SLOT STORAGE-SLOT) (CLASS CLASS)))", ((cpp_function_code)(&yield_slot_writer_tree)), NULL);
    define_function_object("SLOT-VALUE-IS-BARE-LITERAL?", "(DEFUN (SLOT-VALUE-IS-BARE-LITERAL? BOOLEAN) ((SLOT SLOT)))", ((cpp_function_code)(&slot_value_is_bare_literalP)), NULL);
    define_function_object("GENERICALLY-ACCESSIBLE-TYPE?", "(DEFUN (GENERICALLY-ACCESSIBLE-TYPE? BOOLEAN) ((TYPE TYPE)))", ((cpp_function_code)(&generically_accessible_typeP)), NULL);
    define_function_object("GENERIC-SLOT-READER?", "(DEFUN (GENERIC-SLOT-READER? BOOLEAN) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&generic_slot_readerP)), NULL);
    define_function_object("GENERIC-SLOT-WRITER?", "(DEFUN (GENERIC-SLOT-WRITER? BOOLEAN) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&generic_slot_writerP)), NULL);
    define_function_object("SLOT-ACCESS-METHOD?", "(DEFUN (SLOT-ACCESS-METHOD? BOOLEAN) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&slot_access_methodP)), NULL);
    define_function_object("YIELD-GENERIC-SLOT-ACCESSOR-NAME", "(DEFUN (YIELD-GENERIC-SLOT-ACCESSOR-NAME SYMBOL) ((CLASS CLASS)))", ((cpp_function_code)(&yield_generic_slot_accessor_name)), NULL);
    define_function_object("YIELD-GENERIC-SLOT-ACCESSOR-DEFINITION", "(DEFUN (YIELD-GENERIC-SLOT-ACCESSOR-DEFINITION CONS) ((CLASS CLASS)))", ((cpp_function_code)(&yield_generic_slot_accessor_definition)), NULL);
    define_function_object("CREATE-GENERIC-SLOT-ACCESSOR-UNIT", "(DEFUN CREATE-GENERIC-SLOT-ACCESSOR-UNIT ((CLASS CLASS)))", ((cpp_function_code)(&create_generic_slot_accessor_unit)), NULL);
    define_function_object("YIELD-GENERIC-SLOT-ACCESSOR-ATTACHMENT", "(DEFUN (YIELD-GENERIC-SLOT-ACCESSOR-ATTACHMENT CONS) ((CLASS CLASS) (CLASSREF SYMBOL)))", ((cpp_function_code)(&yield_generic_slot_accessor_attachment)), NULL);
    define_function_object("YIELD-MIXIN-CLASS-USERS-TREE", "(DEFUN (YIELD-MIXIN-CLASS-USERS-TREE (CONS OF SYMBOL)) ((CLASS CLASS)))", ((cpp_function_code)(&yield_mixin_class_users_tree)), NULL);
    define_function_object("YIELD-MIXIN-SLOT-READER-NAME", "(DEFUN (YIELD-MIXIN-SLOT-READER-NAME SYMBOL) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&yield_mixin_slot_reader_name)), NULL);
    define_function_object("YIELD-MIXIN-SLOT-WRITER-NAME", "(DEFUN (YIELD-MIXIN-SLOT-WRITER-NAME SYMBOL) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&yield_mixin_slot_writer_name)), NULL);
    define_function_object("YIELD-MIXIN-SLOT-READER-TREE", "(DEFUN (YIELD-MIXIN-SLOT-READER-TREE CONS) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&yield_mixin_slot_reader_tree)), NULL);
    define_function_object("YIELD-MIXIN-SLOT-WRITER-TREE", "(DEFUN (YIELD-MIXIN-SLOT-WRITER-TREE CONS) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&yield_mixin_slot_writer_tree)), NULL);
    define_function_object("YIELD-NATIVE-SLOT-READER-NAME", "(DEFUN (YIELD-NATIVE-SLOT-READER-NAME SYMBOL) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&yield_native_slot_reader_name)), NULL);
    define_function_object("YIELD-NATIVE-SLOT-WRITER-NAME", "(DEFUN (YIELD-NATIVE-SLOT-WRITER-NAME SYMBOL) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&yield_native_slot_writer_name)), NULL);
    define_function_object("YIELD-NATIVE-SLOT-READER-TREE", "(DEFUN (YIELD-NATIVE-SLOT-READER-TREE CONS) ((SLOT STORAGE-SLOT) (CLASS CLASS)))", ((cpp_function_code)(&yield_native_slot_reader_tree)), NULL);
    define_function_object("YIELD-NATIVE-SLOT-WRITER-TREE", "(DEFUN (YIELD-NATIVE-SLOT-WRITER-TREE CONS) ((SLOT STORAGE-SLOT) (CLASS CLASS)))", ((cpp_function_code)(&yield_native_slot_writer_tree)), NULL);
    define_function_object("CREATE-KEYED-EQUALITY-METHOD-UNIT", "(DEFUN CREATE-KEYED-EQUALITY-METHOD-UNIT ((CLASS CLASS)))", ((cpp_function_code)(&create_keyed_equality_method_unit)), NULL);
    define_function_object("TRANSIENT-OBJECT?", "(DEFUN (TRANSIENT-OBJECT? BOOLEAN) ((OBJECT OBJECT)))", ((cpp_function_code)(&transient_objectP)), NULL);
    define_function_object("STARTUP-DEFCLASS", "(DEFUN STARTUP-DEFCLASS () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_defclass)), NULL);
    { Method_Slot* function = lookup_function(SYM_DEFCLASS_STELLA_STARTUP_DEFCLASS);

      set_dynamic_slot_value(function->dynamic_slots, SYM_DEFCLASS_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Defclass"), NULL_STRING_WRAPPER);
    }
  }
}

void startup_defclass() {
  if (current_startup_time_phaseP(0)) {
    oALL_RECYCLE_LISTSo = make_non_recycled_list();
  }
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      help_startup_defclass1();
      help_startup_defclass2();
      SYM_DEFCLASS_STELLA_EQUAL = ((Symbol*)(intern_rigid_symbol_wrt_module("EQUAL", NULL, 0)));
      SYM_DEFCLASS_STELLA_STARTUP_DEFCLASS = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-DEFCLASS", NULL, 0)));
      SYM_DEFCLASS_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
    }
    if (current_startup_time_phaseP(4)) {
      oRECYCLE_LISTS_MAINTENANCE_TIMERo = oRECYCLE_LISTS_MAINTENANCE_INTERVALo;
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      help_startup_defclass3();
      help_startup_defclass4();
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *RECYCLING-ENABLED?* BOOLEAN TRUE :DOCUMENTATION \"If `TRUE' calls to `new' or `free' on classes with :recycle-method;\nspecifications will actually operate on recycle lists.  Otherwise, all\nrecycle list operations will be no-ops.\" :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *ALL-RECYCLE-LISTS* (LIST OF RECYCLE-LIST) NULL)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *MAX-RECYCLE-LIST-BYTES* INTEGER 3000000 :DOCUMENTATION \"Maximum number of bytes to be occupied by recycle lists.\" :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *RECYCLE-LISTS-MAINTENANCE-INTERVAL* INTEGER 5000 :DOCUMENTATION \"Total number of recycle list extensions after which a\nsize maintenance operation is performed to ensure that\nthe `*MAX-RECYCLE-LIST-BYTES*' limit is still met.\" :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *RECYCLE-LISTS-MAINTENANCE-TIMER* INTEGER *RECYCLE-LISTS-MAINTENANCE-INTERVAL* :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *CURRENTLY-RECYCLED-ITEMS* (HASH-TABLE OF OBJECT OBJECT) NULL :DOCUMENTATION \"During recycle list debugging holds all items that live in\nsome free list.  Depending on program behavior, this table can get quite large!\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *TRANSLATE-WITH-RECYCLE-LIST-DEBUGGING?* BOOLEAN FALSE)");
    }
  }
}

Surrogate* SGT_DEFCLASS_STELLA_LITERAL = NULL;

Surrogate* SGT_DEFCLASS_STELLA_SECOND_CLASS_OBJECT = NULL;

Symbol* SYM_DEFCLASS_STELLA_NULL = NULL;

Symbol* SYM_DEFCLASS_STELLA_DEFINE_CLASS_FROM_STRINGIFIED_SOURCE = NULL;

Keyword* KWD_DEFCLASS_COMMON_LISP = NULL;

Symbol* SYM_DEFCLASS_STELLA_CLASS_CL_NATIVE_TYPE = NULL;

Keyword* KWD_DEFCLASS_IDL = NULL;

Symbol* SYM_DEFCLASS_STELLA_CLASS_IDL_NATIVE_TYPE = NULL;

Keyword* KWD_DEFCLASS_JAVA = NULL;

Symbol* SYM_DEFCLASS_STELLA_CLASS_JAVA_NATIVE_TYPE = NULL;

Keyword* KWD_DEFCLASS_CPP = NULL;

Keyword* KWD_DEFCLASS_CPP_STANDALONE = NULL;

Symbol* SYM_DEFCLASS_STELLA_CLASS_CPP_NATIVE_TYPE = NULL;

Surrogate* SGT_DEFCLASS_STELLA_NON_OBJECT = NULL;

Surrogate* SGT_DEFCLASS_STELLA_NATIVE_EXCEPTION = NULL;

Keyword* KWD_DEFCLASS_DYNAMIC = NULL;

Symbol* SYM_DEFCLASS_STELLA_SLOT_INITIAL_VALUE = NULL;

Keyword* KWD_DEFCLASS_BIT = NULL;

Surrogate* SGT_DEFCLASS_STELLA_BOOLEAN = NULL;

Keyword* KWD_DEFCLASS_CLASS = NULL;

Symbol* SYM_DEFCLASS_STELLA_SLOT_RENAMES = NULL;

Surrogate* SGT_DEFCLASS_STELLA_UNKNOWN = NULL;

Symbol* SYM_DEFCLASS_STELLA_SLOT_TYPE_SPECIFIER = NULL;

Surrogate* SGT_DEFCLASS_STELLA_ANCHORED_TYPE_SPECIFIER = NULL;

Symbol* SYM_DEFCLASS_STELLA_NEXT_SWEEP_LIST_OBJECT = NULL;

Keyword* KWD_DEFCLASS_FREE_LIST = NULL;

Keyword* KWD_DEFCLASS_FREE_AND_SWEEP_LIST = NULL;

Keyword* KWD_DEFCLASS_SWEEP_LIST = NULL;

Symbol* SYM_DEFCLASS_STELLA_CLASS_CREATOR = NULL;

Keyword* KWD_DEFCLASS_USE_COMMON_LISP_STRUCTS = NULL;

Keyword* KWD_DEFCLASS_USE_COMMON_LISP_VECTOR_STRUCTS = NULL;

Symbol* SYM_DEFCLASS_STELLA_DEFGLOBAL = NULL;

Symbol* SYM_DEFCLASS_STELLA_RECYCLE_LIST = NULL;

Symbol* SYM_DEFCLASS_STELLA_OF = NULL;

Symbol* SYM_DEFCLASS_STELLA_CREATE_RECYCLE_LIST = NULL;

Symbol* SYM_DEFCLASS_STELLA_SETQ = NULL;

Symbol* SYM_DEFCLASS_STELLA_SELF = NULL;

Symbol* SYM_DEFCLASS_STELLA_MAKE = NULL;

Symbol* SYM_DEFCLASS_STELLA_WHEN = NULL;

Symbol* SYM_DEFCLASS_STELLA_DEFINEDp = NULL;

Symbol* SYM_DEFCLASS_STELLA_FREE = NULL;

Symbol* SYM_DEFCLASS_STELLA_DYNAMIC_SLOTS = NULL;

Symbol* SYM_DEFCLASS_STELLA_CLEAR = NULL;

Symbol* SYM_DEFCLASS_STELLA_UNLESS = NULL;

Symbol* SYM_DEFCLASS_STELLA_RETURN = NULL;

Keyword* KWD_DEFCLASS_FORCE_NULL_VALUE = NULL;

Keyword* KWD_DEFCLASS_NULL_VALUE_ONLY = NULL;

Symbol* SYM_DEFCLASS_STELLA_AND = NULL;

Symbol* SYM_DEFCLASS_STELLA_oRECYCLING_ENABLEDpo = NULL;

Symbol* SYM_DEFCLASS_STELLA_COND = NULL;

Symbol* SYM_DEFCLASS_STELLA___ = NULL;

Symbol* SYM_DEFCLASS_STELLA_CURRENT_LENGTH = NULL;

Symbol* SYM_DEFCLASS_STELLA_OTHERWISE = NULL;

Symbol* SYM_DEFCLASS_STELLA_RECYCLED_ITEMS = NULL;

Symbol* SYM_DEFCLASS_STELLA_POP = NULL;

Symbol* SYM_DEFCLASS_STELLA_LIST_OF_RECYCLED_ITEMS = NULL;

Symbol* SYM_DEFCLASS_STELLA_SETF = NULL;

Symbol* SYM_DEFCLASS_STELLA_SLOT_VALUE = NULL;

Symbol* SYM_DEFCLASS_STELLA_UNUSED_ITEMS = NULL;

Symbol* SYM_DEFCLASS_STELLA_ALL_ITEMS = NULL;

Symbol* SYM_DEFCLASS_STELLA_ii = NULL;

Symbol* SYM_DEFCLASS_STELLA_e = NULL;

Symbol* SYM_DEFCLASS_STELLA_oRECYCLE_LISTS_MAINTENANCE_TIMERo = NULL;

Symbol* SYM_DEFCLASS_STELLA_MAINTAIN_RECYCLE_LISTS = NULL;

Keyword* KWD_DEFCLASS_INITIAL_VALUE_ONLY = NULL;

Keyword* KWD_DEFCLASS_ANY_INITIAL_VALUE = NULL;

Keyword* KWD_DEFCLASS_CONTEXT_SENSITIVEp = NULL;

Symbol* SYM_DEFCLASS_STELLA_FALSE = NULL;

Symbol* SYM_DEFCLASS_STELLA_CLASS_INITIALIZER = NULL;

Symbol* SYM_DEFCLASS_STELLA_RUN_CONSTRUCTOR_DEMONS = NULL;

Symbol* SYM_DEFCLASS_STELLA_DEFUN = NULL;

Keyword* KWD_DEFCLASS_CONSTRUCTORp = NULL;

Symbol* SYM_DEFCLASS_STELLA_TRUE = NULL;

Keyword* KWD_DEFCLASS_PUBLICp = NULL;

Symbol* SYM_DEFCLASS_STELLA_LET = NULL;

Symbol* SYM_DEFCLASS_STELLA_CLASS_DESTRUCTOR = NULL;

Symbol* SYM_DEFCLASS_STELLA_DEFMETHOD = NULL;

Surrogate* SGT_DEFCLASS_STELLA_STANDARD_OBJECT = NULL;

Symbol* SYM_DEFCLASS_STELLA_PUSH = NULL;

Symbol* SYM_DEFCLASS_STELLA_SWEEP = NULL;

Symbol* SYM_DEFCLASS_STELLA_UNUSED = NULL;

Symbol* SYM_DEFCLASS_STELLA_WHILE = NULL;

Symbol* SYM_DEFCLASS_STELLA_NOT = NULL;

Symbol* SYM_DEFCLASS_STELLA_EQLp = NULL;

Symbol* SYM_DEFCLASS_STELLA_UNMAKE = NULL;

Surrogate* SGT_DEFCLASS_STELLA_VOID = NULL;

Surrogate* SGT_DEFCLASS_STELLA_TRANSIENT_MIXIN = NULL;

Symbol* SYM_DEFCLASS_STELLA_RECYCLE_LIST_DEBUGGING_ENABLEDp = NULL;

Symbol* SYM_DEFCLASS_STELLA_REGISTER_RECYCLED_ITEM = NULL;

Symbol* SYM_DEFCLASS_STELLA_UNREGISTER_RECYCLED_ITEM = NULL;

Keyword* KWD_DEFCLASS_NULL_WRAPPER = NULL;

Symbol* SYM_DEFCLASS_STELLA_INLINE_WRAP_BOOLEAN = NULL;

Symbol* SYM_DEFCLASS_STELLA_FALSE_WRAPPER = NULL;

Symbol* SYM_DEFCLASS_STELLA_ANSWER = NULL;

Symbol* SYM_DEFCLASS_STELLA_IF = NULL;

Symbol* SYM_DEFCLASS_STELLA_NULLp = NULL;

Symbol* SYM_DEFCLASS_STELLA_PROGN = NULL;

Symbol* SYM_DEFCLASS_STELLA_ERROR = NULL;

Keyword* KWD_DEFCLASS_AUXILIARYp = NULL;

Symbol* SYM_DEFCLASS_STELLA_VALUE = NULL;

Symbol* SYM_DEFCLASS_STELLA_IGNORE = NULL;

Surrogate* SGT_DEFCLASS_STELLA_OBJECT = NULL;

Keyword* KWD_DEFCLASS_WRAP_FUNCTION = NULL;

Symbol* SYM_DEFCLASS_STELLA_CAST = NULL;

Symbol* SYM_DEFCLASS_STELLA_SETVALUEp = NULL;

Surrogate* SGT_DEFCLASS_STELLA_DYNAMIC_SLOTS_MIXIN = NULL;

Symbol* SYM_DEFCLASS_STELLA_SLOTNAME = NULL;

Symbol* SYM_DEFCLASS_STELLA_QUOTE = NULL;

Symbol* SYM_DEFCLASS_STELLA_CASE = NULL;

Symbol* SYM_DEFCLASS_STELLA_SET_DYNAMIC_SLOT_VALUE = NULL;

Symbol* SYM_DEFCLASS_STELLA_LOOKUP = NULL;

Symbol* SYM_DEFCLASS_STELLA_OBJECT = NULL;

Symbol* SYM_DEFCLASS_STELLA_SYMBOL = NULL;

Symbol* SYM_DEFCLASS_STELLA_BOOLEAN = NULL;

Symbol* SYM_DEFCLASS_STELLA_CLASS_SLOT_ACCESSOR_CODE = NULL;

Symbol* SYM_DEFCLASS_STELLA_THE_CODE = NULL;

Keyword* KWD_DEFCLASS_FUNCTION = NULL;

Symbol* SYM_DEFCLASS_STELLA_TYPECASE = NULL;

Symbol* SYM_DEFCLASS_STELLA_EQp = NULL;

Symbol* SYM_DEFCLASS_STELLA_OTHER = NULL;

Symbol* SYM_DEFCLASS_STELLA_EQUAL = NULL;

Symbol* SYM_DEFCLASS_STELLA_STARTUP_DEFCLASS = NULL;

Symbol* SYM_DEFCLASS_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
