// /tmp/build-stella/stella-3.0.0.beta/native/cpp/stella/demons.hh

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/


namespace stella {

// Class definitions:
class Hook_List : public Key_Value_List {
// Each HOOK-LIST object contains a list of zero or
// more function codes that get funcall'd some procedure.
public:
  char* signature;
  char* documentation;
public:
  virtual Surrogate* primary_type();
};


// Global declarations:
extern String_Hash_Table* oDEMONS_TABLEo;
extern List* oCONSTRUCTOR_DEMONSo;
extern List* oDESTRUCTOR_DEMONSo;
extern List* oALL_SLOTS_DEMONSo;
extern List* oALL_SLOTS_GUARD_DEMONSo;
extern Hook_List* oCHANGE_MODULE_HOOKSo;
extern Hook_List* oCLEAR_MODULE_HOOKSo;
extern Hook_List* oDESTROY_CONTEXT_HOOKSo;
extern Hook_List* oREDEFINE_RELATION_HOOKSo;
extern Hook_List* oFINALIZE_RELATION_HOOKSo;
extern Hook_List* oBLOCK_NATIVE_CLASS_CREATION_HOOKSo;

// Function signatures:
boolean validate_demon_referencesP(Demon* demon);
boolean incorporate_demon_optionsP(Demon* demon, Property_List* options);
Demon* lookup_demon(char* name);
Demon* define_demon(char* name, int options, ...);
List* inject_demon(List* demonlist, Demon* demon);
List* eject_demon(List* demonlist, Demon* demon);
void activate_demon(Demon* demon);
void deactivate_demon(Demon* demon);
Cons* destructure_defdemon_body(Cons* demontree, Cons*& _return1);
void in_place_normalize_demon_options(Cons* optionstree);
Object* defdemon_expander(String_Wrapper* name, Cons* parameterstree, Cons* optionsandbody);
Object* read_slot_value(Standard_Object* self, Storage_Slot* slot);
Object* write_slot_value(Standard_Object* self, Storage_Slot* slot, Object* value);
Object* put_slot_value(Standard_Object* self, Storage_Slot* slot, Object* value);
void drop_slot_value(Standard_Object* self, Storage_Slot* slot, Object* value);
boolean check_for_slot_demonsP(Storage_Slot* slot);
boolean run_slot_guard_demonsP(Standard_Object* self, Storage_Slot* slot, Object* oldvalue, Object* newvalue);
void run_slot_demons(Standard_Object* self, Storage_Slot* slot, Object* oldvalue, Object* newvalue);
void run_constructor_demons(Standard_Object* self);
void run_destructor_demons(Standard_Object* self);
Cons* yield_active_slot_reader_body(Storage_Slot* slot, Object* defaultexpression);
void cant_overwrite_active_collection_slot(Standard_Object* self, Symbol* slotname);
Cons* yield_active_slot_writer_body(Storage_Slot* slot);
Hook_List* new_hook_list();
Object* access_hook_list_slot_value(Hook_List* self, Symbol* slotname, Object* value, boolean setvalueP);
void add_hook(Hook_List* hooklist, Symbol* hookfunction);
void drop_hook(Hook_List* hooklist, Symbol* hookfunction);
void run_hooks(Hook_List* hooklist, Object* argument);
void help_startup_demons1();
void help_startup_demons2();
void startup_demons();

// Auxiliary global declarations:
extern Keyword* KWD_DEMONS_MODIFY;
extern Keyword* KWD_DEMONS_CREATE;
extern Keyword* KWD_DEMONS_DESTROY;
extern Keyword* KWD_DEMONS_ALL;
extern Keyword* KWD_DEMONS_CLASS;
extern Keyword* KWD_DEMONS_SLOT;
extern Keyword* KWD_DEMONS_CODE;
extern Keyword* KWD_DEMONS_METHOD;
extern Keyword* KWD_DEMONS_GUARDp;
extern Keyword* KWD_DEMONS_INHERITEDp;
extern Keyword* KWD_DEMONS_DOCUMENTATION;
extern Keyword* KWD_DEMONS_ACTIVATEp;
extern Symbol* SYM_DEMONS_STELLA_SLOT_GUARD_DEMONS;
extern Symbol* SYM_DEMONS_STELLA_SLOT_DEMONS;
extern Symbol* SYM_DEMONS_STELLA_CLASS_GUARD_CONSTRUCTOR_DEMONS;
extern Symbol* SYM_DEMONS_STELLA_CLASS_CONSTRUCTOR_DEMONS;
extern Symbol* SYM_DEMONS_STELLA_CLASS_GUARD_DESTRUCTOR_DEMONS;
extern Symbol* SYM_DEMONS_STELLA_CLASS_DESTRUCTOR_DEMONS;
extern Symbol* SYM_DEMONS_STELLA_WRAP_LITERAL;
extern Symbol* SYM_DEMONS_STELLA_TRUE;
extern Symbol* SYM_DEMONS_STELLA_WRAP_BOOLEAN;
extern Symbol* SYM_DEMONS_STELLA_FALSE;
extern Symbol* SYM_DEMONS_STELLA_PROGN;
extern Symbol* SYM_DEMONS_STELLA_DEFUN;
extern Symbol* SYM_DEMONS_STELLA_STARTUP_TIME_PROGN;
extern Symbol* SYM_DEMONS_STELLA_DEFINE_DEMON;
extern Symbol* SYM_DEMONS_STELLA_THE_CODE;
extern Keyword* KWD_DEMONS_FUNCTION;
extern Symbol* SYM_DEMONS_STELLA_DEFDEMON;
extern Symbol* SYM_DEMONS_STELLA_DEFDEMON_EXPANDER;
extern Symbol* SYM_DEMONS_STELLA_LET;
extern Symbol* SYM_DEMONS_STELLA_VALUE;
extern Symbol* SYM_DEMONS_STELLA_SLOT_VALUE;
extern Symbol* SYM_DEMONS_STELLA_SELF;
extern Symbol* SYM_DEMONS_STELLA_WHEN;
extern Symbol* SYM_DEMONS_STELLA_DEFINEDp;
extern Symbol* SYM_DEMONS_STELLA_RETURN;
extern Symbol* SYM_DEMONS_STELLA_SETQ;
extern Symbol* SYM_DEMONS_STELLA_NEW;
extern Symbol* SYM_DEMONS_STELLA_SETF;
extern Symbol* SYM_DEMONS_STELLA_OWNER_INSTANCE;
extern Symbol* SYM_DEMONS_STELLA_ACTIVE_SLOT;
extern Symbol* SYM_DEMONS_STELLA_GET_SLOT;
extern Symbol* SYM_DEMONS_STELLA_BQUOTE;
extern Surrogate* SGT_DEMONS_STELLA_BOOLEAN;
extern Symbol* SYM_DEMONS_STELLA_CANT_OVERWRITE_ACTIVE_COLLECTION_SLOT;
extern Surrogate* SGT_DEMONS_STELLA_LITERAL;
extern Symbol* SYM_DEMONS_STELLA_SLOT;
extern Symbol* SYM_DEMONS_STELLA_OLDVALUE;
extern Symbol* SYM_DEMONS_STELLA_NOT;
extern Symbol* SYM_DEMONS_STELLA_EQLp;
extern Symbol* SYM_DEMONS_STELLA_IF;
extern Symbol* SYM_DEMONS_STELLA_CHECK_FOR_SLOT_DEMONSp;
extern Symbol* SYM_DEMONS_STELLA_WRAPPEDOLDVALUE;
extern Symbol* SYM_DEMONS_STELLA_CHOOSE;
extern Symbol* SYM_DEMONS_STELLA_NULL;
extern Symbol* SYM_DEMONS_STELLA_WRAPPEDNEWVALUE;
extern Symbol* SYM_DEMONS_STELLA_RUN_SLOT_GUARD_DEMONSp;
extern Symbol* SYM_DEMONS_STELLA_RUN_SLOT_DEMONS;
extern Symbol* SYM_DEMONS_STELLA_FREE;
extern Symbol* SYM_DEMONS_STELLA_AND;
extern Symbol* SYM_DEMONS_STELLA_EQp;
extern Surrogate* SGT_DEMONS_STELLA_HOOK_LIST;
extern Symbol* SYM_DEMONS_STELLA_SIGNATURE;
extern Symbol* SYM_DEMONS_STELLA_DOCUMENTATION;
extern Symbol* SYM_DEMONS_STELLA_STARTUP_DEMONS;
extern Symbol* SYM_DEMONS_STELLA_METHOD_STARTUP_CLASSNAME;


} // end of namespace stella
