// describe.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

Object* get(Object* name, Cons* type) {
  { Cons* typespec = type;

    return (search_for_object(name, typespec->value));
  }
}

Object* get_evaluator_wrapper(Cons* arguments) {
  return (get(arguments->value, arguments->rest));
}

Object* Object::get_object(Object* type) {
  { Object* self = this;

    return (search_for_object(self, type));
  }
}

Object* string_get_object(char* self, Object* type) {
  { String_Wrapper* wrappedstring = wrap_string(self);
    Object* object = search_for_object(wrappedstring, type);

    wrappedstring->free();
    return (object);
  }
}

Object* integer_get_object(int self, Object* type) {
  { Integer_Wrapper* wrappedinteger = wrap_integer(self);
    Object* object = search_for_object(wrappedinteger, type);

    wrappedinteger->free();
    return (object);
  }
}

// Table of functions (keyed by type of object returned) that can
// be called to search for an object identified by a string.
Key_Value_List* oSTRING_TO_OBJECT_FUNCTIONSo = NULL;

// List of functions that can be called to search for
// an object identified by an integer.
List* oINTEGER_TO_OBJECT_FUNCTIONSo = NULL;

Object* string_search_for_object(char* string, Surrogate* classtype) {
  { Object* object = NULL;

    { Surrogate* type = NULL;
      Code_Wrapper* fn = NULL;
      Kv_Cons* iter_000 = oSTRING_TO_OBJECT_FUNCTIONSo->the_kv_list;

      while (((boolean)(iter_000))) {
        type = ((Surrogate*)(iter_000->key));
        fn = ((Code_Wrapper*)(iter_000->value));
        iter_000 = iter_000->rest;
        if ((!((boolean)(classtype))) ||
            subtype_ofP(type, classtype)) {
          if (subtype_ofP(safe_primary_type(fn), SGT_DESCRIBE_STELLA_FUNCTION_CODE_WRAPPER)) {
            { Code_Wrapper* fn_000 = fn;
              Function_Code_Wrapper* fn = ((Function_Code_Wrapper*)(fn_000));

              object = ((Object*  (*) (char*))fn->wrapper_value)(string);
            }
          }
          else {
            { Output_String_Stream* stream_000 = new_output_string_stream();

              *(stream_000->native_stream) << "`" << safe_primary_type(fn) << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_000->the_string_reader());
            }
          }
          if (((boolean)(object))) {
            return (object);
          }
        }
      }
    }
    return (NULL);
  }
}

Object* search_for_object(Object* self, Object* typeref) {
  // If 'self' is a string or a symbol, search for an object named
  // 'self' of type 'type'.  Otherwise, if 'self' is an object, return it.
  { Surrogate* classtype = NULL;

    if (((boolean)(typeref))) {
      { Surrogate* test_value_000 = safe_primary_type(typeref);

        if (subtype_of_symbolP(test_value_000)) {
          { Object* typeref_000 = typeref;
            Symbol* typeref = ((Symbol*)(typeref_000));

            classtype = lookup_surrogate_in_module(typeref->symbol_name, ((Module*)(typeref->home_context)), FALSE);
          }
        }
        else if (subtype_of_surrogateP(test_value_000)) {
          { Object* typeref_001 = typeref;
            Surrogate* typeref = ((Surrogate*)(typeref_001));

            classtype = typeref;
          }
        }
        else if (subtype_of_stringP(test_value_000)) {
          { Object* typeref_002 = typeref;
            String_Wrapper* typeref = ((String_Wrapper*)(typeref_002));

            classtype = lookup_surrogate(typeref->wrapper_value);
          }
        }
        else if (subtype_of_classP(test_value_000)) {
          { Object* typeref_003 = typeref;
            Class* typeref = ((Class*)(typeref_003));

            classtype = typeref->class_type;
          }
        }
        else {
          cout << "Invalid type reference: " << "`" << typeref << "'" << endl;
        }
      }
      if (((boolean)(classtype)) &&
          ((!((boolean)(classtype->surrogate_value))) ||
           (!isaP(classtype->surrogate_value, SGT_DESCRIBE_STELLA_CLASS)))) {
        cout << "No class exists with name: " << "`" << classtype->symbol_name << "'" << endl;
        classtype = NULL;
      }
    }
    { Surrogate* test_value_001 = safe_primary_type(self);

      if (subtype_of_stringP(test_value_001)) {
        { Object* self_000 = self;
          String_Wrapper* self = ((String_Wrapper*)(self_000));

          { char* string = self->wrapper_value;
            Object* object = string_search_for_object(string, classtype);
            Module* module = NULL;

            if ((!((boolean)(object))) &&
                qualified_nameP(string)) {
              module = compute_module_and_bare_name(string, string);
              object = string_search_for_object(string, classtype);
            }
            if (!((boolean)(object))) {
              string = string_upcase(string);
              object = string_search_for_object(string, classtype);
            }
            return (object);
          }
        }
      }
      else if (subtype_of_symbolP(test_value_001)) {
        { Object* self_001 = self;
          Symbol* self = ((Symbol*)(self_001));

          return (search_for_object(wrap_string(self->symbol_name), classtype));
        }
      }
      else if (subtype_of_integerP(test_value_001)) {
        { Object* self_002 = self;
          Integer_Wrapper* self = ((Integer_Wrapper*)(self_002));

          { int integer = self->wrapper_value;
            Object* object = NULL;

            { Code_Wrapper* fn = NULL;
              Cons* iter_000 = oINTEGER_TO_OBJECT_FUNCTIONSo->the_cons_list;

              while (!(iter_000 == NIL)) {
                fn = ((Code_Wrapper*)(iter_000->value));
                iter_000 = iter_000->rest;
                if (subtype_ofP(safe_primary_type(fn), SGT_DESCRIBE_STELLA_FUNCTION_CODE_WRAPPER)) {
                  { Code_Wrapper* fn_000 = fn;
                    Function_Code_Wrapper* fn = ((Function_Code_Wrapper*)(fn_000));

                    object = ((Object*  (*) (int))fn->wrapper_value)(integer);
                  }
                }
                else {
                  { Output_String_Stream* stream_000 = new_output_string_stream();

                    *(stream_000->native_stream) << "`" << safe_primary_type(fn) << "'" << " is not a valid case option";
                    throw *new_stella_exception(stream_000->the_string_reader());
                  }
                }
                if (((boolean)(object))) {
                  return (object);
                }
              }
            }
            return (NULL);
          }
        }
      }
      else if (subtype_ofP(test_value_001, SGT_DESCRIBE_STELLA_OBJECT)) {
        { Object* self_003 = self;
          Object* self = self_003;

          return (self);
        }
      }
      else {
        return (NULL);
      }
    }
  }
}

boolean filter_string_to_classP(Class* self, All_Purpose_Iterator* iterator) {
  if (iterator->iterator_integer == 1) {
    return (string_eqlP(class_name(self), ((String_Wrapper*)(iterator->iterator_object))->wrapper_value));
  }
  else {
    return (string_search(class_name(self), ((String_Wrapper*)(iterator->iterator_object))->wrapper_value, 0) != NULL_INTEGER);
  }
}

Iterator* string_to_classes(char* string, boolean exactP) {
  { All_Purpose_Iterator* iterator = new_all_purpose_iterator();

    iterator->iterator_nested_iterator = all_classes(NULL, FALSE);
    iterator->iterator_filter_code = ((cpp_function_code)(&filter_string_to_classP));
    iterator->iterator_next_code = ((cpp_function_code)(&filtered_nested_iterator_nextP));
    iterator->iterator_object = wrap_string(string);
    iterator->iterator_integer = (exactP ? 1 : 0);
    return (iterator);
  }
}

Object* string_to_class(char* string) {
  return (string_get_stella_class(string, FALSE));
}

Object* string_to_context(char* string) {
  return (string_get_stella_context_slowly(string));
}

Object* string_to_function(char* string) {
  return (lookup_function_by_name(string));
}

boolean dotted_stringP(char* string) {
  return (string_position(string, '.', 0) != NULL_INTEGER);
}

char* yield_exploded_method_name(char* dottedstring, char*& _return1) {
  { int dotposition = string_position(dottedstring, '.', 0);
    char* firststring = NULL;
    char* secondstring = NULL;

    if (dotposition != NULL_INTEGER) {
      firststring = string_subsequence(dottedstring, 0, dotposition);
      secondstring = string_subsequence(dottedstring, dotposition + 1, NULL_INTEGER);
    }
    _return1 = secondstring;
    return (firststring);
  }
}

Object* string_to_slot(char* string) {
  if (dotted_stringP(string)) {
    { char* classstring = NULL;
      char* slotstring = NULL;

      classstring = yield_exploded_method_name(string, slotstring);
      { Class* renamed_Class = string_get_stella_class(classstring, FALSE);
        Symbol* slotname = NULL;

        if (((boolean)(renamed_Class))) {
          slotname = lookup_symbol(slotstring);
          if (((boolean)(slotname))) {
            return (lookup_slot(renamed_Class, slotname));
          }
        }
      }
    }
  }
  else {
    { Symbol* symbol = lookup_symbol(string);

      if (((boolean)(symbol))) {
        return (lookup_function(symbol));
      }
    }
  }
  return (NULL);
}

Object* string_to_surrogate_value(char* string) {
  { Surrogate* sgt = NULL;
    Iterator* iter_000 = all_surrogates(NULL, FALSE);

    while (iter_000->nextP()) {
      sgt = ((Surrogate*)(iter_000->value));
      if (string_eqlP(sgt->symbol_name, string)) {
        return (sgt->surrogate_value);
      }
    }
  }
  return (NULL);
}

Object* string_to_demon(char* string) {
  return (lookup_demon(string));
}

Object* integer_to_context(int integer) {
  return (integer_get_stella_context_slowly(integer));
}

// Specifies the print mode for 'describe' when no second
// argument is given.
Keyword* oDEFAULT_DESCRIBE_MODEo = NULL;

void describe(Object* name, Cons* mode) {
  // Print a description of an object in :verbose, :terse,
  // or :source modes.
  { Object* object = ((!((boolean)(name))) ? ((Object*)(NULL)) : name->get_object(NULL));
    Object* modespec = mode->value;
    Keyword* validmode = NULL;

    if (!((boolean)(modespec))) {
      modespec = oDEFAULT_DESCRIBE_MODEo;
    }
    if (subtype_ofP(safe_primary_type(modespec), SGT_DESCRIBE_STELLA_GENERALIZED_SYMBOL)) {
      { Object* modespec_000 = modespec;
        Generalized_Symbol* modespec = ((Generalized_Symbol*)(modespec_000));

        { char* test_value_000 = modespec->symbol_name;

          if (string_eqlP(test_value_000, "VERBOSE") ||
              (string_eqlP(test_value_000, "TERSE") ||
               string_eqlP(test_value_000, "SOURCE"))) {
            validmode = intern_keyword(modespec->symbol_name);
          }
          else {
          }
        }
      }
    }
    else {
    }
    if (!((boolean)(validmode))) {
      cout << "Valid modes (the second argument to 'describe') are " << endl << "   :VERBOSE, :TERSE, and :SOURCE" << endl;
      validmode = oDEFAULT_DESCRIBE_MODEo;
    }
    if (!((boolean)(object))) {
      cout << "NULL";
    }
    else {
      object->describe_object(STANDARD_OUTPUT, validmode);
    }
  }
}

void describe_evaluator_wrapper(Cons* arguments) {
  describe(arguments->value, arguments->rest);
}

void describe_tersely(Object* self, Output_Stream* stream) {
  *(stream->native_stream) << self;
}

void pretty_print_stella_tree(Cons* tree, Output_Stream* stream) {
  tree = ((Cons*)(consify_lists_and_iterators(tree)));
  { 
    BIND_STELLA_SPECIAL(oPRINTPRETTYCODEpo, boolean, TRUE);
    print_stella_code(tree, stream->native_stream);
  }
}

void pretty_print_definition_tree(Cons* tree, Output_Stream* stream) {
  tree = ((Cons*)(consify_lists_and_iterators(tree)));
  { 
    BIND_STELLA_SPECIAL(oPRINTPRETTYCODEpo, boolean, TRUE);
    print_stella_definition(tree, stream->native_stream);
  }
}

// Lists longer than the cutoff are truncated during
// pretty printing.
int oPRETTY_PRINT_LIST_CUTOFFo = 5;

Object* consify_lists_and_iterators(Object* tree) {
  if (!((boolean)(tree))) {
    return (tree);
  }
  { Surrogate* test_value_000 = safe_primary_type(tree);

    if (test_value_000 == SGT_DESCRIBE_STELLA_CONS) {
      { Object* tree_000 = tree;
        Cons* tree = ((Cons*)(tree_000));

        { Cons_Iterator* cons = tree->allocate_iterator();

          while (cons->nextP()) {
            cons->value_setter(consify_lists_and_iterators(cons->value));
          }
        }
        return (tree);
      }
    }
    else if (subtype_ofP(test_value_000, SGT_DESCRIBE_STELLA_LIST)) {
      { Object* tree_001 = tree;
        List* tree = ((List*)(tree_001));

        { Cons* conslist = NIL;

          { Object* item = NULL;
            int i = NULL_INTEGER;
            Cons* iter_000 = tree->the_cons_list;
            int iter_001 = 0;
            int upper_bound_000 = oPRETTY_PRINT_LIST_CUTOFFo;
            boolean unboundedP_000 = upper_bound_000 == NULL_INTEGER;
            Cons* collect_000 = NULL;

            while ((!(iter_000 == NIL)) &&
                (unboundedP_000 ||
                 (iter_001 <= upper_bound_000))) {
              item = iter_000->value;
              iter_000 = iter_000->rest;
              i = iter_001;
              iter_001 = iter_001 + 1;
              if (!((boolean)(collect_000))) {
                {
                  collect_000 = cons(((i == oPRETTY_PRINT_LIST_CUTOFFo) ? ELIPSIS : item), NIL);
                  if (conslist == NIL) {
                    conslist = collect_000;
                  }
                  else {
                    add_cons_to_end_of_cons_list(conslist, collect_000);
                  }
                }
              }
              else {
                {
                  collect_000->rest = cons(((i == oPRETTY_PRINT_LIST_CUTOFFo) ? ELIPSIS : item), NIL);
                  collect_000 = collect_000->rest;
                }
              }
            }
          }
          return (conslist);
        }
      }
    }
    else if (subtype_ofP(test_value_000, SGT_DESCRIBE_STELLA_ITERATOR)) {
      { Object* tree_002 = tree;
        Iterator* tree = ((Iterator*)(tree_002));

        { Cons* conslist = NIL;

          { Object* item = NULL;
            int i = NULL_INTEGER;
            Iterator* iter_002 = tree;
            int iter_003 = 0;
            int upper_bound_001 = oPRETTY_PRINT_LIST_CUTOFFo;
            boolean unboundedP_001 = upper_bound_001 == NULL_INTEGER;
            Cons* collect_001 = NULL;

            while (iter_002->nextP() &&
                (unboundedP_001 ||
                 (iter_003 <= upper_bound_001))) {
              item = iter_002->value;
              i = iter_003;
              iter_003 = iter_003 + 1;
              if (!((boolean)(collect_001))) {
                {
                  collect_001 = cons(((i == oPRETTY_PRINT_LIST_CUTOFFo) ? ELIPSIS : item), NIL);
                  if (conslist == NIL) {
                    conslist = collect_001;
                  }
                  else {
                    add_cons_to_end_of_cons_list(conslist, collect_001);
                  }
                }
              }
              else {
                {
                  collect_001->rest = cons(((i == oPRETTY_PRINT_LIST_CUTOFFo) ? ELIPSIS : item), NIL);
                  collect_001 = collect_001->rest;
                }
              }
            }
          }
          return (conslist);
        }
      }
    }
    else {
      return (tree);
    }
  }
}

void Module::print_definition(Output_Stream* stream) {
  { Module* self = this;

    { Cons* options = ((Cons*)(read_s_expression_from_string(self->stringified_options)));

      { 
        BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
        *(stream->native_stream) << "(defmodule \"" << self->module_full_name << "\"";
        while (!(options == NIL)) {
          *(stream->native_stream) << endl << "  " << options->value << " " << options->rest->value;
          options = options->nth_rest(2);
        }
        *(stream->native_stream) << ")" << endl;
      }
    }
  }
}

void print_class_list(Iterator* self, Output_Stream* stream) {
  *(stream->native_stream) << "(";
  if (self->nextP()) {
    *(stream->native_stream) << ((Class*)(self->value))->class_type->symbol_name;
  }
  while (self->nextP()) {
    *(stream->native_stream) << " " << ((Class*)(self->value))->class_type->symbol_name;
  }
  *(stream->native_stream) << ")";
}

void Slot::pretty_print_object(Output_Stream* stream) {
  { Slot* self = this;

    stream = stream;
  }
}

// List of slots containing storage-slot options
DEFINE_STELLA_SPECIAL(oSLOTOPTIONSLOTSo, Cons* , NULL);

// List of slots containing class options
DEFINE_STELLA_SPECIAL(oCLASSOPTIONSLOTSo, Cons* , NULL);

// Plist of unstringifed class options
DEFINE_STELLA_SPECIAL(oCLASSUNSTRINGIFIEDOPTIONSo, Property_List* , NULL);

Cons* get_class_options() {
  { Cons* result = NIL;

    { Storage_Slot* slot = NULL;
      Iterator* iter_000 = private_class_storage_slots(((Class*)(SGT_DESCRIBE_STELLA_CLASS->surrogate_value)));
      Cons* collect_000 = NULL;

      while (iter_000->nextP()) {
        slot = ((Storage_Slot*)(iter_000->value));
        if (((boolean)(((Keyword*)(dynamic_slot_value(slot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_OPTION_KEYWORD, NULL)))))) {
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(slot, NIL);
              if (result == NIL) {
                result = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(result, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(slot, NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
    }
    { Storage_Slot* slot = NULL;
      Iterator* iter_001 = public_class_storage_slots(((Class*)(SGT_DESCRIBE_STELLA_CLASS->surrogate_value)));
      Cons* collect_001 = NULL;

      while (iter_001->nextP()) {
        slot = ((Storage_Slot*)(iter_001->value));
        if (((boolean)(((Keyword*)(dynamic_slot_value(slot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_OPTION_KEYWORD, NULL)))))) {
          if (!((boolean)(collect_001))) {
            {
              collect_001 = cons(slot, NIL);
              if (result == NIL) {
                result = collect_001;
              }
              else {
                add_cons_to_end_of_cons_list(result, collect_001);
              }
            }
          }
          else {
            {
              collect_001->rest = cons(slot, NIL);
              collect_001 = collect_001->rest;
            }
          }
        }
      }
    }
    { Storage_Slot* slot = NULL;
      Iterator* iter_002 = private_class_storage_slots(((Class*)(SGT_DESCRIBE_STELLA_RELATION->surrogate_value)));
      Cons* collect_002 = NULL;

      while (iter_002->nextP()) {
        slot = ((Storage_Slot*)(iter_002->value));
        if (((boolean)(((Keyword*)(dynamic_slot_value(slot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_OPTION_KEYWORD, NULL)))))) {
          if (!((boolean)(collect_002))) {
            {
              collect_002 = cons(slot, NIL);
              if (result == NIL) {
                result = collect_002;
              }
              else {
                add_cons_to_end_of_cons_list(result, collect_002);
              }
            }
          }
          else {
            {
              collect_002->rest = cons(slot, NIL);
              collect_002 = collect_002->rest;
            }
          }
        }
      }
    }
    { Storage_Slot* slot = NULL;
      Iterator* iter_003 = public_class_storage_slots(((Class*)(SGT_DESCRIBE_STELLA_RELATION->surrogate_value)));
      Cons* collect_003 = NULL;

      while (iter_003->nextP()) {
        slot = ((Storage_Slot*)(iter_003->value));
        if (((boolean)(((Keyword*)(dynamic_slot_value(slot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_OPTION_KEYWORD, NULL)))))) {
          if (!((boolean)(collect_003))) {
            {
              collect_003 = cons(slot, NIL);
              if (result == NIL) {
                result = collect_003;
              }
              else {
                add_cons_to_end_of_cons_list(result, collect_003);
              }
            }
          }
          else {
            {
              collect_003->rest = cons(slot, NIL);
              collect_003 = collect_003->rest;
            }
          }
        }
      }
    }
    return (result);
  }
}

Property_List* get_unstringified_class_options(Class* renamed_Class) {
  { Property_List* theoptions = new_property_list();

    theoptions->the_plist = ((Cons*)(read_s_expression_from_string(renamed_Class->class_stringified_source)))->rest->rest->rest;
    return (theoptions);
  }
}

Cons* get_slot_options() {
  { Cons* result = NIL;

    { Storage_Slot* slot = NULL;
      Iterator* iter_000 = private_class_storage_slots(((Class*)(SGT_DESCRIBE_STELLA_STORAGE_SLOT->surrogate_value)));
      Cons* collect_000 = NULL;

      while (iter_000->nextP()) {
        slot = ((Storage_Slot*)(iter_000->value));
        if (((boolean)(((Keyword*)(dynamic_slot_value(slot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_OPTION_KEYWORD, NULL)))))) {
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(slot, NIL);
              if (result == NIL) {
                result = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(result, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(slot, NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
    }
    return (result);
  }
}

void pretty_print_literal(Object* literal, Output_Stream* stream) {
  if (subtype_of_booleanP(safe_primary_type(literal))) {
    { Object* literal_000 = literal;
      Boolean_Wrapper* literal = ((Boolean_Wrapper*)(literal_000));

      if (literal == TRUE_WRAPPER) {
        *(stream->native_stream) << "TRUE";
      }
      else {
        *(stream->native_stream) << "FALSE";
      }
    }
  }
  else {
    { 
      BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
      *(stream->native_stream) << literal;
    }
  }
}

boolean literal_eqlP(Object* x, Object* y) {
  if ((((boolean)(x)) &&
      isaP(x, SGT_DESCRIBE_STELLA_BOOLEAN_WRAPPER)) ||
      (((boolean)(y)) &&
       isaP(y, SGT_DESCRIBE_STELLA_BOOLEAN_WRAPPER))) {
    return (((!((boolean)(((Boolean_Wrapper*)(x))))) &&
        (!coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(y))))) ||
        (((!((boolean)(((Boolean_Wrapper*)(y))))) &&
        (!coerce_wrapped_boolean_to_boolean(((Boolean_Wrapper*)(x))))) ||
         eqlP(x, y)));
  }
  return (eqlP(x, y));
}

Object* convert_to_literal(Object* object) {
  if (!((boolean)(object))) {
    return (object);
  }
  { Surrogate* test_value_000 = safe_primary_type(object);

    if (subtype_ofP(test_value_000, SGT_DESCRIBE_STELLA_LITERAL_WRAPPER)) {
      { Object* object_000 = object;
        Literal_Wrapper* object = ((Literal_Wrapper*)(object_000));

        return (object);
      }
    }
    else if (subtype_of_symbolP(test_value_000)) {
      { Object* object_001 = object;
        Symbol* object = ((Symbol*)(object_001));

        if (object == SYM_DESCRIBE_STELLA_TRUE) {
          return (TRUE_WRAPPER);
        }
        if (object == SYM_DESCRIBE_STELLA_FALSE) {
          return (FALSE_WRAPPER);
        }
        return (object);
      }
    }
    else if (subtype_of_keywordP(test_value_000)) {
      { Object* object_002 = object;
        Keyword* object = ((Keyword*)(object_002));

        return (object);
      }
    }
    else if (subtype_of_surrogateP(test_value_000)) {
      { Object* object_003 = object;
        Surrogate* object = ((Surrogate*)(object_003));

        return (object);
      }
    }
    else {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "`convert-to-literal': Can't handle non-boolean literals" << endl;
      return (NULL);
    }
  }
}

void pretty_print_class_options(Class* renamed_Class, Output_Stream* stream) {
  { Object* optionvalue = NULL;

    { Storage_Slot* optionslot = NULL;
      Cons* iter_000 = oCLASSOPTIONSLOTSo.get();

      while (!(iter_000 == NIL)) {
        optionslot = ((Storage_Slot*)(iter_000->value));
        iter_000 = iter_000->rest;
        optionvalue = read_slot_value(renamed_Class, optionslot);
        if (!((boolean)(optionvalue))) {
          continue;
        }
        { Surrogate* test_value_000 = safe_primary_type(optionvalue);

          if (subtype_ofP(test_value_000, SGT_DESCRIBE_STELLA_LITERAL_WRAPPER)) {
            { Object* optionvalue_000 = optionvalue;
              Literal_Wrapper* optionvalue = ((Literal_Wrapper*)(optionvalue_000));

              if (!(literal_eqlP(convert_to_literal(dynamic_slot_value(optionslot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_INITIAL_VALUE, NULL)), optionvalue) ||
                  literal_eqlP(convert_to_literal(optionslot->default_form()), optionvalue))) {
                *(stream->native_stream) << endl << "  :" << string_downcase(((Keyword*)(dynamic_slot_value(optionslot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_OPTION_KEYWORD, NULL)))->symbol_name) << " ";
                pretty_print_literal(optionvalue, stream);
              }
            }
          }
          else if (subtype_of_symbolP(test_value_000)) {
            { Object* optionvalue_001 = optionvalue;
              Symbol* optionvalue = ((Symbol*)(optionvalue_001));

              if (!(literal_eqlP(convert_to_literal(dynamic_slot_value(optionslot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_INITIAL_VALUE, NULL)), optionvalue) ||
                  literal_eqlP(convert_to_literal(optionslot->default_form()), optionvalue))) {
                *(stream->native_stream) << endl << "  :" << string_downcase(((Keyword*)(dynamic_slot_value(optionslot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_OPTION_KEYWORD, NULL)))->symbol_name) << " ";
                pretty_print_literal(optionvalue, stream);
              }
            }
          }
          else if (subtype_of_keywordP(test_value_000)) {
            { Object* optionvalue_002 = optionvalue;
              Keyword* optionvalue = ((Keyword*)(optionvalue_002));

              if (!(literal_eqlP(convert_to_literal(dynamic_slot_value(optionslot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_INITIAL_VALUE, NULL)), optionvalue) ||
                  literal_eqlP(convert_to_literal(optionslot->default_form()), optionvalue))) {
                *(stream->native_stream) << endl << "  :" << string_downcase(((Keyword*)(dynamic_slot_value(optionslot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_OPTION_KEYWORD, NULL)))->symbol_name) << " ";
                pretty_print_literal(optionvalue, stream);
              }
            }
          }
          else if (subtype_of_surrogateP(test_value_000)) {
            { Object* optionvalue_003 = optionvalue;
              Surrogate* optionvalue = ((Surrogate*)(optionvalue_003));

              if (!(literal_eqlP(convert_to_literal(dynamic_slot_value(optionslot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_INITIAL_VALUE, NULL)), optionvalue) ||
                  literal_eqlP(convert_to_literal(optionslot->default_form()), optionvalue))) {
                *(stream->native_stream) << endl << "  :" << string_downcase(((Keyword*)(dynamic_slot_value(optionslot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_OPTION_KEYWORD, NULL)))->symbol_name) << " ";
                pretty_print_literal(optionvalue, stream);
              }
            }
          }
          else if (subtype_ofP(test_value_000, SGT_DESCRIBE_STELLA_LIST)) {
            { Object* optionvalue_004 = optionvalue;
              List* optionvalue = ((List*)(optionvalue_004));

              if (!(optionvalue->emptyP() ||
                  ((dynamic_slot_value(optionslot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_INITIAL_VALUE, NULL) == optionvalue) ||
                   (optionslot->default_form() == optionvalue)))) {
                { 
                  BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
                  BIND_STELLA_SPECIAL(oPRINTPRETTYpo, boolean, TRUE);
                  *(stream->native_stream) << endl << "  :" << string_downcase(((Keyword*)(dynamic_slot_value(optionslot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_OPTION_KEYWORD, NULL)))->symbol_name) << " ";
                  *(stream->native_stream) << oCLASSUNSTRINGIFIEDOPTIONSo.get()->lookup(((Keyword*)(dynamic_slot_value(optionslot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_OPTION_KEYWORD, NULL))));
                }
              }
            }
          }
          else if (test_value_000 == SGT_DESCRIBE_STELLA_CONS) {
            { Object* optionvalue_005 = optionvalue;
              Cons* optionvalue = ((Cons*)(optionvalue_005));

              if (!((optionvalue == NIL) ||
                  ((dynamic_slot_value(optionslot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_INITIAL_VALUE, NULL) == optionvalue) ||
                   (optionslot->default_form() == optionvalue)))) {
                { 
                  BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
                  BIND_STELLA_SPECIAL(oPRINTPRETTYpo, boolean, TRUE);
                  *(stream->native_stream) << endl << "  :" << string_downcase(((Keyword*)(dynamic_slot_value(optionslot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_OPTION_KEYWORD, NULL)))->symbol_name) << " ";
                  *(stream->native_stream) << optionvalue;
                }
              }
            }
          }
          else {
          }
        }
      }
    }
  }
}

void pretty_print_slot_options(Storage_Slot* slot, Output_Stream* stream) {
  { Object* slotvalue = NULL;

    { Storage_Slot* optionslot = NULL;
      Cons* iter_000 = oSLOTOPTIONSLOTSo.get();

      while (!(iter_000 == NIL)) {
        optionslot = ((Storage_Slot*)(iter_000->value));
        iter_000 = iter_000->rest;
        slotvalue = read_slot_value(slot, optionslot);
        if (!((boolean)(slotvalue))) {
          continue;
        }
        if (subtype_ofP(safe_primary_type(slotvalue), SGT_DESCRIBE_STELLA_LITERAL_WRAPPER)) {
          { Object* slotvalue_000 = slotvalue;
            Literal_Wrapper* slotvalue = ((Literal_Wrapper*)(slotvalue_000));

            if (!literal_eqlP(dynamic_slot_value(optionslot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_INITIAL_VALUE, NULL), slotvalue)) {
              *(stream->native_stream) << " :" << string_downcase(((Keyword*)(dynamic_slot_value(optionslot->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_OPTION_KEYWORD, NULL)))->symbol_name) << " ";
              pretty_print_literal(slotvalue, stream);
            }
          }
        }
        else {
        }
      }
    }
  }
}

void Storage_Slot::pretty_print_object(Output_Stream* stream) {
  { Storage_Slot* self = this;

    *(stream->native_stream) << "(" << string_downcase(self->slot_name->symbol_name) << " :type ";
    *(stream->native_stream) << ((((boolean)(((Compound_Type_Specifier*)(dynamic_slot_value(self->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_TYPE_SPECIFIER, NULL))))) ? stringify(yield_type_spec_tree(((Compound_Type_Specifier*)(dynamic_slot_value(self->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_TYPE_SPECIFIER, NULL))))) : stringify(yield_type_spec_tree(self->slot_base_type))));
    pretty_print_slot_options(self, stream);
    *(stream->native_stream) << ")";
  }
}

void pretty_print_method_parameters(Output_Stream* stream, Method_Slot* method) {
  { int lastargindex = (((Boolean_Wrapper*)(dynamic_slot_value(method->dynamic_slots, SYM_DESCRIBE_STELLA_METHOD_VARIABLE_ARGUMENTSp, FALSE_WRAPPER)))->wrapper_value ? method->method_parameter_names_reader()->length() : -1);

    { 
      BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
      *(stream->native_stream) << "(";
      { Symbol* parameter = NULL;
        Standard_Object* tspec = NULL;
        int i = NULL_INTEGER;
        Cons* iter_000 = method->method_parameter_names_reader()->the_cons_list;
        Cons* iter_001 = method->method_parameter_type_specifiers_reader()->the_cons_list;
        int iter_002 = 1;

        while ((!(iter_000 == NIL)) &&
            (!(iter_001 == NIL))) {
          parameter = ((Symbol*)(iter_000->value));
          iter_000 = iter_000->rest;
          tspec = ((Standard_Object*)(iter_001->value));
          iter_001 = iter_001->rest;
          i = iter_002;
          iter_002 = iter_002 + 1;
          if (i > 1) {
            *(stream->native_stream) << " ";
          }
          if (i == lastargindex) {
            *(stream->native_stream) << "&rest ";
            tspec = variable_arguments_type(method);
          }
          *(stream->native_stream) << "(" << string_downcase(parameter->symbol_name) << " " << yield_type_spec_tree(tspec) << ")";
        }
      }
      *(stream->native_stream) << ")";
    }
  }
}

void Method_Slot::pretty_print_object(Output_Stream* stream) {
  { Method_Slot* self = this;

    *(stream->native_stream) << "(" << string_downcase(self->slot_name->symbol_name) << " ";
    pretty_print_method_parameters(stream, self);
    *(stream->native_stream) << " :type ";
    *(stream->native_stream) << ((((boolean)(((Compound_Type_Specifier*)(dynamic_slot_value(self->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_TYPE_SPECIFIER, NULL))))) ? stringify(yield_type_spec_tree(((Compound_Type_Specifier*)(dynamic_slot_value(self->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_TYPE_SPECIFIER, NULL))))) : stringify(yield_type_spec_tree(self->slot_base_type))));
    *(stream->native_stream) << ")";
  }
}

void pretty_print_slot_block(Output_Stream* stream, Iterator* iterator, char* slotgroup) {
  if (iterator->nextP()) {
    { Slot* slot = ((Slot*)(iterator->value));

      *(stream->native_stream) << endl << "  " << slotgroup << endl << "  (";
      slot->pretty_print_object(stream);
    }
  }
  else {
    return;
  }
  { Slot* slot = NULL;
    Iterator* iter_000 = iterator;

    while (iter_000->nextP()) {
      slot = ((Slot*)(iter_000->value));
      *(stream->native_stream) << endl << "   ";
      slot->pretty_print_object(stream);
    }
  }
  *(stream->native_stream) << ")";
}

void pretty_print_slots(Class* self, Output_Stream* stream) {
  pretty_print_slot_block(stream, public_class_storage_slots(self), ":public-slots");
  pretty_print_slot_block(stream, private_class_storage_slots(self), ":slots");
  pretty_print_slot_block(stream, public_class_methods(self), ":public-methods");
  pretty_print_slot_block(stream, private_class_methods(self), ":methods");
}

void Class::pretty_print_object(Output_Stream* stream) {
  { Class* self = this;

    { 
      BIND_STELLA_SPECIAL(oSLOTOPTIONSLOTSo, Cons*, get_slot_options());
      BIND_STELLA_SPECIAL(oCLASSOPTIONSLOTSo, Cons*, get_class_options());
      BIND_STELLA_SPECIAL(oCLASSUNSTRINGIFIEDOPTIONSo, Property_List*, get_unstringified_class_options(self));
      *(stream->native_stream) << "(defclass " << self->class_type->symbol_name << " ";
      print_class_list(new_types_to_classes_iterator(self->class_direct_supers->the_cons_list), stream);
      pretty_print_class_options(self, stream);
      pretty_print_slots(self, stream);
      *(stream->native_stream) << ")" << endl;
      if (class_tables(self, NIL)->nextP()) {
        *(stream->native_stream) << endl;
        { Table* table = NULL;
          Iterator* iter_000 = class_tables(self, NIL);

          while (iter_000->nextP()) {
            table = ((Table*)(iter_000->value));
            table->pretty_print_object(stream);
            *(stream->native_stream) << endl;
          }
        }
        *(stream->native_stream) << endl;
      }
    }
  }
}

void Object::describe_object(Output_Stream* stream, Keyword* mode) {
  // Prints a description of 'self' to stream 'stream'.  'mode'
  // can be :terse, :verbose, or :source.  The :terse mode is often equivalent
  // to the standard print function.
  { Object* self = this;

    mode = mode;
    describe_tersely(self, stream);
  }
}

void Class::describe_object(Output_Stream* stream, Keyword* mode) {
  { Class* self = this;

    if (mode == KWD_DESCRIBE_SOURCE) {
      pretty_print_definition_tree(((Cons*)(read_s_expression_from_string(self->class_stringified_source))), stream);
    }
    else if (mode == KWD_DESCRIBE_TERSE) {
      describe_tersely(self, stream);
    }
    else if (mode == KWD_DESCRIBE_VERBOSE) {
      *(stream->native_stream) << "SOURCE mode:" << endl;
      self->describe_object(stream, KWD_DESCRIBE_SOURCE);
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "`" << mode << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
}

void Storage_Slot::describe_object(Output_Stream* stream, Keyword* mode) {
  { Storage_Slot* self = this;

    if (mode == KWD_DESCRIBE_SOURCE) {
      *(stream->native_stream) << "TERSE mode:" << endl;
      self->describe_object(stream, KWD_DESCRIBE_TERSE);
    }
    else if (mode == KWD_DESCRIBE_TERSE) {
      describe_tersely(self, stream);
    }
    else if (mode == KWD_DESCRIBE_VERBOSE) {
      pretty_print_definition_tree(listO(3, SYM_DESCRIBE_STELLA_SLOT, self->slot_name, listO(3, KWD_DESCRIBE_OWNER, wrap_string(self->slot_owner->symbol_name), listO(3, KWD_DESCRIBE_TYPE, self->slot_base_type, listO(3, KWD_DESCRIBE_INITIALLY, dynamic_slot_value(self->dynamic_slots, SYM_DESCRIBE_STELLA_SLOT_INITIAL_VALUE, NULL), NIL)))), stream);
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "`" << mode << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
}

void Module::describe_object(Output_Stream* stream, Keyword* mode) {
  { Module* self = this;

    if (mode == KWD_DESCRIBE_SOURCE) {
      pretty_print_stella_tree(((Cons*)(read_s_expression_from_string(xmodule_stringified_source(self)))), stream);
    }
    else if (mode == KWD_DESCRIBE_TERSE) {
      describe_tersely(self, stream);
    }
    else if (mode == KWD_DESCRIBE_VERBOSE) {
      pretty_print_definition_tree(listO(3, SYM_DESCRIBE_STELLA_DEFMODULE, wrap_string(self->module_name), listO(3, KWD_DESCRIBE_INCLUDES, self->parent_modules, listO(3, KWD_DESCRIBE_USES, self->uses, listO(3, KWD_DESCRIBE_SHADOWS, self->shadowed_surrogates_reader(), listO(4, SYM_DESCRIBE_STELLA_gg_INFERRED_SLOTS, KWD_DESCRIBE_CHILDREN, self->child_contexts, listO(3, KWD_DESCRIBE_USED_BY, self->used_by, listO(3, KWD_DESCRIBE_CARDINAL_MODULE, self->cardinal_module, NIL))))))), stream);
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "`" << mode << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
}

char* xmodule_stringified_source(Module* self) {
  return (string_concatenate("(defmodule ", string_concatenate(self->module_name, string_concatenate(" ", string_concatenate(self->stringified_options, ")", 0), 0), 0), 0));
}

char* short_documentation(char* documentation) {
  { int lineend = NULL_INTEGER;
    int sentenceend = NULL_INTEGER;

    if (documentation == NULL) {
      documentation = "Not documented.";
    }
    lineend = string_position(documentation, '\r', 0);
    if (lineend == NULL_INTEGER) {
      lineend = string_position(documentation, '\n', 0);
    }
    if (lineend == NULL_INTEGER) {
      return (documentation);
    }
    if (string_memberP(".!", documentation[(lineend - 1)])) {
      return (string_subsequence(documentation, 0, lineend));
    }
    sentenceend = string_position(documentation, '.', 0);
    if ((sentenceend != NULL_INTEGER) &&
        (sentenceend <= 155)) {
      return (string_subsequence(documentation, 0, sentenceend + 1));
    }
    else {
      return (string_subsequence(documentation, 0, lineend));
    }
  }
}

void print_documentation_string(char* string, int indent, Output_Stream* stream) {
  { boolean freshlineP = TRUE;

    if (indent == NULL_INTEGER) {
      indent = 0;
    }
    { char renamed_Char = NULL_CHARACTER;
      char* vector_000 = string;
      int index_000 = 0;
      int length_000 = strlen(vector_000);

      while (index_000 < length_000) {
        renamed_Char = vector_000[index_000];
        index_000 = index_000 + 1;
        if (freshlineP) {
          { int i = NULL_INTEGER;
            int iter_000 = 1;
            int upper_bound_000 = indent;
            boolean unboundedP_000 = upper_bound_000 == NULL_INTEGER;

            while (unboundedP_000 ||
                (iter_000 <= upper_bound_000)) {
              i = iter_000;
              iter_000 = iter_000 + 1;
              i = i;
              *(stream->native_stream) << " ";
            }
          }
          freshlineP = FALSE;
        }
        *(stream->native_stream) << renamed_Char;
        if (renamed_Char == '\n') {
          freshlineP = TRUE;
        }
      }
    }
    *(stream->native_stream) << endl;
  }
}

void Method_Slot::print_documentation(Output_Stream* stream, boolean shortP) {
  { Method_Slot* self = this;

    if (!((boolean)(stream))) {
      stream = STANDARD_OUTPUT;
    }
    { char* documentation = ((String_Wrapper*)(dynamic_slot_value(self->dynamic_slots, SYM_DESCRIBE_STELLA_DOCUMENTATION, NULL_STRING_WRAPPER)))->wrapper_value;

      *(stream->native_stream) << self->slot_name;
      if (!(shortP)) {
        *(stream->native_stream) << " ";
        pretty_print_method_parameters(stream, self);
      }
      *(stream->native_stream) << ":" << endl;
      print_documentation_string((shortP ? short_documentation(documentation) : documentation), 2, stream);
    }
  }
}

void help_startup_describe1() {
  {
    SGT_DESCRIBE_STELLA_FUNCTION_CODE_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("FUNCTION-CODE-WRAPPER", NULL, 1)));
    SGT_DESCRIBE_STELLA_CLASS = ((Surrogate*)(intern_rigid_symbol_wrt_module("CLASS", NULL, 1)));
    SGT_DESCRIBE_STELLA_OBJECT = ((Surrogate*)(intern_rigid_symbol_wrt_module("OBJECT", NULL, 1)));
    SGT_DESCRIBE_STELLA_CONTEXT = ((Surrogate*)(intern_rigid_symbol_wrt_module("CONTEXT", NULL, 1)));
    SGT_DESCRIBE_STELLA_METHOD_SLOT = ((Surrogate*)(intern_rigid_symbol_wrt_module("METHOD-SLOT", NULL, 1)));
    SGT_DESCRIBE_STELLA_SLOT = ((Surrogate*)(intern_rigid_symbol_wrt_module("SLOT", NULL, 1)));
    SGT_DESCRIBE_STELLA_DEMON = ((Surrogate*)(intern_rigid_symbol_wrt_module("DEMON", NULL, 1)));
    KWD_DESCRIBE_VERBOSE = ((Keyword*)(intern_rigid_symbol_wrt_module("VERBOSE", NULL, 2)));
    SGT_DESCRIBE_STELLA_GENERALIZED_SYMBOL = ((Surrogate*)(intern_rigid_symbol_wrt_module("GENERALIZED-SYMBOL", NULL, 1)));
    SGT_DESCRIBE_STELLA_CONS = ((Surrogate*)(intern_rigid_symbol_wrt_module("CONS", NULL, 1)));
    SGT_DESCRIBE_STELLA_LIST = ((Surrogate*)(intern_rigid_symbol_wrt_module("LIST", NULL, 1)));
    SGT_DESCRIBE_STELLA_ITERATOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("ITERATOR", NULL, 1)));
    SYM_DESCRIBE_STELLA_SLOT_OPTION_KEYWORD = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-OPTION-KEYWORD", NULL, 0)));
    SGT_DESCRIBE_STELLA_RELATION = ((Surrogate*)(intern_rigid_symbol_wrt_module("RELATION", NULL, 1)));
    SGT_DESCRIBE_STELLA_STORAGE_SLOT = ((Surrogate*)(intern_rigid_symbol_wrt_module("STORAGE-SLOT", NULL, 1)));
    SGT_DESCRIBE_STELLA_BOOLEAN_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("BOOLEAN-WRAPPER", NULL, 1)));
    SGT_DESCRIBE_STELLA_LITERAL_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("LITERAL-WRAPPER", NULL, 1)));
    SYM_DESCRIBE_STELLA_TRUE = ((Symbol*)(intern_rigid_symbol_wrt_module("TRUE", NULL, 0)));
    SYM_DESCRIBE_STELLA_FALSE = ((Symbol*)(intern_rigid_symbol_wrt_module("FALSE", NULL, 0)));
    SYM_DESCRIBE_STELLA_SLOT_INITIAL_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-INITIAL-VALUE", NULL, 0)));
    SYM_DESCRIBE_STELLA_SLOT_TYPE_SPECIFIER = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-TYPE-SPECIFIER", NULL, 0)));
    SYM_DESCRIBE_STELLA_METHOD_VARIABLE_ARGUMENTSp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-VARIABLE-ARGUMENTS?", NULL, 0)));
    KWD_DESCRIBE_SOURCE = ((Keyword*)(intern_rigid_symbol_wrt_module("SOURCE", NULL, 2)));
    KWD_DESCRIBE_TERSE = ((Keyword*)(intern_rigid_symbol_wrt_module("TERSE", NULL, 2)));
    SYM_DESCRIBE_STELLA_SLOT = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT", NULL, 0)));
    KWD_DESCRIBE_OWNER = ((Keyword*)(intern_rigid_symbol_wrt_module("OWNER", NULL, 2)));
    KWD_DESCRIBE_TYPE = ((Keyword*)(intern_rigid_symbol_wrt_module("TYPE", NULL, 2)));
    KWD_DESCRIBE_INITIALLY = ((Keyword*)(intern_rigid_symbol_wrt_module("INITIALLY", NULL, 2)));
    SYM_DESCRIBE_STELLA_DEFMODULE = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFMODULE", NULL, 0)));
    KWD_DESCRIBE_INCLUDES = ((Keyword*)(intern_rigid_symbol_wrt_module("INCLUDES", NULL, 2)));
    KWD_DESCRIBE_USES = ((Keyword*)(intern_rigid_symbol_wrt_module("USES", NULL, 2)));
    KWD_DESCRIBE_SHADOWS = ((Keyword*)(intern_rigid_symbol_wrt_module("SHADOWS", NULL, 2)));
    SYM_DESCRIBE_STELLA_gg_INFERRED_SLOTS = ((Symbol*)(intern_rigid_symbol_wrt_module(">>_INFERRED_SLOTS", NULL, 0)));
    KWD_DESCRIBE_CHILDREN = ((Keyword*)(intern_rigid_symbol_wrt_module("CHILDREN", NULL, 2)));
    KWD_DESCRIBE_USED_BY = ((Keyword*)(intern_rigid_symbol_wrt_module("USED-BY", NULL, 2)));
    KWD_DESCRIBE_CARDINAL_MODULE = ((Keyword*)(intern_rigid_symbol_wrt_module("CARDINAL-MODULE", NULL, 2)));
    SYM_DESCRIBE_STELLA_DOCUMENTATION = ((Symbol*)(intern_rigid_symbol_wrt_module("DOCUMENTATION", NULL, 0)));
    SYM_DESCRIBE_STELLA_STARTUP_DESCRIBE = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-DESCRIBE", NULL, 0)));
    SYM_DESCRIBE_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
  }
}

void help_startup_describe2() {
  {
    define_function_object("GET", "(DEFUN (GET OBJECT) ((NAME NAME) |&REST| (TYPE NAME)) :COMMAND? TRUE :PUBLIC? TRUE :EVALUATE-ARGUMENTS? FALSE)", ((cpp_function_code)(&get)), ((cpp_function_code)(&get_evaluator_wrapper)));
    define_method_object("(DEFMETHOD (GET-OBJECT OBJECT) ((SELF OBJECT) (TYPE OBJECT)))", ((cpp_method_code)(&Object::get_object)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (GET-OBJECT OBJECT) ((SELF STRING) (TYPE OBJECT)))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (GET-OBJECT OBJECT) ((SELF INTEGER) (TYPE OBJECT)))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_function_object("STRING-SEARCH-FOR-OBJECT", "(DEFUN (STRING-SEARCH-FOR-OBJECT OBJECT) ((STRING STRING) (CLASSTYPE TYPE)))", ((cpp_function_code)(&string_search_for_object)), NULL);
    define_function_object("SEARCH-FOR-OBJECT", "(DEFUN (SEARCH-FOR-OBJECT OBJECT) ((SELF OBJECT) (TYPEREF OBJECT)) :DOCUMENTATION \"If 'self' is a string or a symbol, search for an object named\n'self' of type 'type'.  Otherwise, if 'self' is an object, return it.\")", ((cpp_function_code)(&search_for_object)), NULL);
    define_function_object("FILTER-STRING-TO-CLASS?", "(DEFUN (FILTER-STRING-TO-CLASS? BOOLEAN) ((SELF CLASS) (ITERATOR ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&filter_string_to_classP)), NULL);
    define_function_object("STRING-TO-CLASSES", "(DEFUN (STRING-TO-CLASSES (ITERATOR OF CLASS)) ((STRING STRING) (EXACT? BOOLEAN)))", ((cpp_function_code)(&string_to_classes)), NULL);
    define_function_object("STRING-TO-CLASS", "(DEFUN (STRING-TO-CLASS OBJECT) ((STRING STRING)))", ((cpp_function_code)(&string_to_class)), NULL);
    define_function_object("STRING-TO-CONTEXT", "(DEFUN (STRING-TO-CONTEXT OBJECT) ((STRING STRING)))", ((cpp_function_code)(&string_to_context)), NULL);
    define_function_object("STRING-TO-FUNCTION", "(DEFUN (STRING-TO-FUNCTION OBJECT) ((STRING STRING)))", ((cpp_function_code)(&string_to_function)), NULL);
    define_function_object("DOTTED-STRING?", "(DEFUN (DOTTED-STRING? BOOLEAN) ((STRING STRING)) :PUBLIC? TRUE)", ((cpp_function_code)(&dotted_stringP)), NULL);
    define_function_object("YIELD-EXPLODED-METHOD-NAME", "(DEFUN (YIELD-EXPLODED-METHOD-NAME STRING STRING) ((DOTTEDSTRING STRING)))", ((cpp_function_code)(&yield_exploded_method_name)), NULL);
    define_function_object("STRING-TO-SLOT", "(DEFUN (STRING-TO-SLOT OBJECT) ((STRING STRING)) :PUBLIC? TRUE)", ((cpp_function_code)(&string_to_slot)), NULL);
    define_function_object("STRING-TO-SURROGATE-VALUE", "(DEFUN (STRING-TO-SURROGATE-VALUE OBJECT) ((STRING STRING)))", ((cpp_function_code)(&string_to_surrogate_value)), NULL);
    define_function_object("STRING-TO-DEMON", "(DEFUN (STRING-TO-DEMON OBJECT) ((STRING STRING)))", ((cpp_function_code)(&string_to_demon)), NULL);
    define_function_object("INTEGER-TO-CONTEXT", "(DEFUN (INTEGER-TO-CONTEXT OBJECT) ((INTEGER INTEGER)))", ((cpp_function_code)(&integer_to_context)), NULL);
    define_function_object("DESCRIBE", "(DEFUN DESCRIBE ((NAME OBJECT) |&REST| (MODE OBJECT)) :DOCUMENTATION \"Print a description of an object in :verbose, :terse,\nor :source modes.\" :PUBLIC? TRUE :COMMAND? TRUE :EVALUATE-ARGUMENTS? FALSE)", ((cpp_function_code)(&describe)), ((cpp_function_code)(&describe_evaluator_wrapper)));
    define_function_object("DESCRIBE-TERSELY", "(DEFUN DESCRIBE-TERSELY ((SELF OBJECT) (STREAM OUTPUT-STREAM)))", ((cpp_function_code)(&describe_tersely)), NULL);
    define_function_object("PRETTY-PRINT-STELLA-TREE", "(DEFUN PRETTY-PRINT-STELLA-TREE ((TREE CONS) (STREAM OUTPUT-STREAM)))", ((cpp_function_code)(&pretty_print_stella_tree)), NULL);
    define_function_object("PRETTY-PRINT-DEFINITION-TREE", "(DEFUN PRETTY-PRINT-DEFINITION-TREE ((TREE CONS) (STREAM OUTPUT-STREAM)))", ((cpp_function_code)(&pretty_print_definition_tree)), NULL);
    define_function_object("CONSIFY-LISTS-AND-ITERATORS", "(DEFUN (CONSIFY-LISTS-AND-ITERATORS OBJECT) ((TREE OBJECT)))", ((cpp_function_code)(&consify_lists_and_iterators)), NULL);
    define_method_object("(DEFMETHOD PRINT-DEFINITION ((SELF MODULE) (STREAM OUTPUT-STREAM)) :PUBLIC? TRUE)", ((cpp_method_code)(&Module::print_definition)), ((cpp_method_code)(NULL)));
    define_function_object("PRINT-CLASS-LIST", "(DEFUN PRINT-CLASS-LIST ((SELF (ITERATOR OF CLASS)) (STREAM OUTPUT-STREAM)))", ((cpp_function_code)(&print_class_list)), NULL);
    define_method_object("(DEFMETHOD PRETTY-PRINT-OBJECT ((SELF SLOT) (STREAM OUTPUT-STREAM)))", ((cpp_method_code)(&Slot::pretty_print_object)), ((cpp_method_code)(NULL)));
    define_function_object("GET-CLASS-OPTIONS", "(DEFUN (GET-CLASS-OPTIONS (CONS OF STORAGE-SLOT)) ())", ((cpp_function_code)(&get_class_options)), NULL);
    define_function_object("GET-UNSTRINGIFIED-CLASS-OPTIONS", "(DEFUN (GET-UNSTRINGIFIED-CLASS-OPTIONS PROPERTY-LIST) ((CLASS CLASS)))", ((cpp_function_code)(&get_unstringified_class_options)), NULL);
    define_function_object("GET-SLOT-OPTIONS", "(DEFUN (GET-SLOT-OPTIONS (CONS OF STORAGE-SLOT)) ())", ((cpp_function_code)(&get_slot_options)), NULL);
    define_function_object("PRETTY-PRINT-LITERAL", "(DEFUN PRETTY-PRINT-LITERAL ((LITERAL OBJECT) (STREAM OUTPUT-STREAM)))", ((cpp_function_code)(&pretty_print_literal)), NULL);
    define_function_object("LITERAL-EQL?", "(DEFUN (LITERAL-EQL? BOOLEAN) ((X OBJECT) (Y OBJECT)))", ((cpp_function_code)(&literal_eqlP)), NULL);
    define_function_object("CONVERT-TO-LITERAL", "(DEFUN (CONVERT-TO-LITERAL OBJECT) ((OBJECT OBJECT)))", ((cpp_function_code)(&convert_to_literal)), NULL);
    define_function_object("PRETTY-PRINT-CLASS-OPTIONS", "(DEFUN PRETTY-PRINT-CLASS-OPTIONS ((CLASS CLASS) (STREAM OUTPUT-STREAM)))", ((cpp_function_code)(&pretty_print_class_options)), NULL);
    define_function_object("PRETTY-PRINT-SLOT-OPTIONS", "(DEFUN PRETTY-PRINT-SLOT-OPTIONS ((SLOT STORAGE-SLOT) (STREAM OUTPUT-STREAM)))", ((cpp_function_code)(&pretty_print_slot_options)), NULL);
    define_method_object("(DEFMETHOD PRETTY-PRINT-OBJECT ((SELF STORAGE-SLOT) (STREAM OUTPUT-STREAM)))", ((cpp_method_code)(&Storage_Slot::pretty_print_object)), ((cpp_method_code)(NULL)));
    define_function_object("PRETTY-PRINT-METHOD-PARAMETERS", "(DEFUN PRETTY-PRINT-METHOD-PARAMETERS ((STREAM OUTPUT-STREAM) (METHOD METHOD-SLOT)))", ((cpp_function_code)(&pretty_print_method_parameters)), NULL);
    define_method_object("(DEFMETHOD PRETTY-PRINT-OBJECT ((SELF METHOD-SLOT) (STREAM OUTPUT-STREAM)))", ((cpp_method_code)(&Method_Slot::pretty_print_object)), ((cpp_method_code)(NULL)));
    define_function_object("PRETTY-PRINT-SLOT-BLOCK", "(DEFUN PRETTY-PRINT-SLOT-BLOCK ((STREAM OUTPUT-STREAM) (ITERATOR (ITERATOR OF SLOT)) (SLOTGROUP STRING)))", ((cpp_function_code)(&pretty_print_slot_block)), NULL);
    define_function_object("PRETTY-PRINT-SLOTS", "(DEFUN PRETTY-PRINT-SLOTS ((SELF CLASS) (STREAM OUTPUT-STREAM)))", ((cpp_function_code)(&pretty_print_slots)), NULL);
    define_method_object("(DEFMETHOD PRETTY-PRINT-OBJECT ((SELF CLASS) (STREAM OUTPUT-STREAM)))", ((cpp_method_code)(&Class::pretty_print_object)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD DESCRIBE-OBJECT ((SELF OBJECT) (STREAM OUTPUT-STREAM) (MODE KEYWORD)) :PUBLIC? TRUE :DOCUMENTATION \"Prints a description of 'self' to stream 'stream'.  'mode'\ncan be :terse, :verbose, or :source.  The :terse mode is often equivalent\nto the standard print function.\")", ((cpp_method_code)(&Object::describe_object)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD DESCRIBE-OBJECT ((SELF CLASS) (STREAM OUTPUT-STREAM) (MODE KEYWORD)) :PUBLIC? TRUE)", ((cpp_method_code)(&Class::describe_object)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD DESCRIBE-OBJECT ((SELF STORAGE-SLOT) (STREAM OUTPUT-STREAM) (MODE KEYWORD)) :PUBLIC? TRUE)", ((cpp_method_code)(&Storage_Slot::describe_object)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD DESCRIBE-OBJECT ((SELF MODULE) (STREAM OUTPUT-STREAM) (MODE KEYWORD)) :PUBLIC? TRUE)", ((cpp_method_code)(&Module::describe_object)), ((cpp_method_code)(NULL)));
    define_function_object("XMODULE-STRINGIFIED-SOURCE", "(DEFUN (XMODULE-STRINGIFIED-SOURCE STRING) ((SELF MODULE)))", ((cpp_function_code)(&xmodule_stringified_source)), NULL);
    define_function_object("SHORT-DOCUMENTATION", "(DEFUN (SHORT-DOCUMENTATION STRING) ((DOCUMENTATION STRING)) :PUBLIC? TRUE)", ((cpp_function_code)(&short_documentation)), NULL);
    define_function_object("PRINT-DOCUMENTATION-STRING", "(DEFUN PRINT-DOCUMENTATION-STRING ((STRING STRING) (INDENT INTEGER) (STREAM OUTPUT-STREAM)))", ((cpp_function_code)(&print_documentation_string)), NULL);
    define_method_object("(DEFMETHOD PRINT-DOCUMENTATION ((SELF METHOD-SLOT) (STREAM OUTPUT-STREAM) (SHORT? BOOLEAN)) :PUBLIC? TRUE)", ((cpp_method_code)(&Method_Slot::print_documentation)), ((cpp_method_code)(NULL)));
    define_function_object("STARTUP-DESCRIBE", "(DEFUN STARTUP-DESCRIBE () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_describe)), NULL);
    { Method_Slot* function = lookup_function(SYM_DESCRIBE_STELLA_STARTUP_DESCRIBE);

      set_dynamic_slot_value(function->dynamic_slots, SYM_DESCRIBE_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Describe"), NULL_STRING_WRAPPER);
    }
  }
}

void startup_describe() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      help_startup_describe1();
    }
    if (current_startup_time_phaseP(4)) {
      oSTRING_TO_OBJECT_FUNCTIONSo = new_key_value_list();
      oINTEGER_TO_OBJECT_FUNCTIONSo = list(0);
      oDEFAULT_DESCRIBE_MODEo = KWD_DESCRIBE_VERBOSE;
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      help_startup_describe2();
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *STRING-TO-OBJECT-FUNCTIONS* (KEY-VALUE-LIST OF TYPE CODE-WRAPPER) (NEW KEY-VALUE-LIST) :DOCUMENTATION \"Table of functions (keyed by type of object returned) that can\nbe called to search for an object identified by a string.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *INTEGER-TO-OBJECT-FUNCTIONS* (LIST OF CODE-WRAPPER) (LIST) :DOCUMENTATION \"List of functions that can be called to search for\nan object identified by an integer.\")");
      oSTRING_TO_OBJECT_FUNCTIONSo->clear();
      oSTRING_TO_OBJECT_FUNCTIONSo->insert_at(SGT_DESCRIBE_STELLA_CLASS, wrap_function_code(((cpp_function_code)(&string_to_class))));
      oSTRING_TO_OBJECT_FUNCTIONSo->insert_at(SGT_DESCRIBE_STELLA_CONTEXT, wrap_function_code(((cpp_function_code)(&string_to_context))));
      oSTRING_TO_OBJECT_FUNCTIONSo->insert_at(SGT_DESCRIBE_STELLA_METHOD_SLOT, wrap_function_code(((cpp_function_code)(&string_to_function))));
      oSTRING_TO_OBJECT_FUNCTIONSo->insert_at(SGT_DESCRIBE_STELLA_SLOT, wrap_function_code(((cpp_function_code)(&string_to_slot))));
      oSTRING_TO_OBJECT_FUNCTIONSo->insert_at(SGT_DESCRIBE_STELLA_OBJECT, wrap_function_code(((cpp_function_code)(&string_to_surrogate_value))));
      oSTRING_TO_OBJECT_FUNCTIONSo->insert_at(SGT_DESCRIBE_STELLA_DEMON, wrap_function_code(((cpp_function_code)(&string_to_demon))));
      oINTEGER_TO_OBJECT_FUNCTIONSo->insert(wrap_function_code(((cpp_function_code)(&integer_to_context))));
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *DEFAULT-DESCRIBE-MODE* KEYWORD :VERBOSE :PUBLIC? TRUE :DOCUMENTATION \"Specifies the print mode for 'describe' when no second\nargument is given.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *PRETTY-PRINT-LIST-CUTOFF* INTEGER 5 :PUBLIC? TRUE :DOCUMENTATION \"Lists longer than the cutoff are truncated during\npretty printing.\")");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *SLOTOPTIONSLOTS* (CONS OF STORAGE-SLOT) NULL :DOCUMENTATION \"List of slots containing storage-slot options\")");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *CLASSOPTIONSLOTS* (CONS OF STORAGE-SLOT) NULL :DOCUMENTATION \"List of slots containing class options\")");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *CLASSUNSTRINGIFIEDOPTIONS* PROPERTY-LIST NULL :DOCUMENTATION \"Plist of unstringifed class options\")");
    }
  }
}

Surrogate* SGT_DESCRIBE_STELLA_FUNCTION_CODE_WRAPPER = NULL;

Surrogate* SGT_DESCRIBE_STELLA_CLASS = NULL;

Surrogate* SGT_DESCRIBE_STELLA_OBJECT = NULL;

Surrogate* SGT_DESCRIBE_STELLA_CONTEXT = NULL;

Surrogate* SGT_DESCRIBE_STELLA_METHOD_SLOT = NULL;

Surrogate* SGT_DESCRIBE_STELLA_SLOT = NULL;

Surrogate* SGT_DESCRIBE_STELLA_DEMON = NULL;

Keyword* KWD_DESCRIBE_VERBOSE = NULL;

Surrogate* SGT_DESCRIBE_STELLA_GENERALIZED_SYMBOL = NULL;

Surrogate* SGT_DESCRIBE_STELLA_CONS = NULL;

Surrogate* SGT_DESCRIBE_STELLA_LIST = NULL;

Surrogate* SGT_DESCRIBE_STELLA_ITERATOR = NULL;

Symbol* SYM_DESCRIBE_STELLA_SLOT_OPTION_KEYWORD = NULL;

Surrogate* SGT_DESCRIBE_STELLA_RELATION = NULL;

Surrogate* SGT_DESCRIBE_STELLA_STORAGE_SLOT = NULL;

Surrogate* SGT_DESCRIBE_STELLA_BOOLEAN_WRAPPER = NULL;

Surrogate* SGT_DESCRIBE_STELLA_LITERAL_WRAPPER = NULL;

Symbol* SYM_DESCRIBE_STELLA_TRUE = NULL;

Symbol* SYM_DESCRIBE_STELLA_FALSE = NULL;

Symbol* SYM_DESCRIBE_STELLA_SLOT_INITIAL_VALUE = NULL;

Symbol* SYM_DESCRIBE_STELLA_SLOT_TYPE_SPECIFIER = NULL;

Symbol* SYM_DESCRIBE_STELLA_METHOD_VARIABLE_ARGUMENTSp = NULL;

Keyword* KWD_DESCRIBE_SOURCE = NULL;

Keyword* KWD_DESCRIBE_TERSE = NULL;

Symbol* SYM_DESCRIBE_STELLA_SLOT = NULL;

Keyword* KWD_DESCRIBE_OWNER = NULL;

Keyword* KWD_DESCRIBE_TYPE = NULL;

Keyword* KWD_DESCRIBE_INITIALLY = NULL;

Symbol* SYM_DESCRIBE_STELLA_DEFMODULE = NULL;

Keyword* KWD_DESCRIBE_INCLUDES = NULL;

Keyword* KWD_DESCRIBE_USES = NULL;

Keyword* KWD_DESCRIBE_SHADOWS = NULL;

Symbol* SYM_DESCRIBE_STELLA_gg_INFERRED_SLOTS = NULL;

Keyword* KWD_DESCRIBE_CHILDREN = NULL;

Keyword* KWD_DESCRIBE_USED_BY = NULL;

Keyword* KWD_DESCRIBE_CARDINAL_MODULE = NULL;

Symbol* SYM_DESCRIBE_STELLA_DOCUMENTATION = NULL;

Symbol* SYM_DESCRIBE_STELLA_STARTUP_DESCRIBE = NULL;

Symbol* SYM_DESCRIBE_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
