// /tmp/build-stella/stella-3.0.0.beta/native/cpp/stella/hierarchy.hh

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/


namespace stella {

// Class definitions:
class Object : public gc {
// A reflective object that can participate in dynamically-typed
// contexts.  Either a STANDARD-OBJECT or a WRAPPER or a GENERIC-OBJECT.
// Also, either an OBJECT-WITH-INHERITED-TYPE or an OBJECT-WITH-OWN-TYPE.
public:
  virtual void print_object(ostream* stream);
  virtual void idl_output_literal();
  virtual Cons* idl_translate_atomic_tree();
  virtual void java_output_literal();
  virtual Cons* java_translate_atomic_tree();
  virtual void cpp_output_literal();
  virtual Cons* cpp_translate_atomic_tree();
  virtual void describe_object(Output_Stream* stream, Keyword* mode);
  virtual Object* get_object(Object* type);
  virtual Object* cl_translate_atomic_tree();
  virtual Object* walk_atomic_tree(Standard_Object*& _return1);
  virtual void sweep();
  virtual Standard_Object* yield_type_specifier();
  virtual Iterator* public_slots();
  virtual void free();
  virtual Class* primary_class();
  virtual Surrogate* surrogatify();
  virtual Object* permanentify();
  virtual boolean object_eqlP(Object* y);
  virtual Cons* consify();
  virtual boolean standard_objectP();
  virtual int hash_code();
  virtual boolean object_equalP(Object* y);
  virtual Surrogate* primary_type();
  virtual Module* home_module();
  virtual boolean deletedP();
  virtual void increment_reference_count();
  virtual void decrement_reference_count();
};

class Standard_Object : public Object {
// Object that used to find its type stored in a :class slot.
// By reimplementing 'primary-type' as a method, the justification for this
// class went away, but we'll keep it for now to keep things working.
public:
  virtual boolean object_eqlP(Object* y);
  virtual Object* copy_wrapped_literal();
  virtual boolean standard_objectP();
  virtual int hash_code();
};

class Reference_Count_Object : public Object {
// A reference count objects contains a slot 'reference-count' 
// that is incremented whenever the object is assigned to a slot, and decremented
// whenever it is deassigned from a slot.
public:
  int reference_count;
public:
  virtual void increment_reference_count();
  virtual void decrement_reference_count();
};

class Context_Sensitive_Object : public Standard_Object {
// Context sensitive objects inherit a slot 'home-context'
// that enables context-dependent access machinery to determine the
// visibility of objects from modules.
public:
  Context* home_context;
public:
  virtual Module* home_module();
};

class Active_Object : public Context_Sensitive_Object {
// Active objects call class and slot triggers in response
// to instance creation/destruction and slot updates.  They include internal
// storage slots needed for class extensions, dynamic storage, and home context.
public:
  Key_Value_List* dynamic_slots;
public:
  virtual void free();
  virtual boolean deletedP_setter(boolean value);
  virtual boolean deletedP();
  virtual Boolean_Wrapper* badP_reader();
};

class Abstract_Iterator : public Standard_Object {
// Instances of ABSTRACT-ITERATOR support iteration over collections.
public:
  boolean first_iterationP;
public:
  virtual Abstract_Iterator* allocate_iterator();
  virtual boolean nextP();
  virtual int length();
};

class Iterator : public Abstract_Iterator {
// Instances of ITERATOR support iteration over 
// collections of OBJECTs.
public:
  Object* value;
public:
  virtual All_Purpose_Iterator* concatenate(Iterator* iterator2, int otheriterators, ...);
  virtual boolean memberP(Object* value);
  virtual boolean emptyP();
  virtual Iterator* advance(int n);
  virtual Object* pop();
  virtual List* listify();
  virtual Cons* consify();
};

class Cons_Iterator : public Iterator {
// Iterator class for the class CONS.
public:
  Cons* cons_iterator_cursor;
public:
  virtual void free();
  virtual Object* value_setter(Object* value);
  virtual boolean emptyP();
  virtual boolean nextP();
  virtual Surrogate* primary_type();
};

class List_Iterator : public Iterator {
// Iterator class for the collection LIST.
public:
  Cons* list_iterator_cursor;
  List* list_iterator_collection;
public:
  virtual void free();
  virtual Object* value_setter(Object* value);
  virtual boolean emptyP();
  virtual boolean nextP();
  virtual Surrogate* primary_type();
};

class Destructive_List_Iterator : public Iterator {
// An iterator that contains a cons list.  Iterates over
// the stored list, and destroys it when the iterator is free'd.
public:
  Cons* the_cons_list;
  Cons* list_iterator_cursor;
public:
  virtual void free();
  virtual void insert(Object* value);
  virtual boolean emptyP();
  virtual boolean nextP();
  virtual Surrogate* primary_type();
};

class Set_Iterator : public Iterator {
// Iterator class for the collection SET.
public:
  Cons* set_iterator_cursor;
  Set* set_iterator_collection;
public:
  virtual void free();
  virtual Object* value_setter(Object* value);
  virtual boolean nextP();
  virtual Surrogate* primary_type();
};

class All_Purpose_Iterator : public Iterator {
// The all-purpose iterator works by storing a 'next?'
// function within itself during iterator allocation.  The method
// 'ALL-PURPOSE-ITERAOR.next?' funcalls the stored 'next?' function.
// An assortment of storage slots are provided for constructing different
// kinds of iterators.
public:
  Object* key;
  cpp_function_code iterator_next_code;
  cpp_function_code iterator_filter_code;
  cpp_function_code iterator_empty_code;
  Iterator* iterator_nested_iterator;
  Cons* iterator_cons_list;
  Object* iterator_object;
  Object* iterator_second_object;
  int iterator_integer;
  int iterator_second_integer;
public:
  virtual void free();
  virtual boolean terminate_all_purpose_iteratorP();
  virtual boolean emptyP();
  virtual boolean nextP();
  virtual Surrogate* primary_type();
};

class Types_To_Classes_Iterator : public Iterator {
// Iterator that returns a class for each type
// in a cons list of types.
public:
  Cons* iterator_cursor;
public:
  virtual Surrogate* primary_type();
  virtual boolean nextP();
};

class Abstract_Dictionary_Iterator : public Abstract_Iterator {
// Instances of ABSTRACT-DICTIONARY-ITERATOR support iteration
// over dictionaries.
};

class Dictionary_Iterator : public Abstract_Dictionary_Iterator {
// Instances of DICTIONARY-ITERATOR support iteration
// over dictionaries with keys and values of type OBJECT.
public:
  Object* key;
  Object* value;
};

class Property_List_Iterator : public Dictionary_Iterator {
// Iterator class for the collection PROPERTY-LIST.
public:
  Cons* plist_iterator_cursor;
  Property_List* plist_iterator_collection;
public:
  virtual Object* key_setter(Object* key);
  virtual Object* value_setter(Object* value);
  virtual boolean nextP();
  virtual Surrogate* primary_type();
};

class Kv_List_Iterator : public Dictionary_Iterator {
public:
  Key_Value_List* the_kv_list;
  Kv_Cons* kv_list_iterator_cursor;
public:
  virtual Object* key_setter(Object* key);
  virtual Object* value_setter(Object* value);
  virtual boolean nextP();
  virtual Surrogate* primary_type();
};

class Interval_Iterator : public Abstract_Iterator {
// An iterator that specifies a (possibly infinite) range of
// values.  Supports 'member?' test as well as iteration methods.
};

class Integer_Interval_Iterator : public Interval_Iterator {
public:
  int interval_cursor;
  int lower_bound;
  int upper_bound;
  int value;
public:
  virtual Reverse_Integer_Interval_Iterator* reverse();
  virtual boolean nextP();
  virtual void initialize_integer_interval_iterator();
  virtual Surrogate* primary_type();
};

class Reverse_Integer_Interval_Iterator : public Interval_Iterator {
public:
  int interval_cursor;
  int lower_bound;
  int upper_bound;
  int value;
public:
  virtual boolean nextP();
  virtual Surrogate* primary_type();
};

class String_Iterator : public Abstract_Iterator {
// Iterator that yields characters from a string.
public:
  char* the_string;
  int cursor;
  int end;
  char value;
public:
  virtual boolean nextP();
  virtual Surrogate* primary_type();
};

class Abstract_Collection : public Standard_Object {
public:
  virtual Abstract_Iterator* allocate_iterator();
  virtual int length();
};

class Collection : public Abstract_Collection {
public:
  virtual boolean orderedP();
  virtual boolean no_duplicatesP();
  virtual boolean memberP(Object* object);
  virtual Abstract_Iterator* allocate_iterator();
  virtual int length();
  virtual void insert(Object* value);
  virtual Abstract_Collection* remove(Object* value);
};

class Sequence : public Collection {
public:
  virtual boolean orderedP();
  virtual boolean memberP(Object* value);
  virtual Abstract_Iterator* allocate_iterator();
  virtual int length();
};

class List : public Sequence {
public:
  Cons* the_cons_list;
public:
  virtual void print_object(ostream* stream);
  virtual Iterator* allocate_destructive_list_iterator();
  virtual List* listify();
  virtual List* subtract(List* otherlist);
  virtual List* difference(List* otherlist);
  virtual List* renamed_Union(List* otherlist);
  virtual List* intersection(List* otherlist);
  virtual boolean equivalent_setsP(List* otherlist);
  virtual List* remove_deleted_members();
  virtual Iterator* but_last();
  virtual Abstract_Iterator* allocate_iterator();
  virtual boolean object_equalP(Object* y);
  virtual List* sort(cpp_function_code predicate);
  virtual void clear();
  virtual List* copy();
  virtual List* prepend(List* list2);
  virtual List* concatenate(List* list2, int otherlists, ...);
  virtual List* substitute(Object* invalue, Object* outvalue);
  virtual List* reverse();
  virtual Object* pop();
  virtual Collection* remove_duplicates();
  virtual List* remove_if(cpp_function_code testP);
  virtual Abstract_Collection* remove(Object* value);
  virtual void insert_last(Object* value);
  virtual void insert_new(Object* value);
  virtual void push(Object* value);
  virtual void insert(Object* value);
  virtual int position(Object* object, int start);
  virtual int length();
  virtual Object* nth_setter(Object* value, int position);
  virtual Object* fifth_setter(Object* value);
  virtual Object* fourth_setter(Object* value);
  virtual Object* third_setter(Object* value);
  virtual Object* second_setter(Object* value);
  virtual Object* first_setter(Object* value);
  virtual Cons* rest();
  virtual Object* last();
  virtual Object* nth(int position);
  virtual Object* fifth();
  virtual Object* fourth();
  virtual Object* third();
  virtual Object* second();
  virtual Object* first();
  virtual boolean membP(Object* object);
  virtual boolean memberP(Object* object);
  virtual boolean non_emptyP();
  virtual boolean emptyP();
  virtual Cons* consify();
  virtual Surrogate* primary_type();
};

class Set : public Collection {
public:
  Cons* the_cons_list;
public:
  virtual void print_object(ostream* stream);
  virtual Abstract_Iterator* allocate_iterator();
  virtual void clear();
  virtual Set* copy();
  virtual Set* reverse();
  virtual Abstract_Collection* remove(Object* value);
  virtual void insert(Object* value);
  virtual int length();
  virtual boolean memberP(Object* object);
  virtual boolean non_emptyP();
  virtual boolean emptyP();
  virtual boolean no_duplicatesP();
  virtual Surrogate* primary_type();
};

class Class_Extension : public List {
public:
  virtual Surrogate* primary_type();
};

class Cons : public Standard_Object {
public:
  Object* value;
  Cons* rest;
public:
  virtual void print_object(ostream* stream);
  virtual void java_output_literal();
  virtual void cpp_output_literal();
  virtual Standard_Object* yield_type_specifier();
  virtual Surrogate* surrogatify();
  virtual Iterator* allocate_destructive_list_iterator();
  virtual List* listify();
  virtual boolean object_equalP(Object* tree2);
  virtual Cons* sort(cpp_function_code predicate);
  virtual Cons* subtract(Cons* otherlist);
  virtual Cons* difference(Cons* otherlist);
  virtual Cons* renamed_Union(Cons* otherlist);
  virtual Cons* intersection(Cons* otherlist);
  virtual boolean equivalent_setsP(Cons* otherlist);
  virtual Cons* remove_deleted_members();
  virtual Cons* consify();
  virtual Iterator* but_last();
  virtual Object* nth_rest_setter(Object* value, int position);
  virtual Object* fifth_setter(Object* value);
  virtual Object* fourth_setter(Object* value);
  virtual Object* nth_setter(Object* value, int position);
  virtual Object* third_setter(Object* value);
  virtual Object* second_setter(Object* value);
  virtual Object* first_setter(Object* value);
  virtual Object* last();
  virtual Cons* nth_rest(int position);
  virtual Object* nth(int position);
  virtual Object* fifth();
  virtual Object* fourth();
  virtual Object* third();
  virtual Object* second();
  virtual Object* first();
  virtual Cons* substitute(Object* invalue, Object* outvalue);
  virtual Cons* reverse();
  virtual Cons* prepend(Cons* list1);
  virtual Cons* concatenate(Cons* list2, int otherlists, ...);
  virtual Cons* remove_duplicates();
  virtual Cons* remove_if(cpp_function_code testP);
  virtual Cons* remove(Object* value);
  virtual boolean membP(Object* object);
  virtual boolean memberP(Object* object);
  virtual int position(Object* object, int start);
  virtual int length();
  virtual boolean non_emptyP();
  virtual boolean emptyP();
  virtual Surrogate* primary_type();
  virtual Cons_Iterator* allocate_iterator();
};

class Abstract_Dictionary : public Abstract_Collection {
public:
  virtual Abstract_Iterator* allocate_iterator();
};

class Dictionary : public Abstract_Dictionary {
public:
  virtual Abstract_Iterator* allocate_iterator();
};

class Property_List : public Dictionary {
public:
  Cons* the_plist;
public:
  virtual void print_object(ostream* stream);
  virtual Abstract_Iterator* allocate_iterator();
  virtual boolean object_equalP(Object* y);
  virtual void clear();
  virtual Property_List* copy();
  virtual boolean non_emptyP();
  virtual boolean emptyP();
  virtual Object* remove_at(Object* key);
  virtual void insert_at(Object* key, Object* value);
  virtual int length();
  virtual Object* lookup(Object* key);
  virtual void initialize_object();
  virtual Surrogate* primary_type();
};

class Kv_Cons : public Standard_Object {
public:
  Object* key;
  Object* value;
  Kv_Cons* rest;
public:
  virtual void print_object(ostream* stream);
  virtual Object* lookup(Object* key);
  virtual Surrogate* primary_type();
};

class Key_Value_List : public Dictionary {
public:
  Kv_Cons* the_kv_list;
public:
  virtual void print_object(ostream* stream);
  virtual List* listify();
  virtual Abstract_Iterator* allocate_iterator();
  virtual boolean object_equalP(Object* y);
  virtual void clear();
  virtual Key_Value_List* copy();
  virtual boolean non_emptyP();
  virtual boolean emptyP();
  virtual Key_Value_List* reverse();
  virtual Object* pop();
  virtual void remove_entry(Object* key, Object* value);
  virtual void insert_entry(Object* key, Object* value);
  virtual Object* remove_at(Object* key);
  virtual void insert_at(Object* key, Object* value);
  virtual int length();
  virtual Object* lookup(Object* key);
  virtual Cons* consify();
  virtual Surrogate* primary_type();
};

class Recycle_List : public Sequence {
public:
  Surrogate* recycle_list_of;
  Object* recycled_items;
  List* list_of_recycled_items;
  Object* all_items;
  Object* unused_items;
  int current_length;
  int item_size;
public:
  virtual void print_object(ostream* stream);
  virtual boolean deletedP_setter(boolean value);
  virtual boolean deletedP();
  virtual Surrogate* primary_type();
};

class Abstract_Hash_Table : public Abstract_Dictionary {
public:
  cpp_hash_table* the_hash_table;
  Stella_Hash_Table* the_stella_hash_table;
public:
  virtual void initialize_hash_table();
  virtual void free_hash_table_values();
};

class Hash_Table : public Abstract_Hash_Table {
public:
  virtual void remove_at(Object* key);
  virtual void insert_at(Object* key, Object* value);
  virtual Object* lookup(Object* key);
  virtual Surrogate* primary_type();
};

class Integer_Hash_Table : public Abstract_Hash_Table {
public:
  virtual void remove_at(int key);
  virtual void insert_at(int key, Object* value);
  virtual Object* lookup(int key);
  virtual void initialize_hash_table();
  virtual Surrogate* primary_type();
};

class Float_Hash_Table : public Abstract_Hash_Table {
public:
  virtual void remove_at(double key);
  virtual void insert_at(double key, Object* value);
  virtual Object* lookup(double key);
  virtual void initialize_hash_table();
  virtual Surrogate* primary_type();
};

class String_Hash_Table : public Abstract_Hash_Table {
public:
  virtual void remove_at(char* key);
  virtual void insert_at(char* key, Object* value);
  virtual Object* lookup(char* key);
  virtual void initialize_hash_table();
  virtual Surrogate* primary_type();
};

class String_To_Integer_Hash_Table : public Abstract_Hash_Table {
public:
  virtual void remove_at(char* key);
  virtual void insert_at(char* key, int value);
  virtual int lookup(char* key);
  virtual void initialize_hash_table();
  virtual Surrogate* primary_type();
};

class Vector : public Sequence {
public:
  int array_size;
  Object** the_array;
public:
  virtual void print_object(ostream* stream);
  virtual Iterator* but_last();
  virtual Abstract_Iterator* allocate_iterator();
  virtual boolean object_equalP(Object* y);
  virtual void clear();
  virtual Vector* copy();
  virtual void insert_at(int offset, Object* value);
  virtual int position(Object* object, int start);
  virtual boolean memberP(Object* object);
  virtual int length();
  virtual Object* last_setter(Object* value);
  virtual Object* nth_setter(Object* value, int position);
  virtual Object* fifth_setter(Object* value);
  virtual Object* fourth_setter(Object* value);
  virtual Object* third_setter(Object* value);
  virtual Object* second_setter(Object* value);
  virtual Object* first_setter(Object* value);
  virtual Object* last();
  virtual Object* nth(int position);
  virtual Object* fifth();
  virtual Object* fourth();
  virtual Object* third();
  virtual Object* second();
  virtual Object* first();
  virtual boolean non_emptyP();
  virtual boolean emptyP();
  virtual void print_vector(ostream* stream);
  virtual List* listify();
  virtual Cons* consify();
  virtual void initialize_vector();
  virtual Surrogate* primary_type();
};

class Extensible_Vector : public Vector {
public:
  virtual void insert_at(int offset, Object* value);
  virtual Surrogate* primary_type();
};

class Extensible_Symbol_Array : public Extensible_Vector {
// Self-extending array with methods for storing and
// accessing symbols within it.
public:
  int top_symbol_offset;
  int potential_free_symbol_offset;
public:
  virtual void insert_at(int offset, Object* value);
  virtual Surrogate* primary_type();
};

class Vector_Sequence : public Vector {
public:
  int sequence_length;
public:
  virtual int length();
  virtual Abstract_Collection* remove(Object* value);
  virtual void insert(Object* value);
  virtual int resize_increment_reader();
};

class Short_Vector_Sequence : public Vector_Sequence {
public:
  virtual Surrogate* primary_type();
  virtual int resize_increment_reader();
};

class Boolean_Vector : public Vector {
public:
  virtual Surrogate* primary_type();
};

class Integer_Vector : public Vector {
public:
  virtual Surrogate* primary_type();
};

class Active_List : public List {
public:
  Storage_Slot* active_slot;
  Standard_Object* owner_instance;
public:
  virtual Abstract_Collection* remove(Object* value);
  virtual void insert(Object* value);
  virtual Surrogate* primary_type();
};

class Active_Set : public List {
public:
  Storage_Slot* active_slot;
  Standard_Object* owner_instance;
public:
  virtual Abstract_Collection* remove(Object* value);
  virtual void insert(Object* value);
  virtual Surrogate* primary_type();
};

class Demon : public Standard_Object {
public:
  char* demon_name;
  Keyword* demon_action;
  List* demon_class_refs;
  List* demon_slot_refs;
  cpp_function_code demon_code;
  Method_Slot* demon_method;
  char* demon_documentation;
  boolean demon_guardP;
  boolean demon_allP;
  boolean demon_inheritP;
public:
  virtual Surrogate* primary_type();
};

class Keyword_Key_Value_List : public Key_Value_List {
public:
  virtual Surrogate* primary_type();
};

class Generalized_Symbol : public Context_Sensitive_Object {
public:
  char* symbol_name;
  int symbol_id;
public:
  virtual Global_Variable* lookup_global_variable();
  virtual char* visible_name();
  virtual char* relative_name();
  virtual char* local_print_name();
  virtual Keyword* keywordify();
  virtual boolean deletedP();
};

class Symbol : public Generalized_Symbol {
public:
  int symbol_slot_offset;
  Cons* symbol_value_and_plist;
public:
  virtual void print_object(ostream* stream);
  virtual void idl_output_literal();
  virtual Cons* idl_translate_atomic_tree();
  virtual void java_output_literal();
  virtual Cons* java_translate_atomic_tree();
  virtual void cpp_output_literal();
  virtual Cons* cpp_translate_atomic_tree();
  virtual Context* get_stella_context_slowly();
  virtual Object* cl_translate_atomic_tree();
  virtual Object* walk_atomic_tree(Standard_Object*& _return1);
  virtual Standard_Object* yield_type_specifier();
  virtual Class* get_stella_class(boolean errorP);
  virtual Class* lookup_class();
  virtual Surrogate* surrogatify();
  virtual Symbol* soft_permanentify();
  virtual Object* permanentify();
  virtual Surrogate* primary_type();
};

class Surrogate : public Generalized_Symbol {
// Rigid surrogate.
public:
  Object* surrogate_value;
public:
  virtual void print_object(ostream* stream);
  virtual void idl_output_literal();
  virtual void java_output_literal();
  virtual void cpp_output_literal();
  virtual Cons* cpp_translate_atomic_tree();
  virtual Object* cl_translate_atomic_tree();
  virtual Object* walk_atomic_tree(Standard_Object*& _return1);
  virtual Surrogate* type_to_wrapped_type();
  virtual Global_Variable* lookup_global_variable();
  virtual Standard_Object* yield_type_specifier();
  virtual void destroy_class();
  virtual Class* get_stella_class(boolean errorP);
  virtual char* local_print_name();
  virtual Surrogate* surrogatify();
  virtual Surrogate* primary_type();
};

class Keyword : public Generalized_Symbol {
public:
  virtual void print_object(ostream* stream);
  virtual void idl_output_literal();
  virtual void java_output_literal();
  virtual void cpp_output_literal();
  virtual Cons* cpp_translate_atomic_tree();
  virtual Object* cl_translate_atomic_tree();
  virtual Object* walk_atomic_tree(Standard_Object*& _return1);
  virtual Keyword* keywordify();
  virtual Surrogate* primary_type();
};

class Transient_Symbol : public Symbol {
public:
  virtual Symbol* soft_permanentify();
  virtual Object* permanentify();
  virtual Surrogate* primary_type();
  virtual boolean transientP_reader();
};

class Mappable_Object : public Standard_Object {
// The class MAPPABLE-OBJECT enables the definition of projections
// from a Stella class, slot, global variable, etc. onto a corresponding native
// entity.
public:
  Key_Value_List* dynamic_slots;
public:
  virtual boolean deletedP_setter(boolean value);
  virtual boolean deletedP();
  virtual Boolean_Wrapper* badP_reader();
  virtual List* projected_from_reader();
  virtual List* projects_onto_reader();
};

class Relation : public Mappable_Object {
public:
  boolean abstractP;
public:
  virtual boolean privateP();
  virtual boolean primitiveP();
  virtual char* name();
  virtual Module* home_module();
  virtual int arity();
  virtual boolean publicP();
  virtual List* direct_supers();
  virtual List* all_supers();
  virtual Iterator* slots();
  virtual List* properties_reader();
};

class Class : public Relation {
public:
  boolean cl_structP;
  boolean mixinP;
  Surrogate* class_type;
  List* class_direct_supers;
  List* class_direct_subs;
  Cons* class_all_super_classes;
  Cons* class_all_slots;
  List* class_local_slots;
  Vector* class_slot_and_method_cache;
  boolean class_collectionP;
  boolean class_publicP;
  boolean class_finalizedP;
  boolean class_slots_finalizedP;
  char* class_stringified_source;
  cpp_function_code class_constructor_code;
  cpp_function_code class_slot_accessor_code;
  boolean class_markedP;
  Taxonomy_Node* class_taxonomy_node;
public:
  virtual void print_object(ostream* stream);
  virtual char* java_make_code_output_file_name(boolean donttruncateP);
  virtual void describe_object(Output_Stream* stream, Keyword* mode);
  virtual void pretty_print_object(Output_Stream* stream);
  virtual Iterator* super_classes();
  virtual Iterator* direct_super_classes();
  virtual List* local_slots();
  virtual Iterator* class_slots();
  virtual Iterator* all_class_slots();
  virtual Iterator* public_slots();
  virtual boolean multiple_parentsP();
  virtual void destroy_class();
  virtual void unbind_from_surrogate();
  virtual Surrogate* primary_type();
  virtual char* name();
  virtual Module* home_module();
  virtual int arity();
  virtual boolean publicP();
  virtual boolean activeP();
  virtual Symbol* creator();
  virtual Symbol* initializer();
  virtual Symbol* terminator();
  virtual Symbol* destructor();
  virtual List* required_slots();
  virtual List* parameters();
  virtual Object* initial_value();
  virtual Class_Extension* extension();
  virtual char* cl_native_type();
  virtual char* cpp_native_type();
  virtual char* idl_native_type();
  virtual char* java_native_type();
  virtual List* class_auxiliary_methods_reader();
  virtual List* class_synonyms_reader();
  virtual List* class_key_reader();
  virtual List* class_destructor_demons_reader();
  virtual List* class_guard_destructor_demons_reader();
  virtual List* class_constructor_demons_reader();
  virtual List* class_guard_constructor_demons_reader();
  virtual List* class_required_slot_names_reader();
  virtual List* class_parameters_reader();
  virtual Keyword* class_recycle_method_reader();
  virtual List* class_cl_struct_slots_reader();
  virtual int class_arity_reader();
};

class Polymorphic_Relation : public Relation {
public:
  Slot* slot_direct_equivalent;
public:
  virtual Surrogate* owner();
  virtual Symbol* renames();
  virtual boolean activeP();
};

class Slot : public Polymorphic_Relation {
public:
  Symbol* slot_name;
  Surrogate* slot_owner;
  Surrogate* slot_base_type;
  Surrogate* slot_slotref;
  boolean slot_publicP;
  boolean slot_renamedP;
  boolean slot_externalP;
  boolean slot_markedP;
public:
  virtual void print_object(ostream* stream);
  virtual void pretty_print_object(Output_Stream* stream);
  virtual Object* finish_walking_call_slot_tree(Cons* tree, Standard_Object* firstargtype, Standard_Object*& _return1);
  virtual Standard_Object* compute_return_type_spec(Standard_Object* firstargtype);
  virtual int method_argument_count();
  virtual Object* system_default_value();
  virtual boolean activeP();
  virtual boolean publicP();
  virtual boolean conforming_signaturesP(Slot* superslot);
  virtual void unfinalize_local_slot();
  virtual void help_finalize_local_slot();
  virtual void finalize_slot_type_computations();
  virtual char* name();
  virtual Surrogate* type();
  virtual Standard_Object* type_specifier();
  virtual Surrogate* owner();
  virtual Module* home_module();
  virtual Symbol* renames();
  virtual boolean collection_valuedP();
  virtual Keyword* closure_assumption();
  virtual List* slot_direct_subs_reader();
  virtual List* slot_direct_supers_reader();
};

class Storage_Slot : public Slot {
public:
  boolean slot_requiredP;
  boolean slot_componentP;
  boolean slot_read_onlyP;
  boolean slot_hardwiredP;
  boolean slot_context_sensitiveP;
public:
  virtual void describe_object(Output_Stream* stream, Keyword* mode);
  virtual void pretty_print_object(Output_Stream* stream);
  virtual Object* finish_walking_call_slot_tree(Cons* tree, Standard_Object* firstargtype, Standard_Object*& _return1);
  virtual Standard_Object* compute_return_type_spec(Standard_Object* firstargtype);
  virtual int arity();
  virtual boolean method_body_argumentP();
  virtual boolean method_variable_argumentsP();
  virtual int method_argument_count();
  virtual Object* system_default_value();
  virtual Object* initial_value();
  virtual boolean dynamic_storageP();
  virtual boolean conforming_signaturesP(Slot* superslot);
  virtual void unfinalize_local_slot();
  virtual void help_finalize_local_slot();
  virtual void finalize_slot_type_computations();
  virtual Surrogate* primary_type();
  virtual Object* initially();
  virtual Keyword* allocation();
  virtual Object* default_form();
  virtual boolean requiredP();
  virtual boolean componentP();
  virtual Symbol* reader();
  virtual Symbol* writer();
  virtual List* slot_demons_reader();
  virtual List* slot_guard_demons_reader();
  virtual Keyword* slot_allocation_reader();
};

class Method_Slot : public Slot {
public:
  boolean method_setterP;
  List* method_parameter_names;
  List* method_parameter_type_specifiers;
  List* method_return_type_specifiers;
  char* method_stringified_source;
  cpp_method_code method_code;
  cpp_function_code function_code;
  boolean method_functionP;
public:
  virtual void print_object(ostream* stream);
  virtual String_Wrapper* java_translate_method_name();
  virtual void print_documentation(Output_Stream* stream, boolean shortP);
  virtual void pretty_print_object(Output_Stream* stream);
  virtual boolean method_startup_functionP();
  virtual Object* finish_walking_call_slot_tree(Cons* tree, Standard_Object* firstargtype, Standard_Object*& _return1);
  virtual Standard_Object* compute_return_type_spec(Standard_Object* firstargtype);
  virtual int arity();
  virtual int method_argument_count();
  virtual void unfinalize_local_slot();
  virtual void help_finalize_local_slot();
  virtual void finalize_slot_type_computations();
  virtual boolean conforming_signaturesP(Slot* superslot);
  virtual Surrogate* primary_type();
  virtual boolean setterP();
  virtual boolean method_evaluate_argumentsP_reader();
  virtual List* method_inlined_functions_reader();
  virtual List* method_return_type_specifiers_reader();
  virtual List* method_parameter_directions_reader();
  virtual List* method_parameter_type_specifiers_reader();
  virtual List* method_parameter_names_reader();
};

class Compound_Type_Specifier : public Standard_Object {
public:
  virtual void print_object(ostream* stream);
};

class Parametric_Type_Specifier : public Compound_Type_Specifier {
public:
  Surrogate* specifier_base_type;
  List* specifier_parameter_types;
  List* specifier_dimensions;
public:
  virtual Standard_Object* yield_type_specifier();
  virtual Surrogate* primary_type();
};

class Anchored_Type_Specifier : public Compound_Type_Specifier {
public:
  Symbol* specifier_parameter_name;
public:
  virtual Standard_Object* yield_type_specifier();
  virtual Surrogate* primary_type();
};

class Table : public Slot {
public:
  List* tuple_domains;
public:
  virtual void print_object(ostream* stream);
  virtual int arity();
  virtual int method_argument_count();
  virtual boolean conforming_signaturesP(Slot* superslot);
  virtual Surrogate* primary_type();
};

class Global_Variable : public Mappable_Object {
public:
  char* documentation;
  Symbol* variable_name;
  Surrogate* variable_type;
  boolean variable_specialP;
  boolean variable_constantP;
  boolean variable_publicP;
  boolean variable_auxiliaryP;
  cpp_function_code variable_get_value_code;
  cpp_function_code variable_set_value_code;
  List* variable_value_stack;
  char* variable_stringified_source;
public:
  virtual Surrogate* primary_type();
  virtual char* name();
  virtual Surrogate* type();
  virtual Module* home_module();
};

class Quoted_Expression : public Standard_Object {
public:
  int quotation_table_offset;
public:
  virtual void idl_output_literal();
  virtual void java_output_literal();
  virtual void cpp_output_literal();
  virtual Cons* cpp_translate_atomic_tree();
  virtual Surrogate* primary_type();
};

class Thing : public Standard_Object {
// Defines a class that must be inherited by any
// class that participates in the PowerLoom side of things.
public:
  Surrogate* surrogate_value_inverse;
  Key_Value_List* dynamic_slots;
public:
  virtual void print_object(ostream* stream);
  virtual boolean deletedP_setter(boolean value);
  virtual boolean deletedP();
  virtual Boolean_Wrapper* badP_reader();
};

class Context : public Thing {
public:
  List* child_contexts;
  Module* base_module;
  Cons* all_super_contexts;
  int context_number;
public:
  virtual void print_object(ostream* stream);
  virtual boolean deletedP();
  virtual void destroy_context();
  virtual Context* change_context();
  virtual char* context_name();
  virtual Iterator* parent_contexts();
  virtual Module* home_module();
};

class Module : public Context {
public:
  List* parent_modules;
  char* documentation;
  List* nicknames;
  List* uses;
  List* used_by;
  List* requires;
  boolean case_sensitiveP;
  char* module_full_name;
  char* module_name;
  char* module_stringified_source;
  char* stringified_options;
  Module* cardinal_module;
  String_To_Integer_Hash_Table* symbol_offset_table;
  String_To_Integer_Hash_Table* surrogate_offset_table;
  World* strict_inference_cache;
  World* default_inference_cache;
  World* prototype_inference_cache;
public:
  virtual void destroy_context();
  virtual void unfinalize_module();
  virtual boolean multiple_parentsP();
  virtual Module* change_module();
  virtual Iterator* parent_contexts();
  virtual void describe_object(Output_Stream* stream, Keyword* mode);
  virtual void print_definition(Output_Stream* stream);
  virtual Surrogate* primary_type();
  virtual char* name();
  virtual Module* parent_module();
  virtual char* lisp_package();
  virtual char* cpp_package();
  virtual boolean clearableP_reader();
  virtual char* java_package_reader();
  virtual char* module_cpp_package_reader();
  virtual char* module_lisp_package_reader();
  virtual List* shadowed_surrogates_reader();
};

class World : public Context {
public:
  Context* parent_context;
public:
  virtual void destroy_context();
  virtual void unfinalize_world();
  virtual Iterator* parent_contexts();
  virtual Surrogate* primary_type();
};

class Cs_Value : public Key_Value_List {
// Contextualized value.  Contains a sorted kv-cons list indexed
// by context. The kv-cons list is never null.  Newer (higher numbered) contexts
// appear first.
public:
  virtual void insert_at(Context* context, Object* newvalue);
  virtual Surrogate* primary_type();
};

class Interval : public Standard_Object {
public:
  int lower_bound;
  int upper_bound;
public:
  virtual void print_object(ostream* stream);
  virtual boolean memberP(int n);
  virtual Surrogate* primary_type();
};

class Taxonomy_Node : public Standard_Object {
public:
  Object* native_object;
  int label;
  Cons* intervals;
  Interval* initial_interval;
  int first_interval_lower_bound;
  int first_interval_upper_bound;
  Cons* parents;
  Cons* children;
  int total_ancestors;
  Cons* tree_children;
  Object* marker;
public:
  virtual void print_object(ostream* stream);
  virtual boolean deletedP_setter(boolean value);
  virtual boolean deletedP();
  virtual Surrogate* primary_type();
};

class Taxonomy_Graph : public Standard_Object {
public:
  boolean renumber_if_out_of_numbersP;
  double renumber_ratio;
  boolean incremental_modeP;
  int largest_postorder_number;
  int number_of_nodes;
  int number_of_foreign_interval_nodes;
  List* roots;
  List* broken_links;
  List* added_links;
  List* removed_links;
public:
  virtual void print_object(ostream* stream);
  virtual Surrogate* primary_type();
};

class Stella_Exception : public gc, public runtime_error {
public:
  Stella_Exception(const string& msg) : runtime_error(msg) {
}

};

class Read_Exception : public Stella_Exception {
public:
  Read_Exception(const string& msg) : Stella_Exception(msg) {
}

};

class Unhandled_Exception : public Stella_Exception {
public:
  Unhandled_Exception(const string& msg) : Stella_Exception(msg) {
}

};

class Evaluation_Exception : public Stella_Exception {
public:
  Evaluation_Exception(const string& msg) : Stella_Exception(msg) {
}

};

class No_Such_Object_Exception : public Stella_Exception {
public:
  No_Such_Object_Exception(const string& msg) : Stella_Exception(msg) {
}

};

class No_Such_Context_Exception : public No_Such_Object_Exception {
public:
  No_Such_Context_Exception(const string& msg) : No_Such_Object_Exception(msg) {
}

};

class Wrapper : public Object {
public:
  virtual Object* walk_atomic_tree(Standard_Object*& _return1);
  virtual boolean terminate_wrapperP();
  virtual int hash_code();
  virtual boolean object_equalP(Object* y);
};

class Literal_Wrapper : public Wrapper {
public:
  virtual Object* permanentify();
  virtual Object* copy_wrapped_literal();
};

class Number_Wrapper : public Literal_Wrapper {
public:
  virtual double number_wrapper_to_float();
};

class Integer_Wrapper : public Number_Wrapper {
public:
  int wrapper_value;
public:
  virtual void print_object(ostream* stream);
  virtual void java_output_literal();
  virtual void cpp_output_literal();
  virtual double number_wrapper_to_float();
  virtual boolean object_eqlP(Object* y);
  virtual Object* copy_wrapped_literal();
  virtual boolean terminate_wrapperP();
  virtual int hash_code();
  virtual Surrogate* primary_type();
};

class Float_Wrapper : public Number_Wrapper {
public:
  double wrapper_value;
public:
  virtual void print_object(ostream* stream);
  virtual void java_output_literal();
  virtual void cpp_output_literal();
  virtual Object* cl_translate_atomic_tree();
  virtual double number_wrapper_to_float();
  virtual boolean object_eqlP(Object* y);
  virtual Object* copy_wrapped_literal();
  virtual int hash_code();
  virtual Surrogate* primary_type();
};

class String_Wrapper : public Literal_Wrapper {
public:
  char* wrapper_value;
public:
  virtual void print_object(ostream* stream);
  virtual void idl_output_literal();
  virtual void java_output_literal();
  virtual void cpp_output_literal();
  virtual Keyword* keywordify();
  virtual boolean object_eqlP(Object* y);
  virtual Object* copy_wrapped_literal();
  virtual int hash_code();
  virtual Surrogate* primary_type();
};

class Mutable_String_Wrapper : public Literal_Wrapper {
public:
  char* wrapper_value;
public:
  virtual void print_object(ostream* stream);
  virtual void idl_output_literal();
  virtual void java_output_literal();
  virtual void cpp_output_literal();
  virtual boolean object_eqlP(Object* y);
  virtual Object* copy_wrapped_literal();
  virtual Surrogate* primary_type();
};

class Character_Wrapper : public Literal_Wrapper {
public:
  char wrapper_value;
public:
  virtual void print_object(ostream* stream);
  virtual void java_output_literal();
  virtual void cpp_output_literal();
  virtual boolean object_eqlP(Object* y);
  virtual Object* copy_wrapped_literal();
  virtual int hash_code();
  virtual Surrogate* primary_type();
};

class Boolean_Wrapper : public Literal_Wrapper {
// Objectified version of the BOOLEAN data type.  The NULL
// value can be used for a 3-valued semantics (see THREE-VALUED-BOOLEAN).
public:
  boolean wrapper_value;
public:
  virtual void print_object(ostream* stream);
  virtual void java_output_literal();
  virtual void cpp_output_literal();
  virtual boolean object_eqlP(Object* y);
  virtual int hash_code();
  virtual Surrogate* primary_type();
};

class Code_Wrapper : public Wrapper {
};

class Function_Code_Wrapper : public Code_Wrapper {
public:
  cpp_function_code wrapper_value;
public:
  virtual Object* copy_wrapped_literal();
  virtual Surrogate* primary_type();
};

class Method_Code_Wrapper : public Code_Wrapper {
public:
  cpp_method_code wrapper_value;
public:
  virtual Object* copy_wrapped_literal();
  virtual Surrogate* primary_type();
};

class Verbatim_String_Wrapper : public String_Wrapper {
// Wrapper class used to hold verbatim native code strings.
public:
  virtual void print_object(ostream* stream);
  virtual Surrogate* primary_type();
};

class Lisp_Code : public gc {
// Used to indicate variables that input or output Common Lisp~
//       structures.
};

class System_Definition : public Standard_Object {
// The System-Definition class is used to define systems of files
// that constitute Stella applications.
public:
  char* name;
  char* directory;
  Cons* files;
  Cons* lisp_only_files;
  Cons* cpp_only_files;
  Cons* java_only_files;
  Cons* preprocessed_files;
  Cons* required_systems;
  boolean loadedP;
  boolean up_to_dateP;
  char* cardinal_module;
  char* source_root_directory;
  char* native_root_directory;
  char* binary_root_directory;
  char* banner;
  Cons* production_settings;
  Cons* development_settings;
  Symbol* finalization_function;
public:
  virtual void print_object(ostream* stream);
  virtual char* system_startup_function_name();
  virtual Surrogate* primary_type();
};

class Stream : public Standard_Object {
public:
  Keyword* state;
};

class Output_Stream : public Stream {
public:
  ostream* native_stream;
public:
  virtual void free();
  virtual Surrogate* primary_type();
};

class Input_Stream : public Stream {
public:
  istream* native_stream;
  Output_Stream* echo_stream;
  Tokenizer_Stream_State* tokenizer_state;
  Keyword* buffering_scheme;
public:
  virtual void free();
  virtual Surrogate* primary_type();
};

class Output_File_Stream : public Output_Stream {
public:
  char* filename;
  Keyword* if_exists_action;
public:
  virtual void print_object(ostream* stream);
  virtual void free();
  virtual Surrogate* primary_type();
};

class Input_File_Stream : public Input_Stream {
public:
  char* filename;
  Keyword* if_not_exists_action;
public:
  virtual void print_object(ostream* stream);
  virtual void free();
  virtual Surrogate* primary_type();
};

class Output_String_Stream : public Output_Stream {
public:
  char* the_string;
public:
  virtual void free();
  virtual char* the_string_reader();
  virtual Surrogate* primary_type();
};

class Input_String_Stream : public Input_Stream {
public:
  char* the_string;
public:
  virtual void free();
  virtual Surrogate* primary_type();
};



// Function signatures:
ostream& operator<< (ostream& stream, Object* self);
Object* access_reference_count_object_slot_value(Reference_Count_Object* self, Symbol* slotname, Object* value, boolean setvalueP);
Object* access_context_sensitive_object_slot_value(Context_Sensitive_Object* self, Symbol* slotname, Object* value, boolean setvalueP);
Object* access_active_object_slot_value(Active_Object* self, Symbol* slotname, Object* value, boolean setvalueP);
Object* access_abstract_iterator_slot_value(Abstract_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP);
Cons_Iterator* new_cons_iterator();
Object* access_cons_iterator_slot_value(Cons_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP);
List_Iterator* new_list_iterator();
Object* access_list_iterator_slot_value(List_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP);
Destructive_List_Iterator* new_destructive_list_iterator();
Object* access_destructive_list_iterator_slot_value(Destructive_List_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP);
Set_Iterator* new_set_iterator();
Object* access_set_iterator_slot_value(Set_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP);
All_Purpose_Iterator* new_all_purpose_iterator();
Object* access_all_purpose_iterator_slot_value(All_Purpose_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP);
Types_To_Classes_Iterator* new_types_to_classes_iterator(Cons* iterator_cursor);
Object* access_types_to_classes_iterator_slot_value(Types_To_Classes_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP);
Property_List_Iterator* new_property_list_iterator();
Object* access_property_list_iterator_slot_value(Property_List_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP);
Kv_List_Iterator* new_kv_list_iterator();
Object* access_kv_list_iterator_slot_value(Kv_List_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP);
Integer_Interval_Iterator* new_integer_interval_iterator(int lower_bound, int upper_bound);
Object* access_integer_interval_iterator_slot_value(Integer_Interval_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP);
Reverse_Integer_Interval_Iterator* new_reverse_integer_interval_iterator(int lower_bound, int upper_bound);
Object* access_reverse_integer_interval_iterator_slot_value(Reverse_Integer_Interval_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP);
String_Iterator* new_string_iterator();
Object* access_string_iterator_slot_value(String_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP);
List* new_list();
Object* access_list_slot_value(List* self, Symbol* slotname, Object* value, boolean setvalueP);
Set* new_set();
Object* access_set_slot_value(Set* self, Symbol* slotname, Object* value, boolean setvalueP);
Class_Extension* new_class_extension();
Cons* new_cons();
Object* access_cons_slot_value(Cons* self, Symbol* slotname, Object* value, boolean setvalueP);
Property_List* new_property_list();
Object* access_property_list_slot_value(Property_List* self, Symbol* slotname, Object* value, boolean setvalueP);
Kv_Cons* new_kv_cons();
Object* access_kv_cons_slot_value(Kv_Cons* self, Symbol* slotname, Object* value, boolean setvalueP);
Key_Value_List* new_key_value_list();
Object* access_key_value_list_slot_value(Key_Value_List* self, Symbol* slotname, Object* value, boolean setvalueP);
Recycle_List* new_recycle_list();
Object* access_recycle_list_slot_value(Recycle_List* self, Symbol* slotname, Object* value, boolean setvalueP);
Object* access_abstract_hash_table_slot_value(Abstract_Hash_Table* self, Symbol* slotname, Object* value, boolean setvalueP);
Hash_Table* new_hash_table();
Integer_Hash_Table* new_integer_hash_table();
Float_Hash_Table* new_float_hash_table();
String_Hash_Table* new_string_hash_table();
String_To_Integer_Hash_Table* new_string_to_integer_hash_table();
Vector* new_vector(int array_size);
Object* access_vector_slot_value(Vector* self, Symbol* slotname, Object* value, boolean setvalueP);
Extensible_Vector* new_extensible_vector(int array_size);
Extensible_Symbol_Array* new_extensible_symbol_array(int array_size);
Object* access_extensible_symbol_array_slot_value(Extensible_Symbol_Array* self, Symbol* slotname, Object* value, boolean setvalueP);
Object* access_vector_sequence_slot_value(Vector_Sequence* self, Symbol* slotname, Object* value, boolean setvalueP);
Short_Vector_Sequence* new_short_vector_sequence(int array_size);
Object* access_short_vector_sequence_slot_value(Short_Vector_Sequence* self, Symbol* slotname, Object* value, boolean setvalueP);
Boolean_Vector* new_boolean_vector(int array_size);
Integer_Vector* new_integer_vector(int array_size);
Active_List* new_active_list();
Active_Set* new_active_set();
Demon* new_demon();
Object* access_demon_slot_value(Demon* self, Symbol* slotname, Object* value, boolean setvalueP);
Keyword_Key_Value_List* new_keyword_key_value_list();
Object* access_generalized_symbol_slot_value(Generalized_Symbol* self, Symbol* slotname, Object* value, boolean setvalueP);
Symbol* new_symbol(char* symbol_name);
Object* access_symbol_slot_value(Symbol* self, Symbol* slotname, Object* value, boolean setvalueP);
Surrogate* new_surrogate(char* symbol_name);
Object* access_surrogate_slot_value(Surrogate* self, Symbol* slotname, Object* value, boolean setvalueP);
Keyword* new_keyword(char* symbol_name);
Object* access_keyword_slot_value(Keyword* self, Symbol* slotname, Object* value, boolean setvalueP);
Transient_Symbol* new_transient_symbol(char* symbol_name);
Object* access_mappable_object_slot_value(Mappable_Object* self, Symbol* slotname, Object* value, boolean setvalueP);
Object* access_relation_slot_value(Relation* self, Symbol* slotname, Object* value, boolean setvalueP);
Class* new_class();
Object* access_class_slot_value(Class* self, Symbol* slotname, Object* value, boolean setvalueP);
Object* access_polymorphic_relation_slot_value(Polymorphic_Relation* self, Symbol* slotname, Object* value, boolean setvalueP);
Object* access_slot_slot_value(Slot* self, Symbol* slotname, Object* value, boolean setvalueP);
Storage_Slot* new_storage_slot();
Object* access_storage_slot_slot_value(Storage_Slot* self, Symbol* slotname, Object* value, boolean setvalueP);
Method_Slot* new_method_slot();
Object* access_method_slot_slot_value(Method_Slot* self, Symbol* slotname, Object* value, boolean setvalueP);
Parametric_Type_Specifier* new_parametric_type_specifier();
Object* access_parametric_type_specifier_slot_value(Parametric_Type_Specifier* self, Symbol* slotname, Object* value, boolean setvalueP);
Anchored_Type_Specifier* new_anchored_type_specifier();
Object* access_anchored_type_specifier_slot_value(Anchored_Type_Specifier* self, Symbol* slotname, Object* value, boolean setvalueP);
Table* new_table();
Object* access_table_slot_value(Table* self, Symbol* slotname, Object* value, boolean setvalueP);
Global_Variable* new_global_variable();
Object* access_global_variable_slot_value(Global_Variable* self, Symbol* slotname, Object* value, boolean setvalueP);
Quoted_Expression* new_quoted_expression();
Object* access_quoted_expression_slot_value(Quoted_Expression* self, Symbol* slotname, Object* value, boolean setvalueP);
Object* access_thing_slot_value(Thing* self, Symbol* slotname, Object* value, boolean setvalueP);
Object* access_context_slot_value(Context* self, Symbol* slotname, Object* value, boolean setvalueP);
Module* new_module();
Object* access_module_slot_value(Module* self, Symbol* slotname, Object* value, boolean setvalueP);
World* new_world();
Object* access_world_slot_value(World* self, Symbol* slotname, Object* value, boolean setvalueP);
Cs_Value* new_cs_value();
Interval* new_interval();
Object* access_interval_slot_value(Interval* self, Symbol* slotname, Object* value, boolean setvalueP);
Taxonomy_Node* new_taxonomy_node();
Object* access_taxonomy_node_slot_value(Taxonomy_Node* self, Symbol* slotname, Object* value, boolean setvalueP);
Taxonomy_Graph* new_taxonomy_graph();
Object* access_taxonomy_graph_slot_value(Taxonomy_Graph* self, Symbol* slotname, Object* value, boolean setvalueP);
Stella_Exception* new_stella_exception(char* message);
Read_Exception* new_read_exception(char* message);
Unhandled_Exception* new_unhandled_exception(char* message);
Evaluation_Exception* new_evaluation_exception(char* message);
No_Such_Object_Exception* new_no_such_object_exception(char* message);
No_Such_Context_Exception* new_no_such_context_exception(char* message);
Integer_Wrapper* new_integer_wrapper(int wrapper_value);
Object* access_integer_wrapper_slot_value(Integer_Wrapper* self, Symbol* slotname, Object* value, boolean setvalueP);
Float_Wrapper* new_float_wrapper(double wrapper_value);
Object* access_float_wrapper_slot_value(Float_Wrapper* self, Symbol* slotname, Object* value, boolean setvalueP);
String_Wrapper* new_string_wrapper(char* wrapper_value);
Object* access_string_wrapper_slot_value(String_Wrapper* self, Symbol* slotname, Object* value, boolean setvalueP);
Mutable_String_Wrapper* new_mutable_string_wrapper(char* wrapper_value);
Character_Wrapper* new_character_wrapper(char wrapper_value);
Object* access_character_wrapper_slot_value(Character_Wrapper* self, Symbol* slotname, Object* value, boolean setvalueP);
Boolean_Wrapper* new_boolean_wrapper(boolean wrapper_value);
Object* access_boolean_wrapper_slot_value(Boolean_Wrapper* self, Symbol* slotname, Object* value, boolean setvalueP);
Function_Code_Wrapper* new_function_code_wrapper(cpp_function_code wrapper_value);
Object* access_function_code_wrapper_slot_value(Function_Code_Wrapper* self, Symbol* slotname, Object* value, boolean setvalueP);
Method_Code_Wrapper* new_method_code_wrapper(cpp_method_code wrapper_value);
Object* access_method_code_wrapper_slot_value(Method_Code_Wrapper* self, Symbol* slotname, Object* value, boolean setvalueP);
Verbatim_String_Wrapper* new_verbatim_string_wrapper(char* wrapper_value);
System_Definition* new_system_definition();
Object* access_system_definition_slot_value(System_Definition* self, Symbol* slotname, Object* value, boolean setvalueP);
Object* access_stream_slot_value(Stream* self, Symbol* slotname, Object* value, boolean setvalueP);
Output_Stream* new_output_stream();
Input_Stream* new_input_stream();
Object* access_input_stream_slot_value(Input_Stream* self, Symbol* slotname, Object* value, boolean setvalueP);
Output_File_Stream* new_output_file_stream(char* filename);
Object* access_output_file_stream_slot_value(Output_File_Stream* self, Symbol* slotname, Object* value, boolean setvalueP);
Input_File_Stream* new_input_file_stream(char* filename);
Object* access_input_file_stream_slot_value(Input_File_Stream* self, Symbol* slotname, Object* value, boolean setvalueP);
Output_String_Stream* new_output_string_stream();
Object* access_output_string_stream_slot_value(Output_String_Stream* self, Symbol* slotname, Object* value, boolean setvalueP);
Input_String_Stream* new_input_string_stream(char* the_string);
Object* access_input_string_stream_slot_value(Input_String_Stream* self, Symbol* slotname, Object* value, boolean setvalueP);
void help_startup_hierarchy1();
void help_startup_hierarchy2();
void help_startup_hierarchy3();
void help_startup_hierarchy4();
void help_startup_hierarchy5();
void help_startup_hierarchy6();
void help_startup_hierarchy7();
void help_startup_hierarchy8();
void startup_hierarchy();

// Auxiliary global declarations:
extern Symbol* SYM_HIERARCHY_STELLA_TYPE_SPEC;
extern boolean oHARDWIRED_TRANSIENTp_ON_TRANSIENT_MIXINo;
extern Symbol* SYM_HIERARCHY_STELLA_REFERENCE_COUNT;
extern Symbol* SYM_HIERARCHY_STELLA_HOME_CONTEXT;
extern Symbol* SYM_HIERARCHY_STELLA_BADp;
extern Symbol* SYM_HIERARCHY_STELLA_DELETED_OBJECTp;
extern Symbol* SYM_HIERARCHY_STELLA_FIRST_ITERATIONp;
extern Symbol* SYM_HIERARCHY_STELLA_OBJECT_ITERATOR;
extern Surrogate* SGT_HIERARCHY_STELLA_CONS_ITERATOR;
extern Symbol* SYM_HIERARCHY_STELLA_CONS_ITERATOR_CURSOR;
extern Surrogate* SGT_HIERARCHY_STELLA_LIST_ITERATOR;
extern Symbol* SYM_HIERARCHY_STELLA_LIST_ITERATOR_CURSOR;
extern Symbol* SYM_HIERARCHY_STELLA_LIST_ITERATOR_COLLECTION;
extern Surrogate* SGT_HIERARCHY_STELLA_DESTRUCTIVE_LIST_ITERATOR;
extern Symbol* SYM_HIERARCHY_STELLA_THE_CONS_LIST;
extern Surrogate* SGT_HIERARCHY_STELLA_SET_ITERATOR;
extern Symbol* SYM_HIERARCHY_STELLA_SET_ITERATOR_CURSOR;
extern Symbol* SYM_HIERARCHY_STELLA_SET_ITERATOR_COLLECTION;
extern Surrogate* SGT_HIERARCHY_STELLA_ALL_PURPOSE_ITERATOR;
extern Symbol* SYM_HIERARCHY_STELLA_KEY;
extern Symbol* SYM_HIERARCHY_STELLA_ITERATOR_NEXT_CODE;
extern Symbol* SYM_HIERARCHY_STELLA_ITERATOR_FILTER_CODE;
extern Symbol* SYM_HIERARCHY_STELLA_ITERATOR_EMPTY_CODE;
extern Symbol* SYM_HIERARCHY_STELLA_ITERATOR_NESTED_ITERATOR;
extern Symbol* SYM_HIERARCHY_STELLA_ITERATOR_CONS_LIST;
extern Symbol* SYM_HIERARCHY_STELLA_ITERATOR_OBJECT;
extern Symbol* SYM_HIERARCHY_STELLA_ITERATOR_SECOND_OBJECT;
extern Symbol* SYM_HIERARCHY_STELLA_ITERATOR_INTEGER;
extern Symbol* SYM_HIERARCHY_STELLA_ITERATOR_SECOND_INTEGER;
extern Surrogate* SGT_HIERARCHY_STELLA_TYPES_TO_CLASSES_ITERATOR;
extern Symbol* SYM_HIERARCHY_STELLA_ITERATOR_CURSOR;
extern Symbol* SYM_HIERARCHY_STELLA_OBJECT_DICTIONARY_ITERATOR;
extern Surrogate* SGT_HIERARCHY_STELLA_PROPERTY_LIST_ITERATOR;
extern Symbol* SYM_HIERARCHY_STELLA_PLIST_ITERATOR_CURSOR;
extern Symbol* SYM_HIERARCHY_STELLA_PLIST_ITERATOR_COLLECTION;
extern Surrogate* SGT_HIERARCHY_STELLA_KV_LIST_ITERATOR;
extern Symbol* SYM_HIERARCHY_STELLA_THE_KV_LIST;
extern Symbol* SYM_HIERARCHY_STELLA_KV_LIST_ITERATOR_CURSOR;
extern Surrogate* SGT_HIERARCHY_STELLA_INTEGER_INTERVAL_ITERATOR;
extern Symbol* SYM_HIERARCHY_STELLA_INTERVAL_CURSOR;
extern Symbol* SYM_HIERARCHY_STELLA_LOWER_BOUND;
extern Symbol* SYM_HIERARCHY_STELLA_UPPER_BOUND;
extern Surrogate* SGT_HIERARCHY_STELLA_REVERSE_INTEGER_INTERVAL_ITERATOR;
extern Surrogate* SGT_HIERARCHY_STELLA_STRING_ITERATOR;
extern Symbol* SYM_HIERARCHY_STELLA_THE_STRING;
extern Symbol* SYM_HIERARCHY_STELLA_CURSOR;
extern Symbol* SYM_HIERARCHY_STELLA_END;
extern Symbol* SYM_HIERARCHY_STELLA_OBJECT_COLLECTION;
extern Symbol* SYM_HIERARCHY_STELLA_OBJECT_SEQUENCE;
extern Surrogate* SGT_HIERARCHY_STELLA_LIST;
extern Surrogate* SGT_HIERARCHY_STELLA_SET;
extern Surrogate* SGT_HIERARCHY_STELLA_CLASS_EXTENSION;
extern Surrogate* SGT_HIERARCHY_STELLA_CONS;
extern Symbol* SYM_HIERARCHY_STELLA_VALUE;
extern Symbol* SYM_HIERARCHY_STELLA_REST;
extern Symbol* SYM_HIERARCHY_STELLA_OBJECT_TO_OBJECT_DICTIONARY;
extern Surrogate* SGT_HIERARCHY_STELLA_PROPERTY_LIST;
extern Symbol* SYM_HIERARCHY_STELLA_THE_PLIST;
extern Surrogate* SGT_HIERARCHY_STELLA_KV_CONS;
extern Surrogate* SGT_HIERARCHY_STELLA_KEY_VALUE_LIST;
extern Surrogate* SGT_HIERARCHY_STELLA_RECYCLE_LIST;
extern Symbol* SYM_HIERARCHY_STELLA_RECYCLE_LIST_OF;
extern Symbol* SYM_HIERARCHY_STELLA_RECYCLED_ITEMS;
extern Symbol* SYM_HIERARCHY_STELLA_LIST_OF_RECYCLED_ITEMS;
extern Symbol* SYM_HIERARCHY_STELLA_ALL_ITEMS;
extern Symbol* SYM_HIERARCHY_STELLA_UNUSED_ITEMS;
extern Symbol* SYM_HIERARCHY_STELLA_CURRENT_LENGTH;
extern Symbol* SYM_HIERARCHY_STELLA_ITEM_SIZE;
extern Symbol* SYM_HIERARCHY_STELLA_THE_STELLA_HASH_TABLE;
extern Surrogate* SGT_HIERARCHY_STELLA_HASH_TABLE;
extern Symbol* SYM_HIERARCHY_STELLA_OBJECT_TO_OBJECT_HASH_TABLE;
extern Surrogate* SGT_HIERARCHY_STELLA_INTEGER_HASH_TABLE;
extern Surrogate* SGT_HIERARCHY_STELLA_FLOAT_HASH_TABLE;
extern Surrogate* SGT_HIERARCHY_STELLA_STRING_HASH_TABLE;
extern Surrogate* SGT_HIERARCHY_STELLA_STRING_TO_INTEGER_HASH_TABLE;
extern Surrogate* SGT_HIERARCHY_STELLA_VECTOR;
extern Symbol* SYM_HIERARCHY_STELLA_ARRAY_SIZE;
extern Surrogate* SGT_HIERARCHY_STELLA_EXTENSIBLE_VECTOR;
extern Surrogate* SGT_HIERARCHY_STELLA_EXTENSIBLE_SYMBOL_ARRAY;
extern Symbol* SYM_HIERARCHY_STELLA_TOP_SYMBOL_OFFSET;
extern Symbol* SYM_HIERARCHY_STELLA_POTENTIAL_FREE_SYMBOL_OFFSET;
extern int oHARDWIRED_RESIZE_INCREMENT_ON_VECTOR_SEQUENCEo;
extern Symbol* SYM_HIERARCHY_STELLA_RESIZE_INCREMENT;
extern Symbol* SYM_HIERARCHY_STELLA_SEQUENCE_LENGTH;
extern Symbol* SYM_HIERARCHY_STELLA_VSEQ;
extern Surrogate* SGT_HIERARCHY_STELLA_SHORT_VECTOR_SEQUENCE;
extern int oHARDWIRED_RESIZE_INCREMENT_ON_SHORT_VECTOR_SEQUENCEo;
extern Symbol* SYM_HIERARCHY_STELLA_SVSEQ;
extern Surrogate* SGT_HIERARCHY_STELLA_BOOLEAN_VECTOR;
extern Surrogate* SGT_HIERARCHY_STELLA_INTEGER_VECTOR;
extern Surrogate* SGT_HIERARCHY_STELLA_ACTIVE_LIST;
extern Surrogate* SGT_HIERARCHY_STELLA_ACTIVE_SET;
extern Surrogate* SGT_HIERARCHY_STELLA_DEMON;
extern Symbol* SYM_HIERARCHY_STELLA_DEMON_NAME;
extern Symbol* SYM_HIERARCHY_STELLA_DEMON_ACTION;
extern Symbol* SYM_HIERARCHY_STELLA_DEMON_CLASS_REFS;
extern Symbol* SYM_HIERARCHY_STELLA_DEMON_SLOT_REFS;
extern Symbol* SYM_HIERARCHY_STELLA_DEMON_CODE;
extern Symbol* SYM_HIERARCHY_STELLA_DEMON_METHOD;
extern Symbol* SYM_HIERARCHY_STELLA_DEMON_DOCUMENTATION;
extern Symbol* SYM_HIERARCHY_STELLA_DEMON_GUARDp;
extern Symbol* SYM_HIERARCHY_STELLA_DEMON_ALLp;
extern Symbol* SYM_HIERARCHY_STELLA_DEMON_INHERITp;
extern Surrogate* SGT_HIERARCHY_STELLA_KEYWORD_KEY_VALUE_LIST;
extern Symbol* SYM_HIERARCHY_STELLA_SYMBOL_NAME;
extern Symbol* SYM_HIERARCHY_STELLA_SYMBOL_ID;
extern Symbol* SYM_HIERARCHY_STELLA_INTERNED_IN;
extern Surrogate* SGT_HIERARCHY_STELLA_SYMBOL;
extern Symbol* SYM_HIERARCHY_STELLA_SYMBOL_SLOT_OFFSET;
extern Symbol* SYM_HIERARCHY_STELLA_SYMBOL_VALUE_AND_PLIST;
extern Surrogate* SGT_HIERARCHY_STELLA_SURROGATE;
extern Symbol* SYM_HIERARCHY_STELLA_SURROGATE_VALUE;
extern Symbol* SYM_HIERARCHY_STELLA_SURROGATE_NAME;
extern Symbol* SYM_HIERARCHY_STELLA_TYPE_CLASS;
extern Symbol* SYM_HIERARCHY_STELLA_TYPE_NAME;
extern Symbol* SYM_HIERARCHY_STELLA_SLOTREF_SLOT;
extern Symbol* SYM_HIERARCHY_STELLA_TYPE;
extern Symbol* SYM_HIERARCHY_STELLA_SLOTREF;
extern Surrogate* SGT_HIERARCHY_STELLA_KEYWORD;
extern Symbol* SYM_HIERARCHY_STELLA_KEYWORD_NAME;
extern Surrogate* SGT_HIERARCHY_STELLA_TRANSIENT_SYMBOL;
extern Symbol* SYM_HIERARCHY_STELLA_PROJECTS_ONTO;
extern Symbol* SYM_HIERARCHY_STELLA_PROJECTED_FROM;
extern Symbol* SYM_HIERARCHY_STELLA_PROPERTIES;
extern Symbol* SYM_HIERARCHY_STELLA_ABSTRACTp;
extern Surrogate* SGT_HIERARCHY_STELLA_CLASS;
extern Symbol* SYM_HIERARCHY_STELLA_STORED_ACTIVEp;
extern Surrogate* SGT_HIERARCHY_STELLA_ACTIVE_OBJECT;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_CREATOR;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_INITIALIZER;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_TERMINATOR;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_DESTRUCTOR;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_PARAMETERS;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_INITIAL_VALUE;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_EXTENSION;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_CL_NATIVE_TYPE;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_CPP_NATIVE_TYPE;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_JAVA_NATIVE_TYPE;
extern int oHARDWIRED_CLASS_ARITY_ON_CLASSo;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_CL_STRUCT_SLOTS;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_RECYCLE_METHOD;
extern Keyword* KWD_HIERARCHY_NONE;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_REQUIRED_SLOT_NAMES;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_GUARD_CONSTRUCTOR_DEMONS;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_CONSTRUCTOR_DEMONS;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_GUARD_DESTRUCTOR_DEMONS;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_DESTRUCTOR_DEMONS;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_KEY;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_SYNONYMS;
extern Symbol* SYM_HIERARCHY_STELLA_CL_STRUCTp;
extern Symbol* SYM_HIERARCHY_STELLA_MIXINp;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_TYPE;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_ARITY;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_DIRECT_SUPERS;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_DIRECT_SUBS;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_ALL_SUPER_CLASSES;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_ALL_SLOTS;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_LOCAL_SLOTS;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_SLOT_AND_METHOD_CACHE;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_ABSTRACTp;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_MIXINp;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_COLLECTIONp;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_CL_STRUCTp;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_PUBLICp;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_FINALIZEDp;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_SLOTS_FINALIZEDp;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_STRINGIFIED_SOURCE;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_CONSTRUCTOR_CODE;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_SLOT_ACCESSOR_CODE;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_MARKEDp;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_TAXONOMY_NODE;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_DOCUMENTATION;
extern Symbol* SYM_HIERARCHY_STELLA_CLASS_PRINT_FORM;
extern Symbol* SYM_HIERARCHY_STELLA_DOCUMENTATION;
extern Symbol* SYM_HIERARCHY_STELLA_PRINT_FORM;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_DIRECT_EQUIVALENT;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_TYPE_SPECIFIER;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_HOME_MODULE;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_RENAMES;
extern Surrogate* SGT_HIERARCHY_STELLA_COLLECTION;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_CLOSURE_ASSUMPTION;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_DIRECT_SUPERS;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_DIRECT_SUBS;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_NAME;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_OWNER;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_BASE_TYPE;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_SLOTREF;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_PUBLICp;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_RENAMEDp;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_EXTERNALp;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_MARKEDp;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_DOCUMENTATION;
extern Surrogate* SGT_HIERARCHY_STELLA_STORAGE_SLOT;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_ALLOCATION;
extern Keyword* KWD_HIERARCHY_INSTANCE;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_DEFAULT_EXPRESSION;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_READER;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_WRITER;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_GUARD_DEMONS;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_DEMONS;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_REQUIREDp;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_COMPONENTp;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_READ_ONLYp;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_HARDWIREDp;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_CONTEXT_SENSITIVEp;
extern Surrogate* SGT_HIERARCHY_STELLA_METHOD_SLOT;
extern Symbol* SYM_HIERARCHY_STELLA_METHOD_PARAMETER_DIRECTIONS;
extern Symbol* SYM_HIERARCHY_STELLA_METHOD_INLINED_FUNCTIONS;
extern Symbol* SYM_HIERARCHY_STELLA_METHOD_EVALUATE_ARGUMENTSp;
extern Symbol* SYM_HIERARCHY_STELLA_METHOD_SETTERp;
extern Symbol* SYM_HIERARCHY_STELLA_METHOD_PARAMETER_NAMES;
extern Symbol* SYM_HIERARCHY_STELLA_METHOD_PARAMETER_TYPE_SPECIFIERS;
extern Symbol* SYM_HIERARCHY_STELLA_METHOD_RETURN_TYPE_SPECIFIERS;
extern Symbol* SYM_HIERARCHY_STELLA_METHOD_STRINGIFIED_SOURCE;
extern Symbol* SYM_HIERARCHY_STELLA_METHOD_CODE;
extern Symbol* SYM_HIERARCHY_STELLA_FUNCTION_CODE;
extern Symbol* SYM_HIERARCHY_STELLA_METHOD_FUNCTIONp;
extern Symbol* SYM_HIERARCHY_STELLA_METHOD_DOCUMENTATION;
extern Symbol* SYM_HIERARCHY_STELLA_METHOD_AUXILIARYp;
extern Symbol* SYM_HIERARCHY_STELLA_METHOD_STORAGE_SLOT;
extern Symbol* SYM_HIERARCHY_STELLA_SLOT_AUXILIARYp;
extern Symbol* SYM_HIERARCHY_STELLA_STORAGE_SLOT;
extern Symbol* SYM_HIERARCHY_STELLA_FUNCTION;
extern Surrogate* SGT_HIERARCHY_STELLA_PARAMETRIC_TYPE_SPECIFIER;
extern Symbol* SYM_HIERARCHY_STELLA_SPECIFIER_BASE_TYPE;
extern Symbol* SYM_HIERARCHY_STELLA_SPECIFIER_PARAMETER_TYPES;
extern Symbol* SYM_HIERARCHY_STELLA_SPECIFIER_DIMENSIONS;
extern Surrogate* SGT_HIERARCHY_STELLA_ANCHORED_TYPE_SPECIFIER;
extern Symbol* SYM_HIERARCHY_STELLA_SPECIFIER_PARAMETER_NAME;
extern Surrogate* SGT_HIERARCHY_STELLA_TABLE;
extern Symbol* SYM_HIERARCHY_STELLA_TUPLE_DOMAINS;
extern Surrogate* SGT_HIERARCHY_STELLA_GLOBAL_VARIABLE;
extern Symbol* SYM_HIERARCHY_STELLA_VARIABLE_HOME_MODULE;
extern Symbol* SYM_HIERARCHY_STELLA_VARIABLE_NAME;
extern Symbol* SYM_HIERARCHY_STELLA_VARIABLE_TYPE;
extern Symbol* SYM_HIERARCHY_STELLA_VARIABLE_SPECIALp;
extern Symbol* SYM_HIERARCHY_STELLA_VARIABLE_CONSTANTp;
extern Symbol* SYM_HIERARCHY_STELLA_VARIABLE_PUBLICp;
extern Symbol* SYM_HIERARCHY_STELLA_VARIABLE_AUXILIARYp;
extern Symbol* SYM_HIERARCHY_STELLA_VARIABLE_GET_VALUE_CODE;
extern Symbol* SYM_HIERARCHY_STELLA_VARIABLE_SET_VALUE_CODE;
extern Symbol* SYM_HIERARCHY_STELLA_VARIABLE_VALUE_STACK;
extern Symbol* SYM_HIERARCHY_STELLA_VARIABLE_DOCUMENTATION;
extern Symbol* SYM_HIERARCHY_STELLA_VARIABLE_STRINGIFIED_SOURCE;
extern Surrogate* SGT_HIERARCHY_STELLA_QUOTED_EXPRESSION;
extern Symbol* SYM_HIERARCHY_STELLA_QUOTATION_TABLE_OFFSET;
extern Symbol* SYM_HIERARCHY_STELLA_SURROGATE_VALUE_INVERSE;
extern Surrogate* SGT_HIERARCHY_STELLA_MODULE;
extern Surrogate* SGT_HIERARCHY_STELLA_WORLD;
extern Symbol* SYM_HIERARCHY_STELLA_WORLD_NAME;
extern Symbol* SYM_HIERARCHY_STELLA_CHILD_CONTEXTS;
extern Symbol* SYM_HIERARCHY_STELLA_BASE_MODULE;
extern Symbol* SYM_HIERARCHY_STELLA_ALL_SUPER_CONTEXTS;
extern Symbol* SYM_HIERARCHY_STELLA_CONTEXT_NUMBER;
extern Symbol* SYM_HIERARCHY_STELLA_MODULE_LISP_PACKAGE;
extern Symbol* SYM_HIERARCHY_STELLA_MODULE_CPP_PACKAGE;
extern Symbol* SYM_HIERARCHY_STELLA_SHADOWED_SURROGATES;
extern Symbol* SYM_HIERARCHY_STELLA_JAVA_PACKAGE;
extern Symbol* SYM_HIERARCHY_STELLA_CLEARABLEp;
extern Symbol* SYM_HIERARCHY_STELLA_PARENT_MODULES;
extern Symbol* SYM_HIERARCHY_STELLA_NICKNAMES;
extern Symbol* SYM_HIERARCHY_STELLA_USES;
extern Symbol* SYM_HIERARCHY_STELLA_USED_BY;
extern Symbol* SYM_HIERARCHY_STELLA_REQUIRES;
extern Symbol* SYM_HIERARCHY_STELLA_CASE_SENSITIVEp;
extern Symbol* SYM_HIERARCHY_STELLA_MODULE_FULL_NAME;
extern Symbol* SYM_HIERARCHY_STELLA_MODULE_NAME;
extern Symbol* SYM_HIERARCHY_STELLA_MODULE_STRINGIFIED_SOURCE;
extern Symbol* SYM_HIERARCHY_STELLA_STRINGIFIED_OPTIONS;
extern Symbol* SYM_HIERARCHY_STELLA_CARDINAL_MODULE;
extern Symbol* SYM_HIERARCHY_STELLA_SYMBOL_OFFSET_TABLE;
extern Symbol* SYM_HIERARCHY_STELLA_SURROGATE_OFFSET_TABLE;
extern Symbol* SYM_HIERARCHY_STELLA_STRICT_INFERENCE_CACHE;
extern Symbol* SYM_HIERARCHY_STELLA_DEFAULT_INFERENCE_CACHE;
extern Symbol* SYM_HIERARCHY_STELLA_PROTOTYPE_INFERENCE_CACHE;
extern Symbol* SYM_HIERARCHY_STELLA_PARENT_CONTEXT;
extern Surrogate* SGT_HIERARCHY_STELLA_CS_VALUE;
extern Surrogate* SGT_HIERARCHY_STELLA_INTERVAL;
extern Surrogate* SGT_HIERARCHY_STELLA_TAXONOMY_NODE;
extern Symbol* SYM_HIERARCHY_STELLA_NATIVE_OBJECT;
extern Symbol* SYM_HIERARCHY_STELLA_LABEL;
extern Symbol* SYM_HIERARCHY_STELLA_INTERVALS;
extern Symbol* SYM_HIERARCHY_STELLA_INITIAL_INTERVAL;
extern Symbol* SYM_HIERARCHY_STELLA_FIRST_INTERVAL_LOWER_BOUND;
extern Symbol* SYM_HIERARCHY_STELLA_FIRST_INTERVAL_UPPER_BOUND;
extern Symbol* SYM_HIERARCHY_STELLA_PARENTS;
extern Symbol* SYM_HIERARCHY_STELLA_CHILDREN;
extern Symbol* SYM_HIERARCHY_STELLA_TOTAL_ANCESTORS;
extern Symbol* SYM_HIERARCHY_STELLA_TREE_CHILDREN;
extern Symbol* SYM_HIERARCHY_STELLA_MARKER;
extern Surrogate* SGT_HIERARCHY_STELLA_TAXONOMY_GRAPH;
extern Symbol* SYM_HIERARCHY_STELLA_RENUMBER_IF_OUT_OF_NUMBERSp;
extern Symbol* SYM_HIERARCHY_STELLA_RENUMBER_RATIO;
extern Symbol* SYM_HIERARCHY_STELLA_INCREMENTAL_MODEp;
extern Symbol* SYM_HIERARCHY_STELLA_LARGEST_POSTORDER_NUMBER;
extern Symbol* SYM_HIERARCHY_STELLA_NUMBER_OF_NODES;
extern Symbol* SYM_HIERARCHY_STELLA_NUMBER_OF_FOREIGN_INTERVAL_NODES;
extern Symbol* SYM_HIERARCHY_STELLA_ROOTS;
extern Symbol* SYM_HIERARCHY_STELLA_BROKEN_LINKS;
extern Symbol* SYM_HIERARCHY_STELLA_ADDED_LINKS;
extern Symbol* SYM_HIERARCHY_STELLA_REMOVED_LINKS;
extern Symbol* SYM_HIERARCHY_STELLA_FILE_NAME;
extern Surrogate* SGT_HIERARCHY_STELLA_INTEGER_WRAPPER;
extern Symbol* SYM_HIERARCHY_STELLA_WRAPPER_VALUE;
extern Surrogate* SGT_HIERARCHY_STELLA_FLOAT_WRAPPER;
extern Surrogate* SGT_HIERARCHY_STELLA_STRING_WRAPPER;
extern Surrogate* SGT_HIERARCHY_STELLA_MUTABLE_STRING_WRAPPER;
extern Surrogate* SGT_HIERARCHY_STELLA_CHARACTER_WRAPPER;
extern Surrogate* SGT_HIERARCHY_STELLA_BOOLEAN_WRAPPER;
extern Symbol* SYM_HIERARCHY_STELLA_TRUE;
extern Symbol* SYM_HIERARCHY_STELLA_FALSE;
extern Surrogate* SGT_HIERARCHY_STELLA_FUNCTION_CODE_WRAPPER;
extern Surrogate* SGT_HIERARCHY_STELLA_METHOD_CODE_WRAPPER;
extern Surrogate* SGT_HIERARCHY_STELLA_VERBATIM_STRING_WRAPPER;
extern Symbol* SYM_HIERARCHY_STELLA_THREE_VALUED_BOOLEAN;
extern Surrogate* SGT_HIERARCHY_STELLA_SYSTEM_DEFINITION;
extern Symbol* SYM_HIERARCHY_STELLA_NAME;
extern Symbol* SYM_HIERARCHY_STELLA_DIRECTORY;
extern Symbol* SYM_HIERARCHY_STELLA_FILES;
extern Symbol* SYM_HIERARCHY_STELLA_LISP_ONLY_FILES;
extern Symbol* SYM_HIERARCHY_STELLA_CPP_ONLY_FILES;
extern Symbol* SYM_HIERARCHY_STELLA_JAVA_ONLY_FILES;
extern Symbol* SYM_HIERARCHY_STELLA_PREPROCESSED_FILES;
extern Symbol* SYM_HIERARCHY_STELLA_REQUIRED_SYSTEMS;
extern Symbol* SYM_HIERARCHY_STELLA_LOADEDp;
extern Symbol* SYM_HIERARCHY_STELLA_UP_TO_DATEp;
extern Symbol* SYM_HIERARCHY_STELLA_SOURCE_ROOT_DIRECTORY;
extern Symbol* SYM_HIERARCHY_STELLA_NATIVE_ROOT_DIRECTORY;
extern Symbol* SYM_HIERARCHY_STELLA_BINARY_ROOT_DIRECTORY;
extern Symbol* SYM_HIERARCHY_STELLA_BANNER;
extern Symbol* SYM_HIERARCHY_STELLA_COPYRIGHT_HEADER;
extern Symbol* SYM_HIERARCHY_STELLA_PRODUCTION_SETTINGS;
extern Symbol* SYM_HIERARCHY_STELLA_DEVELOPMENT_SETTINGS;
extern Symbol* SYM_HIERARCHY_STELLA_FINALIZATION_FUNCTION;
extern Symbol* SYM_HIERARCHY_STELLA_STATE;
extern Surrogate* SGT_HIERARCHY_STELLA_OUTPUT_STREAM;
extern Keyword* KWD_HIERARCHY_LINE;
extern Surrogate* SGT_HIERARCHY_STELLA_INPUT_STREAM;
extern Symbol* SYM_HIERARCHY_STELLA_ECHO_STREAM;
extern Symbol* SYM_HIERARCHY_STELLA_TOKENIZER_STATE;
extern Symbol* SYM_HIERARCHY_STELLA_BUFFERING_SCHEME;
extern Keyword* KWD_HIERARCHY_SUPERSEDE;
extern Surrogate* SGT_HIERARCHY_STELLA_OUTPUT_FILE_STREAM;
extern Symbol* SYM_HIERARCHY_STELLA_FILENAME;
extern Symbol* SYM_HIERARCHY_STELLA_IF_EXISTS_ACTION;
extern Symbol* SYM_HIERARCHY_STELLA_FILE_OUTPUT_STREAM;
extern Keyword* KWD_HIERARCHY_BLOCK;
extern Keyword* KWD_HIERARCHY_ERROR;
extern Surrogate* SGT_HIERARCHY_STELLA_INPUT_FILE_STREAM;
extern Symbol* SYM_HIERARCHY_STELLA_IF_NOT_EXISTS_ACTION;
extern Symbol* SYM_HIERARCHY_STELLA_FILE_INPUT_STREAM;
extern Surrogate* SGT_HIERARCHY_STELLA_OUTPUT_STRING_STREAM;
extern Symbol* SYM_HIERARCHY_STELLA_STRING_OUTPUT_STREAM;
extern Surrogate* SGT_HIERARCHY_STELLA_INPUT_STRING_STREAM;
extern Symbol* SYM_HIERARCHY_STELLA_STRING_INPUT_STREAM;
extern Symbol* SYM_HIERARCHY_STELLA_STARTUP_HIERARCHY;
extern Symbol* SYM_HIERARCHY_STELLA_METHOD_STARTUP_CLASSNAME;


} // end of namespace stella
