// java-translate-file.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

void clt() {
  string_change_module("STELLA");
  set_translator_output_language(KWD_JAVA_TRANSLATE_FILE_COMMON_LISP);
}

void jt() {
  string_change_module("STELLA");
  set_translator_output_language(KWD_JAVA_TRANSLATE_FILE_JAVA);
}

void jptrans(Object* statement) {
  // Translate `statement' to C++ and print the result.
  { 
    BIND_STELLA_SPECIAL(oTRANSLATOROUTPUTLANGUAGEo, Keyword*, oTRANSLATOROUTPUTLANGUAGEo.get());
    BIND_STELLA_SPECIAL(oCURRENT_STREAMo, Output_Stream*, STANDARD_OUTPUT);
    set_translator_output_language(KWD_JAVA_TRANSLATE_FILE_JAVA);
    incrementally_translate(statement);
  }
}

char* string_java_make_code_output_file_name(char* barefile, boolean donttruncateP) {
  if (donttruncateP) {
    { 
      BIND_STELLA_SPECIAL(oDONTTRUNCATEFILENAMESpo, boolean, TRUE);
      return (make_file_name_from_relative_path(wrap_string(barefile), KWD_JAVA_TRANSLATE_FILE_JAVA));
    }
  }
  else {
    return (make_file_name_from_relative_path(wrap_string(barefile), KWD_JAVA_TRANSLATE_FILE_JAVA));
  }
}

char* Class::java_make_code_output_file_name(boolean donttruncateP) {
  { Class* source = this;

    { String_Wrapper* barefile = java_translate_class_name(intern_symbol_in_module(source->class_type->symbol_name, ((Module*)(source->class_type->home_context)), TRUE));

      return (string_java_make_code_output_file_name(barefile->wrapper_value, donttruncateP));
    }
  }
}

char* java_make_global_output_file_name(Module* module, boolean donttruncateP) {
  return (string_java_make_code_output_file_name(java_yield_flotsam_class_name(module), donttruncateP));
}

void java_output_flotsam_units_to_file(Hash_Table* globalsht, Hash_Table* functionht, Cons* codemodulelist) {
  { Cons* functions = NIL;
    Cons* globals = NIL;

    { Module* module = NULL;
      Cons* iter_000 = codemodulelist;

      while (!(iter_000 == NIL)) {
        module = ((Module*)(iter_000->value));
        iter_000 = iter_000->rest;
        globals = ((Cons*)(globalsht->lookup(module)));
        if (!(((boolean)(globals)))) {
          globals = NIL;
        }
        functions = ((Cons*)(functionht->lookup(module)));
        if (!(((boolean)(functions)))) {
          functions = NIL;
        }
        java_output_flotsam_units_to_file_for_module(module, NULL, globals, functions);
      }
    }
  }
}

void java_output_flotsam_units_to_file_for_module(Module* module, Translation_Unit* classunit, Cons* globals, Cons* functions) {
  { char* classoutputfile = NULL;
    Output_File_Stream* classoutputstream = NULL;
    Cons* translation = NIL;
    Class* renamed_Class = NULL;
    Cons* classtranslation = NIL;

    { 
      BIND_STELLA_SPECIAL(oMODULEo, Module*, module);
      BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
      classoutputfile = java_make_global_output_file_name(module, FALSE);
      classoutputstream = new_output_file_stream(classoutputfile);
      if (((boolean)(classunit))) {
        renamed_Class = ((Class*)(classunit->the_object));
        cout << "Translating " << "`" << renamed_Class << "'" << "..." << endl;
        classtranslation = java_translate_define_native_class(renamed_Class)->rest;
      }
      { 
        BIND_STELLA_SPECIAL(oCURRENT_STREAMo, Output_Stream*, classoutputstream);
        if (oTRANSLATIONVERBOSITYLEVELo.get() >= 1) {
          if (((boolean)(classunit))) {
            cout << "    Writing " << "`" << classoutputfile << "'" << " ..." << endl;
          }
          else {
            cout << "Writing " << "`" << classoutputfile << "'" << "..." << endl;
          }
        }
        java_output_file_header(classoutputstream, java_make_global_output_file_name(module, TRUE));
        if (((boolean)(classunit))) {
          java_output_class_declaration(classtranslation);
        }
        else {
          *(oCURRENT_STREAMo.get()->native_stream) << "public class " << java_yield_flotsam_class_name(module) << " ";
        }
        *(oCURRENT_STREAMo.get()->native_stream) << "{" << endl;
        java_bump_indent();
        if (((boolean)(classunit))) {
          java_output_class_variable_definitions(((Cons*)(classtranslation->nth(6))));
        }
        { Translation_Unit* global = NULL;
          Cons* iter_000 = globals;

          while (!(iter_000 == NIL)) {
            global = ((Translation_Unit*)(iter_000->value));
            iter_000 = iter_000->rest;
            translation = java_translate_unit(global);
            { Symbol* test_value_000 = ((Symbol*)(translation->value));

              if (test_value_000 == SYM_JAVA_TRANSLATE_FILE_STELLA_JAVA_GLOBAL) {
                java_output_global_definition(translation->rest);
              }
              else if (test_value_000 == SYM_JAVA_TRANSLATE_FILE_STELLA_JAVA_SPECIAL) {
                java_output_special_definition(translation->rest);
              }
              else {
                { Output_String_Stream* stream_000 = new_output_string_stream();

                  *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
                  throw *new_stella_exception(stream_000->the_string_reader());
                }
              }
            }
            global->translation = NULL;
            global->code_register = NULL;
          }
        }
        if (((boolean)(classunit))) {
          java_output_class_constructors(((Cons*)(classtranslation->nth(7))), java_yield_flotsam_class_name(module), ((boolean)(renamed_Class)) &&
              exception_classP(renamed_Class));
          { Translation_Unit* statement = NULL;
            Cons* iter_001 = ((Cons*)(classtranslation->nth(8)));

            while (!(iter_001 == NIL)) {
              statement = ((Translation_Unit*)(iter_001->value));
              iter_001 = iter_001->rest;
              java_output_statement(java_translate_unit(statement));
              statement->translation = NULL;
              statement->code_register = NULL;
            }
          }
        }
        { Translation_Unit* function = NULL;
          Cons* iter_002 = functions;

          while (!(iter_002 == NIL)) {
            function = ((Translation_Unit*)(iter_002->value));
            iter_002 = iter_002->rest;
            java_output_method(java_translate_unit(function)->rest);
            function->translation = NULL;
            function->code_register = NULL;
          }
        }
        java_unbump_indent();
        *(oCURRENT_STREAMo.get()->native_stream) << "}" << endl;
        classoutputstream->free();
      }
    }
  }
}

void java_output_file_header(Output_Stream* stream, char* filename) {
  *(stream->native_stream) << "//  -*- Mode: Java -*-" << endl << "//" << endl;
  *(stream->native_stream) << "// " << filename << endl << endl;
  { char* packagename = java_package_prefix(oMODULEo.get(), ".");

    output_copyright_header(stream);
    if (!((packagename == NULL) ||
        string_eqlP(packagename, ""))) {
      packagename = string_subsequence(packagename, 0, strlen(packagename) - 1);
      *(stream->native_stream) << "package " << packagename << ";" << endl << endl;
    }
    if (!(oMODULEo.get() == oSTELLA_MODULEo)) {
      *(stream->native_stream) << "import " << java_stella_package() << ".*;" << endl;
    }
    *(stream->native_stream) << "import " << java_stella_package() << ".javalib.*;" << endl << endl;
  }
}

void java_output_class_to_file(Class* renamed_Class) {
  { 
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, renamed_Class->home_module());
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oCONTEXTo.get()->base_module);
    { char* classoutputfile = renamed_Class->java_make_code_output_file_name(FALSE);
      Output_File_Stream* classoutputstream = new_output_file_stream(classoutputfile);
      Cons* translation = NIL;

      { 
        BIND_STELLA_SPECIAL(oCURRENT_STREAMo, Output_Stream*, classoutputstream);
        cout << "Translating " << "`" << renamed_Class << "'" << "..." << endl;
        translation = java_translate_define_native_class(renamed_Class);
        if (oTRANSLATIONVERBOSITYLEVELo.get() >= 1) {
          cout << "    Writing " << "`" << classoutputfile << "'" << "..." << endl;
        }
        java_output_file_header(classoutputstream, renamed_Class->java_make_code_output_file_name(TRUE));
        java_output_class(translation->rest, exception_classP(renamed_Class));
        classoutputstream->free();
      }
    }
  }
}

void java_output_class_unit_to_file(Translation_Unit* classunit) {
  java_output_class_to_file(((Class*)(classunit->the_object)));
}

boolean java_flotsam_functionP(Method_Slot* method) {
  return ((!((Boolean_Wrapper*)(dynamic_slot_value(method->dynamic_slots, SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_CONSTRUCTORp, FALSE_WRAPPER)))->wrapper_value) &&
      ((!((Boolean_Wrapper*)(dynamic_slot_value(method->dynamic_slots, SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_NATIVEp, FALSE_WRAPPER)))->wrapper_value) &&
       ((!method->method_startup_functionP()) &&
        (zero_argument_functionP(method) ||
         (java_method_object_defined_on_native_typeP(method) ||
          java_function_in_different_moduleP(method))))));
}

void java_output_startup_units_to_file(String_Hash_Table* startupht, Cons* keylist) {
  { char* classoutputfile = NULL;
    Output_File_Stream* classoutputstream = NULL;
    Cons* startupfunctions = NULL;

    { 
      BIND_STELLA_SPECIAL(oCURRENT_STREAMo, Output_Stream*, NULL);
      { String_Wrapper* classname = NULL;
        Cons* iter_000 = keylist;

        while (!(iter_000 == NIL)) {
          classname = ((String_Wrapper*)(iter_000->value));
          iter_000 = iter_000->rest;
          startupfunctions = ((Cons*)(startupht->lookup(classname->wrapper_value)));
          if (((boolean)(startupfunctions)) &&
              (!(!((boolean)(startupfunctions))))) {
            { 
              BIND_STELLA_SPECIAL(oMODULEo, Module*, ((Translation_Unit*)(startupfunctions->value))->home_module());
              BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
              classoutputfile = string_java_make_code_output_file_name(classname->wrapper_value, FALSE);
              classoutputstream = new_output_file_stream(classoutputfile);
              oCURRENT_STREAMo.set(classoutputstream);
              if (oTRANSLATIONVERBOSITYLEVELo.get() >= 1) {
                cout << "Writing " << "`" << classoutputfile << "'" << "..." << endl;
              }
              java_output_file_header(classoutputstream, string_java_make_code_output_file_name(classname->wrapper_value, TRUE));
              *(oCURRENT_STREAMo.get()->native_stream) << "public class " << classname->wrapper_value << " {" << endl;
              java_bump_indent();
              { Translation_Unit* function = NULL;
                Cons* iter_001 = startupfunctions;

                while (!(iter_001 == NIL)) {
                  function = ((Translation_Unit*)(iter_001->value));
                  iter_001 = iter_001->rest;
                  java_output_method(java_translate_unit(function)->rest);
                  function->translation = NULL;
                  function->code_register = NULL;
                }
              }
              java_unbump_indent();
              *(oCURRENT_STREAMo.get()->native_stream) << "}" << endl;
              classoutputstream->free();
            }
          }
        }
      }
    }
  }
}

void java_push_into_hash_table(Hash_Table* ht, Object* key, Object* object) {
  { Cons* value = ((Cons*)(ht->lookup(key)));

    if (((boolean)(value))) {
      ht->insert_at(key, cons(object, value));
    }
    else {
      ht->insert_at(key, cons(object, NIL));
    }
  }
}

void java_push_into_string_hash_table(String_Hash_Table* ht, char* key, Object* object) {
  { Cons* value = ((Cons*)(ht->lookup(key)));

    if (((boolean)(value))) {
      ht->insert_at(key, cons(object, value));
    }
    else {
      ht->insert_at(key, cons(object, NIL));
    }
  }
}

boolean java_class_unit_defines_flotsam_classP(Translation_Unit* classunit) {
  { Class* renamed_Class = ((Class*)(classunit->the_object));
    Module* module = classunit->home_module();

    return (string_eqlP(java_translate_class_namestring(wrap_string(intern_symbol_in_module(renamed_Class->class_type->symbol_name, ((Module*)(renamed_Class->class_type->home_context)), TRUE)->symbol_name))->wrapper_value, java_yield_flotsam_class_name(module)));
  }
}

void java_output_all_units_to_file() {
  { String_Wrapper* startupclassname = NULL;
    String_Hash_Table* startupht = new_string_hash_table();
    Cons* startupclasslist = NIL;
    Cons* methods = NIL;
    Hash_Table* flotsamfunctionht = new_hash_table();
    Hash_Table* globalsht = new_hash_table();
    List* codemoduleslist = list(0);
    Cons* verbatimstatements = NIL;
    Cons* classes = NIL;
    Module* codeoutputmodule = NULL;

    oJAVA_STELLA_PACKAGE_MAPPINGo->insert_at(wrap_string("STELLAROOT"), wrap_string(java_stella_package()));
    oTRANSLATIONUNITSo.set(oTRANSLATIONUNITSo.get()->reverse());
    { Translation_Unit* unit = NULL;
      Cons* iter_000 = oTRANSLATIONUNITSo.get()->the_cons_list;

      while (!(iter_000 == NIL)) {
        unit = ((Translation_Unit*)(iter_000->value));
        iter_000 = iter_000->rest;
        { Symbol* test_value_000 = unit->category;

          if ((test_value_000 == SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD) ||
              (test_value_000 == SYM_JAVA_TRANSLATE_FILE_STELLA_MACRO)) {
            if (java_flotsam_functionP(((Method_Slot*)(unit->the_object)))) {
              codeoutputmodule = unit->the_object->home_module();
              java_push_into_hash_table(flotsamfunctionht, codeoutputmodule, unit);
              codemoduleslist->insert_new(codeoutputmodule);
            }
            else if (((Method_Slot*)(unit->the_object))->method_startup_functionP()) {
              startupclassname = java_translate_class_namestring(wrap_string(((String_Wrapper*)(dynamic_slot_value(((Method_Slot*)(unit->the_object))->dynamic_slots, SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_STARTUP_CLASSNAME, NULL_STRING_WRAPPER)))->wrapper_value));
              java_push_into_string_hash_table(startupht, startupclassname->wrapper_value, unit);
              if (!(startupclasslist->memberP(startupclassname))) {
                startupclasslist = cons(startupclassname, startupclasslist);
              }
            }
            else {
              methods = cons(unit, methods);
            }
          }
          else if (test_value_000 == SYM_JAVA_TRANSLATE_FILE_STELLA_PRINT_METHOD) {
            methods = cons(unit, methods);
          }
          else if (test_value_000 == SYM_JAVA_TRANSLATE_FILE_STELLA_TYPE) {
          }
          else if (test_value_000 == SYM_JAVA_TRANSLATE_FILE_STELLA_VERBATIM) {
            verbatimstatements = cons(unit, verbatimstatements);
          }
          else if (test_value_000 == SYM_JAVA_TRANSLATE_FILE_STELLA_CLASS) {
            classes = cons(unit, classes);
          }
          else if (test_value_000 == SYM_JAVA_TRANSLATE_FILE_STELLA_GLOBAL_VARIABLE) {
            codeoutputmodule = unit->the_object->home_module();
            java_push_into_hash_table(globalsht, codeoutputmodule, unit);
            codemoduleslist->insert_new(codeoutputmodule);
          }
          else {
            { Output_String_Stream* stream_000 = new_output_string_stream();

              *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_000->the_string_reader());
            }
          }
        }
      }
    }
    { Translation_Unit* classunit = NULL;
      Cons* iter_001 = classes;

      while (!(iter_001 == NIL)) {
        classunit = ((Translation_Unit*)(iter_001->value));
        iter_001 = iter_001->rest;
        if (java_class_unit_defines_flotsam_classP(classunit)) {
          { Module* module = classunit->the_object->home_module();
            Cons* globals = NIL;
            Cons* functions = NIL;

            globals = ((Cons*)(globalsht->lookup(module)));
            if (!(((boolean)(globals)))) {
              globals = NIL;
            }
            functions = ((Cons*)(flotsamfunctionht->lookup(module)));
            if (!(((boolean)(functions)))) {
              functions = NIL;
            }
            java_output_flotsam_units_to_file_for_module(module, classunit, globals, functions);
            codemoduleslist->remove(module);
          }
        }
        else {
          java_output_class_unit_to_file(classunit);
        }
      }
    }
    java_output_flotsam_units_to_file(globalsht, flotsamfunctionht, codemoduleslist->the_cons_list);
    java_output_startup_units_to_file(startupht, startupclasslist);
  }
}

void java_initialize_file_translation() {
  oJAVA_INDENT_CHARSo.set(0);
}

void java_translate_file(char* filename) {
  translate_file(filename, KWD_JAVA_TRANSLATE_FILE_JAVA, FALSE);
}

void java_translate_file_as_part_of_system(char* filename) {
  translate_file(filename, KWD_JAVA_TRANSLATE_FILE_JAVA, TRUE);
}

void java_translate_walked_system_units(Key_Value_List* systemunits) {
  { 
    BIND_STELLA_SPECIAL(oTRANSLATIONUNITSo, List*, concatenate_system_units(systemunits));
    if (oTRANSLATIONVERBOSITYLEVELo.get() >= 1) {
      cout << "Generating Java translations..." << endl;
    }
    java_output_all_units_to_file();
    clean_up_translation_units_special();
  }
}

void java_translate_system(char* systemname) {
  // Translate the system `systemName' to Java.
  translate_system(systemname, KWD_JAVA_TRANSLATE_FILE_JAVA, 6, KWD_JAVA_TRANSLATE_FILE_TWO_PASSp, TRUE_WRAPPER, KWD_JAVA_TRANSLATE_FILE_FORCE_TRANSLATIONp, TRUE_WRAPPER, KWD_JAVA_TRANSLATE_FILE_PRODUCTION_SETTINGSp, TRUE_WRAPPER);
}

void startup_java_translate_file() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      KWD_JAVA_TRANSLATE_FILE_COMMON_LISP = ((Keyword*)(intern_rigid_symbol_wrt_module("COMMON-LISP", NULL, 2)));
      KWD_JAVA_TRANSLATE_FILE_JAVA = ((Keyword*)(intern_rigid_symbol_wrt_module("JAVA", NULL, 2)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_JAVA_GLOBAL = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_GLOBAL", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_JAVA_SPECIAL = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_SPECIAL", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_CONSTRUCTORp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-CONSTRUCTOR?", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_NATIVEp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-NATIVE?", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_MACRO = ((Symbol*)(intern_rigid_symbol_wrt_module("MACRO", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_PRINT_METHOD = ((Symbol*)(intern_rigid_symbol_wrt_module("PRINT-METHOD", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("TYPE", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_VERBATIM = ((Symbol*)(intern_rigid_symbol_wrt_module("VERBATIM", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_CLASS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_GLOBAL_VARIABLE = ((Symbol*)(intern_rigid_symbol_wrt_module("GLOBAL-VARIABLE", NULL, 0)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
      KWD_JAVA_TRANSLATE_FILE_TWO_PASSp = ((Keyword*)(intern_rigid_symbol_wrt_module("TWO-PASS?", NULL, 2)));
      KWD_JAVA_TRANSLATE_FILE_FORCE_TRANSLATIONp = ((Keyword*)(intern_rigid_symbol_wrt_module("FORCE-TRANSLATION?", NULL, 2)));
      KWD_JAVA_TRANSLATE_FILE_PRODUCTION_SETTINGSp = ((Keyword*)(intern_rigid_symbol_wrt_module("PRODUCTION-SETTINGS?", NULL, 2)));
      SYM_JAVA_TRANSLATE_FILE_STELLA_STARTUP_JAVA_TRANSLATE_FILE = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-JAVA-TRANSLATE-FILE", NULL, 0)));
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      define_function_object("CLT", "(DEFUN CLT ())", ((cpp_function_code)(&clt)), NULL);
      define_function_object("JT", "(DEFUN JT ())", ((cpp_function_code)(&jt)), NULL);
      define_function_object("JPTRANS", "(DEFUN JPTRANS ((STATEMENT OBJECT)) :COMMAND? TRUE :PUBLIC? TRUE :EVALUATE-ARGUMENTS? FALSE :DOCUMENTATION \"Translate `statement' to C++ and print the result.\")", ((cpp_function_code)(&jptrans)), NULL);
      define_method_object("(DEFMETHOD (JAVA-MAKE-CODE-OUTPUT-FILE-NAME FILE-NAME) ((BAREFILE FILE-NAME) (DONTTRUNCATE? BOOLEAN)))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (JAVA-MAKE-CODE-OUTPUT-FILE-NAME FILE-NAME) ((SOURCE CLASS) (DONTTRUNCATE? BOOLEAN)))", ((cpp_method_code)(&Class::java_make_code_output_file_name)), ((cpp_method_code)(NULL)));
      define_function_object("JAVA-MAKE-GLOBAL-OUTPUT-FILE-NAME", "(DEFUN (JAVA-MAKE-GLOBAL-OUTPUT-FILE-NAME FILE-NAME) ((MODULE MODULE) (DONTTRUNCATE? BOOLEAN)))", ((cpp_function_code)(&java_make_global_output_file_name)), NULL);
      define_function_object("JAVA-OUTPUT-FLOTSAM-UNITS-TO-FILE", "(DEFUN JAVA-OUTPUT-FLOTSAM-UNITS-TO-FILE ((GLOBALSHT HASH-TABLE) (FUNCTIONHT HASH-TABLE) (CODEMODULELIST (CONS OF MODULE))))", ((cpp_function_code)(&java_output_flotsam_units_to_file)), NULL);
      define_function_object("JAVA-OUTPUT-FLOTSAM-UNITS-TO-FILE-FOR-MODULE", "(DEFUN JAVA-OUTPUT-FLOTSAM-UNITS-TO-FILE-FOR-MODULE ((MODULE MODULE) (CLASSUNIT TRANSLATION-UNIT) (GLOBALS (CONS OF TRANSLATION-UNIT)) (FUNCTIONS (CONS OF TRANSLATION-UNIT))))", ((cpp_function_code)(&java_output_flotsam_units_to_file_for_module)), NULL);
      define_function_object("JAVA-OUTPUT-FILE-HEADER", "(DEFUN JAVA-OUTPUT-FILE-HEADER ((STREAM OUTPUT-STREAM) (FILENAME FILE-NAME)))", ((cpp_function_code)(&java_output_file_header)), NULL);
      define_function_object("JAVA-OUTPUT-CLASS-TO-FILE", "(DEFUN JAVA-OUTPUT-CLASS-TO-FILE ((CLASS CLASS)))", ((cpp_function_code)(&java_output_class_to_file)), NULL);
      define_function_object("JAVA-OUTPUT-CLASS-UNIT-TO-FILE", "(DEFUN JAVA-OUTPUT-CLASS-UNIT-TO-FILE ((CLASSUNIT TRANSLATION-UNIT)))", ((cpp_function_code)(&java_output_class_unit_to_file)), NULL);
      define_function_object("JAVA-FLOTSAM-FUNCTION?", "(DEFUN (JAVA-FLOTSAM-FUNCTION? BOOLEAN) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&java_flotsam_functionP)), NULL);
      define_function_object("JAVA-OUTPUT-STARTUP-UNITS-TO-FILE", "(DEFUN JAVA-OUTPUT-STARTUP-UNITS-TO-FILE ((STARTUPHT STRING-HASH-TABLE) (KEYLIST (CONS OF STRING-WRAPPER))))", ((cpp_function_code)(&java_output_startup_units_to_file)), NULL);
      define_function_object("JAVA-PUSH-INTO-HASH-TABLE", "(DEFUN JAVA-PUSH-INTO-HASH-TABLE ((HT HASH-TABLE) (KEY OBJECT) (OBJECT OBJECT)))", ((cpp_function_code)(&java_push_into_hash_table)), NULL);
      define_function_object("JAVA-PUSH-INTO-STRING-HASH-TABLE", "(DEFUN JAVA-PUSH-INTO-STRING-HASH-TABLE ((HT STRING-HASH-TABLE) (KEY STRING) (OBJECT OBJECT)))", ((cpp_function_code)(&java_push_into_string_hash_table)), NULL);
      define_function_object("JAVA-CLASS-UNIT-DEFINES-FLOTSAM-CLASS?", "(DEFUN (JAVA-CLASS-UNIT-DEFINES-FLOTSAM-CLASS? BOOLEAN) ((CLASSUNIT TRANSLATION-UNIT)))", ((cpp_function_code)(&java_class_unit_defines_flotsam_classP)), NULL);
      define_function_object("JAVA-OUTPUT-ALL-UNITS-TO-FILE", "(DEFUN JAVA-OUTPUT-ALL-UNITS-TO-FILE ())", ((cpp_function_code)(&java_output_all_units_to_file)), NULL);
      define_function_object("JAVA-INITIALIZE-FILE-TRANSLATION", "(DEFUN JAVA-INITIALIZE-FILE-TRANSLATION ())", ((cpp_function_code)(&java_initialize_file_translation)), NULL);
      define_function_object("JAVA-TRANSLATE-FILE", "(DEFUN JAVA-TRANSLATE-FILE ((FILENAME FILE-NAME)) :PUBLIC? TRUE)", ((cpp_function_code)(&java_translate_file)), NULL);
      define_function_object("JAVA-TRANSLATE-FILE-AS-PART-OF-SYSTEM", "(DEFUN JAVA-TRANSLATE-FILE-AS-PART-OF-SYSTEM ((FILENAME FILE-NAME)))", ((cpp_function_code)(&java_translate_file_as_part_of_system)), NULL);
      define_function_object("JAVA-TRANSLATE-WALKED-SYSTEM-UNITS", "(DEFUN JAVA-TRANSLATE-WALKED-SYSTEM-UNITS ((SYSTEMUNITS SYSTEM-UNITS-ALIST)))", ((cpp_function_code)(&java_translate_walked_system_units)), NULL);
      define_function_object("JAVA-TRANSLATE-SYSTEM", "(DEFUN JAVA-TRANSLATE-SYSTEM ((SYSTEMNAME STRING)) :DOCUMENTATION \"Translate the system `systemName' to Java.\" :PUBLIC? TRUE)", ((cpp_function_code)(&java_translate_system)), NULL);
      define_function_object("STARTUP-JAVA-TRANSLATE-FILE", "(DEFUN STARTUP-JAVA-TRANSLATE-FILE () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_java_translate_file)), NULL);
      { Method_Slot* function = lookup_function(SYM_JAVA_TRANSLATE_FILE_STELLA_STARTUP_JAVA_TRANSLATE_FILE);

        set_dynamic_slot_value(function->dynamic_slots, SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Java-Translate-File"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
  }
}

Keyword* KWD_JAVA_TRANSLATE_FILE_COMMON_LISP = NULL;

Keyword* KWD_JAVA_TRANSLATE_FILE_JAVA = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_JAVA_GLOBAL = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_JAVA_SPECIAL = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_CONSTRUCTORp = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_NATIVEp = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_MACRO = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_PRINT_METHOD = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_TYPE = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_VERBATIM = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_CLASS = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_GLOBAL_VARIABLE = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

Keyword* KWD_JAVA_TRANSLATE_FILE_TWO_PASSp = NULL;

Keyword* KWD_JAVA_TRANSLATE_FILE_FORCE_TRANSLATIONp = NULL;

Keyword* KWD_JAVA_TRANSLATE_FILE_PRODUCTION_SETTINGSp = NULL;

Symbol* SYM_JAVA_TRANSLATE_FILE_STELLA_STARTUP_JAVA_TRANSLATE_FILE = NULL;

} // end of namespace stella
