// /tmp/build-stella/stella-3.0.0.beta/native/cpp/stella/memoize.hh

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/


namespace stella {

// Class definitions:
class Memoization_Table : public Standard_Object {
// Table that maps argument values onto computed results for
// one individual memoization site.
public:
  Integer_Hash_Table* hash_table;
  Surrogate* table_name;
  Cons* current_timestamp;
  Cons* timestamps;
public:
  virtual Surrogate* primary_type();
};

class Mru_Memoization_Table : public Memoization_Table {
public:
  Vector* mru_buckets_vector;
  Vector* lru_buckets_vector;
  Object** mru_buckets;
  Object** lru_buckets;
  Cons* mru_timestamp;
  Cons* lru_timestamp;
  int nof_buckets;
  int free_entries;
  int max_entries;
public:
  virtual Surrogate* primary_type();
};


// Global declarations:
extern List* oALL_MEMOIZATION_TABLESo;
extern boolean oMEMOIZATION_ENABLEDpo;
extern Symbol* MEMOIZED_NULL_VALUE;
extern int oTOTAL_MEMOIZATION_LOOKUPSo;
extern int oFAILED_MEMOIZATION_LOOKUPSo;

// Function signatures:
Memoization_Table* new_memoization_table();
Object* access_memoization_table_slot_value(Memoization_Table* self, Symbol* slotname, Object* value, boolean setvalueP);
void enable_memoization();
void disable_memoization();
int hash_memoized_arguments(Object* arg1, Object* arg2, Object* arg3, Object* arg4, int eqvector);
int hash_memoized_argumentsn(Cons* tuple, int eqvector);
Cons* make_memoized_value_entry(Object* value, Object* arg1, Object* arg2, Object* arg3, Object* arg4);
Cons* make_memoized_value_entryn(Object* value, Cons* args);
Cons* find_memoized_value_entry(Cons* bucket, Object* arg1, Object* arg2, Object* arg3, Object* arg4, int eqvector, boolean deleteentryP);
Cons* find_memoized_value_entryn(Cons* bucket, Cons* tuple, int eqvector, boolean deleteentryP);
Cons* lookup_memoized_value(Memoization_Table* memotable, Object* arg1, Object* arg2, Object* arg3, Object* arg4, int eqvector);
Cons* lookup_memoized_valuen(Memoization_Table* memotable, Cons* args, int eqvector);
Mru_Memoization_Table* new_mru_memoization_table();
Object* access_mru_memoization_table_slot_value(Mru_Memoization_Table* self, Symbol* slotname, Object* value, boolean setvalueP);
void initialize_mru_bucket_tables(Mru_Memoization_Table* memotable);
Cons* lookup_mru_memoized_value(Mru_Memoization_Table* memotable, Object* arg1, Object* arg2, Object* arg3, Object* arg4, int eqvector);
Cons* lookup_mru_memoized_valuen(Mru_Memoization_Table* memotable, Cons* args, int eqvector);
void initialize_memoization_table(Surrogate* memotablesurrogate, char* options);
void clear_all_memoization_tables();
void clear_memoization_tables(Keyword* timestampname);
void bump_memoization_timestamp(Keyword* timestampname);
Surrogate* make_memoization_table_surrogate(Symbol* memoname);
Property_List* parse_memoize_options(Cons* options);
Object* memoize_expander(Cons* inputargs, Cons* body);
void help_startup_memoize1();
void startup_memoize();

// Auxiliary global declarations:
extern Surrogate* SGT_MEMOIZE_STELLA_MEMOIZATION_TABLE;
extern Symbol* SYM_MEMOIZE_STELLA_HASH_TABLE;
extern Symbol* SYM_MEMOIZE_STELLA_TABLE_NAME;
extern Symbol* SYM_MEMOIZE_STELLA_CURRENT_TIMESTAMP;
extern Symbol* SYM_MEMOIZE_STELLA_TIMESTAMPS;
extern Surrogate* SGT_MEMOIZE_STELLA_MRU_MEMOIZATION_TABLE;
extern Symbol* SYM_MEMOIZE_STELLA_MRU_BUCKETS_VECTOR;
extern Symbol* SYM_MEMOIZE_STELLA_LRU_BUCKETS_VECTOR;
extern Symbol* SYM_MEMOIZE_STELLA_MRU_TIMESTAMP;
extern Symbol* SYM_MEMOIZE_STELLA_LRU_TIMESTAMP;
extern Symbol* SYM_MEMOIZE_STELLA_NOF_BUCKETS;
extern Symbol* SYM_MEMOIZE_STELLA_FREE_ENTRIES;
extern Symbol* SYM_MEMOIZE_STELLA_MAX_ENTRIES;
extern Keyword* KWD_MEMOIZE_MAX_VALUES;
extern Keyword* KWD_MEMOIZE_TIMESTAMPS;
extern Keyword* KWD_MEMOIZE_NAME;
extern Surrogate* SGT_MEMOIZE_STELLA_GENERALIZED_SYMBOL;
extern Surrogate* SGT_MEMOIZE_STELLA_STANDARD_OBJECT;
extern Surrogate* SGT_MEMOIZE_STELLA_BOOLEAN_WRAPPER;
extern Surrogate* SGT_MEMOIZE_STELLA_BOOLEAN;
extern Symbol* SYM_MEMOIZE_STELLA_LOOKUP_MEMOIZED_VALUE;
extern Symbol* SYM_MEMOIZE_STELLA_LOOKUP_MEMOIZED_VALUEN;
extern Symbol* SYM_MEMOIZE_STELLA_LOOKUP_MRU_MEMOIZED_VALUE;
extern Symbol* SYM_MEMOIZE_STELLA_LOOKUP_MRU_MEMOIZED_VALUEN;
extern Surrogate* SGT_MEMOIZE_STELLA_OBJECT;
extern Symbol* SYM_MEMOIZE_STELLA_MEMOIZED_NULL_VALUE;
extern Symbol* SYM_MEMOIZE_STELLA_NULL;
extern Symbol* SYM_MEMOIZE_STELLA_CONS_LIST;
extern Symbol* SYM_MEMOIZE_STELLA_VRLET;
extern Symbol* SYM_MEMOIZE_STELLA_MEMOIZATION_TABLE;
extern Symbol* SYM_MEMOIZE_STELLA_SURROGATE_VALUE;
extern Symbol* SYM_MEMOIZE_STELLA_CONS;
extern Symbol* SYM_MEMOIZE_STELLA_OBJECT;
extern Symbol* SYM_MEMOIZE_STELLA_WHEN;
extern Symbol* SYM_MEMOIZE_STELLA_NULLp;
extern Symbol* SYM_MEMOIZE_STELLA_INITIALIZE_MEMOIZATION_TABLE;
extern Symbol* SYM_MEMOIZE_STELLA_SETQ;
extern Symbol* SYM_MEMOIZE_STELLA_oMEMOIZATION_ENABLEDpo;
extern Symbol* SYM_MEMOIZE_STELLA_FIRST;
extern Symbol* SYM_MEMOIZE_STELLA_COND;
extern Symbol* SYM_MEMOIZE_STELLA_DEFINEDp;
extern Symbol* SYM_MEMOIZE_STELLA_EQLp;
extern Symbol* SYM_MEMOIZE_STELLA_OTHERWISE;
extern Symbol* SYM_MEMOIZE_STELLA_SETF;
extern Symbol* SYM_MEMOIZE_STELLA_VALUE;
extern Symbol* SYM_MEMOIZE_STELLA_CHOOSE;
extern Symbol* SYM_MEMOIZE_STELLA_MEMOIZE;
extern Symbol* SYM_MEMOIZE_STELLA_MEMOIZE_EXPANDER;
extern Symbol* SYM_MEMOIZE_STELLA_STARTUP_MEMOIZE;
extern Symbol* SYM_MEMOIZE_STELLA_METHOD_STARTUP_CLASSNAME;


} // end of namespace stella
