// /tmp/build-stella/stella-3.0.0.beta/native/cpp/stella/primal.hh

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/



namespace stella {

// Global declarations:
extern boolean TRUE;
extern boolean FALSE;
extern double PI;
extern int NULL_INTEGER;
extern short int NULL_SHORT_INTEGER;
extern long int NULL_LONG_INTEGER;
extern unsigned short int NULL_UNSIGNED_SHORT_INTEGER;
extern unsigned long int NULL_UNSIGNED_LONG_INTEGER;
extern double NULL_FLOAT;
extern float NULL_SINGLE_FLOAT;
extern double NULL_DOUBLE_FLOAT;
extern char NULL_CHARACTER;
extern char NULL_BYTE;
extern char NULL_OCTET;
extern DECLARE_STELLA_SPECIAL(oTRANSIENTOBJECTSpo, boolean );
extern int* oHASH_BYTE_RANDOM_TABLEo;
extern List* oHASH_TABLE_SIZE_PRIMESo;

// Function signatures:
boolean eqlP(Object* x, Object* y);
boolean equalP(Object* x, Object* y);
boolean zeroP(int x);
boolean plusP(int x);
boolean evenP(int x);
boolean oddP(int x);
int mod(int x, int modulo);
int random_(int n);
void seed_random_number_generator();
double sqrt_(double n);
double cos_(double n);
double sin_(double n);
double tan_(double n);
double log_(double n);
int min_(int x, int y);
int max_(int x, int y);
int integer_abs(int x);
double float_abs(double x);
int character_code(char character);
char code_character(int code);
char character_downcase(char c);
char character_upcase(char c);
char character_capitalize(char c);
boolean string_eqlP(char* x, char* y);
boolean string_equalP(char* x, char* y);
int string_compare(char* x, char* y, boolean case_sensitiveP);
boolean stringL(char* x, char* y);
boolean stringLE(char* x, char* y);
boolean stringGE(char* x, char* y);
boolean stringG(char* x, char* y);
boolean string_lessP(char* x, char* y);
boolean string_less_equalP(char* x, char* y);
boolean string_greater_equalP(char* x, char* y);
boolean string_greaterP(char* x, char* y);
char* make_mutable_string(int size, char initchar);
char* make_raw_mutable_string(int size);
char* string_concatenate(char* string1, char* string2, int otherstrings, ...);
char* replace_substrings(char* string, char* renamed_New, char* old);
int insert_string(char* source, int start, int end, char* target, int target_index, Keyword* case_conversion);
char* character_to_string(char c);
char* format_with_padding(char* input, int length, char padchar, Keyword* align, boolean truncateP);
char string_nth(char* self, int position);
char mutable_string_nth(char* self, int position);
char mutable_string_nth_setter(char* self, char ch, int position);
int string_length(char* self);
int mutable_string_length(char* self);
Object* unstringify(char* string);
int object_hash_code(Object* self);
int string_hash_code(char* self);
int integer_hash_code(int self);
int float_hash_code(double self);
int character_hash_code(char self);
int hash_string(char* string, int seedcode);
int pick_hash_table_size_prime(int minsize);
void resize_vector(Vector* self, int size);
Object* native_vector_nth(Object** self, int position);
Object* native_vector_nth_setter(Object** self, Object* value, int position);
void unmake(Object* self);
void copy_file(char* fromfile, char* tofile);
int lognot(int arg);
int logand(int arg1, int arg2);
int logor(int arg1, int arg2);
int logxor(int arg1, int arg2);
int shift_left(int arg, int count);
int shift_right(int arg, int count);
Object* stellify(Object* self);
boolean stella_objectP(Object* self);
boolean running_as_lispP();
void sleep_(double seconds);
char* exception_message(exception* e);
void print_exception_context(exception* e, Output_Stream* stream);
char* make_process_lock();
void help_startup_primal1();
void help_startup_primal2();
void help_startup_primal3();
void help_startup_primal4();
void startup_primal();

// Auxiliary global declarations:
extern Symbol* SYM_PRIMAL_STELLA_FLOOR;
extern Keyword* KWD_PRIMAL_CPP;
extern Keyword* KWD_PRIMAL_FUNCTION;
extern Symbol* SYM_PRIMAL_STELLA_RANDOM;
extern Symbol* SYM_PRIMAL_STELLA_SQRT;
extern Symbol* SYM_PRIMAL_STELLA_COS;
extern Symbol* SYM_PRIMAL_STELLA_SIN;
extern Symbol* SYM_PRIMAL_STELLA_TAN;
extern Symbol* SYM_PRIMAL_STELLA_LOG;
extern Symbol* SYM_PRIMAL_STELLA_MIN;
extern Symbol* SYM_PRIMAL_STELLA_MAX;
extern Keyword* KWD_PRIMAL_UPCASE;
extern Keyword* KWD_PRIMAL_DOWNCASE;
extern Keyword* KWD_PRIMAL_CAPITALIZE;
extern Keyword* KWD_PRIMAL_PRESERVE;
extern Keyword* KWD_PRIMAL_LEFT;
extern Keyword* KWD_PRIMAL_RIGHT;
extern Keyword* KWD_PRIMAL_CENTER;
extern Symbol* SYM_PRIMAL_STELLA_HASH_CODE;
extern Keyword* KWD_PRIMAL_JAVA;
extern Symbol* SYM_PRIMAL_STELLA_SLEEP;
extern Symbol* SYM_PRIMAL_STELLA_STARTUP_PRIMAL;
extern Symbol* SYM_PRIMAL_STELLA_METHOD_STARTUP_CLASSNAME;


} // end of namespace stella
