// /tmp/build-stella/stella-3.0.0.beta/native/cpp/stella/read.hh

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/


namespace stella {

// Class definitions:
class Tokenizer_Table : public Standard_Object {
public:
  char* transitions;
  Vector* unique_state_names;
  Vector* state_names;
  Vector* legal_eof_states;
public:
  virtual Surrogate* primary_type();
};

class Tokenizer_Token : public Standard_Object {
public:
  Keyword* type;
  char* content;
  Tokenizer_Token* next;
public:
  virtual Surrogate* primary_type();
};

class Tokenizer_Stream_State : public Standard_Object {
public:
  char* buffer;
  int buffer_size;
  int cursor;
  int end;
  int state;
  Tokenizer_Token* token_list;
public:
  virtual Surrogate* primary_type();
};

class Stream_Tokenizer : public Iterator {
// Iterator that generates tokens by tokenizing a `stream'
// according to a particular tokenization `table'.
public:
  Input_Stream* stream;
  Tokenizer_Table* table;
public:
  virtual boolean nextP();
  virtual Surrogate* primary_type();
};

class Stella_Token : public Tokenizer_Token {
public:
  Keyword* logical_token_type;
  char* module;
  Keyword* escape_mode;
public:
  virtual Surrogate* primary_type();
};

class Xml_Expression_Iterator : public Stream_Iterator {
// Iterator that yields XML expressions from an input stream.
// If the value of `region-tag' is non-`null', only XML expressions enclosed within
// matching region tags will be returned (there can be multiple such regions).
// What is a match is determined by `region-match-function'.  Usually, region tags
// will/should be at the top level, but this is not necessarily so and also not
// enforced by the iterator.
public:
  Cons* region_tag;
  cpp_function_code region_match_function;
  Object* value;
public:
  virtual void free();
  virtual boolean nextP();
  virtual Surrogate* primary_type();
};


// Global declarations:
extern int oMAX_TOKENIZER_STATESo;
extern int oMAX_TOKENIZER_CHARACTERSo;
extern int oTOKENIZER_INITIAL_BUFFER_SIZEo;
extern DECLARE_STELLA_SPECIAL(oWITHTOKENIZERINPUTTYPEo, Surrogate* );
extern Cons* oSTELLA_TOKENIZER_TABLE_DEFINITIONo;
extern Tokenizer_Table* oSTELLA_TOKENIZER_TABLEo;
extern Vector* oSTELLA_LOGICAL_STATE_NAMESo;
extern Property_List* oSTELLA_LOGICAL_STATE_NAMES_TABLEo;
extern int oSTELLA_TOKENIZER_WHITE_SPACE_STATEo;
extern DECLARE_STELLA_SPECIAL(oUSER_QUERY_ACTIONo, Keyword* );
extern Cons* oXML_TOKENIZER_TABLE_DEFINITIONo;
extern Tokenizer_Table* oXML_TOKENIZER_TABLEo;

// Function signatures:
Tokenizer_Table* new_tokenizer_table();
Object* access_tokenizer_table_slot_value(Tokenizer_Table* self, Symbol* slotname, Object* value, boolean setvalueP);
Tokenizer_Table* parse_tokenizer_definition(Cons* definition);
boolean legal_tokenizer_stateP(Object* x);
boolean legal_tokenizer_from_stateP(Object* x);
boolean legal_tokenizer_to_stateP(Object* x);
Generalized_Symbol* tokenizer_to_state_name(Object* state);
Generalized_Symbol* tokenizer_to_state_alias(Object* state);
int parse_tokenizer_state_modifiers(Object* modifier);
boolean tokenizer_include_specP(Object* x);
Cons* parse_tokenizer_character_spec(Object* characterspec, List* allcharactersets);
char* stringify_tokenizer_table(Tokenizer_Table* table);
Tokenizer_Table* unstringify_tokenizer_table(char* table);
char* make_tokenizer_byte_array(int size);
char* string_to_tokenizer_byte_array(char* string);
char* tokenizer_byte_array_to_string(char* bytes);
char tokenizer_byte_array_byte_array_nth(char* buffer, int position);
char tokenizer_byte_array_byte_array_nth_setter(char* buffer, char ch, int position);
int byte_array_read_sequence(char* buffer, istream* stream, int start, int end);
void ensure_tokenizer_buffer_size(Tokenizer_Stream_State* state, int currenttokenstart, int requiredspace);
boolean read_into_tokenizer_buffer(Input_Stream* stream, Tokenizer_Stream_State* state, int currenttokenstart);
char read_character_from_tokenizer_buffer(Input_Stream* stream, boolean& _return1);
void unread_character_from_tokenizer_buffer(char renamed_Char, Input_Stream* stream);
char* read_line_from_tokenizer_buffer(Input_Stream* stream, boolean& _return1);
Tokenizer_Token* new_tokenizer_token();
Object* access_tokenizer_token_slot_value(Tokenizer_Token* self, Symbol* slotname, Object* value, boolean setvalueP);
Tokenizer_Stream_State* new_tokenizer_stream_state();
Object* access_tokenizer_stream_state_slot_value(Tokenizer_Stream_State* self, Symbol* slotname, Object* value, boolean setvalueP);
Object* with_tokenizer_expander(Object* table, Object* input, Cons* body);
boolean with_tokenizer_string_inputP();
boolean inside_with_tokenizerP();
Object* end_of_tokensP_expander();
Object* get_token_text_expander(Cons* options);
char* get_token_text_internal(char* buffer, int start, int end, int size, boolean upcaseP);
Object* get_token_type_expander();
Object* save_tokenizer_stream_state_expander();
Object* get_next_token_expander(Cons* options);
char* unescape_token_string(char* token, char escapechar, boolean upcaseP);
Stream_Tokenizer* new_stream_tokenizer(Input_Stream* stream, Tokenizer_Table* table);
Object* access_stream_tokenizer_slot_value(Stream_Tokenizer* self, Symbol* slotname, Object* value, boolean setvalueP);
Object* with_stella_tokenizer_expander(Object* input, Cons* body);
Object* get_next_stella_token_expander(Cons* options);
Object* get_stella_token_type_expander();
Object* get_qualified_symbol_separator_position_expander();
int get_qualified_symbol_separator_position_internal(char* buffer, int tokenstart, int tokenend, int size);
Stella_Token* new_stella_token();
Object* access_stella_token_slot_value(Stella_Token* self, Symbol* slotname, Object* value, boolean setvalueP);
Stella_Token* tokenize_s_expression(Input_Stream* stream, Stella_Token* tokenlist);
Object* stella_token_list_to_s_expression(Stella_Token* tokenlist);
void expand_quote_macro_token(Stella_Token* quotedlist);
char string_to_character(char* name);
Object* read_s_expression(Input_Stream* stream, boolean& _return1);
boolean eat_next_character_if_whitespace(Input_Stream* stream);
boolean consume_whitespace(Input_Stream* stream);
Object* read_s_expression_from_string(char* string);
Input_String_Stream* make_tokenizer_string_stream(char* string);
char* read_line(Input_Stream* inputstream, boolean& _return1);
char read_character(Input_Stream* inputstream, boolean& _return1);
void unread_character(char ch, Input_Stream* inputstream);
boolean yes_or_noP(char* message);
boolean y_or_nP(char* message);
Tokenizer_Token* tokenize_xml_expression(Input_Stream* stream, Tokenizer_Token* tokenlist, char* regiontagname, boolean skiptoregionP, boolean& _return1);
Object* xml_token_list_to_s_expression(Stella_Token* tokenlist);
Object* read_xml_expression(Input_Stream* stream, char* starttagname, boolean& _return1);
void print_xml_expression(Output_Stream* stream, Cons* xml_expression);
Xml_Expression_Iterator* new_xml_expression_iterator();
Object* access_xml_expression_iterator_slot_value(Xml_Expression_Iterator* self, Symbol* slotname, Object* value, boolean setvalueP);
boolean xml_region_matchesP(Cons* regionspec, Cons* regiontag);
Xml_Expression_Iterator* xml_expressions(Input_Stream* stream, Cons* regiontag);
void help_startup_read1();
void help_startup_read2();
void help_startup_read3();
void help_startup_read4();
void help_startup_read5();
void help_startup_read6();
void help_startup_read7();
void startup_read();

// Auxiliary global declarations:
extern Surrogate* SGT_READ_STELLA_TOKENIZER_TABLE;
extern Symbol* SYM_READ_STELLA_TRANSITIONS;
extern Symbol* SYM_READ_STELLA_UNIQUE_STATE_NAMES;
extern Symbol* SYM_READ_STELLA_STATE_NAMES;
extern Symbol* SYM_READ_STELLA_LEGAL_EOF_STATES;
extern Symbol* SYM_READ_STELLA_CHARACTER_SET;
extern Keyword* KWD_READ_ERROR;
extern Keyword* KWD_READ_START;
extern Keyword* KWD_READ_EOF;
extern Surrogate* SGT_READ_STELLA_GENERALIZED_SYMBOL;
extern Surrogate* SGT_READ_STELLA_CONS;
extern Keyword* KWD_READ_BLOCK;
extern Keyword* KWD_READ_LINE;
extern Keyword* KWD_READ_CHARACTER;
extern Surrogate* SGT_READ_STELLA_TOKENIZER_TOKEN;
extern Symbol* SYM_READ_STELLA_TYPE;
extern Symbol* SYM_READ_STELLA_CONTENT;
extern Symbol* SYM_READ_STELLA_NEXT;
extern Surrogate* SGT_READ_STELLA_TOKENIZER_STREAM_STATE;
extern Symbol* SYM_READ_STELLA_BUFFER_SIZE;
extern Symbol* SYM_READ_STELLA_CURSOR;
extern Symbol* SYM_READ_STELLA_END;
extern Symbol* SYM_READ_STELLA_STATE;
extern Symbol* SYM_READ_STELLA_TOKEN_LIST;
extern Surrogate* SGT_READ_STELLA_INPUT_STREAM;
extern Surrogate* SGT_READ_STELLA_STRING;
extern Surrogate* SGT_READ_STELLA_VOID;
extern Surrogate* SGT_READ_STELLA_MUTABLE_STRING;
extern Symbol* SYM_READ_STELLA_LET;
extern Symbol* SYM_READ_STELLA_TOK_TABLE_;
extern Symbol* SYM_READ_STELLA_TOK_TRANSITIONS_;
extern Symbol* SYM_READ_STELLA_TOK_STATENAMES_;
extern Symbol* SYM_READ_STELLA_THE_ARRAY;
extern Symbol* SYM_READ_STELLA_TOK_TOKENSTART_;
extern Symbol* SYM_READ_STELLA_TOK_ENDOFTOKENSp_;
extern Symbol* SYM_READ_STELLA_FALSE;
extern Symbol* SYM_READ_STELLA_TOK_STREAMSTATE_;
extern Symbol* SYM_READ_STELLA_TOKENIZER_STREAM_STATE;
extern Symbol* SYM_READ_STELLA_NULL;
extern Symbol* SYM_READ_STELLA_TOK_BUFFER_;
extern Symbol* SYM_READ_STELLA_STRING_TO_TOKENIZER_BYTE_ARRAY;
extern Symbol* SYM_READ_STELLA_TOK_STATE_;
extern Symbol* SYM_READ_STELLA_TOK_NEXTSTATE_;
extern Symbol* SYM_READ_STELLA_TOK_CURSOR_;
extern Symbol* SYM_READ_STELLA_TOK_SIZE_;
extern Symbol* SYM_READ_STELLA_LENGTH;
extern Symbol* SYM_READ_STELLA_TOK_END_;
extern Symbol* SYM_READ_STELLA_TOK_INPUTSTREAM_;
extern Symbol* SYM_READ_STELLA_TOK_ECHOSTREAM_;
extern Symbol* SYM_READ_STELLA_ECHO_STREAM;
extern Symbol* SYM_READ_STELLA_CHOOSE;
extern Symbol* SYM_READ_STELLA_NULLp;
extern Symbol* SYM_READ_STELLA_TOKENIZER_STATE;
extern Symbol* SYM_READ_STELLA_SETF;
extern Symbol* SYM_READ_STELLA_NEW;
extern Symbol* SYM_READ_STELLA_BUFFER;
extern Symbol* SYM_READ_STELLA_TOK_CHECKPOINT_;
extern Symbol* SYM_READ_STELLA_le;
extern Symbol* SYM_READ_STELLA_WITH_TOKENIZER;
extern Symbol* SYM_READ_STELLA_WITH_TOKENIZER_EXPANDER;
extern Symbol* SYM_READ_STELLA_END_OF_TOKENSp;
extern Symbol* SYM_READ_STELLA_END_OF_TOKENSp_EXPANDER;
extern Symbol* SYM_READ_STELLA_GET_TOKEN_TEXT;
extern Symbol* SYM_READ_STELLA_GET_TOKEN_TEXT_INTERNAL;
extern Symbol* SYM_READ_STELLA_i;
extern Symbol* SYM_READ_STELLA_l;
extern Symbol* SYM_READ_STELLA_GET_TOKEN_TEXT_EXPANDER;
extern Symbol* SYM_READ_STELLA_GET_TOKEN_TYPE;
extern Symbol* SYM_READ_STELLA_NTH;
extern Symbol* SYM_READ_STELLA_GET_TOKEN_TYPE_EXPANDER;
extern Symbol* SYM_READ_STELLA_WHEN;
extern Symbol* SYM_READ_STELLA_DEFINEDp;
extern Symbol* SYM_READ_STELLA_SAVE_TOKENIZER_STREAM_STATE;
extern Symbol* SYM_READ_STELLA_SAVE_TOKENIZER_STREAM_STATE_EXPANDER;
extern Symbol* SYM_READ_STELLA_GET_NEXT_TOKEN;
extern Symbol* SYM_READ_STELLA_TRUE;
extern Symbol* SYM_READ_STELLA_PROGN;
extern Symbol* SYM_READ_STELLA_SETQ;
extern Symbol* SYM_READ_STELLA_LOOP;
extern Symbol* SYM_READ_STELLA_CHARACTER_CODE;
extern Symbol* SYM_READ_STELLA_LOGOR;
extern Symbol* SYM_READ_STELLA_SHIFT_LEFT;
extern Symbol* SYM_READ_STELLA_BYTE_ARRAY_NTH;
extern Symbol* SYM_READ_STELLA_COND;
extern Symbol* SYM_READ_STELLA_e;
extern Symbol* SYM_READ_STELLA_LOGAND;
extern Symbol* SYM_READ_STELLA_ii;
extern Symbol* SYM_READ_STELLA_OTHERWISE;
extern Symbol* SYM_READ_STELLA_BREAK;
extern Symbol* SYM_READ_STELLA_READ_INTO_TOKENIZER_BUFFER;
extern Symbol* SYM_READ_STELLA_MOD;
extern Symbol* SYM_READ_STELLA_UNLESS;
extern Symbol* SYM_READ_STELLA_IF;
extern Symbol* SYM_READ_STELLA_ge;
extern Symbol* SYM_READ_STELLA_PRINT_STREAM;
extern Symbol* SYM_READ_STELLA_1_;
extern Symbol* SYM_READ_STELLA_GET_NEXT_TOKEN_EXPANDER;
extern Surrogate* SGT_READ_STELLA_STREAM_TOKENIZER;
extern Symbol* SYM_READ_STELLA_STREAM;
extern Symbol* SYM_READ_STELLA_TABLE;
extern Keyword* KWD_READ_INCLUDE;
extern Keyword* KWD_READ_DELIMITER;
extern Keyword* KWD_READ_ATOM;
extern Symbol* SYM_READ_STELLA_o;
extern Keyword* KWD_READ_OPEN_PAREN;
extern Keyword* KWD_READ_CLOSE_PAREN;
extern Keyword* KWD_READ_OPEN_STRING;
extern Keyword* KWD_READ_STRING;
extern Keyword* KWD_READ_SINGLE_QUOTE;
extern Keyword* KWD_READ_BACK_QUOTE;
extern Keyword* KWD_READ_COMMA;
extern Symbol* SYM_READ_STELLA_x;
extern Keyword* KWD_READ_COMMENT;
extern Keyword* KWD_READ_HASH;
extern Keyword* KWD_READ_WHITE_SPACE;
extern Keyword* KWD_READ_OTHERWISE;
extern Keyword* KWD_READ_PAREN_COMMENT;
extern Keyword* KWD_READ_PAREN_COMMENT_ESCAPE;
extern Keyword* KWD_READ_PAREN_COMMENT_BAR;
extern Keyword* KWD_READ_ANY;
extern Keyword* KWD_READ_COMMA_SPLICE;
extern Keyword* KWD_READ_SYMBOL_OR_SIGNED_NUMBER;
extern Keyword* KWD_READ_SYMBOL;
extern Keyword* KWD_READ_SYMBOL_OR_NUMBER;
extern Keyword* KWD_READ_INTEGER;
extern Keyword* KWD_READ_SYMBOL_OR_CL_SYMBOL;
extern Keyword* KWD_READ_OPEN_FULLY_ESCAPED_SYMBOL;
extern Keyword* KWD_READ_FULLY_ESCAPED_SYMBOL;
extern Keyword* KWD_READ_ESCAPED_SYMBOL_ESCAPE;
extern Keyword* KWD_READ_QUALIFIED_NAME;
extern Keyword* KWD_READ_OPEN_KEYWORD;
extern Keyword* KWD_READ_KEYWORD;
extern Keyword* KWD_READ_OPEN_SURROGATE;
extern Keyword* KWD_READ_SURROGATE;
extern Keyword* KWD_READ_CLOSE_STRING;
extern Keyword* KWD_READ_STRING_ESCAPE;
extern Keyword* KWD_READ_ESCAPED_STRING;
extern Keyword* KWD_READ_CHARACTER_CONSTANT;
extern Keyword* KWD_READ_SYMBOL_OR_MANTISSA;
extern Keyword* KWD_READ_SYMBOL_OR_MANTISSA2;
extern Keyword* KWD_READ_FLOAT;
extern Keyword* KWD_READ_SYMBOL_OR_EXPONENT_DELIMITER;
extern Keyword* KWD_READ_SYMBOL_OR_EXPONENT;
extern Keyword* KWD_READ_SYMBOL_OR_EXPONENT2;
extern Keyword* KWD_READ_SYMBOL_OR_CL_SYMBOL2;
extern Keyword* KWD_READ_SYMBOL_OR_CL_SYMBOL3;
extern Keyword* KWD_READ_CL_SYMBOL;
extern Keyword* KWD_READ_ESCAPED_SYMBOL;
extern Keyword* KWD_READ_CLOSE_FULLY_ESCAPED_NAME;
extern Keyword* KWD_READ_FULLY_ESCAPED_SYMBOL_ESCAPE;
extern Keyword* KWD_READ_FULLY_ESCAPED_KEYWORD;
extern Keyword* KWD_READ_ESCAPED_KEYWORD_ESCAPE;
extern Keyword* KWD_READ_ESCAPED_KEYWORD;
extern Keyword* KWD_READ_FULLY_ESCAPED_KEYWORD_ESCAPE;
extern Keyword* KWD_READ_FULLY_ESCAPED_SURROGATE;
extern Keyword* KWD_READ_ESCAPED_SURROGATE_ESCAPE;
extern Keyword* KWD_READ_ESCAPED_SURROGATE;
extern Keyword* KWD_READ_FULLY_ESCAPED_SURROGATE_ESCAPE;
extern Keyword* KWD_READ_QUALIFIED_SURROGATE;
extern Keyword* KWD_READ_QUALIFIED_ESCAPED_SYMBOL_ESCAPE;
extern Keyword* KWD_READ_QUALIFIED_FULLY_ESCAPED_SYMBOL;
extern Keyword* KWD_READ_QUALIFIED_SYMBOL;
extern Keyword* KWD_READ_QUALIFIED_ESCAPED_SURROGATE_ESCAPE;
extern Keyword* KWD_READ_QUALIFIED_FULLY_ESCAPED_SURROGATE;
extern Keyword* KWD_READ_QUALIFIED_ESCAPED_SYMBOL;
extern Keyword* KWD_READ_QUALIFIED_ESCAPED_SURROGATE;
extern Keyword* KWD_READ_QUALIFIED_FULLY_ESCAPED_SYMBOL_ESCAPE;
extern Keyword* KWD_READ_QUALIFIED_FULLY_ESCAPED_SURROGATE_ESCAPE;
extern Keyword* KWD_READ_CLOSE_BALANCED_QUOTE;
extern Symbol* SYM_READ_STELLA_oSTELLA_TOKENIZER_TABLEo;
extern Symbol* SYM_READ_STELLA_TOK_STELLASTATENAME_;
extern Symbol* SYM_READ_STELLA_KEYWORD;
extern Symbol* SYM_READ_STELLA_TOK_STELLALOGICALSTATENAMES_;
extern Symbol* SYM_READ_STELLA_oSTELLA_LOGICAL_STATE_NAMESo;
extern Symbol* SYM_READ_STELLA_TOK_STELLALOGICALSTATENAME_;
extern Symbol* SYM_READ_STELLA_WITH_STELLA_TOKENIZER;
extern Symbol* SYM_READ_STELLA_WITH_STELLA_TOKENIZER_EXPANDER;
extern Symbol* SYM_READ_STELLA_GET_NEXT_STELLA_TOKEN;
extern Symbol* SYM_READ_STELLA_GET_NEXT_STELLA_TOKEN_EXPANDER;
extern Symbol* SYM_READ_STELLA_GET_STELLA_TOKEN_TYPE;
extern Symbol* SYM_READ_STELLA_GET_STELLA_TOKEN_TYPE_EXPANDER;
extern Symbol* SYM_READ_STELLA_GET_QUALIFIED_SYMBOL_SEPARATOR_POSITION_INTERNAL;
extern Symbol* SYM_READ_STELLA_GET_QUALIFIED_SYMBOL_SEPARATOR_POSITION;
extern Symbol* SYM_READ_STELLA_GET_QUALIFIED_SYMBOL_SEPARATOR_POSITION_EXPANDER;
extern Surrogate* SGT_READ_STELLA_STELLA_TOKEN;
extern Symbol* SYM_READ_STELLA_LOGICAL_TOKEN_TYPE;
extern Symbol* SYM_READ_STELLA_MODULE;
extern Symbol* SYM_READ_STELLA_ESCAPE_MODE;
extern Keyword* KWD_READ_FULL;
extern Keyword* KWD_READ_PARTIAL;
extern Symbol* SYM_READ_STELLA_a;
extern Symbol* SYM_READ_STELLA_aa;
extern Keyword* KWD_READ_ASK;
extern Keyword* KWD_READ_YES;
extern Keyword* KWD_READ_NO;
extern Keyword* KWD_READ_YES_VERBOSE;
extern Keyword* KWD_READ_NO_VERBOSE;
extern Keyword* KWD_READ_SKIP_WHITESPACE;
extern Keyword* KWD_READ_OPEN_TAG;
extern Keyword* KWD_READ_CONTENT;
extern Keyword* KWD_READ_START_TAG_END;
extern Keyword* KWD_READ_OPEN_END_TAG;
extern Keyword* KWD_READ_OPEN_PI_TAG;
extern Keyword* KWD_READ_OPEN_DECLARATION_TAG;
extern Keyword* KWD_READ_START_TAG;
extern Keyword* KWD_READ_OPEN_EMPTY_TAG_END;
extern Keyword* KWD_READ_SKIP_TO_ATTRIBUTE_NAME;
extern Keyword* KWD_READ_START_PI_TAG;
extern Keyword* KWD_READ_OPEN_PI_TAG_END;
extern Keyword* KWD_READ_PI_TAG_DATA;
extern Keyword* KWD_READ_PI_TAG_DATA_OR_END;
extern Keyword* KWD_READ_EMPTY_TAG_END;
extern Keyword* KWD_READ_END_TAG_END;
extern Keyword* KWD_READ_END_TAG;
extern Keyword* KWD_READ_ATTRIBUTE_NAME;
extern Keyword* KWD_READ_SKIP_TO_ATTRIBUTE_VALUE;
extern Keyword* KWD_READ_SINGLE_QUOTED_ATTRIBUTE_VALUE;
extern Keyword* KWD_READ_DOUBLE_QUOTED_ATTRIBUTE_VALUE;
extern Keyword* KWD_READ_UNQUOTED_ATTRIBUTE_VALUE;
extern Keyword* KWD_READ_QUOTED_ATTRIBUTE_VALUE;
extern Keyword* KWD_READ_START_TAG_OR_COMMENT;
extern Keyword* KWD_READ_OPEN_SPECIAL_TAG;
extern Keyword* KWD_READ_START_DECLARATION_TAG;
extern Keyword* KWD_READ_DECLARATION_TAG_DATA;
extern Keyword* KWD_READ_SINGLE_QUOTED_DECLARATION_TAG_DATA;
extern Keyword* KWD_READ_DOUBLE_QUOTED_DECLARATION_TAG_DATA;
extern Keyword* KWD_READ_START_SPECIAL_TAG;
extern Keyword* KWD_READ_SPECIAL_TAG_DATA;
extern Keyword* KWD_READ_SPECIAL_TAG_DATA_OR_END;
extern Keyword* KWD_READ_SPECIAL_TAG_DATA_OR_END2;
extern Keyword* KWD_READ_COMMENT_BODY;
extern Keyword* KWD_READ_END_COMMENT_OR_COMMENT;
extern Keyword* KWD_READ_END_COMMENT_OR_COMMENT2;
extern Keyword* KWD_READ_ATTRIBUTE_VALUE;
extern Symbol* SYM_READ_STELLA_p;
extern Symbol* SYM_READ_STELLA_j;
extern Surrogate* SGT_READ_STELLA_XML_EXPRESSION_ITERATOR;
extern Symbol* SYM_READ_STELLA_REGION_TAG;
extern Symbol* SYM_READ_STELLA_REGION_MATCH_FUNCTION;
extern Symbol* SYM_READ_STELLA_STARTUP_READ;
extern Symbol* SYM_READ_STELLA_METHOD_STARTUP_CLASSNAME;


} // end of namespace stella
