// /tmp/build-stella/stella-3.0.0.beta/native/cpp/stella/systems.hh

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/



namespace stella {

// Global declarations:
extern List* oSYSTEMDEFINITIONSo;
extern Key_Value_List* oSYSTEM_CONFIGURATION_TABLEo;

// Function signatures:
Object* evaluate_command(Object* command, boolean finalizeP);
void load_file(char* file);
void load_file_evaluator_wrapper(Cons* arguments);
char* make_system_definition_file_name(char* name);
Cons* parse_list_of_file_paths(Cons* files);
System_Definition* define_system(Object* name, Cons* options);
System_Definition* defsystem(Symbol* name, Cons* options);
System_Definition* defsystem_evaluator_wrapper(Cons* arguments);
System_Definition* get_system_definition(char* name);
Module* get_cardinal_module(System_Definition* system);
void help_get_system_files(char* filename, List* collection, boolean probefileP);
List* get_system_files(System_Definition* system, Keyword* type, boolean probefilesP);
void clean_system(char* systemname);
void compute_optimization_levels(System_Definition* system, boolean productionmodeP);
Keyword* lisp_keyword_to_stella_keyword(Keyword* key);
Property_List* vet_options(Object* plist, Cons* legaloptions);
boolean make_system(char* systemname, Keyword* language, Cons* options);
Boolean_Wrapper* make_system_evaluator_wrapper(Cons* arguments);
void run_system_finalization(System_Definition* system);
Cons* system_definition_source_files(System_Definition* system);
Cons* files_plus_system_startup(Cons* files);
Cons* system_definition_native_files(System_Definition* system);
boolean system_loadedP(char* name);
void load_preprocessed_files(char* systemname);
boolean load_system(char* systemname, Keyword* language, int options, ...);
boolean compile_and_load_file(char* file, Keyword* language, boolean forcerecompilationP);
void run_system_startup_function(System_Definition* system);
char* string_system_startup_function_name(char* systemname);
Symbol* system_startup_function_symbol(System_Definition* system);
Symbol* startup_name_from_file(char* file);
Cons* startup_names_from_files(Cons* files);
char* system_startup_file_name(System_Definition* system);
boolean system_startup_fileP(char* file);
boolean system_started_upP(char* systemname, char* systemmodulename);
Cons* yield_startup_required_systems(System_Definition* system);
Cons* collect_startup_forms_from_system_file(System_Definition* system);
void create_system_startup_file(System_Definition* system);
void make_stella(boolean forcerecompilationP);
String_Wrapper* parse_configuration_file_line(char* line, Wrapper*& _return1);
Key_Value_List* load_configuration_file(char* file);
Wrapper* lookup_configuration_property(char* property, Wrapper* defaultvalue, Key_Value_List* configuration);
Wrapper* set_configuration_property(char* property, Wrapper* value, Key_Value_List* configuration);
void configure_stella(char* file);
void help_startup_systems1();
void startup_systems();

// Auxiliary global declarations:
extern Keyword* KWD_SYSTEMS_DEFINE;
extern Surrogate* SGT_SYSTEMS_STELLA_CONS;
extern Symbol* SYM_SYSTEMS_STELLA_IN_MODULE;
extern Keyword* KWD_SYSTEMS_FINALIZE;
extern Symbol* SYM_SYSTEMS_STELLA_DEFMODULE;
extern Symbol* SYM_SYSTEMS_STELLA_DEFSYSTEM;
extern Symbol* SYM_SYSTEMS_STELLA_DEFCLASS;
extern Symbol* SYM_SYSTEMS_STELLA_DEFSLOT;
extern Symbol* SYM_SYSTEMS_STELLA_DEFUN;
extern Symbol* SYM_SYSTEMS_STELLA_DEFMETHOD;
extern Keyword* KWD_SYSTEMS_DIRECTORY;
extern Keyword* KWD_SYSTEMS_FILES;
extern Keyword* KWD_SYSTEMS_REQUIRED_SYSTEMS;
extern Keyword* KWD_SYSTEMS_LISP_ONLY_FILES;
extern Keyword* KWD_SYSTEMS_CPP_ONLY_FILES;
extern Keyword* KWD_SYSTEMS_JAVA_ONLY_FILES;
extern Keyword* KWD_SYSTEMS_PREPROCESSED_FILES;
extern Keyword* KWD_SYSTEMS_CARDINAL_MODULE;
extern Keyword* KWD_SYSTEMS_ROOT_SOURCE_DIRECTORY;
extern Keyword* KWD_SYSTEMS_ROOT_NATIVE_DIRECTORY;
extern Keyword* KWD_SYSTEMS_ROOT_BINARY_DIRECTORY;
extern Keyword* KWD_SYSTEMS_BANNER;
extern Keyword* KWD_SYSTEMS_COPYRIGHT_HEADER;
extern Keyword* KWD_SYSTEMS_PRODUCTION_SETTINGS;
extern Keyword* KWD_SYSTEMS_DEVELOPMENT_SETTINGS;
extern Keyword* KWD_SYSTEMS_FINALIZATION_FUNCTION;
extern Keyword* KWD_SYSTEMS_JAVA;
extern Keyword* KWD_SYSTEMS_LISP;
extern Keyword* KWD_SYSTEMS_CPP;
extern Keyword* KWD_SYSTEMS_LISP_BINARY;
extern Surrogate* SGT_SYSTEMS_STELLA_PROPERTY_LIST;
extern Surrogate* SGT_SYSTEMS_STELLA_LIST;
extern Keyword* KWD_SYSTEMS_COMMON_LISP;
extern Keyword* KWD_SYSTEMS_DEVELOPMENT_SETTINGSp;
extern Keyword* KWD_SYSTEMS_FORCE_TRANSLATIONp;
extern Keyword* KWD_SYSTEMS_FORCE_RECOMPILATIONp;
extern Keyword* KWD_SYSTEMS_TWO_PASSp;
extern Keyword* KWD_SYSTEMS_LOAD_SYSTEMp;
extern Keyword* KWD_SYSTEMS_STARTUPp;
extern Keyword* KWD_SYSTEMS_USE_COMMON_LISP_STRUCTS;
extern Keyword* KWD_SYSTEMS_STELLA;
extern Keyword* KWD_SYSTEMS_WARN_ABOUT_UNDEFINED_METHODS;
extern Symbol* SYM_SYSTEMS_STELLA_UNLESS;
extern Symbol* SYM_SYSTEMS_STELLA_SYSTEM_STARTED_UPp;
extern Symbol* SYM_SYSTEMS_STELLA_STARTUP_TIME_PROGN;
extern Keyword* KWD_SYSTEMS_EARLY_INITS;
extern Keyword* KWD_SYSTEMS_MODULES;
extern Keyword* KWD_SYSTEMS_PUBLICp;
extern Symbol* SYM_SYSTEMS_STELLA_TRUE;
extern Symbol* SYM_SYSTEMS_STELLA_STARTUP;
extern Symbol* SYM_SYSTEMS_STELLA_FALSE;
extern Symbol* SYM_SYSTEMS_STELLA_FOREACH;
extern Symbol* SYM_SYSTEMS_STELLA_PHASE;
extern Symbol* SYM_SYSTEMS_STELLA_IN;
extern Symbol* SYM_SYSTEMS_STELLA_INTERVAL;
extern Symbol* SYM_SYSTEMS_STELLA_PHASE_TO_INTEGER;
extern Keyword* KWD_SYSTEMS_FINAL;
extern Symbol* SYM_SYSTEMS_STELLA_DO;
extern Symbol* SYM_SYSTEMS_STELLA_SETQ;
extern Symbol* SYM_SYSTEMS_STELLA_oSTARTUP_TIME_PHASEo;
extern Keyword* KWD_SYSTEMS_WHITE_SPACE;
extern Symbol* SYM_SYSTEMS_STELLA_CONFIGURATION_TABLE;
extern Keyword* KWD_SYSTEMS_ROOT_DIRECTORY;
extern Symbol* SYM_SYSTEMS_STELLA_STARTUP_SYSTEMS;
extern Symbol* SYM_SYSTEMS_STELLA_METHOD_STARTUP_CLASSNAME;


} // end of namespace stella
