// /tmp/build-stella/stella-3.0.0.beta/native/cpp/stella/taxonomies.hh

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/



namespace stella {

// Global declarations:
extern int oNUMBERING_INTERVALo;
extern int MARKER_LABEL;
extern int DELETED_LABEL;
extern DECLARE_STELLA_SPECIAL(oTAXONOMY_POSTORDER_NUMBERo, int );

// Function signatures:
boolean taxonomy_rootP(Taxonomy_Node* node);
void add_taxonomy_root(Taxonomy_Graph* graph, Taxonomy_Node* root);
void remove_taxonomy_root(Taxonomy_Graph* graph, Taxonomy_Node* root);
void update_taxonomy_roots(Taxonomy_Graph* graph, Taxonomy_Node* node);
Taxonomy_Node* create_taxonomy_node(Taxonomy_Graph* graph, Taxonomy_Node* node, Object* nativeobject, boolean rootP);
void add_taxonomy_node(Taxonomy_Graph* graph, Taxonomy_Node* node, boolean rootP);
void remove_taxonomy_node(Taxonomy_Graph* graph, Taxonomy_Node* node);
void create_taxonomy_link(Taxonomy_Graph* graph, Taxonomy_Node* parent, Taxonomy_Node* child);
void link_taxonomy_nodes(Taxonomy_Graph* graph, Taxonomy_Node* parent, Taxonomy_Node* child);
void incrementally_link_taxonomy_nodes(Taxonomy_Graph* graph, Taxonomy_Node* parent, Taxonomy_Node* child);
Interval* create_next_postorder_interval(Taxonomy_Graph* graph);
Interval* allocate_interval_for_new_leaf_node(Taxonomy_Node* parent);
void unlink_taxonomy_nodes(Taxonomy_Graph* graph, Taxonomy_Node* parent, Taxonomy_Node* child);
void incrementally_unlink_taxonomy_nodes(Taxonomy_Graph* graph, Taxonomy_Node* parent, Taxonomy_Node* child);
Iterator* all_taxonomy_nodes(Taxonomy_Graph* graph);
boolean all_taxonomy_nodes_nextP(All_Purpose_Iterator* self);
Taxonomy_Node* find_taxonomy_node(Taxonomy_Graph* graph, int label);
boolean labeled_taxonomy_nodeP(Taxonomy_Node* node);
void clear_taxonomy_node(Taxonomy_Node* node);
void initialize_taxonomy_node(Taxonomy_Node* node);
void help_clear_taxonomy_graph(Taxonomy_Node* node);
void clear_taxonomy_graph(Taxonomy_Graph* graph);
void initialize_taxonomy_graph(Taxonomy_Graph* graph);
int help_create_taxonomy_tree_intervals(Taxonomy_Node* self);
void create_taxonomy_tree_intervals(Taxonomy_Graph* graph);
boolean subinterval_ofP(Interval* subinterval, Interval* superinterval);
Interval* merge_intervals(Interval* leftinterval, Interval* rightinterval);
Cons* all_taxonomy_node_intervals(Taxonomy_Node* node);
Interval* intern_taxonomy_node_interval(Taxonomy_Node* node, int lowerbound, int upperbound);
void add_taxonomy_node_interval(Taxonomy_Node* node, Interval* interval);
void remove_taxonomy_node_interval(Taxonomy_Node* node, Interval* interval);
boolean adjoin_taxonomy_node_intervalP(Taxonomy_Node* node, Interval* interval);
void propagate_foreign_interval(Taxonomy_Node* node, Interval* interval);
void uncompute_total_ancestors(Taxonomy_Node* node);
void help_compute_total_ancestors(Taxonomy_Node* node);
void compute_total_ancestors(Taxonomy_Node* node);
void create_taxonomy_spanning_tree(Taxonomy_Graph* graph, Taxonomy_Node* node);
void create_taxonomy_spanning_forest(Taxonomy_Graph* graph);
void finalize_taxonomy_graph(Taxonomy_Graph* graph);
void finalize_taxonomy_graph_nonincrementally(Taxonomy_Graph* graph);
boolean taxonomy_subnode_ofP(Taxonomy_Node* sub_node, Taxonomy_Node* super_node);
boolean slow_taxonomy_subnode_ofP(Taxonomy_Node* subnode, Taxonomy_Node* supernode);
void print_taxonomy_tree(Taxonomy_Node* node, int indent, Output_Stream* stream);
void print_taxonomy_graph(Taxonomy_Graph* graph, Output_Stream* stream);
void print_taxonomy_spanning_tree(Taxonomy_Node* node, int indent, Output_Stream* stream);
void print_taxonomy_spanning_forest(Taxonomy_Graph* graph, Output_Stream* stream);
void startup_taxonomies();

// Auxiliary global declarations:
extern Symbol* SYM_TAXONOMIES_STELLA_STARTUP_TAXONOMIES;
extern Symbol* SYM_TAXONOMIES_STELLA_METHOD_STARTUP_CLASSNAME;


} // end of namespace stella
