// java-translate.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

char* java_yield_flotsam_class_name(Module* mod) {
  { char* flotsamclassname = ((String_Wrapper*)(dynamic_slot_value(mod->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_JAVA_FLOTSAM_CLASS, NULL_STRING_WRAPPER)))->wrapper_value;

    if ((flotsamclassname != NULL) &&
        (!(flotsamclassname == ""))) {
      return (flotsamclassname);
    }
    else {
      {
        flotsamclassname = java_translate_class_namestring(wrap_string(mod->name()))->wrapper_value;
        set_dynamic_slot_value(mod->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_JAVA_FLOTSAM_CLASS, wrap_string(flotsamclassname), NULL_STRING_WRAPPER);
        return (flotsamclassname);
      }
    }
  }
}

char* java_yield_full_native_class_name() {
  return (string_concatenate(java_stella_package(), ".javalib.Native", 0));
}

Cons* java_translate_unit(Translation_Unit* unit) {
  { Symbol* test_value_000 = unit->category;

    if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_GLOBAL_VARIABLE) {
      return (java_translate_define_global_variable_unit(unit));
    }
    else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_CLASS) {
      return (java_translate_define_native_class_unit(unit));
    }
    else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_TYPE) {
      return (NIL);
    }
    else if ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_METHOD) ||
        (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_MACRO)) {
      return (java_translate_define_method_unit(unit));
    }
    else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_PRINT_METHOD) {
      return (java_translate_defprint_unit(unit));
    }
    else if ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_STARTUP_TIME_PROGN) ||
        (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_VERBATIM)) {
      return (((Cons*)(java_translate_a_tree(unit->the_object))));
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
}

Cons* java_translate_define_global_variable_unit(Translation_Unit* unit) {
  { Global_Variable* global = ((Global_Variable*)(unit->the_object));

    { 
      BIND_STELLA_SPECIAL(oCONTEXTo, Context*, global->home_module());
      BIND_STELLA_SPECIAL(oMODULEo, Module*, oCONTEXTo.get()->base_module);
      { Cons* typelist = cons((global->variable_specialP ? wrap_string("InheritableThreadLocal") : java_translate_type_spec(global_variable_type_spec(global))), NIL);
        Object* initialvaluetree = java_translate_a_tree(unit->code_register);
        String_Wrapper* variablename = java_translate_global_name(global->variable_name, TRUE);

        typelist = cons(wrap_string("static"), typelist);
        if (global->variable_constantP ||
            global->variable_specialP) {
          typelist = cons(wrap_string("final"), typelist);
        }
        if (global->variable_publicP) {
          typelist = cons(wrap_string("public"), typelist);
        }
        if ((unit->code_register == KWD_JAVA_TRANSLATE_UNBOUND_SPECIAL_VARIABLE) ||
            global->variable_specialP) {
          return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_SPECIAL, wrap_string(global->documentation), listO(3, cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_TYPE, typelist->concatenate(NIL, 0)), variablename, NIL)));
        }
        else {
          return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_GLOBAL, wrap_string(global->documentation), listO(3, cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_TYPE, typelist->concatenate(NIL, 0)), variablename, cons(initialvaluetree, NIL))));
        }
      }
    }
  }
}

Cons* wrap_method_body_with_java_auxiliary_declarations(Cons* methodbody, Cons* declarations) {
  { Cons* odeclarations = NIL;
    Cons* otree = NULL;

    declarations = declarations->reverse();
    { Object* declaration = NULL;
      Cons* iter_000 = declarations;

      while (!(iter_000 == NIL)) {
        declaration = iter_000->value;
        iter_000 = iter_000->rest;
        { Cons* d = ((Cons*)(declaration));

          odeclarations = cons(cons(java_translate_type_spec(((Standard_Object*)(d->rest->value))), cons(java_translate_a_tree(d->value), ((((boolean)(d->rest->rest->value)) ? cons_list(1, java_translate_a_tree(d->rest->rest->value)) : NIL))->concatenate(NIL, 0))), odeclarations);
        }
      }
    }
    otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_BLOCK, odeclarations->reverse(), cons(methodbody, NIL)), NIL);
    return (otree);
  }
}

boolean java_synchronized_methodP(Method_Slot* method) {
  return (((Boolean_Wrapper*)(dynamic_slot_value(method->home_module()->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_APIp, FALSE_WRAPPER)))->wrapper_value);
}

Cons* java_translate_define_method_unit(Translation_Unit* unit) {
  { 
    BIND_STELLA_SPECIAL(oLOCALGENSYMTABLEo, Key_Value_List*, new_key_value_list());
    BIND_STELLA_SPECIAL(oMETHODBEINGWALKEDo, Method_Slot*, NULL);
    BIND_STELLA_SPECIAL(oCURRENTDUMMYINDEXo, int, 0);
    BIND_STELLA_SPECIAL(oDUMMYDECLARATIONSo, Cons*, NIL);
    { Method_Slot* method = ((Method_Slot*)(unit->the_object));
      boolean unitisfunction = java_method_object_is_functionP(method);
      Cons* bodytree = ((Cons*)(unit->code_register));
      Cons* returntype = NULL;
      Cons* modifiers = NIL;
      Cons* otree = NIL;
      Cons* translatedmethodbody = NULL;

      if (((boolean)(unit->translation)) &&
          (!(!((boolean)(unit->translation))))) {
        otree = ((Cons*)(unit->translation));
        unit->translation = NIL;
        return (otree);
      }
      oMETHODBEINGWALKEDo.set(method);
      bodytree = delete_quoted_null_statements(bodytree);
      if (!unitisfunction) {
        oDUMMYDECLARATIONSo.set(cons(cons(((Symbol*)(method->method_parameter_names_reader()->first())), cons(method->slot_owner, cons(SYM_JAVA_TRANSLATE_STELLA_THIS, NIL))), oDUMMYDECLARATIONSo.get()));
      }
      translatedmethodbody = cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, java_translate_list_of_trees(bodytree)->concatenate(NIL, 0));
      if (!(oDUMMYDECLARATIONSo.get() == NIL)) {
        translatedmethodbody = wrap_method_body_with_java_auxiliary_declarations(translatedmethodbody, oDUMMYDECLARATIONSo.get());
      }
      if (((String_Wrapper*)(dynamic_slot_value(method->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_DOCUMENTATION, NULL_STRING_WRAPPER)))->wrapper_value != NULL) {
        translatedmethodbody->rest = cons(listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_COMMENT, wrap_string(((String_Wrapper*)(dynamic_slot_value(method->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_DOCUMENTATION, NULL_STRING_WRAPPER)))->wrapper_value), NIL), translatedmethodbody->rest->concatenate(NIL, 0));
      }
      if (java_synchronized_methodP(method)) {
        modifiers = cons(wrap_string("synchronized"), modifiers);
      }
      if (unitisfunction) {
        modifiers = cons(wrap_string("static"), modifiers);
      }
      if (method->abstractP) {
        modifiers = cons(wrap_string("abstract"), modifiers);
        translatedmethodbody = NIL;
      }
      if (method->publicP()) {
        modifiers = cons(wrap_string("public"), modifiers);
      }
      returntype = cons(java_translate_type_spec(compute_most_general_return_type(method, method->compute_return_type_spec(method->slot_owner))), NIL);
      otree = cons((unitisfunction ? SYM_JAVA_TRANSLATE_STELLA_JAVA_FUNCTION : SYM_JAVA_TRANSLATE_STELLA_JAVA_METHOD), cons(modifiers, cons(returntype, cons(method->java_translate_method_name(), cons(java_translate_method_parameters(method), cons(translatedmethodbody, NIL))))));
      unit->translation = NIL;
      return (otree);
    }
  }
}

Cons* java_translate_defprint_unit(Translation_Unit* unit) {
  { 
    BIND_STELLA_SPECIAL(oMETHODBEINGWALKEDo, Method_Slot*, new_method_slot());
    BIND_STELLA_SPECIAL(oLOCALGENSYMTABLEo, Key_Value_List*, new_key_value_list());
    BIND_STELLA_SPECIAL(oCURRENTDUMMYINDEXo, int, 0);
    BIND_STELLA_SPECIAL(oDUMMYDECLARATIONSo, Cons*, NIL);
    { Class* renamed_Class = ((Class*)(unit->the_object));
      Cons* bodytree = ((Cons*)(unit->code_register));
      Cons* otree = NIL;
      Cons* translatedmethodbody = NULL;

      { 
        BIND_STELLA_SPECIAL(oCONTEXTo, Context*, renamed_Class->home_module());
        BIND_STELLA_SPECIAL(oMODULEo, Module*, oCONTEXTo.get()->base_module);
        bodytree = delete_quoted_null_statements(bodytree);
        oDUMMYDECLARATIONSo.set(cons(listO(3, SYM_JAVA_TRANSLATE_STELLA_SELF, canonical_type(renamed_Class->class_type), cons(SYM_JAVA_TRANSLATE_STELLA_THIS, NIL)), oDUMMYDECLARATIONSo.get()));
        translatedmethodbody = wrap_method_body_with_java_auxiliary_declarations(listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, java_translate_a_tree(bodytree), NIL), oDUMMYDECLARATIONSo.get());
        otree = listO(5, SYM_JAVA_TRANSLATE_STELLA_JAVA_METHOD, cons(wrap_string("public"), NIL), cons(wrap_string("String"), NIL), wrap_string("toString"), listO(3, NIL, translatedmethodbody, NIL));
        unit->translation = NIL;
        return (otree);
      }
    }
  }
}

Cons* java_translate_list_of_trees(Cons* trees) {
  { Cons* cursor = trees;

    while (!(cursor == NIL)) {
      cursor->value = java_translate_a_tree(cursor->value);
      cursor = cursor->rest;
    }
    return (trees);
  }
}

Object* java_translate_a_tree(Object* tree) {
  if (safe_primary_type(tree) == SGT_JAVA_TRANSLATE_STELLA_CONS) {
    { Object* tree_000 = tree;
      Cons* tree = ((Cons*)(tree_000));

      if (tree == NIL) {
        return (tree->java_translate_atomic_tree());
      }
      { Generalized_Symbol* test_value_000 = ((Generalized_Symbol*)(tree->value));

        if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_SPECIAL) {
          return (java_translate_special_tree(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_VERBATIM) {
          return (java_translate_verbatim_tree(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_CATCH) {
          return (java_translate_catch(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_THROW) {
          return (java_translate_throw(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_SYS_UNWIND_PROTECT) {
          return (java_translate_unwind_protect(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_SYS_SIGNAL) {
          return (java_translate_signal(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_SYS_HANDLER_CASE) {
          return (java_translate_handler_case(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_SYS_HANDLE_EXCEPTION) {
          return (java_translate_handle_exception(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_VOID_SYS) {
          return (java_translate_void_sys(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_TYPED_SYS) {
          return (java_translate_typed_sys(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_BAD_SYS) {
          return (java_translate_bad_sys(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_SETQ) {
          return (java_translate_setq_tree(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_SYS_SLOT_VALUE) {
          return (java_translate_sys_slot_value(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_SYS_REFERENCED_SLOT_VALUE) {
          return (java_translate_sys_referenced_slot_value(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_SYS_SLOT_VALUE_SETTER) {
          return (java_translate_sys_slot_value_setter(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_SYS_CALL_METHOD) {
          return (java_translate_method_call(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_SYS_CALL_FUNCTION) {
          return (java_translate_function_call(tree, NULL));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_PROGN) {
          return (java_translate_progn(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_WITH_PROCESS_LOCK) {
          return (java_translate_with_process_lock(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_LOOP) {
          return (java_translate_loop(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_SYS_FOREACH) {
          return (java_translate_foreach_tree(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_LET) {
          return (java_translate_let_tree(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_STARTUP_TIME_PROGN) {
          return (java_translate_startup_time_progn(tree));
        }
        else if ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_CAST) ||
            (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_SAFE_CAST)) {
          return (java_translate_cast(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_RETURN) {
          return (java_translate_return_tree(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_BREAK) {
          return (java_translate_break(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_CONTINUE) {
          return (java_translate_continue(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_WHILE) {
          return (java_translate_while(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_IF) {
          return (java_translate_if_tree(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_CHOOSE) {
          return (java_translate_choose_tree(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_WHEN) {
          return (java_translate_when_tree(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_UNLESS) {
          return (java_translate_unless_tree(tree));
        }
        else if ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_AND) ||
            ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_OR) ||
             ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_NOT) ||
              ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_EQp) ||
               ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_g) ||
                ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_ge) ||
                 ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_l) ||
                  ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_le) ||
                   ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_ii) ||
                    (test_value_000 == SYM_JAVA_TRANSLATE_STELLA___)))))))))) {
          return (java_translate_operator_tree(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_COND) {
          return (java_translate_cond_tree(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_CASE) {
          return (java_translate_case_tree(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_MAKE) {
          return (java_translate_make_tree(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_SYS_NEW) {
          return (java_translate_sys_new(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_MV_SETQ) {
          return (java_translate_mv_setq(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_SYS_CALL_FUNCTION_CODE) {
          return (java_translate_call_function_code(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_SYS_CALL_METHOD_CODE) {
          return (java_translate_call_method_code(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_SYS_INLINE_CALL) {
          return (java_translate_inline_call_tree(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_THE_CODE) {
          return (java_translate_the_code(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_SYS_POINTER_TO_FUNCTION) {
          return (java_translate_sys_pointer_to_function(tree));
        }
        else if ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_INLINE) ||
            (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_IGNORE)) {
          return (java_translate_ignore_tree(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_PRINT_STREAM) {
          return (java_translate_print_stream(tree));
        }
        else if (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_PRINT_NATIVE_STREAM) {
          return (java_translate_print_native_stream(tree));
        }
        else {
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        }
      }
    }
  }
  else {
    if (!((boolean)(tree))) {
      return (java_translate_null());
    }
    return (tree->java_translate_atomic_tree());
  }
}

// A list of new vectors generated for variable-length argument 
//              lists
DEFINE_STELLA_SPECIAL(oVARARGSTATEMENTSo, Cons* , NIL);

// A list of vector-pushes generated for variable-length argument 
//              lists
DEFINE_STELLA_SPECIAL(oVARARGDECLSo, Cons* , NIL);

// The current index of the variable length arguments
DEFINE_STELLA_SPECIAL(oCURRENTVARARGINDEXo, int , 1);

// Mapping from STELLA operators to Java operators
Key_Value_List* oJAVA_OPERATOR_TABLEo = NULL;

Cons* java_lookup_operator_table(Symbol* renamed_Operator) {
  { Cons* entry = ((Cons*)(oJAVA_OPERATOR_TABLEo->lookup(renamed_Operator)));
    Cons* copiedentry = NIL;

    if (((boolean)(entry))) {
      { String_Wrapper* wrapper = NULL;
        Cons* iter_000 = entry;
        Cons* collect_000 = NULL;

        while (!(iter_000 == NIL)) {
          wrapper = ((String_Wrapper*)(iter_000->value));
          iter_000 = iter_000->rest;
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(wrap_string(wrapper->wrapper_value), NIL);
              if (copiedentry == NIL) {
                copiedentry = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(copiedentry, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(wrap_string(wrapper->wrapper_value), NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
      return (copiedentry);
    }
    return (NULL);
  }
}

char* java_create_character_substitution_table() {
  { char* table = make_string(256, '_');

    { int code = NULL_INTEGER;
      int iter_000 = (int)(unsigned char) '0';
      int upper_bound_000 = (int)(unsigned char) '9';
      boolean unboundedP_000 = upper_bound_000 == NULL_INTEGER;

      while (unboundedP_000 ||
          (iter_000 <= upper_bound_000)) {
        code = iter_000;
        iter_000 = iter_000 + 1;
        table[code] = '=';
      }
    }
    { int code = NULL_INTEGER;
      int iter_001 = (int)(unsigned char) 'A';
      int upper_bound_001 = (int)(unsigned char) 'Z';
      boolean unboundedP_001 = upper_bound_001 == NULL_INTEGER;

      while (unboundedP_001 ||
          (iter_001 <= upper_bound_001)) {
        code = iter_001;
        iter_001 = iter_001 + 1;
        table[code] = '=';
      }
    }
    { int code = NULL_INTEGER;
      int iter_002 = (int)(unsigned char) 'a';
      int upper_bound_002 = (int)(unsigned char) 'z';
      boolean unboundedP_002 = upper_bound_002 == NULL_INTEGER;

      while (unboundedP_002 ||
          (iter_002 <= upper_bound_002)) {
        code = iter_002;
        iter_002 = iter_002 + 1;
        table[code] = '=';
      }
    }
    table[((int)(unsigned char) '_')] = '=';
    table[((int)(unsigned char) ' ')] = '_';
    table[((int)(unsigned char) '!')] = 'X';
    table[((int)(unsigned char) '"')] = '_';
    table[((int)(unsigned char) '#')] = 'H';
    table[((int)(unsigned char) '$')] = 'B';
    table[((int)(unsigned char) '%')] = 'R';
    table[((int)(unsigned char) '&')] = 'A';
    table[((int)(unsigned char) '\'')] = 'Q';
    table[((int)(unsigned char) '(')] = '_';
    table[((int)(unsigned char) ')')] = '_';
    table[((int)(unsigned char) '*')] = '$';
    table[((int)(unsigned char) '+')] = 'I';
    table[((int)(unsigned char) ',')] = '_';
    table[((int)(unsigned char) '-')] = '_';
    table[((int)(unsigned char) '.')] = 'D';
    table[((int)(unsigned char) '/')] = 'S';
    table[((int)(unsigned char) ':')] = 'C';
    table[((int)(unsigned char) ';')] = '_';
    table[((int)(unsigned char) '<')] = 'L';
    table[((int)(unsigned char) '=')] = 'E';
    table[((int)(unsigned char) '>')] = 'G';
    table[((int)(unsigned char) '?')] = 'P';
    table[((int)(unsigned char) '@')] = 'M';
    table[((int)(unsigned char) '[')] = 'J';
    table[((int)(unsigned char) '\\')] = '_';
    table[((int)(unsigned char) ']')] = 'K';
    table[((int)(unsigned char) '^')] = 'U';
    table[((int)(unsigned char) '`')] = '_';
    table[((int)(unsigned char) '{')] = 'Y';
    table[((int)(unsigned char) '|')] = 'V';
    table[((int)(unsigned char) '}')] = 'Z';
    table[((int)(unsigned char) '~')] = 'T';
    return (table);
  }
}

char* oJAVA_CHARACTER_SUBSTITUTION_TABLEo = NULL;

char* java_substitute_forbidden_characters(char* name, Keyword* caseconvention) {
  { char* result = NULL;
    char substitution = NULL_CHARACTER;

    switch (name[0]) {
      case '0': 
        result = string_to_mutable_string(replace_first_character_with_string(name, "zero"));
      break;
      case '1': 
        result = string_to_mutable_string(replace_first_character_with_string(name, "one"));
      break;
      case '2': 
        result = string_to_mutable_string(replace_first_character_with_string(name, "two"));
      break;
      case '3': 
        result = string_to_mutable_string(replace_first_character_with_string(name, "three"));
      break;
      case '4': 
        result = string_to_mutable_string(replace_first_character_with_string(name, "four"));
      break;
      case '5': 
        result = string_to_mutable_string(replace_first_character_with_string(name, "five"));
      break;
      case '6': 
        result = string_to_mutable_string(replace_first_character_with_string(name, "six"));
      break;
      case '7': 
        result = string_to_mutable_string(replace_first_character_with_string(name, "seven"));
      break;
      case '8': 
        result = string_to_mutable_string(replace_first_character_with_string(name, "eight"));
      break;
      case '9': 
        result = string_to_mutable_string(replace_first_character_with_string(name, "nine"));
      break;
      default:
        result = strcpy(new (GC) char[strlen(name)+1], name);
      break;
    }
    { int i = NULL_INTEGER;
      int iter_000 = 0;
      int upper_bound_000 = strlen(result) - 1;

      while (iter_000 <= upper_bound_000) {
        i = iter_000;
        iter_000 = iter_000 + 1;
        substitution = oJAVA_CHARACTER_SUBSTITUTION_TABLEo[((int)(unsigned char) (result[i]))];
        if (!(substitution == '=')) {
          if (caseconvention == KWD_JAVA_TRANSLATE_UPPERCASE) {
            result[i] = (oCHARACTER_DOWNCASE_TABLEo[((int)(unsigned char) substitution)]);
          }
          else if (caseconvention == KWD_JAVA_TRANSLATE_LOWERCASE) {
            result[i] = (oCHARACTER_UPCASE_TABLEo[((int)(unsigned char) substitution)]);
          }
          else if (caseconvention == KWD_JAVA_TRANSLATE_CAPITALIZED) {
            result[i] = (oCHARACTER_UPCASE_TABLEo[((int)(unsigned char) substitution)]);
          }
          else {
            { Output_String_Stream* stream_000 = new_output_string_stream();

              *(stream_000->native_stream) << "`" << caseconvention << "'" << " is not a valid case option";
              throw *new_stella_exception(stream_000->the_string_reader());
            }
          }
        }
      }
    }
    return (result);
  }
}

char* java_change_case(Generalized_Symbol* symbol, Keyword* caseconvention) {
  { Module* module = ((Module*)(symbol->home_context));
    char* name = symbol->symbol_name;

    if (!((boolean)(module))) {
      module = oMODULEo.get();
    }
    if (module->case_sensitiveP ||
        (!(compute_symbol_escape_code(name, FALSE) == KWD_JAVA_TRANSLATE_UNESCAPED))) {
      return (make_matching_name_full(name, KWD_JAVA_TRANSLATE_NO, KWD_JAVA_TRANSLATE_NO, "- ", "", "", "", "", KWD_JAVA_TRANSLATE_PRESERVE, "_"));
    }
    if (caseconvention == KWD_JAVA_TRANSLATE_UPPERCASE) {
      return (make_matching_name_full(name, KWD_JAVA_TRANSLATE_NO, KWD_JAVA_TRANSLATE_NO, "- ", "", "", "", "", KWD_JAVA_TRANSLATE_UPCASE, "_"));
    }
    else if (caseconvention == KWD_JAVA_TRANSLATE_LOWERCASE) {
      return (make_matching_name_full(name, KWD_JAVA_TRANSLATE_CLEVER, KWD_JAVA_TRANSLATE_CLEVER, "- ", "", "", "", "", KWD_JAVA_TRANSLATE_TITLECASEX, ""));
    }
    else if (caseconvention == KWD_JAVA_TRANSLATE_CAPITALIZED) {
      return (make_matching_name_full(name, KWD_JAVA_TRANSLATE_CLEVER, KWD_JAVA_TRANSLATE_CLEVER, "- ", "", "", "", "", KWD_JAVA_TRANSLATE_TITLECASE, ""));
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "`" << caseconvention << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
}

boolean java_literalP(Standard_Object* typespec) {
  return (sub_type_spec_ofP(typespec, SGT_JAVA_TRANSLATE_STELLA_LITERAL) &&
      ((!sub_type_spec_ofP(typespec, SGT_JAVA_TRANSLATE_STELLA_STRING)) &&
       ((!sub_type_spec_ofP(typespec, SGT_JAVA_TRANSLATE_STELLA_MUTABLE_STRING)) &&
        (!sub_type_spec_ofP(typespec, SGT_JAVA_TRANSLATE_STELLA_CODE)))));
}

String_Wrapper* java_native_literal_wrapper_names(Standard_Object* typespec, String_Wrapper*& _return1) {
  { Generalized_Symbol* test_value_000 = ((Generalized_Symbol*)(typespec));

    if (test_value_000 == SGT_JAVA_TRANSLATE_STELLA_BOOLEAN) {
      { String_Wrapper* _return_temp = wrap_string("Boolean");

        _return1 = wrap_string("booleanValue");
        return (_return_temp);
      }
    }
    else if ((test_value_000 == SGT_JAVA_TRANSLATE_STELLA_INTEGER) ||
        ((test_value_000 == SGT_JAVA_TRANSLATE_STELLA_SHORT_INTEGER) ||
         ((test_value_000 == SGT_JAVA_TRANSLATE_STELLA_UNSIGNED_SHORT_INTEGER) ||
          ((test_value_000 == SGT_JAVA_TRANSLATE_STELLA_BYTE) ||
           ((test_value_000 == SGT_JAVA_TRANSLATE_STELLA_OCTET) ||
            (test_value_000 == SGT_JAVA_TRANSLATE_STELLA_THIRY_TWO_BIT_VECTOR)))))) {
      { String_Wrapper* _return_temp = wrap_string("Integer");

        _return1 = wrap_string("intValue");
        return (_return_temp);
      }
    }
    else if ((test_value_000 == SGT_JAVA_TRANSLATE_STELLA_FLOAT) ||
        (test_value_000 == SGT_JAVA_TRANSLATE_STELLA_DOUBLE)) {
      { String_Wrapper* _return_temp = wrap_string("Double");

        _return1 = wrap_string("doubleValue");
        return (_return_temp);
      }
    }
    else if (test_value_000 == SGT_JAVA_TRANSLATE_STELLA_CHARACTER) {
      { String_Wrapper* _return_temp = wrap_string("Character");

        _return1 = wrap_string("charValue");
        return (_return_temp);
      }
    }
    else if (test_value_000 == SGT_JAVA_TRANSLATE_STELLA_LONG_INTEGER) {
      { String_Wrapper* _return_temp = wrap_string("Long");

        _return1 = wrap_string("longValue");
        return (_return_temp);
      }
    }
    else if (test_value_000 == SGT_JAVA_TRANSLATE_STELLA_SINGLE_FLOAT) {
      { String_Wrapper* _return_temp = wrap_string("Float");

        _return1 = wrap_string("floatValue");
        return (_return_temp);
      }
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
}

String_Wrapper* java_special_setter_name(Standard_Object* typespec) {
  { Generalized_Symbol* test_value_000 = ((Generalized_Symbol*)(typespec));

    if (test_value_000 == SGT_JAVA_TRANSLATE_STELLA_BOOLEAN) {
      return (wrap_string("setBooleanSpecial"));
    }
    else if ((test_value_000 == SGT_JAVA_TRANSLATE_STELLA_INTEGER) ||
        ((test_value_000 == SGT_JAVA_TRANSLATE_STELLA_SHORT_INTEGER) ||
         ((test_value_000 == SGT_JAVA_TRANSLATE_STELLA_UNSIGNED_SHORT_INTEGER) ||
          ((test_value_000 == SGT_JAVA_TRANSLATE_STELLA_BYTE) ||
           ((test_value_000 == SGT_JAVA_TRANSLATE_STELLA_OCTET) ||
            (test_value_000 == SGT_JAVA_TRANSLATE_STELLA_THIRY_TWO_BIT_VECTOR)))))) {
      return (wrap_string("setIntSpecial"));
    }
    else if ((test_value_000 == SGT_JAVA_TRANSLATE_STELLA_FLOAT) ||
        (test_value_000 == SGT_JAVA_TRANSLATE_STELLA_DOUBLE)) {
      return (wrap_string("setDoubleSpecial"));
    }
    else if (test_value_000 == SGT_JAVA_TRANSLATE_STELLA_CHARACTER) {
      return (wrap_string("setCharSpecial"));
    }
    else if (test_value_000 == SGT_JAVA_TRANSLATE_STELLA_LONG_INTEGER) {
      return (wrap_string("setLongSpecial"));
    }
    else if (test_value_000 == SGT_JAVA_TRANSLATE_STELLA_SINGLE_FLOAT) {
      return (wrap_string("setFloatSpecial"));
    }
    else {
      return (wrap_string("setSpecial"));
    }
  }
}

String_Wrapper* java_translate_symbol_constant_name(Generalized_Symbol* symbol) {
  { Generalized_Symbol* globalsymbol = ((Generalized_Symbol*)(oSYMBOL_REGISTRYo->lookup(symbol)));
    char* translatedname = "";

    if (!(((boolean)(globalsymbol)))) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "Couldn't lookup symbol name for " << "`" << symbol << "'" << endl << "     in java-translate-symbol-constant-name" << endl;
    }
    translatedname = string_concatenate(java_yield_flotsam_class_name(globalsymbol->home_module()), ".", 1, java_substitute_forbidden_characters(globalsymbol->symbol_name, KWD_JAVA_TRANSLATE_UPPERCASE));
    if (!in_current_java_packageP(globalsymbol->home_module())) {
      translatedname = string_concatenate(java_package_prefix(globalsymbol->home_module(), "."), translatedname, 0);
    }
    return (wrap_string(translatedname));
  }
}

String_Wrapper* java_translate_symbol_constant_id(Generalized_Symbol* symbol) {
  { String_Wrapper* translatedsymbol = java_translate_symbol_constant_name(symbol);

    translatedsymbol->wrapper_value = string_concatenate(translatedsymbol->wrapper_value, "_id", 0);
    return (translatedsymbol);
  }
}

String_Wrapper* java_translate_symbol_name(Symbol* symbol) {
  return (java_translate_symbol_constant_name(symbol));
}

String_Wrapper* java_translate_symbol_id(Symbol* symbol) {
  return (java_translate_symbol_constant_id(symbol));
}

String_Wrapper* java_translate_surrogate_name(Surrogate* surrogate) {
  return (java_translate_symbol_constant_name(surrogate));
}

String_Wrapper* java_translate_surrogate_id(Surrogate* surrogate) {
  return (java_translate_symbol_constant_id(surrogate));
}

String_Wrapper* java_translate_keyword_name(Keyword* keyword) {
  return (java_translate_symbol_constant_name(keyword));
}

String_Wrapper* java_translate_keyword_id(Keyword* keyword) {
  return (java_translate_symbol_constant_id(keyword));
}

String_Wrapper* java_translate_class_namestring(String_Wrapper* namestring) {
  { char* translatedname = java_substitute_forbidden_characters(make_matching_name_full(namestring->wrapper_value, KWD_JAVA_TRANSLATE_CLEVER, KWD_JAVA_TRANSLATE_CLEVER, "- ", "", "", "", "", KWD_JAVA_TRANSLATE_TITLECASE, ""), KWD_JAVA_TRANSLATE_CAPITALIZED);

    if (java_lookup_reserved_word_table(translatedname) != NULL) {
      return (wrap_string(string_concatenate("Stella_", translatedname, 0)));
    }
    else {
      return (wrap_string(translatedname));
    }
  }
}

String_Wrapper* java_translate_class_name(Generalized_Symbol* namesymbol) {
  return (java_translate_type_spec(namesymbol->surrogatify()));
}

boolean in_current_java_packageP(Module* module) {
  return ((module == oMODULEo.get()) ||
      string_eqlP(java_package_prefix(module, "."), java_package_prefix(oMODULEo.get(), ".")));
}

String_Wrapper* java_translate_type_spec(Standard_Object* typespec) {
  { Standard_Object* finaltypespec = NULL;
    Surrogate* finaltype = NULL;

    { Surrogate* test_value_000 = safe_primary_type(typespec);

      if (subtype_of_parametric_type_specifierP(test_value_000)) {
        { Standard_Object* typespec_000 = typespec;
          Parametric_Type_Specifier* typespec = ((Parametric_Type_Specifier*)(typespec_000));

          if (array_type_specifierP(typespec)) {
            return (java_translate_array_type(typespec));
          }
          finaltypespec = typespec->specifier_base_type;
          if (in_current_java_packageP(finaltypespec->home_module())) {
            return (java_translate_type_spec(finaltypespec));
          }
          else {
            return (java_yield_fully_qualified_type_name(typespec));
          }
        }
      }
      else if (subtype_of_surrogateP(test_value_000)) {
        { Standard_Object* typespec_001 = typespec;
          Surrogate* typespec = ((Surrogate*)(typespec_001));

          if (((String_Wrapper*)(dynamic_slot_value(((Class*)(typespec->surrogate_value))->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_CLASS_JAVA_NATIVE_TYPE, NULL_STRING_WRAPPER)))->wrapper_value != NULL) {
            return (wrap_string(((Class*)(typespec->surrogate_value))->java_native_type()));
          }
          else {
            finaltype = ((Class*)(typespec->surrogate_value))->class_type;
            if (in_current_java_packageP(finaltype->home_module())) {
              return (java_translate_class_namestring(wrap_string(finaltype->symbol_name)));
            }
            else {
              return (wrap_string(string_concatenate(java_package_prefix(finaltype->home_module(), "."), java_translate_class_namestring(wrap_string(finaltype->symbol_name))->wrapper_value, 0)));
            }
          }
        }
      }
      else {
        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      }
    }
  }
}

String_Wrapper* java_translate_array_type(Parametric_Type_Specifier* arraytype) {
  { boolean dummy1;

    { List* dimensions = array_type_dimensions(arraytype);
      Standard_Object* elementtype = extract_parameter_type(arraytype, SYM_JAVA_TRANSLATE_STELLA_ANY_VALUE, dummy1);
      char* translatedtype = java_translate_type_spec(elementtype)->wrapper_value;

      switch (dimensions->length()) {
        case 1: 
          return (wrap_string(string_concatenate(translatedtype, "[]", 0)));
        case 2: 
          return (wrap_string(string_concatenate(translatedtype, "[][]", 0)));
        case 3: 
          return (wrap_string(string_concatenate(translatedtype, "[][][]", 0)));
        default:
          { Integer_Wrapper* unused_dim = NULL;
            Cons* iter_000 = dimensions->the_cons_list;

            while (!(iter_000 == NIL)) {
              unused_dim = ((Integer_Wrapper*)(iter_000->value));
              iter_000 = iter_000->rest;
              translatedtype = string_concatenate(translatedtype, "[]", 0);
            }
          }
          return (wrap_string(translatedtype));
      }
    }
  }
}

String_Wrapper* java_translate_array_of_type_spec(Standard_Object* typespec, boolean fullyqualifiedP) {
  { char* basetypestring = (fullyqualifiedP ? java_yield_fully_qualified_type_name(typespec)->wrapper_value : java_translate_type_spec(typespec)->wrapper_value);

    return (wrap_string(string_concatenate(basetypestring, " []", 0)));
  }
}

String_Wrapper* java_yield_renamed_reserved_word(Symbol* word) {
  return (wrap_string(string_concatenate("renamed_", string_capitalize(word->symbol_name), 0)));
}

String_Hash_Table* java_create_reserved_word_table() {
  { String_Hash_Table* table = new_string_hash_table();

    { Object* word = NULL;
      Cons* iter_000 = listO(48, SYM_JAVA_TRANSLATE_STELLA_OBJECT, SYM_JAVA_TRANSLATE_STELLA_ABSTRACT, SYM_JAVA_TRANSLATE_STELLA_BOOLEAN, SYM_JAVA_TRANSLATE_STELLA_BREAK, SYM_JAVA_TRANSLATE_STELLA_BYTE, SYM_JAVA_TRANSLATE_STELLA_CASE, SYM_JAVA_TRANSLATE_STELLA_CATCH, SYM_JAVA_TRANSLATE_STELLA_CHAR, SYM_JAVA_TRANSLATE_STELLA_CLASS, SYM_JAVA_TRANSLATE_STELLA_CONST, SYM_JAVA_TRANSLATE_STELLA_CONTINUE, SYM_JAVA_TRANSLATE_STELLA_DEFAULT, SYM_JAVA_TRANSLATE_STELLA_DO, SYM_JAVA_TRANSLATE_STELLA_DOUBLE, SYM_JAVA_TRANSLATE_STELLA_ELSE, SYM_JAVA_TRANSLATE_STELLA_EXTENDS, SYM_JAVA_TRANSLATE_STELLA_FINAL, SYM_JAVA_TRANSLATE_STELLA_FINALLY, SYM_JAVA_TRANSLATE_STELLA_FLOAT, SYM_JAVA_TRANSLATE_STELLA_FOR, SYM_JAVA_TRANSLATE_STELLA_GOTO, SYM_JAVA_TRANSLATE_STELLA_IF, SYM_JAVA_TRANSLATE_STELLA_IMPLEMENTS, SYM_JAVA_TRANSLATE_STELLA_IMPORT, SYM_JAVA_TRANSLATE_STELLA_INSTANCEOF, SYM_JAVA_TRANSLATE_STELLA_INT, SYM_JAVA_TRANSLATE_STELLA_INTERFACE, SYM_JAVA_TRANSLATE_STELLA_LONG, SYM_JAVA_TRANSLATE_STELLA_NATIVE, SYM_JAVA_TRANSLATE_STELLA_NEW, SYM_JAVA_TRANSLATE_STELLA_PACKAGE, SYM_JAVA_TRANSLATE_STELLA_PRIVATE, SYM_JAVA_TRANSLATE_STELLA_PROTECTED, SYM_JAVA_TRANSLATE_STELLA_PUBLIC, SYM_JAVA_TRANSLATE_STELLA_RETURN, SYM_JAVA_TRANSLATE_STELLA_SHORT, SYM_JAVA_TRANSLATE_STELLA_STATIC, SYM_JAVA_TRANSLATE_STELLA_SUPER, SYM_JAVA_TRANSLATE_STELLA_SWITCH, SYM_JAVA_TRANSLATE_STELLA_SYNCHRONIZED, SYM_JAVA_TRANSLATE_STELLA_THROW, SYM_JAVA_TRANSLATE_STELLA_THROWS, SYM_JAVA_TRANSLATE_STELLA_TRANSIENT, SYM_JAVA_TRANSLATE_STELLA_TRY, SYM_JAVA_TRANSLATE_STELLA_VOID, SYM_JAVA_TRANSLATE_STELLA_VOLATILE, SYM_JAVA_TRANSLATE_STELLA_WHILE, NIL);

      while (!(iter_000 == NIL)) {
        word = iter_000->value;
        iter_000 = iter_000->rest;
        table->insert_at(string_downcase(((Symbol*)(word))->symbol_name), java_yield_renamed_reserved_word(((Symbol*)(word))));
        table->insert_at(string_capitalize(((Symbol*)(word))->symbol_name), java_yield_renamed_reserved_word(((Symbol*)(word))));
      }
    }
    table->insert_at("++", wrap_string("stella_Increment"));
    table->insert_at("--", wrap_string("stella_Decrement"));
    return (table);
  }
}

String_Hash_Table* oJAVA_RESERVED_WORD_TABLEo = NULL;

char* java_lookup_reserved_word_table(char* word) {
  { String_Wrapper* entry = ((String_Wrapper*)(oJAVA_RESERVED_WORD_TABLEo->lookup(word)));

    if (((boolean)(entry))) {
      return (entry->wrapper_value);
    }
    else {
      return (NULL);
    }
  }
}

char* java_translate_namestring(char* namestring) {
  { char* reservedname = java_lookup_reserved_word_table(namestring);

    if (namestring == NULL) {
      return ("NULL");
    }
    if (reservedname != NULL) {
      return (reservedname);
    }
    return (java_substitute_forbidden_characters(namestring, KWD_JAVA_TRANSLATE_LOWERCASE));
  }
}

String_Wrapper* java_translate_name(Symbol* namesymbol) {
  return (wrap_string(java_translate_namestring(java_change_case(namesymbol, KWD_JAVA_TRANSLATE_LOWERCASE))));
}

Cons* java_translate_special_tree(Cons* tree) {
  { Cons* savedeclarations = NIL;
    Cons* setforms = NIL;
    Cons* restoreforms = NIL;
    Cons* otree = NULL;

    { Object* declaration = NULL;
      Cons* iter_000 = ((Cons*)(tree->rest->value));

      while (!(iter_000 == NIL)) {
        declaration = iter_000->value;
        iter_000 = iter_000->rest;
        { Cons* d = ((Cons*)(declaration));
          Symbol* specialvariablename = ((Symbol*)(d->value));
          Object* oldvaluevariable = java_translate_a_tree(local_gensym(string_concatenate("OLD-", specialvariablename->symbol_name, 0)));

          savedeclarations = cons(cons(wrap_string("Object"), cons(oldvaluevariable, cons(listO(6, SYM_JAVA_TRANSLATE_STELLA_JAVA_METHOD_CALL, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string(""), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string("get"), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, java_translate_global_name(specialvariablename, FALSE), NIL), cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_ACTUALS, NIL), NIL), NIL))), savedeclarations);
          setforms = cons(java_translate_a_tree(listO(3, SYM_JAVA_TRANSLATE_STELLA_SETQ, d->value, cons(d->rest->rest->value, NIL))), setforms);
          restoreforms = cons(listO(6, SYM_JAVA_TRANSLATE_STELLA_JAVA_METHOD_CALL, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string(""), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string("set"), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, java_translate_global_name(specialvariablename, FALSE), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_ACTUALS, oldvaluevariable, NIL), NIL), restoreforms);
        }
      }
    }
    otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_BLOCK, savedeclarations->reverse(), cons(listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_UNWIND_PROTECT, cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, setforms->reverse()->concatenate(java_translate_list_of_trees(tree->rest->rest)->concatenate(NIL, 0), 0)), restoreforms->concatenate(NIL, 0)), NIL), NIL));
    return (otree);
  }
}

Cons* java_translate_verbatim_tree(Cons* tree) {
  { Cons* cursor = tree->rest;
    Object* verbatimcode = NULL;

    while (!(cursor == NIL)) {
      if (cursor->value == KWD_JAVA_TRANSLATE_JAVA) {
        verbatimcode = cursor->rest->value;
        return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_VERBATIM, verbatimcode, NIL));
      }
      cursor = cursor->rest->rest;
    }
    throw *new_stella_exception("Verbatim has no :java option");
  }
}

Object* java_translate_inline_call_tree(Cons* tree) {
  { Cons* calltree = ((Cons*)(tree->rest->value));
    Cons* arguments = ((calltree->value == SYM_JAVA_TRANSLATE_STELLA_SYS_CALL_FUNCTION) ? calltree->nth_rest(2) : calltree->nth_rest(3));
    Cons* tokenizedbody = tokenize_verbatim_method_body(calltree);

    if (((boolean)(tokenizedbody))) {
      { List* inlinedparameters = new_list();
        char* inlinedbody = NULL;
        boolean successP = TRUE;

        { 
          BIND_STELLA_SPECIAL(oCURRENT_STREAMo, Output_Stream*, new_output_string_stream());
          { Object* token = NULL;
            Cons* iter_000 = tokenizedbody;

            while (!(iter_000 == NIL)) {
              token = iter_000->value;
              iter_000 = iter_000->rest;
              { Surrogate* test_value_000 = safe_primary_type(token);

                if (subtype_of_stringP(test_value_000)) {
                  { Object* token_000 = token;
                    String_Wrapper* token = ((String_Wrapper*)(token_000));

                    *(oCURRENT_STREAMo.get()->native_stream) << token->wrapper_value;
                  }
                }
                else if (test_value_000 == SGT_JAVA_TRANSLATE_STELLA_CONS) {
                  { Object* token_001 = token;
                    Cons* token = ((Cons*)(token_001));

                    { String_Wrapper* parameter = ((String_Wrapper*)(token->value));
                      Integer_Wrapper* position = ((Integer_Wrapper*)(token->rest->value));
                      Object* argument = arguments->nth(position->wrapper_value);

                      if (!side_effect_free_expressionP(argument)) {
                        if (inlinedparameters->memberP(parameter)) {
                          {
                            successP = FALSE;
                            break;
                          }
                        }
                        else {
                          inlinedparameters->insert(parameter);
                        }
                      }
                      if (!(atomic_expressionP(argument))) {
                        *(oCURRENT_STREAMo.get()->native_stream) << "(";
                      }
                      java_output_statement(java_translate_a_tree(copy_cons_tree(argument)));
                      if (!(atomic_expressionP(argument))) {
                        *(oCURRENT_STREAMo.get()->native_stream) << ")";
                      }
                    }
                  }
                }
                else {
                  { Output_String_Stream* stream_000 = new_output_string_stream();

                    *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
                    throw *new_stella_exception(stream_000->the_string_reader());
                  }
                }
              }
            }
          }
          inlinedbody = ((Output_String_Stream*)(oCURRENT_STREAMo.get()))->the_string_reader();
          if (successP) {
            return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_VERBATIM, new_verbatim_string_wrapper(inlinedbody), NIL));
          }
        }
      }
    }
    return (java_translate_a_tree(calltree));
  }
}

Object* java_translate_catch(Cons* tree) {
  *(STANDARD_WARNING->native_stream) << "Warning: " << "java-translate-catch not yet implemented" << endl;
  return (NIL);
}

Object* java_translate_throw(Cons* tree) {
  *(STANDARD_WARNING->native_stream) << "Warning: " << "java-translate-throw not yet implemented" << endl;
  return (NIL);
}

Cons* java_translate_unwind_protect(Cons* tree) {
  tree->first_setter(SYM_JAVA_TRANSLATE_STELLA_JAVA_UNWIND_PROTECT);
  tree->rest = java_translate_list_of_trees(tree->rest);
  return (tree);
}

Cons* java_translate_signal(Cons* tree) {
  tree->first_setter(SYM_JAVA_TRANSLATE_STELLA_JAVA_SIGNAL);
  tree->second_setter(java_translate_a_tree(tree->rest->value));
  return (tree);
}

Cons* java_translate_handler_case(Cons* tree) {
  tree->first_setter(SYM_JAVA_TRANSLATE_STELLA_JAVA_HANDLER_CASE);
  tree->rest = java_translate_list_of_trees(tree->rest);
  return (tree);
}

Cons* java_translate_handle_exception(Cons* tree) {
  tree->first_setter(SYM_JAVA_TRANSLATE_STELLA_JAVA_CATCH);
  tree->second_setter(cons(java_translate_type_spec(((Standard_Object*)(((Cons*)(tree->rest->value))->rest->value))), cons(java_translate_name(((Symbol*)(((Cons*)(tree->rest->value))->value))), NIL)));
  tree->rest->rest = java_translate_list_of_trees(tree->rest->rest);
  return (tree);
}

Cons* java_translate_void_sys(Cons* tree) {
  return (((Cons*)(java_translate_a_tree(tree->rest->value))));
}

Cons* java_translate_typed_sys(Cons* tree) {
  return (((Cons*)(java_translate_a_tree(tree->rest->value))));
}

Object* java_translate_bad_sys(Cons* tree) {
  return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_VERBATIM, wrap_string(string_concatenate("*** TRANSLATION FAILURE FROM ", stringify(tree->rest->value), 0)), NIL));
}

Object* java_translate_setq_tree(Cons* tree) {
  { Symbol* variablename = ((Symbol*)(tree->rest->value));
    Global_Variable* globalvar = variablename->soft_permanentify()->lookup_global_variable();

    if (((boolean)(globalvar)) &&
        globalvar->variable_specialP) {
      return (listO(5, SYM_JAVA_TRANSLATE_STELLA_JAVA_FUNCTION_CALL, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string(java_yield_full_native_class_name()), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, java_special_setter_name(global_variable_type_spec(globalvar)), NIL), listO(4, SYM_JAVA_TRANSLATE_STELLA_JAVA_ACTUALS, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, java_translate_global_name(variablename, FALSE), NIL), java_translate_a_tree(tree->rest->rest->value), NIL), NIL));
    }
    else {
      return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_ASSIGN, java_translate_a_tree(tree->rest->value), cons(java_translate_a_tree(tree->rest->rest->value), NIL)));
    }
  }
}

Object* java_translate_sys_referenced_slot_value(Cons* tree) {
  *(STANDARD_WARNING->native_stream) << "Warning: " << "java-translate-sys-referenced-slot-value not yet implemented" << endl;
  return (NIL);
}

Cons* java_translate_sys_slot_value_setter(Cons* tree) {
  { Object* objectref = tree->fourth();
    Surrogate* objecttype = ((Surrogate*)(tree->rest->value));
    Symbol* slotname = true_slot_name(((Symbol*)(tree->rest->rest->value)), objecttype);
    Object* valueref = java_translate_a_tree(tree->fifth());
    Cons* otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_SLOT_VALUE_SETTER, java_translate_a_tree(objectref), cons(java_translate_name(slotname), cons(valueref, NIL)));

    tree->third_setter(NULL);
    tree->fourth_setter(NULL);
    tree->fifth_setter(NULL);
    return (otree);
  }
}

Cons* java_translate_sys_slot_value(Cons* tree) {
  { Surrogate* objecttype = ((Surrogate*)(tree->rest->value));
    Object* objectref = tree->fourth();
    Symbol* slotname = true_slot_name(((Symbol*)(tree->rest->rest->value)), objecttype);
    Cons* otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_SLOT_VALUE, java_translate_a_tree(objectref), cons(java_translate_name(slotname), NIL));

    return (otree);
  }
}

Cons* java_translate_progn(Cons* tree) {
  return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_PROGN, cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, java_translate_list_of_trees(tree->rest)->concatenate(NIL, 0)), NIL));
}

Cons* java_translate_with_process_lock(Cons* tree) {
  return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_WITH_PROCESS_LOCK, java_translate_a_tree(tree->rest->value), cons(cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, java_translate_list_of_trees(tree->rest->rest)->concatenate(NIL, 0)), NIL)));
}

DEFINE_STELLA_SPECIAL(oJAVA_LOOP_NAMEo, Symbol* , NULL);

DEFINE_STELLA_SPECIAL(oJAVA_LOOP_NAME_USEDpo, boolean , FALSE);

Cons* java_translate_loop(Cons* tree) {
  { 
    BIND_STELLA_SPECIAL(oJAVA_LOOP_NAMEo, Symbol*, local_gensym("LOOP"));
    BIND_STELLA_SPECIAL(oJAVA_LOOP_NAME_USEDpo, boolean, FALSE);
    { Cons* body = java_translate_list_of_trees(tree->rest);

      if (oJAVA_LOOP_NAME_USEDpo.get()) {
        return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_NAMED_STATEMENT, java_translate_a_tree(oJAVA_LOOP_NAMEo.get()), cons(listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_LOOP, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_PROGN, cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, body->concatenate(NIL, 0)), NIL), NIL), NIL)));
      }
      else {
        return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_LOOP, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_PROGN, cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, body->concatenate(NIL, 0)), NIL), NIL));
      }
    }
  }
}

Cons* java_translate_argument_list_iterator(Cons* iteratorbindings, Cons* cursorbindings, Cons* body) {
  { Object* cursorvar = ((Cons*)(cursorbindings->value))->value;
    Object* cursortype = ((Cons*)(cursorbindings->value))->rest->value;
    Object* dummyarg = ((Cons*)(((Cons*)(iteratorbindings->value))->rest->rest->value))->fourth();
    Object* iteratorconstructor = NULL;
    Cons* declarations = listO(3, cons(wrap_string("Enumeration"), cons(java_translate_a_tree(SYM_JAVA_TRANSLATE_STELLA_ARG_ENUM), NIL)), cons(java_translate_type_spec(((Standard_Object*)(cursortype))), cons(java_translate_a_tree(cursorvar), NIL)), NIL);

    { Cons* binding = NULL;
      Cons* iter_000 = iteratorbindings->rest;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        binding = ((Cons*)(iter_000->value));
        iter_000 = iter_000->rest;
        iteratorconstructor = java_translate_a_tree(((Cons*)(binding))->rest->rest->value);
        binding->third_setter(NULL);
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(cons(java_translate_type_spec(((Standard_Object*)(((Cons*)(binding))->rest->value))), cons(java_translate_a_tree(((Cons*)(binding))->value), cons(iteratorconstructor, NIL))), NIL);
            if (declarations == NIL) {
              declarations = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(declarations, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(cons(java_translate_type_spec(((Standard_Object*)(((Cons*)(binding))->rest->value))), cons(java_translate_a_tree(((Cons*)(binding))->value), cons(iteratorconstructor, NIL))), NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
    }
    return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_BLOCK, declarations, cons(listO(4, SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, listO(4, SYM_JAVA_TRANSLATE_STELLA_JAVA_ASSIGN, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, java_translate_name(SYM_JAVA_TRANSLATE_STELLA_ARG_ENUM), NIL), listO(5, SYM_JAVA_TRANSLATE_STELLA_JAVA_METHOD_CALL, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string(""), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, java_translate_name(SYM_JAVA_TRANSLATE_STELLA_ELEMENTS), NIL), java_translate_a_tree(dummyarg), cons(cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_ACTUALS, NIL), NIL)), NIL), listO(4, SYM_JAVA_TRANSLATE_STELLA_JAVA_WHILE, listO(6, SYM_JAVA_TRANSLATE_STELLA_JAVA_METHOD_CALL, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string(""), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string("hasMoreElements"), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, java_translate_name(SYM_JAVA_TRANSLATE_STELLA_ARG_ENUM), NIL), cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_ACTUALS, NIL), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_PROGN, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_ASSIGN, java_translate_a_tree(cursorvar), cons(listO(4, SYM_JAVA_TRANSLATE_STELLA_JAVA_CAST, listO(6, SYM_JAVA_TRANSLATE_STELLA_JAVA_METHOD_CALL, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string(""), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string("nextElement"), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, java_translate_name(SYM_JAVA_TRANSLATE_STELLA_ARG_ENUM), NIL), cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_ACTUALS, NIL), NIL), java_translate_type_spec(SGT_JAVA_TRANSLATE_STELLA_OBJECT), NIL), NIL)), body->concatenate(NIL, 0)), NIL), NIL), NIL), NIL)));
  }
}

Cons* java_translate_foreach_tree(Cons* tree) {
  { 
    BIND_STELLA_SPECIAL(oJAVA_LOOP_NAMEo, Symbol*, local_gensym("LOOP"));
    BIND_STELLA_SPECIAL(oJAVA_LOOP_NAME_USEDpo, boolean, FALSE);
    { Cons* iteratorbindings = ((Cons*)(tree->rest->value));
      Cons* cursorbindings = ((Cons*)(tree->rest->rest->value));
      Object* continuationtest = java_translate_a_tree(tree->fourth());
      Cons* body = java_translate_list_of_trees(tree->nth_rest(4));
      Cons* declarations = NIL;
      Cons* nextassignments = NIL;
      Object* iteratorconstructor = NULL;

      tree->nth_rest_setter(NIL, 4);
      if (((Cons*)(((Cons*)(iteratorbindings->value))))->rest->value == SGT_JAVA_TRANSLATE_STELLA_LISP_CONS_ITERATOR) {
        return (java_translate_argument_list_iterator(iteratorbindings, cursorbindings, body));
      }
      { Cons* binding = NULL;
        Cons* iter_000 = iteratorbindings;
        Cons* collect_000 = NULL;

        while (!(iter_000 == NIL)) {
          binding = ((Cons*)(iter_000->value));
          iter_000 = iter_000->rest;
          iteratorconstructor = java_translate_a_tree(binding->rest->rest->value);
          binding->third_setter(NULL);
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(cons(java_translate_type_spec(((Standard_Object*)(((Cons*)(binding))->rest->value))), cons(java_translate_a_tree(((Cons*)(binding))->value), cons(iteratorconstructor, NIL))), NIL);
              if (declarations == NIL) {
                declarations = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(declarations, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(cons(java_translate_type_spec(((Standard_Object*)(((Cons*)(binding))->rest->value))), cons(java_translate_a_tree(((Cons*)(binding))->value), cons(iteratorconstructor, NIL))), NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
      { Cons* binding = NULL;
        Cons* iter_001 = cursorbindings;

        while (!(iter_001 == NIL)) {
          binding = ((Cons*)(iter_001->value));
          iter_001 = iter_001->rest;
          nextassignments = cons(java_translate_a_tree(binding->rest->rest->value), nextassignments);
          binding->third_setter(NULL);
          if (!(duplicate_cursor_variableP(((Symbol*)(binding->value)), iteratorbindings))) {
            declarations = cons(cons(java_translate_type_spec(((Standard_Object*)(((Cons*)(binding))->rest->value))), cons(java_translate_a_tree(((Cons*)(binding))->value), cons(java_translate_a_tree(type_to_walked_null_value_tree(type_spec_to_base_type(((Standard_Object*)(binding->rest->value))))), NIL))), declarations);
          }
        }
      }
      if (oJAVA_LOOP_NAME_USEDpo.get()) {
        return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_BLOCK, declarations, cons(listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_NAMED_STATEMENT, java_translate_a_tree(oJAVA_LOOP_NAMEo.get()), cons(listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_FOREACH, continuationtest, cons(nextassignments->reverse(), cons(body, NIL))), NIL)), NIL), NIL)));
      }
      else {
        return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_BLOCK, declarations, cons(listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_FOREACH, continuationtest, cons(nextassignments->reverse(), cons(body, NIL))), NIL), NIL)));
      }
    }
  }
}

Cons* java_translate_let_tree(Cons* tree) {
  { Cons* odeclarations = NIL;
    Cons* otree = NULL;

    { Object* declaration = NULL;
      Cons* iter_000 = ((Cons*)(tree->rest->value));

      while (!(iter_000 == NIL)) {
        declaration = iter_000->value;
        iter_000 = iter_000->rest;
        { Cons* d = ((Cons*)(declaration));

          odeclarations = cons(cons(java_translate_type_spec(((Standard_Object*)(d->rest->value))), cons(java_translate_a_tree(d->value), cons(java_translate_a_tree(d->rest->rest->value), NIL))), odeclarations);
        }
      }
    }
    otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_BLOCK, odeclarations->reverse(), cons(cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, java_translate_list_of_trees(tree->rest->rest)->concatenate(NIL, 0)), NIL));
    return (otree);
  }
}

Object* java_translate_startup_time_progn(Cons* tree) {
  *(STANDARD_WARNING->native_stream) << "Warning: " << "java-translate-startup-time-progn not yet implemented" << endl;
  return (NIL);
}

Cons* java_translate_cast(Cons* tree) {
  { Object* expression = tree->rest->value;
    Surrogate* type = type_spec_to_base_type(((Standard_Object*)(tree->rest->rest->value)));

    if (subtype_ofP(type, SGT_JAVA_TRANSLATE_STELLA_FLOAT) &&
        isaP(expression, SGT_JAVA_TRANSLATE_STELLA_INTEGER_WRAPPER)) {
      return (((Cons*)(java_translate_a_tree(wrap_float(((double)(((Integer_Wrapper*)(expression))->wrapper_value)))))));
    }
    if (subtype_ofP(type, SGT_JAVA_TRANSLATE_STELLA_INTEGER) &&
        isaP(expression, SGT_JAVA_TRANSLATE_STELLA_FLOAT_WRAPPER)) {
      return (((Cons*)(java_translate_a_tree(wrap_integer(((int)(((Float_Wrapper*)(expression))->wrapper_value)))))));
    }
    tree->first_setter(SYM_JAVA_TRANSLATE_STELLA_JAVA_CAST);
    tree->second_setter(java_translate_a_tree(tree->rest->value));
    tree->third_setter(java_translate_type_spec(((Standard_Object*)(tree->rest->rest->value))));
    return (tree);
  }
}

Cons* java_translate_return_tree(Cons* tree) {
  { Cons* otree = NULL;
    Cons* returnassignments = NIL;
    Cons* methodotherreturntypes = oMETHODBEINGWALKEDo.get()->method_return_type_specifiers_reader()->rest();
    Object* returnexpression = NULL;
    Cons* translatedreturnexpression = NIL;
    Cons* mvreturnparametertranslation = NIL;

    { Object* exp = NULL;
      int parameterindex = NULL_INTEGER;
      Standard_Object* parametertype = NULL;
      Cons* iter_000 = tree->rest->rest;
      int iter_001 = 0;
      Cons* iter_002 = methodotherreturntypes;

      while ((!(iter_000 == NIL)) &&
          (!(iter_002 == NIL))) {
        exp = iter_000->value;
        iter_000 = iter_000->rest;
        parameterindex = iter_001;
        iter_001 = iter_001 + 1;
        parametertype = ((Standard_Object*)(iter_002->value));
        iter_002 = iter_002->rest;
        if (subtype_ofP(type_spec_to_base_type(parametertype), SGT_JAVA_TRANSLATE_STELLA_LITERAL)) {
          mvreturnparametertranslation = ((Cons*)(java_translate_a_tree(listO(3, SYM_JAVA_TRANSLATE_STELLA_SYS_CALL_FUNCTION, lookup_literal_type_info(type_spec_to_base_type(parametertype), KWD_JAVA_TRANSLATE_WRAP_FUNCTION), cons(exp, NIL)))));
        }
        else {
          mvreturnparametertranslation = ((Cons*)(java_translate_a_tree(exp)));
        }
        returnassignments = cons(listO(4, SYM_JAVA_TRANSLATE_STELLA_JAVA_ASSIGN, listO(4, SYM_JAVA_TRANSLATE_STELLA_JAVA_AREF, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string("MV_returnarray"), NIL), wrap_integer(parameterindex), NIL), mvreturnparametertranslation, NIL), returnassignments);
      }
    }
    returnassignments = returnassignments->reverse();
    returnexpression = tree->rest->value;
    if ((methodotherreturntypes->length() > 0) &&
        ((tree->rest->rest == NIL) &&
         (consP(returnexpression) &&
          (((boolean)(((Cons*)(returnexpression))->value)) &&
           ((((Cons*)(returnexpression))->value == SYM_JAVA_TRANSLATE_STELLA_SYS_CALL_FUNCTION) ||
            (((Cons*)(returnexpression))->value == SYM_JAVA_TRANSLATE_STELLA_SYS_CALL_METHOD)))))) {
      translatedreturnexpression = java_translate_mv_function_call(((Cons*)(returnexpression)), SYM_JAVA_TRANSLATE_STELLA_MV_RETURNARRAY);
    }
    else {
      translatedreturnexpression = ((Cons*)(java_translate_a_tree(returnexpression)));
    }
    if (returnassignments == NIL) {
      if (tree->rest == NIL) {
        otree = cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_RETURN, NIL);
      }
      else {
        otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_RETURN, translatedreturnexpression, NIL);
      }
    }
    else {
      { char* tempvarname = "_return_temp";

        otree = listO(4, SYM_JAVA_TRANSLATE_STELLA_JAVA_BLOCK, cons(cons(java_translate_type_spec(oMETHODBEINGWALKEDo.get()->compute_return_type_spec(oMETHODBEINGWALKEDo.get()->slot_owner)), listO(3, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string(tempvarname), NIL), translatedreturnexpression, NIL)), NIL), cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, returnassignments->concatenate(cons(listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_RETURN, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string(tempvarname), NIL), NIL), NIL), 0)), NIL);
      }
    }
    return (otree);
  }
}

Cons* java_translate_break(Cons* tree) {
  if (!((boolean)(oJAVA_LOOP_NAMEo.get()))) {
    break_program("No loop name in (BREAK)");
  }
  oJAVA_LOOP_NAME_USEDpo.set(TRUE);
  tree->first_setter(SYM_JAVA_TRANSLATE_STELLA_JAVA_BREAK);
  tree->rest = cons(java_translate_a_tree(oJAVA_LOOP_NAMEo.get()), NIL);
  return (tree);
}

Cons* java_translate_continue(Cons* tree) {
  if (!((boolean)(oJAVA_LOOP_NAMEo.get()))) {
    break_program("No loop name in (CONTINUE)");
  }
  oJAVA_LOOP_NAME_USEDpo.set(TRUE);
  tree->first_setter(SYM_JAVA_TRANSLATE_STELLA_JAVA_CONTINUE);
  tree->rest = cons(java_translate_a_tree(oJAVA_LOOP_NAMEo.get()), NIL);
  return (tree);
}

Cons* java_translate_if_tree(Cons* tree) {
  { Object* test = java_translate_a_tree(tree->rest->value);
    Cons* truebody = ((Cons*)(java_translate_a_tree(tree->rest->rest->value)));
    Cons* falsebody = ((Cons*)(java_translate_a_tree(tree->rest->rest->rest->value)));

    tree->rest->rest = NIL;
    if (consP(truebody->value) &&
        (!(truebody->rest == NIL))) {
      truebody = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_PROGN, cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, truebody->concatenate(NIL, 0)), NIL);
    }
    else {
      truebody = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_PROGN, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, truebody, NIL), NIL);
    }
    if (consP(falsebody->value) &&
        (!(falsebody->rest == NIL))) {
      falsebody = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_PROGN, cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, falsebody->concatenate(NIL, 0)), NIL);
    }
    else {
      falsebody = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_PROGN, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, falsebody, NIL), NIL);
    }
    return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IF, test, cons(truebody, cons(falsebody, NIL))));
  }
}

boolean java_symbol_caseP(Object* keyform) {
  if (safe_primary_type(keyform) == SGT_JAVA_TRANSLATE_STELLA_CONS) {
    { Object* keyform_000 = keyform;
      Cons* keyform = ((Cons*)(keyform_000));

      return ((keyform->value == SYM_JAVA_TRANSLATE_STELLA_SYS_SLOT_VALUE) &&
          (keyform->rest->rest->value == SYM_JAVA_TRANSLATE_STELLA_SYMBOL_ID));
    }
  }
  else {
    return (FALSE);
  }
}

Cons* java_yield_symbol_id_form(int symbolid) {
  { Symbol* symbol = get_sym(symbolid);

    return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, java_translate_symbol_id(symbol), NIL));
  }
}

Cons* java_translate_condition(Cons* condition, boolean symbolcasep) {
  { Cons* translatedactions = cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, java_translate_list_of_trees(condition->rest)->concatenate(NIL, 0));
    Object* keys = condition->value;
    Object* translatedkeys = NULL;
    Cons* translatedkeyslist = NIL;

    if (symbolcasep) {
      if (consP(keys)) {
        {
          { Object* key = NULL;
            Cons* iter_000 = ((Cons*)(keys));
            Cons* collect_000 = NULL;

            while (!(iter_000 == NIL)) {
              key = iter_000->value;
              iter_000 = iter_000->rest;
              if (!((boolean)(collect_000))) {
                {
                  collect_000 = cons(java_yield_symbol_id_form(((Integer_Wrapper*)(key))->wrapper_value), NIL);
                  if (translatedkeyslist == NIL) {
                    translatedkeyslist = collect_000;
                  }
                  else {
                    add_cons_to_end_of_cons_list(translatedkeyslist, collect_000);
                  }
                }
              }
              else {
                {
                  collect_000->rest = cons(java_yield_symbol_id_form(((Integer_Wrapper*)(key))->wrapper_value), NIL);
                  collect_000 = collect_000->rest;
                }
              }
            }
          }
          translatedkeys = translatedkeyslist;
        }
      }
      else {
        translatedkeys = java_yield_symbol_id_form(((Integer_Wrapper*)(keys))->wrapper_value);
      }
    }
    else {
      translatedkeys = (consP(keys) ? java_translate_list_of_trees(((Cons*)(keys))) : java_translate_a_tree(keys));
    }
    return (cons(translatedkeys, cons(translatedactions, NIL)));
  }
}

Cons* java_translate_case_tree(Cons* tree) {
  { Cons* otherwisecondition = NULL;
    Cons* conditions = NIL;
    Object* keyform = NULL;
    boolean symbolcasep = FALSE;

    symbolcasep = java_symbol_caseP(tree->rest->value);
    keyform = java_translate_a_tree(tree->rest->value);
    { Object* condition = NULL;
      Cons* iter_000 = tree->rest->rest;

      while (!(iter_000 == NIL)) {
        condition = iter_000->value;
        iter_000 = iter_000->rest;
        if (safe_primary_type(condition) == SGT_JAVA_TRANSLATE_STELLA_CONS) {
          { Object* condition_000 = condition;
            Cons* condition = ((Cons*)(condition_000));

            if (condition->value == SYM_JAVA_TRANSLATE_STELLA_OTHERWISE) {
              otherwisecondition = cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, java_translate_list_of_trees(condition->rest)->concatenate(NIL, 0));
            }
            else {
              conditions = cons(java_translate_condition(condition, symbolcasep), conditions);
            }
          }
        }
        else {
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "`" << safe_primary_type(condition) << "'" << " is not a valid case option";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        }
      }
    }
    tree->rest->rest = NIL;
    return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_CASE, keyform, cons(otherwisecondition, cons(conditions->reverse(), NIL))));
  }
}

Cons* java_translate_cond_tree(Cons* tree) {
  { Cons* otherwisecondition = NULL;
    Cons* conditions = NIL;

    { Object* condition = NULL;
      Cons* iter_000 = tree->rest;

      while (!(iter_000 == NIL)) {
        condition = iter_000->value;
        iter_000 = iter_000->rest;
        if (safe_primary_type(condition) == SGT_JAVA_TRANSLATE_STELLA_CONS) {
          { Object* condition_000 = condition;
            Cons* condition = ((Cons*)(condition_000));

            if (condition->value == SYM_JAVA_TRANSLATE_STELLA_OTHERWISE) {
              otherwisecondition = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_PROGN, cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, java_translate_list_of_trees(condition->rest)->concatenate(NIL, 0)), NIL);
            }
            else {
              conditions = cons(cons(java_translate_a_tree(condition->value), cons(listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_PROGN, cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, java_translate_list_of_trees(condition->rest)->concatenate(NIL, 0)), NIL), NIL)), conditions);
            }
          }
        }
        else {
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "`" << safe_primary_type(condition) << "'" << " is not a valid case option";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        }
      }
    }
    return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_COND, otherwisecondition, cons(conditions->reverse(), NIL)));
  }
}

Cons* java_translate_while(Cons* tree) {
  { 
    BIND_STELLA_SPECIAL(oJAVA_LOOP_NAMEo, Symbol*, local_gensym("LOOP"));
    BIND_STELLA_SPECIAL(oJAVA_LOOP_NAME_USEDpo, boolean, FALSE);
    { Object* test = java_translate_a_tree(tree->rest->value);
      Cons* body = java_translate_list_of_trees(tree->rest->rest);

      tree->rest->rest = NIL;
      if (oJAVA_LOOP_NAME_USEDpo.get()) {
        return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_NAMED_STATEMENT, java_translate_a_tree(oJAVA_LOOP_NAMEo.get()), cons(listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_WHILE, test, cons(listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_PROGN, cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, body->concatenate(NIL, 0)), NIL), NIL)), NIL)));
      }
      else {
        return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_WHILE, test, cons(listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_PROGN, cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, body->concatenate(NIL, 0)), NIL), NIL)));
      }
    }
  }
}

Cons* java_translate_choose_tree(Cons* tree) {
  { Object* test = tree->rest->value;
    Object* truebody = tree->rest->rest->value;
    Object* falsebody = tree->rest->rest->rest->value;

    tree->rest->rest = NIL;
    return (java_translate_operator_call(cons(wrap_string("?"), cons(wrap_string(":"), NIL)), cons(test, cons(truebody, cons(falsebody, NIL))), 3));
  }
}

Cons* java_translate_when_tree(Cons* tree) {
  { Object* test = java_translate_a_tree(tree->rest->value);
    Cons* body = java_translate_list_of_trees(tree->rest->rest);

    tree->rest->rest = NIL;
    return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_WHEN, test, cons(cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, body->concatenate(NIL, 0)), NIL)));
  }
}

Cons* java_translate_unless_tree(Cons* tree) {
  { Object* test = java_translate_a_tree(tree->rest->value);
    Cons* body = java_translate_list_of_trees(tree->rest->rest);

    tree->rest->rest = NIL;
    return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_UNLESS, test, cons(cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, body->concatenate(NIL, 0)), NIL)));
  }
}

Cons* java_translate_make_tree(Cons* tree) {
  { Class* renamed_Class = ((Symbol*)(tree->rest->value))->get_stella_class(TRUE);
    String_Wrapper* classname = ((((String_Wrapper*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_CLASS_JAVA_NATIVE_TYPE, NULL_STRING_WRAPPER)))->wrapper_value != NULL) ? wrap_string(renamed_Class->java_native_type()) : java_translate_class_name(((Generalized_Symbol*)(tree->rest->value))));
    Cons* otree = listO(4, SYM_JAVA_TRANSLATE_STELLA_JAVA_MAKE, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, classname, NIL), java_translate_actual_parameters(tree->rest->rest), NIL);

    return (otree);
  }
}

Cons* java_translate_sys_new(Cons* tree) {
  { Standard_Object* typespec = ((Standard_Object*)(tree->rest->value));

    if (array_type_specifierP(typespec)) {
      return (java_translate_new_array(tree));
    }
    else {
      return (listO(5, SYM_JAVA_TRANSLATE_STELLA_JAVA_FUNCTION_CALL, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, java_translate_type_spec(typespec), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, java_translate_name(yield_constructor_name(type_spec_to_class(typespec))), NIL), java_translate_actual_parameters(tree->rest->rest), NIL));
    }
  }
}

Cons* java_translate_new_array(Cons* tree) {
  { boolean dummy1;

    { Parametric_Type_Specifier* arraytype = ((Parametric_Type_Specifier*)(tree->rest->value));
      Object* initialelement = tree->rest->rest->value;
      Cons* dimensions = java_translate_list_of_trees(tree->rest->rest->rest);
      Standard_Object* elementtype = extract_parameter_type(arraytype, SYM_JAVA_TRANSLATE_STELLA_ANY_VALUE, dummy1);

      return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_MAKE_ARRAY, java_translate_type_spec(elementtype), dimensions->concatenate(NIL, 0)));
    }
  }
}

Cons* java_translate_mv_setq(Cons* tree) {
  { Cons* variables = ((Cons*)(tree->rest->value));
    Cons* functioncall = ((Cons*)(tree->rest->rest->value));
    Cons* mvassignments = NIL;
    Cons* otree = NULL;
    Cons* otherreturntypes = NIL;
    Cons* mvreturnedvalue = NIL;

    if (functioncall->value == SYM_JAVA_TRANSLATE_STELLA_SYS_CALL_FUNCTION) {
      otherreturntypes = lookup_function(((Symbol*)(functioncall->rest->value)))->method_return_type_specifiers_reader()->rest();
    }
    else {
      otherreturntypes = ((Method_Slot*)(lookup_slot(((Class*)(((Surrogate*)(functioncall->rest->value))->surrogate_value)), ((Symbol*)(functioncall->rest->rest->value)))))->method_return_type_specifiers_reader()->rest();
    }
    { Object* variable = NULL;
      int currentindex = NULL_INTEGER;
      Object* returntype = NULL;
      Cons* iter_000 = variables->rest;
      int iter_001 = 0;
      Cons* iter_002 = otherreturntypes;

      while ((!(iter_000 == NIL)) &&
          (!(iter_002 == NIL))) {
        variable = iter_000->value;
        iter_000 = iter_000->rest;
        currentindex = iter_001;
        iter_001 = iter_001 + 1;
        returntype = iter_002->value;
        iter_002 = iter_002->rest;
        mvreturnedvalue = listO(4, SYM_JAVA_TRANSLATE_STELLA_JAVA_AREF, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string("caller_MV_returnarray"), NIL), wrap_integer(currentindex), NIL);
        if (subtype_ofP(type_spec_to_base_type(((Standard_Object*)(returntype))), SGT_JAVA_TRANSLATE_STELLA_LITERAL)) {
          mvreturnedvalue = listO(4, SYM_JAVA_TRANSLATE_STELLA_JAVA_SLOT_VALUE, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_CAST, mvreturnedvalue, cons(java_translate_type_spec(type_spec_to_base_type(((Standard_Object*)(returntype)))->type_to_wrapped_type()->yield_type_specifier()), NIL)), wrap_string("wrapperValue"), NIL);
        }
        mvassignments = cons(listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_ASSIGN, java_translate_a_tree(variable), cons(listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_CAST, mvreturnedvalue, cons(java_translate_type_spec(((Standard_Object*)(returntype))), NIL)), NIL)), mvassignments);
      }
    }
    otree = listO(4, SYM_JAVA_TRANSLATE_STELLA_JAVA_BLOCK, cons(cons(java_translate_array_of_type_spec(SGT_JAVA_TRANSLATE_STELLA_NATIVE_OBJECT_POINTER, FALSE), listO(3, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string("caller_MV_returnarray"), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_MAKE_ARRAY, java_translate_type_spec(SGT_JAVA_TRANSLATE_STELLA_NATIVE_OBJECT_POINTER), cons(wrap_integer(otherreturntypes->length()), NIL)), NIL)), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_ASSIGN, java_translate_a_tree(variables->value), cons(java_translate_mv_function_call(functioncall, SYM_JAVA_TRANSLATE_STELLA_CALLER_MV_RETURNARRAY), NIL)), mvassignments->reverse()->concatenate(NIL, 0)), NIL);
    return (otree);
  }
}

Cons* java_translate_mv_function_call(Cons* functioncall, Symbol* mvvectorname) {
  return (((Cons*)(java_translate_a_tree(functioncall->concatenate(cons(mvvectorname, NIL), 0)))));
}

Cons* java_translate_code_signature(Cons* signature) {
  { Cons* translatedreturntypes = NIL;
    Cons* translatedparametertypes = NIL;

    { Object* returntype = NULL;
      Cons* iter_000 = ((Cons*)(signature->value));
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        returntype = iter_000->value;
        iter_000 = iter_000->rest;
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(java_translate_type_spec(((Standard_Object*)(returntype))), NIL);
            if (translatedreturntypes == NIL) {
              translatedreturntypes = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(translatedreturntypes, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(java_translate_type_spec(((Standard_Object*)(returntype))), NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
    }
    { Object* parametertype = NULL;
      Cons* iter_001 = signature->rest;
      Cons* collect_001 = NULL;

      while (!(iter_001 == NIL)) {
        parametertype = iter_001->value;
        iter_001 = iter_001->rest;
        if (!((boolean)(collect_001))) {
          {
            collect_001 = cons(java_translate_type_spec(((Standard_Object*)(parametertype))), NIL);
            if (translatedparametertypes == NIL) {
              translatedparametertypes = collect_001;
            }
            else {
              add_cons_to_end_of_cons_list(translatedparametertypes, collect_001);
            }
          }
        }
        else {
          {
            collect_001->rest = cons(java_translate_type_spec(((Standard_Object*)(parametertype))), NIL);
            collect_001 = collect_001->rest;
          }
        }
      }
    }
    return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_FUNCTION_SIGNATURE, translatedreturntypes, cons(translatedparametertypes, NIL)));
  }
}

Cons* java_translate_with_native_wrapper(Object* tree, Standard_Object* type) {
  { String_Wrapper* dummy1;

    if (java_literalP(type)) {
      return (listO(4, SYM_JAVA_TRANSLATE_STELLA_JAVA_MAKE, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, java_native_literal_wrapper_names(type, dummy1), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_ACTUALS, java_translate_a_tree(tree), NIL), NIL));
    }
    else {
      return (((Cons*)(java_translate_a_tree(tree))));
    }
  }
}

Cons* java_translate_actual_parameters_with_native_wrappers(Cons* trees, Cons* parametertypes) {
  { Cons* cursor = trees;
    Cons* typecursor = parametertypes;

    while (!(cursor == NIL)) {
      cursor->value = java_translate_with_native_wrapper(cursor->value, ((Standard_Object*)(typecursor->value)));
      cursor = cursor->rest;
      typecursor = typecursor->rest;
    }
    return (trees);
  }
}

Cons* java_wrap_with_native_value_extraction(Cons* tree, Standard_Object* primaryreturntype) {
  { String_Wrapper* wrappertype = NULL;
    String_Wrapper* extractorname = NULL;

    wrappertype = java_native_literal_wrapper_names(primaryreturntype, extractorname);
    return (listO(6, SYM_JAVA_TRANSLATE_STELLA_JAVA_METHOD_CALL, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string(""), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, extractorname, NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_CAST, tree, cons(wrappertype, NIL)), cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_ACTUALS, NIL), NIL));
  }
}

Cons* java_translate_object_to_return_type(Cons* tree, Standard_Object* returntype) {
  if (java_literalP(returntype)) {
    return (java_wrap_with_native_value_extraction(tree, returntype));
  }
  else if ((returntype == SGT_JAVA_TRANSLATE_STELLA_STRING) ||
      (returntype == SGT_JAVA_TRANSLATE_STELLA_MUTABLE_STRING)) {
    return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_CAST, tree, cons(java_translate_type_spec(returntype), NIL)));
  }
  else if (returntype == SGT_JAVA_TRANSLATE_STELLA_VOID) {
    return (tree);
  }
  else {
    return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_CAST, tree, cons(java_translate_type_spec(returntype), NIL)));
  }
}

Cons* java_translate_call_function_code(Cons* tree) {
  { Cons* signature = java_translate_code_signature(((Cons*)(tree->rest->value)));
    Object* functionnameexpr = java_translate_a_tree(tree->rest->rest->value);
    Cons* functionargs = tree->rest->rest->rest;
    Standard_Object* primaryreturntype = ((Standard_Object*)(((Cons*)(((Cons*)(tree->rest->value))->value))->value));
    Cons* otree = NULL;

    otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_FUNCALL, signature, cons(functionnameexpr, cons(cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_ACTUALS, java_translate_actual_parameters_with_native_wrappers(functionargs, ((Cons*)(tree->rest->value))->rest)->concatenate(NIL, 0)), NIL)));
    return (java_translate_object_to_return_type(otree, primaryreturntype));
  }
}

Cons* java_translate_call_method_code(Cons* tree) {
  { Cons* signature = java_translate_code_signature(((Cons*)(tree->rest->value)));
    Object* methodnameexpr = java_translate_a_tree(tree->rest->rest->value);
    Standard_Object* primaryreturntype = ((Standard_Object*)(((Cons*)(((Cons*)(tree->rest->value))->value))->value));
    Object* theobject = tree->rest->rest->rest->value;
    Cons* methodargs = tree->rest->rest->rest->rest;
    Cons* otree = NULL;

    otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_METHOD_CODE_CALL, signature, cons(methodnameexpr, cons(java_translate_a_tree(theobject), cons(cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_ACTUALS, java_translate_actual_parameters_with_native_wrappers(methodargs, ((Cons*)(tree->rest->value))->rest->rest)->concatenate(NIL, 0)), NIL))));
    return (java_translate_object_to_return_type(otree, primaryreturntype));
  }
}

char* java_package_prefix(Module* module, char* separator) {
  { char* result = "";
    char separator_char = separator[0];
    char* package_prefix = module->java_package_reader();

    if (package_prefix != NULL) {
      if (separator_char == '.') {
        result = package_prefix;
      }
      else {
        result = string_substitute(package_prefix, separator_char, '.');
      }
      return (string_concatenate(result, separator, 0));
    }
    result = string_concatenate(java_translate_namestring(string_downcase(module->name())), separator, 0);
    { Context* mod = NULL;
      Iterator* iter_000 = module->parent_contexts();

      while (iter_000->nextP()) {
        mod = ((Context*)(iter_000->value));
        if (isaP(mod, SGT_JAVA_TRANSLATE_STELLA_MODULE) &&
            (!(mod == oROOT_MODULEo))) {
          result = string_concatenate(java_package_prefix(((Module*)(mod)), separator), result, 0);
          break;
        }
      }
    }
    if (separator_char == '.') {
      set_dynamic_slot_value(module->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_JAVA_PACKAGE, wrap_string(string_subsequence(result, 0, strlen(result) - 1)), NULL_STRING_WRAPPER);
    }
    return (result);
  }
}

String_Wrapper* java_yield_fully_qualified_type_name(Standard_Object* typespec) {
  { Surrogate* basetypespec = type_spec_to_base_type(typespec);
    String_Wrapper* translatedtype = NULL;
    char* nativetype = ((Class*)(basetypespec->surrogate_value))->java_native_type();

    if (basetypespec == SGT_JAVA_TRANSLATE_STELLA_ARGUMENT_LIST) {
      basetypespec = SGT_JAVA_TRANSLATE_STELLA_CONS;
      nativetype = NULL;
    }
    if (nativetype != NULL) {
      if (string_memberP(nativetype, '.') ||
          java_literalP(typespec)) {
        return (wrap_string(nativetype));
      }
      else {
        return (wrap_string(string_concatenate("java.lang.", nativetype, 0)));
      }
    }
    translatedtype = java_translate_type_spec(basetypespec);
    if (string_memberP(translatedtype->wrapper_value, '.')) {
      return (translatedtype);
    }
    else {
      return (wrap_string(string_concatenate(java_package_prefix(basetypespec->home_module(), "."), translatedtype->wrapper_value, 0)));
    }
  }
}

String_Wrapper* java_yield_translated_class_and_function_names(Method_Slot* f, String_Wrapper*& _return1) {
  if (java_flotsam_functionP(f)) {
    { String_Wrapper* _return_temp = wrap_string(string_concatenate(java_package_prefix(f->home_module(), "."), java_yield_flotsam_class_name(f->home_module()), 0));

      _return1 = java_translate_function_name(f);
      return (_return_temp);
    }
  }
  else if (f->method_startup_functionP()) {
    { String_Wrapper* _return_temp = wrap_string(string_concatenate(java_package_prefix(f->home_module(), "."), java_translate_class_namestring(wrap_string(((String_Wrapper*)(dynamic_slot_value(f->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_METHOD_STARTUP_CLASSNAME, NULL_STRING_WRAPPER)))->wrapper_value))->wrapper_value, 0));

      _return1 = java_translate_function_name(f);
      return (_return_temp);
    }
  }
  else if (((Boolean_Wrapper*)(dynamic_slot_value(f->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_METHOD_NATIVEp, FALSE_WRAPPER)))->wrapper_value) {
    { String_Wrapper* _return_temp = wrap_string(java_yield_full_native_class_name());

      _return1 = java_translate_function_name(f);
      return (_return_temp);
    }
  }
  else if (((Boolean_Wrapper*)(dynamic_slot_value(f->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_METHOD_CONSTRUCTORp, FALSE_WRAPPER)))->wrapper_value) {
    { String_Wrapper* _return_temp = java_yield_fully_qualified_type_name(((Standard_Object*)(f->method_return_type_specifiers_reader()->first())));

      _return1 = java_translate_function_name(f);
      return (_return_temp);
    }
  }
  else {
    { String_Wrapper* _return_temp = java_yield_fully_qualified_type_name(((Standard_Object*)(f->method_parameter_type_specifiers_reader()->first())));

      _return1 = java_translate_function_name(f);
      return (_return_temp);
    }
  }
}

char* java_yield_translated_class_and_method_names(Standard_Object* owner, Method_Slot* f, char*& _return1) {
  if (((Boolean_Wrapper*)(dynamic_slot_value(f->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_METHOD_NATIVEp, FALSE_WRAPPER)))->wrapper_value) {
    { char* _return_temp = java_yield_full_native_class_name();

      _return1 = java_translate_function_name(f)->wrapper_value;
      return (_return_temp);
    }
  }
  else if (java_flotsam_functionP(f)) {
    { char* _return_temp = string_concatenate(java_package_prefix(f->home_module(), "."), java_yield_flotsam_class_name(f->home_module()), 0);

      _return1 = java_translate_function_name(f)->wrapper_value;
      return (_return_temp);
    }
  }
  else {
    { char* _return_temp = java_yield_fully_qualified_type_name(owner)->wrapper_value;

      _return1 = java_translate_function_name(f)->wrapper_value;
      return (_return_temp);
    }
  }
}

Cons* java_translate_class_object_lookup(char* classname) {
  return (listO(5, SYM_JAVA_TRANSLATE_STELLA_JAVA_FUNCTION_CALL, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string(java_yield_full_native_class_name()), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string("find_java_class"), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_ACTUALS, wrap_string(classname), NIL), NIL));
}

Cons* java_yield_class_object_array_expression(Standard_Object* typespec, int ndimensions) {
  { char* prefix = make_string(ndimensions, '[');
    Surrogate* classtype = NULL;
    char* classid = NULL;

    { Surrogate* test_value_000 = safe_primary_type(typespec);

      if (subtype_of_parametric_type_specifierP(test_value_000)) {
        { Standard_Object* typespec_000 = typespec;
          Parametric_Type_Specifier* typespec = ((Parametric_Type_Specifier*)(typespec_000));

          classtype = typespec->specifier_base_type;
        }
      }
      else if (subtype_of_surrogateP(test_value_000)) {
        { Standard_Object* typespec_001 = typespec;
          Surrogate* typespec = ((Surrogate*)(typespec_001));

          classtype = ((Class*)(typespec->surrogate_value))->class_type;
        }
      }
      else {
        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      }
    }
    if (classtype == SGT_JAVA_TRANSLATE_STELLA_VOID) {
      throw *new_stella_exception("VOID is an invalid array base type.");
    }
    else if (classtype == SGT_JAVA_TRANSLATE_STELLA_BOOLEAN) {
      classid = "Z";
    }
    else if (classtype == SGT_JAVA_TRANSLATE_STELLA_INTEGER) {
      classid = "I";
    }
    else if (classtype == SGT_JAVA_TRANSLATE_STELLA_CHARACTER) {
      classid = "C";
    }
    else if ((classtype == SGT_JAVA_TRANSLATE_STELLA_BYTE) ||
        (classtype == SGT_JAVA_TRANSLATE_STELLA_OCTET)) {
      classid = "B";
    }
    else if ((classtype == SGT_JAVA_TRANSLATE_STELLA_FLOAT) ||
        (classtype == SGT_JAVA_TRANSLATE_STELLA_DOUBLE_FLOAT)) {
      classid = "D";
    }
    else if ((classtype == SGT_JAVA_TRANSLATE_STELLA_SHORT_INTEGER) ||
        (classtype == SGT_JAVA_TRANSLATE_STELLA_UNSIGNED_SHORT_INTEGER)) {
      classid = "S";
    }
    else if ((classtype == SGT_JAVA_TRANSLATE_STELLA_LONG_INTEGER) ||
        (classtype == SGT_JAVA_TRANSLATE_STELLA_UNSIGNED_LONG_INTEGER)) {
      classid = "J";
    }
    else if (classtype == SGT_JAVA_TRANSLATE_STELLA_SINGLE_FLOAT) {
      classid = "F";
    }
    else if (classtype == SGT_JAVA_TRANSLATE_STELLA_STRING) {
      classid = "Ljava.lang.String;";
    }
    else if (classtype == SGT_JAVA_TRANSLATE_STELLA_MUTABLE_STRING) {
      classid = "Ljava.lang.StringBuffer;";
    }
    else {
      classid = string_concatenate("L", java_yield_fully_qualified_type_name(typespec)->wrapper_value, 1, ";");
    }
    return (java_translate_class_object_lookup(string_concatenate(prefix, classid, 0)));
  }
}

Cons* java_yield_class_object_expression(Standard_Object* typespec) {
  { boolean dummy1;

    { Cons* otree = NULL;
      Surrogate* classtype = NULL;

      { Surrogate* test_value_000 = safe_primary_type(typespec);

        if (subtype_of_parametric_type_specifierP(test_value_000)) {
          { Standard_Object* typespec_000 = typespec;
            Parametric_Type_Specifier* typespec = ((Parametric_Type_Specifier*)(typespec_000));

            if (array_type_specifierP(typespec)) {
              return (java_yield_class_object_array_expression(extract_parameter_type(typespec, SYM_JAVA_TRANSLATE_STELLA_ANY_VALUE, dummy1), array_type_dimensions(typespec)->length()));
            }
            else {
              classtype = typespec->specifier_base_type;
            }
          }
        }
        else if (subtype_of_surrogateP(test_value_000)) {
          { Standard_Object* typespec_001 = typespec;
            Surrogate* typespec = ((Surrogate*)(typespec_001));

            classtype = ((Class*)(typespec->surrogate_value))->class_type;
          }
        }
        else {
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        }
      }
      if (classtype == SGT_JAVA_TRANSLATE_STELLA_VOID) {
        otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_VERBATIM, wrap_string("java.lang.Void.TYPE"), NIL);
      }
      else if (classtype == SGT_JAVA_TRANSLATE_STELLA_BOOLEAN) {
        otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_VERBATIM, wrap_string("java.lang.Boolean.TYPE"), NIL);
      }
      else if ((classtype == SGT_JAVA_TRANSLATE_STELLA_INTEGER) ||
          (classtype == SGT_JAVA_TRANSLATE_STELLA_THIRTY_TWO_BIT_VECTOR)) {
        otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_VERBATIM, wrap_string("java.lang.Integer.TYPE"), NIL);
      }
      else if (classtype == SGT_JAVA_TRANSLATE_STELLA_CHARACTER) {
        otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_VERBATIM, wrap_string("java.lang.Character.TYPE"), NIL);
      }
      else if ((classtype == SGT_JAVA_TRANSLATE_STELLA_BYTE) ||
          (classtype == SGT_JAVA_TRANSLATE_STELLA_OCTET)) {
        otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_VERBATIM, wrap_string("java.lang.Byte.TYPE"), NIL);
      }
      else if ((classtype == SGT_JAVA_TRANSLATE_STELLA_FLOAT) ||
          (classtype == SGT_JAVA_TRANSLATE_STELLA_DOUBLE_FLOAT)) {
        otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_VERBATIM, wrap_string("java.lang.Double.TYPE"), NIL);
      }
      else if ((classtype == SGT_JAVA_TRANSLATE_STELLA_SHORT_INTEGER) ||
          (classtype == SGT_JAVA_TRANSLATE_STELLA_UNSIGNED_SHORT_INTEGER)) {
        otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_VERBATIM, wrap_string("java.lang.Short.TYPE"), NIL);
      }
      else if ((classtype == SGT_JAVA_TRANSLATE_STELLA_LONG_INTEGER) ||
          ((classtype == SGT_JAVA_TRANSLATE_STELLA_UNSIGNED_LONG_INTEGER) ||
           ((classtype == SGT_JAVA_TRANSLATE_STELLA_TICKTOCK) ||
            (classtype == SGT_JAVA_TRANSLATE_STELLA_NATIVE_DATE_TIME)))) {
        otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_VERBATIM, wrap_string("java.lang.Long.TYPE"), NIL);
      }
      else if (classtype == SGT_JAVA_TRANSLATE_STELLA_SINGLE_FLOAT) {
        otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_VERBATIM, wrap_string("java.lang.Float.TYPE"), NIL);
      }
      else if (classtype == SGT_JAVA_TRANSLATE_STELLA_STRING) {
        otree = java_translate_class_object_lookup("java.lang.String");
      }
      else if (classtype == SGT_JAVA_TRANSLATE_STELLA_MUTABLE_STRING) {
        otree = java_translate_class_object_lookup("java.lang.StringBuffer");
      }
      else {
        if ((!(classtype == SGT_JAVA_TRANSLATE_STELLA_ARGUMENT_LIST)) &&
            (((String_Wrapper*)(dynamic_slot_value(((Class*)(classtype->surrogate_value))->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_CLASS_JAVA_NATIVE_TYPE, NULL_STRING_WRAPPER)))->wrapper_value != NULL)) {
          otree = java_translate_class_object_lookup(yield_class_object_lookup_name_for_native_type(((Class*)(classtype->surrogate_value))->java_native_type()));
        }
        else {
          otree = java_translate_class_object_lookup(java_yield_fully_qualified_type_name(typespec)->wrapper_value);
        }
      }
      return (otree);
    }
  }
}

Key_Value_List* oJAVA_PRIMITIVE_ARRAY_TYPE_NAMESo = NULL;

char* yield_class_object_lookup_name_for_native_type(char* nativetype) {
  { char* basetype = nativetype;
    char* package = "java.lang.";
    int arraydimensions = 0;
    int index = -1;
    int arraymarkerindex = NULL_INTEGER;

    { char ch = NULL_CHARACTER;
      int i = NULL_INTEGER;
      char* vector_000 = nativetype;
      int index_000 = 0;
      int length_000 = strlen(vector_000);
      int iter_000 = 0;

      while (index_000 < length_000) {
        ch = vector_000[index_000];
        index_000 = index_000 + 1;
        i = iter_000;
        iter_000 = iter_000 + 1;
        if (ch == '.') {
          index = i;
        }
        if (ch == '[') {
          if (arraymarkerindex == NULL_INTEGER) {
            arraymarkerindex = i;
          }
          arraydimensions = arraydimensions + 1;
        }
      }
    }
    if (index >= 0) {
      package = string_subsequence(nativetype, 0, index + 1);
    }
    basetype = string_subsequence(nativetype, index + 1, arraymarkerindex);
    if (((boolean)(oJAVA_PRIMITIVE_ARRAY_TYPE_NAMESo->lookup(wrap_string(basetype))))) {
      basetype = ((String_Wrapper*)(oJAVA_PRIMITIVE_ARRAY_TYPE_NAMESo->lookup(wrap_string(basetype))))->wrapper_value;
    }
    else if (arraydimensions > 0) {
      basetype = string_concatenate("L", package, 2, basetype, ";");
    }
    else {
      basetype = string_concatenate(package, basetype, 0);
    }
    switch (arraydimensions) {
      case 0: 
        return (basetype);
      case 1: 
        return (string_concatenate("[", basetype, 0));
      case 2: 
        return (string_concatenate("[[", basetype, 0));
      default:
        return (string_concatenate(make_string(arraydimensions, '['), basetype, 0));
    }
  }
}

Cons* java_translate_param_type_specifications(Method_Slot* fn, Standard_Object* owner) {
  { Cons* paramclasses = NIL;
    boolean skipparameterP = (!fn->method_functionP) &&
        (!java_method_object_defined_on_native_typeP(fn));

    { Standard_Object* ts = NULL;
      Cons* iter_000 = fn->method_parameter_type_specifiers_reader()->the_cons_list;

      while (!(iter_000 == NIL)) {
        ts = ((Standard_Object*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (skipparameterP) {
          skipparameterP = FALSE;
        }
        else if (anchored_type_specifierP(ts)) {
          paramclasses = cons(java_yield_class_object_expression(compute_anchored_type_spec(owner, ((Anchored_Type_Specifier*)(ts)))), paramclasses);
        }
        else {
          paramclasses = cons(java_yield_class_object_expression(ts), paramclasses);
        }
      }
    }
    if (!(fn->method_return_type_specifiers_reader()->rest() == NIL)) {
      paramclasses = cons(java_yield_class_object_array_expression(SGT_JAVA_TRANSLATE_STELLA_NATIVE_OBJECT_POINTER, 1), paramclasses);
    }
    return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_ANONYMOUS_ARRAY, wrap_string("java.lang.Class"), paramclasses->reverse()->concatenate(NIL, 0)));
  }
}

Object* java_translate_the_code(Cons* tree) {
  { Cons* otree = NULL;
    Method_Slot* fn = NULL;
    char* classname = "";
    char* functionname = "";

    { Generalized_Symbol* test_value_000 = ((Generalized_Symbol*)(tree->rest->value));

      if (test_value_000 == KWD_JAVA_TRANSLATE_FUNCTION) {
        fn = lookup_function(((Symbol*)(tree->rest->rest->value)));
        if (!(((boolean)(fn)))) {
          fn = ((Method_Slot*)(tree->fourth()));
        }
        if (!(((boolean)(fn)))) {
          *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't translate the-code form for function " << "`" << tree->rest->rest->value << "'" << endl << " because it there is no corresponding function object" << endl;
          return (java_translate_null());
        }
        { String_Wrapper* value_000 = NULL;
          String_Wrapper* value_001 = NULL;

          value_000 = java_yield_translated_class_and_function_names(fn, value_001);
          {
            classname = value_000->wrapper_value;
            functionname = value_001->wrapper_value;
          }
        }
        otree = listO(5, SYM_JAVA_TRANSLATE_STELLA_JAVA_FUNCTION_CALL, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string(java_yield_full_native_class_name()), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string("find_java_method"), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_ACTUALS, wrap_string(classname), cons(wrap_string(functionname), cons(java_translate_param_type_specifications(fn, NULL), NIL))), NIL);
      }
      else if (test_value_000 == KWD_JAVA_TRANSLATE_METHOD) {
        fn = ((Method_Slot*)(lookup_slot(((Class*)(((Surrogate*)(tree->rest->rest->value))->surrogate_value)), ((Symbol*)(tree->fourth())))));
        if (!(((boolean)(fn)))) {
          *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't translate the-code form for method " << "`" << tree->fourth() << "'" << endl << " because it there is no corresponding function object" << endl;
          return (java_translate_null());
        }
        classname = java_yield_translated_class_and_method_names(((Standard_Object*)(tree->rest->rest->value)), fn, functionname);
        otree = listO(5, SYM_JAVA_TRANSLATE_STELLA_JAVA_FUNCTION_CALL, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string(java_yield_full_native_class_name()), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string("find_java_method"), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_ACTUALS, wrap_string(classname), cons(wrap_string(functionname), cons(java_translate_param_type_specifications(fn, ((Standard_Object*)(tree->rest->rest->value))), NIL))), NIL);
      }
      else {
        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      }
    }
    return (otree);
  }
}

String_Wrapper* java_translate_function_name(Method_Slot* function) {
  { Symbol* functionname = yield_renamed_name_if_native(function->slot_name, KWD_JAVA_TRANSLATE_JAVA, KWD_JAVA_TRANSLATE_FUNCTION);
    char* translatedname = NULL;

    if (java_method_object_is_overloaded_functionP(function)) {
      functionname = java_create_overloaded_function_name(functionname, function->slot_owner);
    }
    translatedname = java_translate_name(functionname)->wrapper_value;
    if (function->method_setterP) {
      translatedname = string_concatenate(translatedname, "_setter", 0);
    }
    if (slot_readerP(function)) {
      translatedname = string_concatenate(translatedname, "_reader", 0);
    }
    return (wrap_string(translatedname));
  }
}

Object* java_translate_sys_pointer_to_function(Cons* tree) {
  *(STANDARD_WARNING->native_stream) << "Warning: " << "java-translate-sys-pointer-to-function not yet implemented" << endl;
  return (java_translate_null());
}

Object* java_translate_ignore_tree(Cons* tree) {
  return (java_translate_null());
}

boolean java_stream_is_standard_outputP(Object* tree) {
  if (safe_primary_type(tree) == SGT_JAVA_TRANSLATE_STELLA_CONS) {
    { Object* tree_000 = tree;
      Cons* tree = ((Cons*)(tree_000));

      return (equal_cons_treesP(tree, get_quoted_tree("((SYS-SLOT-VALUE @OUTPUT-STREAM NATIVE-STREAM STANDARD-OUTPUT) \"/STELLA\")", "/STELLA")));
    }
  }
  else {
    return (FALSE);
  }
}

Cons* java_translate_print_stream(Cons* tree) {
  tree->first_setter(SYM_JAVA_TRANSLATE_STELLA_JAVA_PRINT_STREAM);
  if (subtype_of_keywordP(safe_primary_type(tree->rest->value))) {
    { Generalized_Symbol* test_value_000 = ((Generalized_Symbol*)(tree->rest->value));

      if ((test_value_000 == KWD_JAVA_TRANSLATE_WARN) ||
          ((test_value_000 == KWD_JAVA_TRANSLATE_ERROR) ||
           (test_value_000 == KWD_JAVA_TRANSLATE_CONTINUABLE_ERROR))) {
        tree->second_setter(SYM_JAVA_TRANSLATE_STELLA_JAVA_STANDARD_ERROR);
      }
      else {
        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      }
    }
  }
  else {
    if (java_stream_is_standard_outputP(tree->rest->value)) {
      tree->second_setter(SYM_JAVA_TRANSLATE_STELLA_JAVA_STANDARD_OUT);
    }
    else {
      tree->second_setter(java_translate_a_tree(tree->rest->value));
    }
  }
  java_translate_list_of_trees(tree->rest->rest);
  return (tree);
}

Cons* java_translate_print_native_stream(Cons* tree) {
  { Cons* otree = java_translate_print_stream(tree);

    otree->first_setter(SYM_JAVA_TRANSLATE_STELLA_JAVA_PRINT_NATIVE_STREAM);
    return (otree);
  }
}

Cons* java_translate_null() {
  return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string("null"), NIL));
}

Cons* java_translate_eol() {
  return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_EOL, java_translate_global_name(SYM_JAVA_TRANSLATE_STELLA_EOL, FALSE), NIL));
}

Cons* Object::java_translate_atomic_tree() {
  { Object* tree = this;

    return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_LITERAL, tree, NIL));
  }
}

char* java_prepend_global_class_name(char* namestring, Module* module) {
  return (string_concatenate(java_yield_flotsam_class_name(module), ".", 1, namestring));
}

String_Wrapper* java_translate_global_name(Symbol* namesymbol, boolean definitionP) {
  if (!((boolean)(namesymbol))) {
    return (wrap_string("NULL"));
  }
  { char* rawname = java_substitute_forbidden_characters(java_change_case(namesymbol, KWD_JAVA_TRANSLATE_UPPERCASE), KWD_JAVA_TRANSLATE_UPPERCASE);
    Module* module = namesymbol->home_module();

    if (definitionP) {
      return (wrap_string(rawname));
    }
    else if (in_current_java_packageP(module)) {
      return (wrap_string(java_prepend_global_class_name(rawname, module)));
    }
    else {
      return (wrap_string(string_concatenate(java_package_prefix(module, "."), java_prepend_global_class_name(rawname, module), 0)));
    }
  }
}

Cons* java_translate_general_symbol(Symbol* tree) {
  { Global_Variable* globalvar = tree->soft_permanentify()->lookup_global_variable();

    if (!((boolean)(globalvar))) {
      return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, java_translate_name(tree), NIL));
    }
    else if (globalvar->variable_specialP) {
      return (java_translate_object_to_return_type(listO(6, SYM_JAVA_TRANSLATE_STELLA_JAVA_METHOD_CALL, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string(""), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string("get"), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, java_translate_global_name(tree, FALSE), NIL), cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_ACTUALS, NIL), NIL), global_variable_type_spec(globalvar)));
    }
    else {
      return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, java_translate_global_name(tree, FALSE), NIL));
    }
  }
}

Cons* Symbol::java_translate_atomic_tree() {
  { Symbol* tree = this;

    if (tree == SYM_JAVA_TRANSLATE_STELLA_NULL) {
      return (java_translate_null());
    }
    else if (tree == SYM_JAVA_TRANSLATE_STELLA_EOL) {
      return (java_translate_eol());
    }
    else {
      return (java_translate_general_symbol(tree));
    }
  }
}

Object* java_translate_method_parameter(Symbol* name, Standard_Object* type, Method_Slot* method) {
  type = compute_relative_type_spec(type, method->slot_owner);
  if (((Boolean_Wrapper*)(dynamic_slot_value(method->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_METHOD_VARIABLE_ARGUMENTSp, FALSE_WRAPPER)))->wrapper_value &&
      ((variable_arguments_name(method) == name) &&
       (pass_variable_arguments_as_listP(method) ||
        string_concatenate_methodP(method)))) {
    type = yield_listified_variable_arguments_type(method);
  }
  return (cons(java_translate_type_spec(type), cons(java_translate_name(name), NIL)));
}

Cons* java_translate_return_parameter() {
  return (listO(3, wrap_string("Object []"), wrap_string("MV_returnarray"), NIL));
}

Cons* java_translate_method_parameters(Method_Slot* method) {
  { boolean skipfirstparameterP = !java_method_object_is_functionP(method);
    Cons* translatedparameters = NIL;

    { Symbol* pname = NULL;
      Standard_Object* ptype = NULL;
      int i = NULL_INTEGER;
      Cons* iter_000 = method->method_parameter_names_reader()->the_cons_list;
      Cons* iter_001 = method->method_parameter_type_specifiers_reader()->the_cons_list;
      int iter_002 = 1;
      Cons* collect_000 = NULL;

      while ((!(iter_000 == NIL)) &&
          (!(iter_001 == NIL))) {
        pname = ((Symbol*)(iter_000->value));
        iter_000 = iter_000->rest;
        ptype = ((Standard_Object*)(iter_001->value));
        iter_001 = iter_001->rest;
        i = iter_002;
        iter_002 = iter_002 + 1;
        if ((!skipfirstparameterP) ||
            (i > 1)) {
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(java_translate_method_parameter(pname, ptype, method), NIL);
              if (translatedparameters == NIL) {
                translatedparameters = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(translatedparameters, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(java_translate_method_parameter(pname, ptype, method), NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
    }
    if (!(method->method_return_type_specifiers_reader()->rest() == NIL)) {
      translatedparameters = translatedparameters->concatenate(cons(java_translate_return_parameter(), NIL), 0);
    }
    return (translatedparameters);
  }
}

boolean zero_argument_functionP(Method_Slot* method) {
  return (method->method_functionP &&
      method->method_parameter_names_reader()->emptyP());
}

boolean java_method_object_defined_on_native_typeP(Method_Slot* method) {
  { Standard_Object* firstparametertype = ((Standard_Object*)(method->method_parameter_type_specifiers_reader()->first()));

    return (((boolean)(firstparametertype)) &&
        (((String_Wrapper*)(dynamic_slot_value(type_spec_to_class(firstparametertype)->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_CLASS_JAVA_NATIVE_TYPE, NULL_STRING_WRAPPER)))->wrapper_value != NULL));
  }
}

boolean java_method_object_is_overloaded_functionP(Method_Slot* method) {
  return ((!method->method_functionP) &&
      (java_method_object_defined_on_native_typeP(method) &&
       ((!((Boolean_Wrapper*)(dynamic_slot_value(method->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_METHOD_NATIVEp, FALSE_WRAPPER)))->wrapper_value) ||
        subtype_ofP(method->slot_owner, SGT_JAVA_TRANSLATE_STELLA_LITERAL))));
}

boolean java_method_object_is_functionP(Method_Slot* method) {
  return (method->method_functionP ||
      java_method_object_is_overloaded_functionP(method));
}

boolean java_function_in_different_moduleP(Method_Slot* method) {
  { Standard_Object* firstparametertype = ((Standard_Object*)(method->method_parameter_type_specifiers_reader()->first()));

    if ((!((boolean)(firstparametertype))) ||
        (method->home_module() == type_spec_to_class(firstparametertype)->home_module())) {
      return (FALSE);
    }
    else {
      {
        if (!(java_method_object_is_functionP(method))) {
          *(STANDARD_WARNING->native_stream) << "Warning: " << "Method " << "`" << method << "'" << " being defined on type " << "`" << firstparametertype << "'" << ", but they are not in the same module!" << endl;
        }
        return (TRUE);
      }
    }
  }
}

Symbol* java_create_overloaded_function_name(Symbol* functionname, Surrogate* classtype) {
  return (intern_symbol(string_concatenate(classtype->symbol_name, "_", 1, functionname->symbol_name)));
}

String_Wrapper* Method_Slot::java_translate_method_name() {
  { Method_Slot* method = this;

    { Symbol* methodname = yield_renamed_name_if_native(method->slot_name, KWD_JAVA_TRANSLATE_JAVA, KWD_JAVA_TRANSLATE_FUNCTION);
      char* translatedname = NULL;

      if (java_method_object_is_overloaded_functionP(method)) {
        methodname = java_create_overloaded_function_name(methodname, method->slot_owner);
      }
      translatedname = java_translate_name(methodname)->wrapper_value;
      return (wrap_string(translatedname));
    }
  }
}

Cons* java_delete_quoted_null_statements(Cons* trees) {
  return (trees->remove(SYM_JAVA_TRANSLATE_STELLA_NULL));
}

Cons* java_wrap_method_body_with_vararg_declarations(Cons* methodbody) {
  return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_BLOCK, oVARARGDECLSo.get()->reverse(), cons(methodbody, NIL)));
}

Cons* java_wrap_method_body_with_vararg_value_setup(Cons* methodbody) {
  return (cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, oVARARGSTATEMENTSo.get()->reverse()->concatenate(cons(methodbody, NIL), 0)));
}

Cons* java_translate_method(Method_Slot* method) {
  { 
    BIND_STELLA_SPECIAL(oMETHODBEINGWALKEDo, Method_Slot*, NULL);
    BIND_STELLA_SPECIAL(oCURRENTDUMMYINDEXo, int, 0);
    BIND_STELLA_SPECIAL(oDUMMYDECLARATIONSo, Cons*, NIL);
    { boolean unitisfunction = java_method_object_is_functionP(method);
      Cons* bodytree = NULL;
      Cons* modifiers = NIL;
      Cons* returntype = NULL;
      Method_Slot* mostgeneralmethod = method;
      Cons* otree = NIL;
      Cons* translatedmethodbody = NULL;

      oMETHODBEINGWALKEDo.set(method);
      bodytree = java_delete_quoted_null_statements(bodytree);
      if (!(unitisfunction)) {
        substitute_cons_tree(bodytree, SYM_JAVA_TRANSLATE_STELLA_THIS, ((Symbol*)(method->method_parameter_names_reader()->first())));
      }
      while (((boolean)(mostgeneralmethod->slot_direct_equivalent)) &&
          (!(method_contains_anchored_typeP(((Method_Slot*)(mostgeneralmethod->slot_direct_equivalent))) ||
          method_contains_unknown_typeP(((Method_Slot*)(mostgeneralmethod->slot_direct_equivalent)))))) {
        mostgeneralmethod = ((Method_Slot*)(mostgeneralmethod->slot_direct_equivalent));
      }
      translatedmethodbody = cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS, java_translate_list_of_trees(bodytree)->concatenate(NIL, 0));
      if (method->publicP()) {
        modifiers = cons(wrap_string("public"), modifiers);
      }
      returntype = cons(java_translate_type_spec(mostgeneralmethod->slot_base_type), NIL);
      otree = cons((unitisfunction ? SYM_JAVA_TRANSLATE_STELLA_JAVA_FUNCTION : SYM_JAVA_TRANSLATE_STELLA_JAVA_METHOD), cons(modifiers, cons(returntype, cons(method->java_translate_method_name(), cons(java_translate_method_parameters(method), cons(translatedmethodbody, NIL))))));
      return (otree);
    }
  }
}

String_Wrapper* java_translate_variable_length_arg_name(Symbol* namesymbol, int parameternumber) {
  { Symbol* returnsymbol = intern_symbol(string_concatenate(namesymbol->symbol_name, integer_to_string(parameternumber), 0));
    String_Wrapper* translatedname = java_translate_name(returnsymbol);

    return (translatedname);
  }
}

Cons* java_translate_variable_length_actuals(Cons* actuals, Method_Slot* unused_method) {
  return (java_translate_actual_parameters(actuals));
}

Cons* java_translate_actual_parameters(Cons* tree) {
  return (cons(SYM_JAVA_TRANSLATE_STELLA_JAVA_ACTUALS, java_translate_list_of_trees(tree)->concatenate(NIL, 0)));
}

char* java_yield_class_name_for_function(Method_Slot* function) {
  if (java_flotsam_functionP(function)) {
    if (in_current_java_packageP(function->home_module())) {
      return (java_yield_flotsam_class_name(function->home_module()));
    }
    else {
      return (string_concatenate(java_package_prefix(function->home_module(), "."), java_yield_flotsam_class_name(function->home_module()), 0));
    }
  }
  else if (((Boolean_Wrapper*)(dynamic_slot_value(function->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_METHOD_NATIVEp, FALSE_WRAPPER)))->wrapper_value) {
    return ("Native");
  }
  else if (function->method_startup_functionP()) {
    if (in_current_java_packageP(function->home_module())) {
      return (java_translate_class_namestring(wrap_string(((String_Wrapper*)(dynamic_slot_value(function->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_METHOD_STARTUP_CLASSNAME, NULL_STRING_WRAPPER)))->wrapper_value))->wrapper_value);
    }
    else {
      return (string_concatenate(java_package_prefix(function->home_module(), "."), java_translate_class_namestring(wrap_string(((String_Wrapper*)(dynamic_slot_value(function->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_METHOD_STARTUP_CLASSNAME, NULL_STRING_WRAPPER)))->wrapper_value))->wrapper_value, 0));
    }
  }
  else {
    return (java_translate_type_spec(((Standard_Object*)(function->method_parameter_type_specifiers_reader()->first())))->wrapper_value);
  }
}

Cons* java_translate_function_call(Cons* tree, Method_Slot* method) {
  tree = tree->rest;
  { Symbol* functionname = ((Symbol*)(tree->value));
    Cons* functionargs = copy_cons_list(tree->rest);
    Object* firstarg = functionargs->value;
    Method_Slot* function = (((boolean)(method)) ? method : lookup_function(functionname));
    Cons* renamed_Operator = java_lookup_operator_table(functionname->soft_permanentify());
    Cons* otree = NULL;

    if ((functionname == SYM_JAVA_TRANSLATE_STELLA_GET_SYM) &&
        isaP(firstarg, SGT_JAVA_TRANSLATE_STELLA_INTEGER_WRAPPER)) {
      return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_SYMBOL, get_sym(((Integer_Wrapper*)(firstarg))->wrapper_value), NIL));
    }
    else if ((functionname == SYM_JAVA_TRANSLATE_STELLA_GET_KWD) &&
        isaP(firstarg, SGT_JAVA_TRANSLATE_STELLA_INTEGER_WRAPPER)) {
      return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_SYMBOL, get_kwd(((Integer_Wrapper*)(firstarg))->wrapper_value), NIL));
    }
    else if ((functionname == SYM_JAVA_TRANSLATE_STELLA_GET_SGT) &&
        isaP(firstarg, SGT_JAVA_TRANSLATE_STELLA_INTEGER_WRAPPER)) {
      return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_SYMBOL, get_sgt(((Integer_Wrapper*)(firstarg))->wrapper_value), NIL));
    }
    if (((boolean)(renamed_Operator))) {
      otree = java_translate_operator_call(renamed_Operator, functionargs, functionargs->length());
    }
    else if (!((boolean)(function))) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't translate function call: Undefined function or method " << "`" << functionname << "'" << endl;
      otree = NIL;
    }
    else {
      { 
        BIND_STELLA_SPECIAL(oVARARGSTATEMENTSo, Cons*, NIL);
        { Cons* actuals = (((Boolean_Wrapper*)(dynamic_slot_value(function->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_METHOD_VARIABLE_ARGUMENTSp, FALSE_WRAPPER)))->wrapper_value ? java_translate_variable_length_actuals(functionargs, function) : java_translate_actual_parameters(functionargs));

          if (!((function->method_return_type_specifiers_reader()->rest() == NIL) ||
              ((actuals->length() - 1) > function->method_parameter_names_reader()->length()))) {
            actuals = actuals->concatenate(cons(listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_MAKE_ARRAY, java_translate_type_spec(SGT_JAVA_TRANSLATE_STELLA_NATIVE_OBJECT_POINTER), cons(wrap_integer(function->method_return_type_specifiers_reader()->length() - 1), NIL)), NIL), 0);
          }
          otree = listO(5, SYM_JAVA_TRANSLATE_STELLA_JAVA_FUNCTION_CALL, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, wrap_string(java_yield_class_name_for_function(function)), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, java_translate_function_name(function), NIL), actuals, NIL);
          if (!(oVARARGSTATEMENTSo.get() == NIL)) {
            otree = java_wrap_method_body_with_vararg_value_setup(otree);
          }
        }
      }
    }
    return (otree);
  }
}

Cons* java_translate_defined_or_null(Surrogate* classtype, Object* object, boolean nullP) {
  { char* renamed_Operator = (nullP ? (char*)"==" : (char*)"!=");

    if (subtype_ofP(classtype, SGT_JAVA_TRANSLATE_STELLA_OBJECT)) {
      return (java_translate_operator_call(cons(wrap_string(renamed_Operator), NIL), cons(object, cons(SYM_JAVA_TRANSLATE_STELLA_NULL, NIL)), 2));
    }
    else if (subtype_ofP(classtype, SGT_JAVA_TRANSLATE_STELLA_INTEGER)) {
      return (java_translate_operator_call(cons(wrap_string(renamed_Operator), NIL), cons(object, cons(SYM_JAVA_TRANSLATE_STELLA_NULL_INTEGER, NIL)), 2));
    }
    else if (subtype_ofP(classtype, SGT_JAVA_TRANSLATE_STELLA_FLOAT)) {
      return (java_translate_operator_call(cons(wrap_string(renamed_Operator), NIL), cons(object, cons(SYM_JAVA_TRANSLATE_STELLA_NULL_FLOAT, NIL)), 2));
    }
    else if (subtype_ofP(classtype, SGT_JAVA_TRANSLATE_STELLA_SINGLE_FLOAT)) {
      return (java_translate_operator_call(cons(wrap_string(renamed_Operator), NIL), cons(object, cons(SYM_JAVA_TRANSLATE_STELLA_NULL_SINGLE_FLOAT, NIL)), 2));
    }
    else if (subtype_ofP(classtype, SGT_JAVA_TRANSLATE_STELLA_CHARACTER)) {
      return (java_translate_operator_call(cons(wrap_string(renamed_Operator), NIL), cons(object, cons(SYM_JAVA_TRANSLATE_STELLA_NULL_CHARACTER, NIL)), 2));
    }
    else {
      return (java_translate_operator_call(cons(wrap_string(renamed_Operator), NIL), cons(object, cons(SYM_JAVA_TRANSLATE_STELLA_NULL, NIL)), 2));
    }
  }
}

Cons* java_translate_method_call(Cons* tree) {
  { Symbol* methodname = ((Symbol*)(tree->rest->rest->value));
    Surrogate* classtype = ((Surrogate*)(tree->rest->value));
    Slot* slot = lookup_slot(((Class*)(classtype->surrogate_value)), methodname);
    Method_Slot* methodslot = ((((boolean)(slot)) &&
        isaP(slot, SGT_JAVA_TRANSLATE_STELLA_METHOD_SLOT)) ? ((Method_Slot*)(slot)) : NULL);
    Cons* allargs = tree->rest->rest->rest;
    Object* object = tree->fourth();
    Cons* renamed_Operator = java_lookup_operator_table(methodname->soft_permanentify());
    Cons* otree = NULL;

    if (methodname == SYM_JAVA_TRANSLATE_STELLA_DEFINEDp) {
      return (java_translate_defined_or_null(classtype, object, FALSE));
    }
    else if (methodname == SYM_JAVA_TRANSLATE_STELLA_NULLp) {
      return (java_translate_defined_or_null(classtype, object, TRUE));
    }
    else if (methodname == SYM_JAVA_TRANSLATE_STELLA_GET_SYM) {
      if (isaP(object, SGT_JAVA_TRANSLATE_STELLA_INTEGER_WRAPPER)) {
        return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_SYMBOL, get_sym(((Integer_Wrapper*)(object))->wrapper_value), NIL));
      }
    }
    else if (methodname == SYM_JAVA_TRANSLATE_STELLA_GET_KWD) {
      if (isaP(object, SGT_JAVA_TRANSLATE_STELLA_INTEGER_WRAPPER)) {
        return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_SYMBOL, get_kwd(((Integer_Wrapper*)(object))->wrapper_value), NIL));
      }
    }
    else if (methodname == SYM_JAVA_TRANSLATE_STELLA_GET_SGT) {
      if (isaP(object, SGT_JAVA_TRANSLATE_STELLA_INTEGER_WRAPPER)) {
        return (listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_SYMBOL, get_sgt(((Integer_Wrapper*)(object))->wrapper_value), NIL));
      }
    }
    else if (methodname == SYM_JAVA_TRANSLATE_STELLA_CONCATENATE) {
      if (classtype == SGT_JAVA_TRANSLATE_STELLA_STRING) {
        return (java_translate_operator_call(get_quoted_tree("((\"+\") \"/STELLA\")", "/STELLA"), allargs, 2));
      }
    }
    else if ((methodname == SYM_JAVA_TRANSLATE_STELLA_AREF) ||
        (methodname == SYM_JAVA_TRANSLATE_STELLA_AREF_SETTER)) {
      return (java_translate_aref_method_call(methodname, classtype, allargs));
    }
    else {
      if (((boolean)(methodslot)) &&
          java_method_object_is_overloaded_functionP(methodslot)) {
        return (java_translate_function_call(listO(3, SYM_JAVA_TRANSLATE_STELLA_SYS_CALL_FUNCTION, java_create_overloaded_function_name(methodname, classtype), tree->rest->rest->rest->concatenate(NIL, 0)), methodslot));
      }
    }
    if (((boolean)(renamed_Operator))) {
      otree = java_translate_operator_call(renamed_Operator, allargs, allargs->length());
    }
    else {
      otree = listO(5, SYM_JAVA_TRANSLATE_STELLA_JAVA_METHOD_CALL, listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, java_translate_class_name(intern_symbol_in_module(classtype->symbol_name, ((Module*)(classtype->home_context)), TRUE)), NIL), listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT, (((boolean)(methodslot)) ? methodslot->java_translate_method_name() : java_translate_name(methodname)), NIL), java_translate_a_tree(object), cons(((((boolean)(methodslot)) &&
          ((!(methodslot->method_return_type_specifiers_reader()->rest() == NIL)) &&
           (allargs->length() <= methodslot->method_parameter_names_reader()->length()))) ? java_translate_actual_parameters(allargs->rest)->concatenate(cons(listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_MAKE_ARRAY, java_translate_type_spec(SGT_JAVA_TRANSLATE_STELLA_NATIVE_OBJECT_POINTER), cons(wrap_integer(methodslot->method_return_type_specifiers_reader()->length() - 1), NIL)), NIL), 0) : java_translate_actual_parameters(allargs->rest)), NIL));
    }
    return (otree);
  }
}

Cons* java_translate_operator_tree(Cons* tree) {
  { Cons* renamed_Operator = java_lookup_operator_table(((Symbol*)(tree->value)));
    Cons* args = tree->rest;
    int arity = NULL_INTEGER;

    { Generalized_Symbol* test_value_000 = ((Generalized_Symbol*)(tree->value));

      if ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_AND) ||
          ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_OR) ||
           ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_EQp) ||
            ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_g) ||
             ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_ge) ||
              ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_l) ||
               (test_value_000 == SYM_JAVA_TRANSLATE_STELLA_le))))))) {
        arity = 2;
      }
      else if ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_NOT) ||
          ((test_value_000 == SYM_JAVA_TRANSLATE_STELLA_ii) ||
           (test_value_000 == SYM_JAVA_TRANSLATE_STELLA___))) {
        arity = 1;
      }
      else {
        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      }
    }
    return (java_translate_operator_call(renamed_Operator, args, arity));
  }
}

Cons* java_translate_operator_call(Cons* operatornames, Cons* arguments, int arity) {
  { Cons* otree = NULL;

    switch (arity) {
      case 1: 
        otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_UNARY_OP, operatornames->value, cons(java_translate_a_tree(arguments->value), NIL));
      break;
      case 2: 
        if (arguments->length() > 2) {
          otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_BINARY_OP, java_translate_a_tree(arguments->value), cons(operatornames->value, cons(java_translate_operator_call(operatornames, arguments->rest, 2), NIL)));
        }
        else {
          otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_BINARY_OP, java_translate_a_tree(arguments->value), cons(operatornames->value, cons(java_translate_a_tree(arguments->rest->value), NIL)));
        }
      break;
      case 3: 
        otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_TERNARY_OP, java_translate_a_tree(arguments->value), cons(operatornames->value, cons(java_translate_a_tree(arguments->rest->value), cons(operatornames->rest->value, cons(java_translate_a_tree(arguments->rest->rest->value), NIL)))));
      break;
      default:
        { Output_String_Stream* stream_000 = new_output_string_stream();

          *(stream_000->native_stream) << "`" << arity << "'" << " is not a valid case option";
          throw *new_stella_exception(stream_000->the_string_reader());
        }
      break;
    }
    return (otree);
  }
}

Cons* java_translate_aref_method_call(Symbol* renamed_Operator, Surrogate* owner, Cons* arguments) {
  if (owner == SGT_JAVA_TRANSLATE_STELLA_ARRAY) {
    { boolean setterP = renamed_Operator == SYM_JAVA_TRANSLATE_STELLA_AREF_SETTER;
      Cons* otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_AREF, java_translate_a_tree(arguments->value), java_translate_list_of_trees((setterP ? arguments->rest->rest : arguments->rest))->concatenate(NIL, 0));

      if (setterP) {
        otree = listO(3, SYM_JAVA_TRANSLATE_STELLA_JAVA_ASSIGN, otree, cons(java_translate_a_tree(arguments->rest->value), NIL));
      }
      return (otree);
    }
  }
  else {
    return (NIL);
  }
}

void help_startup_java_translate1() {
  {
    SYM_JAVA_TRANSLATE_STELLA_JAVA_FLOTSAM_CLASS = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA-FLOTSAM-CLASS", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_GLOBAL_VARIABLE = ((Symbol*)(intern_rigid_symbol_wrt_module("GLOBAL-VARIABLE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_CLASS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("TYPE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_METHOD = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_MACRO = ((Symbol*)(intern_rigid_symbol_wrt_module("MACRO", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_PRINT_METHOD = ((Symbol*)(intern_rigid_symbol_wrt_module("PRINT-METHOD", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_STARTUP_TIME_PROGN = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-TIME-PROGN", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_VERBATIM = ((Symbol*)(intern_rigid_symbol_wrt_module("VERBATIM", NULL, 0)));
    KWD_JAVA_TRANSLATE_UNBOUND_SPECIAL_VARIABLE = ((Keyword*)(intern_rigid_symbol_wrt_module("UNBOUND-SPECIAL-VARIABLE", NULL, 2)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_SPECIAL = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_SPECIAL", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_TYPE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_GLOBAL = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_GLOBAL", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_STATEMENTS", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_BLOCK = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_BLOCK", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_APIp = ((Symbol*)(intern_rigid_symbol_wrt_module("API?", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_THIS = ((Symbol*)(intern_rigid_symbol_wrt_module("THIS", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_DOCUMENTATION = ((Symbol*)(intern_rigid_symbol_wrt_module("DOCUMENTATION", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_COMMENT = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_COMMENT", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_FUNCTION = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_FUNCTION", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_METHOD = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_METHOD", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SELF = ((Symbol*)(intern_rigid_symbol_wrt_module("SELF", NULL, 0)));
    SGT_JAVA_TRANSLATE_STELLA_CONS = ((Surrogate*)(intern_rigid_symbol_wrt_module("CONS", NULL, 1)));
    SYM_JAVA_TRANSLATE_STELLA_SPECIAL = ((Symbol*)(intern_rigid_symbol_wrt_module("SPECIAL", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_CATCH = ((Symbol*)(intern_rigid_symbol_wrt_module("CATCH", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_THROW = ((Symbol*)(intern_rigid_symbol_wrt_module("THROW", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SYS_UNWIND_PROTECT = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-UNWIND-PROTECT", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SYS_SIGNAL = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-SIGNAL", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SYS_HANDLER_CASE = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-HANDLER-CASE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SYS_HANDLE_EXCEPTION = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-HANDLE-EXCEPTION", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_VOID_SYS = ((Symbol*)(intern_rigid_symbol_wrt_module("VOID-SYS", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_TYPED_SYS = ((Symbol*)(intern_rigid_symbol_wrt_module("TYPED-SYS", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_BAD_SYS = ((Symbol*)(intern_rigid_symbol_wrt_module("BAD-SYS", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SETQ = ((Symbol*)(intern_rigid_symbol_wrt_module("SETQ", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SYS_SLOT_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-SLOT-VALUE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SYS_REFERENCED_SLOT_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-REFERENCED-SLOT-VALUE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SYS_SLOT_VALUE_SETTER = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-SLOT-VALUE-SETTER", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SYS_CALL_METHOD = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-CALL-METHOD", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SYS_CALL_FUNCTION = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-CALL-FUNCTION", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_PROGN = ((Symbol*)(intern_rigid_symbol_wrt_module("PROGN", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_WITH_PROCESS_LOCK = ((Symbol*)(intern_rigid_symbol_wrt_module("WITH-PROCESS-LOCK", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_LOOP = ((Symbol*)(intern_rigid_symbol_wrt_module("LOOP", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SYS_FOREACH = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-FOREACH", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_LET = ((Symbol*)(intern_rigid_symbol_wrt_module("LET", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_CAST = ((Symbol*)(intern_rigid_symbol_wrt_module("CAST", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SAFE_CAST = ((Symbol*)(intern_rigid_symbol_wrt_module("SAFE-CAST", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_RETURN = ((Symbol*)(intern_rigid_symbol_wrt_module("RETURN", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_BREAK = ((Symbol*)(intern_rigid_symbol_wrt_module("BREAK", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_CONTINUE = ((Symbol*)(intern_rigid_symbol_wrt_module("CONTINUE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_WHILE = ((Symbol*)(intern_rigid_symbol_wrt_module("WHILE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_IF = ((Symbol*)(intern_rigid_symbol_wrt_module("IF", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_CHOOSE = ((Symbol*)(intern_rigid_symbol_wrt_module("CHOOSE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_WHEN = ((Symbol*)(intern_rigid_symbol_wrt_module("WHEN", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_UNLESS = ((Symbol*)(intern_rigid_symbol_wrt_module("UNLESS", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_AND = ((Symbol*)(intern_rigid_symbol_wrt_module("AND", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_OR = ((Symbol*)(intern_rigid_symbol_wrt_module("OR", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_NOT = ((Symbol*)(intern_rigid_symbol_wrt_module("NOT", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_EQp = ((Symbol*)(intern_rigid_symbol_wrt_module("EQ?", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_g = ((Symbol*)(intern_rigid_symbol_wrt_module(">", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_ge = ((Symbol*)(intern_rigid_symbol_wrt_module(">=", NULL, 0)));
  }
}

void help_startup_java_translate2() {
  {
    SYM_JAVA_TRANSLATE_STELLA_l = ((Symbol*)(intern_rigid_symbol_wrt_module("<", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_le = ((Symbol*)(intern_rigid_symbol_wrt_module("<=", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_ii = ((Symbol*)(intern_rigid_symbol_wrt_module("++", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA___ = ((Symbol*)(intern_rigid_symbol_wrt_module("--", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_COND = ((Symbol*)(intern_rigid_symbol_wrt_module("COND", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_CASE = ((Symbol*)(intern_rigid_symbol_wrt_module("CASE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_MAKE = ((Symbol*)(intern_rigid_symbol_wrt_module("MAKE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SYS_NEW = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-NEW", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_MV_SETQ = ((Symbol*)(intern_rigid_symbol_wrt_module("MV-SETQ", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SYS_CALL_FUNCTION_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-CALL-FUNCTION-CODE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SYS_CALL_METHOD_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-CALL-METHOD-CODE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SYS_INLINE_CALL = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-INLINE-CALL", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_THE_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("THE-CODE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SYS_POINTER_TO_FUNCTION = ((Symbol*)(intern_rigid_symbol_wrt_module("SYS-POINTER-TO-FUNCTION", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_INLINE = ((Symbol*)(intern_rigid_symbol_wrt_module("INLINE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_IGNORE = ((Symbol*)(intern_rigid_symbol_wrt_module("IGNORE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_PRINT_STREAM = ((Symbol*)(intern_rigid_symbol_wrt_module("PRINT-STREAM", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_PRINT_NATIVE_STREAM = ((Symbol*)(intern_rigid_symbol_wrt_module("PRINT-NATIVE-STREAM", NULL, 0)));
    SGT_JAVA_TRANSLATE_STELLA_KEY_VALUE_LIST = ((Surrogate*)(intern_rigid_symbol_wrt_module("KEY-VALUE-LIST", NULL, 1)));
    SYM_JAVA_TRANSLATE_STELLA_i = ((Symbol*)(intern_rigid_symbol_wrt_module("+", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA__ = ((Symbol*)(intern_rigid_symbol_wrt_module("-", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_o = ((Symbol*)(intern_rigid_symbol_wrt_module("*", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_s = ((Symbol*)(intern_rigid_symbol_wrt_module("/", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_eg = ((Symbol*)(intern_rigid_symbol_wrt_module("=>", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_el = ((Symbol*)(intern_rigid_symbol_wrt_module("=<", NULL, 0)));
    KWD_JAVA_TRANSLATE_UPPERCASE = ((Keyword*)(intern_rigid_symbol_wrt_module("UPPERCASE", NULL, 2)));
    KWD_JAVA_TRANSLATE_LOWERCASE = ((Keyword*)(intern_rigid_symbol_wrt_module("LOWERCASE", NULL, 2)));
    KWD_JAVA_TRANSLATE_CAPITALIZED = ((Keyword*)(intern_rigid_symbol_wrt_module("CAPITALIZED", NULL, 2)));
    KWD_JAVA_TRANSLATE_UNESCAPED = ((Keyword*)(intern_rigid_symbol_wrt_module("UNESCAPED", NULL, 2)));
    KWD_JAVA_TRANSLATE_NO = ((Keyword*)(intern_rigid_symbol_wrt_module("NO", NULL, 2)));
    KWD_JAVA_TRANSLATE_PRESERVE = ((Keyword*)(intern_rigid_symbol_wrt_module("PRESERVE", NULL, 2)));
    KWD_JAVA_TRANSLATE_UPCASE = ((Keyword*)(intern_rigid_symbol_wrt_module("UPCASE", NULL, 2)));
    KWD_JAVA_TRANSLATE_CLEVER = ((Keyword*)(intern_rigid_symbol_wrt_module("CLEVER", NULL, 2)));
    KWD_JAVA_TRANSLATE_TITLECASEX = ((Keyword*)(intern_rigid_symbol_wrt_module("TITLECASEX", NULL, 2)));
    KWD_JAVA_TRANSLATE_TITLECASE = ((Keyword*)(intern_rigid_symbol_wrt_module("TITLECASE", NULL, 2)));
    SGT_JAVA_TRANSLATE_STELLA_LITERAL = ((Surrogate*)(intern_rigid_symbol_wrt_module("LITERAL", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_STRING = ((Surrogate*)(intern_rigid_symbol_wrt_module("STRING", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_MUTABLE_STRING = ((Surrogate*)(intern_rigid_symbol_wrt_module("MUTABLE-STRING", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_CODE = ((Surrogate*)(intern_rigid_symbol_wrt_module("CODE", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_BOOLEAN = ((Surrogate*)(intern_rigid_symbol_wrt_module("BOOLEAN", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_INTEGER = ((Surrogate*)(intern_rigid_symbol_wrt_module("INTEGER", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_SHORT_INTEGER = ((Surrogate*)(intern_rigid_symbol_wrt_module("SHORT-INTEGER", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_UNSIGNED_SHORT_INTEGER = ((Surrogate*)(intern_rigid_symbol_wrt_module("UNSIGNED-SHORT-INTEGER", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_BYTE = ((Surrogate*)(intern_rigid_symbol_wrt_module("BYTE", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_OCTET = ((Surrogate*)(intern_rigid_symbol_wrt_module("OCTET", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_THIRY_TWO_BIT_VECTOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("THIRY-TWO-BIT-VECTOR", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_FLOAT = ((Surrogate*)(intern_rigid_symbol_wrt_module("FLOAT", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_DOUBLE = ((Surrogate*)(intern_rigid_symbol_wrt_module("DOUBLE", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_CHARACTER = ((Surrogate*)(intern_rigid_symbol_wrt_module("CHARACTER", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_LONG_INTEGER = ((Surrogate*)(intern_rigid_symbol_wrt_module("LONG-INTEGER", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_SINGLE_FLOAT = ((Surrogate*)(intern_rigid_symbol_wrt_module("SINGLE-FLOAT", NULL, 1)));
    SYM_JAVA_TRANSLATE_STELLA_CLASS_JAVA_NATIVE_TYPE = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-JAVA-NATIVE-TYPE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_ANY_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("ANY-VALUE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_OBJECT = ((Symbol*)(intern_rigid_symbol_wrt_module("OBJECT", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_ABSTRACT = ((Symbol*)(intern_rigid_symbol_wrt_module("ABSTRACT", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_BOOLEAN = ((Symbol*)(intern_rigid_symbol_wrt_module("BOOLEAN", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_BYTE = ((Symbol*)(intern_rigid_symbol_wrt_module("BYTE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_CHAR = ((Symbol*)(intern_rigid_symbol_wrt_module("CHAR", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_CONST = ((Symbol*)(intern_rigid_symbol_wrt_module("CONST", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_DEFAULT = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFAULT", NULL, 0)));
  }
}

void help_startup_java_translate3() {
  {
    SYM_JAVA_TRANSLATE_STELLA_DO = ((Symbol*)(intern_rigid_symbol_wrt_module("DO", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_DOUBLE = ((Symbol*)(intern_rigid_symbol_wrt_module("DOUBLE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_ELSE = ((Symbol*)(intern_rigid_symbol_wrt_module("ELSE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_EXTENDS = ((Symbol*)(intern_rigid_symbol_wrt_module("EXTENDS", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_FINAL = ((Symbol*)(intern_rigid_symbol_wrt_module("FINAL", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_FINALLY = ((Symbol*)(intern_rigid_symbol_wrt_module("FINALLY", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_FLOAT = ((Symbol*)(intern_rigid_symbol_wrt_module("FLOAT", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_FOR = ((Symbol*)(intern_rigid_symbol_wrt_module("FOR", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_GOTO = ((Symbol*)(intern_rigid_symbol_wrt_module("GOTO", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_IMPLEMENTS = ((Symbol*)(intern_rigid_symbol_wrt_module("IMPLEMENTS", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_IMPORT = ((Symbol*)(intern_rigid_symbol_wrt_module("IMPORT", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_INSTANCEOF = ((Symbol*)(intern_rigid_symbol_wrt_module("INSTANCEOF", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_INT = ((Symbol*)(intern_rigid_symbol_wrt_module("INT", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_INTERFACE = ((Symbol*)(intern_rigid_symbol_wrt_module("INTERFACE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_LONG = ((Symbol*)(intern_rigid_symbol_wrt_module("LONG", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_NATIVE = ((Symbol*)(intern_rigid_symbol_wrt_module("NATIVE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_NEW = ((Symbol*)(intern_rigid_symbol_wrt_module("NEW", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_PACKAGE = ((Symbol*)(intern_rigid_symbol_wrt_module("PACKAGE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_PRIVATE = ((Symbol*)(intern_rigid_symbol_wrt_module("PRIVATE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_PROTECTED = ((Symbol*)(intern_rigid_symbol_wrt_module("PROTECTED", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_PUBLIC = ((Symbol*)(intern_rigid_symbol_wrt_module("PUBLIC", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SHORT = ((Symbol*)(intern_rigid_symbol_wrt_module("SHORT", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_STATIC = ((Symbol*)(intern_rigid_symbol_wrt_module("STATIC", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SUPER = ((Symbol*)(intern_rigid_symbol_wrt_module("SUPER", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SWITCH = ((Symbol*)(intern_rigid_symbol_wrt_module("SWITCH", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SYNCHRONIZED = ((Symbol*)(intern_rigid_symbol_wrt_module("SYNCHRONIZED", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_THROWS = ((Symbol*)(intern_rigid_symbol_wrt_module("THROWS", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_TRANSIENT = ((Symbol*)(intern_rigid_symbol_wrt_module("TRANSIENT", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_TRY = ((Symbol*)(intern_rigid_symbol_wrt_module("TRY", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_VOID = ((Symbol*)(intern_rigid_symbol_wrt_module("VOID", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_VOLATILE = ((Symbol*)(intern_rigid_symbol_wrt_module("VOLATILE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_METHOD_CALL = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_METHOD_CALL", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_IDENT", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_ACTUALS = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_ACTUALS", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_UNWIND_PROTECT = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_UNWIND_PROTECT", NULL, 0)));
    KWD_JAVA_TRANSLATE_JAVA = ((Keyword*)(intern_rigid_symbol_wrt_module("JAVA", NULL, 2)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_VERBATIM = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_VERBATIM", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_SIGNAL = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_SIGNAL", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_HANDLER_CASE = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_HANDLER_CASE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_CATCH = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_CATCH", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_FUNCTION_CALL = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_FUNCTION_CALL", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_ASSIGN = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_ASSIGN", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_SLOT_VALUE_SETTER = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_SLOT_VALUE_SETTER", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_SLOT_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_SLOT_VALUE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_PROGN = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_PROGN", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_WITH_PROCESS_LOCK = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_WITH_PROCESS_LOCK", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_NAMED_STATEMENT = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_NAMED_STATEMENT", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_LOOP = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_LOOP", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_ARG_ENUM = ((Symbol*)(intern_rigid_symbol_wrt_module("ARG_ENUM", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_WHILE = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_WHILE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_CAST = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_CAST", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_ELEMENTS = ((Symbol*)(intern_rigid_symbol_wrt_module("ELEMENTS", NULL, 0)));
    SGT_JAVA_TRANSLATE_STELLA_OBJECT = ((Surrogate*)(intern_rigid_symbol_wrt_module("OBJECT", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_LISP_CONS_ITERATOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("LISP-CONS-ITERATOR", NULL, 1)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_FOREACH = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_FOREACH", NULL, 0)));
    SGT_JAVA_TRANSLATE_STELLA_INTEGER_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("INTEGER-WRAPPER", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_FLOAT_WRAPPER = ((Surrogate*)(intern_rigid_symbol_wrt_module("FLOAT-WRAPPER", NULL, 1)));
    KWD_JAVA_TRANSLATE_WRAP_FUNCTION = ((Keyword*)(intern_rigid_symbol_wrt_module("WRAP-FUNCTION", NULL, 2)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_AREF = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_AREF", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_MV_RETURNARRAY = ((Symbol*)(intern_rigid_symbol_wrt_module("MV_returnarray", NULL, 0)));
  }
}

void help_startup_java_translate4() {
  {
    SYM_JAVA_TRANSLATE_STELLA_JAVA_RETURN = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_RETURN", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_BREAK = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_BREAK", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_CONTINUE = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_CONTINUE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_IF = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_IF", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_SYMBOL_ID = ((Symbol*)(intern_rigid_symbol_wrt_module("SYMBOL-ID", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_OTHERWISE = ((Symbol*)(intern_rigid_symbol_wrt_module("OTHERWISE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_CASE = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_CASE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_COND = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_COND", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_WHEN = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_WHEN", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_UNLESS = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_UNLESS", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_MAKE = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_MAKE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_MAKE_ARRAY = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_MAKE_ARRAY", NULL, 0)));
    SGT_JAVA_TRANSLATE_STELLA_NATIVE_OBJECT_POINTER = ((Surrogate*)(intern_rigid_symbol_wrt_module("NATIVE-OBJECT-POINTER", NULL, 1)));
    SYM_JAVA_TRANSLATE_STELLA_CALLER_MV_RETURNARRAY = ((Symbol*)(intern_rigid_symbol_wrt_module("caller_MV_returnarray", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_FUNCTION_SIGNATURE = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_FUNCTION_SIGNATURE", NULL, 0)));
    SGT_JAVA_TRANSLATE_STELLA_VOID = ((Surrogate*)(intern_rigid_symbol_wrt_module("VOID", NULL, 1)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_FUNCALL = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_FUNCALL", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_METHOD_CODE_CALL = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_METHOD_CODE_CALL", NULL, 0)));
    SGT_JAVA_TRANSLATE_STELLA_MODULE = ((Surrogate*)(intern_rigid_symbol_wrt_module("MODULE", NULL, 1)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_PACKAGE = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA-PACKAGE", NULL, 0)));
    SGT_JAVA_TRANSLATE_STELLA_ARGUMENT_LIST = ((Surrogate*)(intern_rigid_symbol_wrt_module("ARGUMENT-LIST", NULL, 1)));
    SYM_JAVA_TRANSLATE_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_METHOD_NATIVEp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-NATIVE?", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_METHOD_CONSTRUCTORp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-CONSTRUCTOR?", NULL, 0)));
    SGT_JAVA_TRANSLATE_STELLA_DOUBLE_FLOAT = ((Surrogate*)(intern_rigid_symbol_wrt_module("DOUBLE-FLOAT", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_UNSIGNED_LONG_INTEGER = ((Surrogate*)(intern_rigid_symbol_wrt_module("UNSIGNED-LONG-INTEGER", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_THIRTY_TWO_BIT_VECTOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("THIRTY-TWO-BIT-VECTOR", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_TICKTOCK = ((Surrogate*)(intern_rigid_symbol_wrt_module("TICKTOCK", NULL, 1)));
    SGT_JAVA_TRANSLATE_STELLA_NATIVE_DATE_TIME = ((Surrogate*)(intern_rigid_symbol_wrt_module("NATIVE-DATE-TIME", NULL, 1)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_ANONYMOUS_ARRAY = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_ANONYMOUS_ARRAY", NULL, 0)));
    KWD_JAVA_TRANSLATE_FUNCTION = ((Keyword*)(intern_rigid_symbol_wrt_module("FUNCTION", NULL, 2)));
    KWD_JAVA_TRANSLATE_METHOD = ((Keyword*)(intern_rigid_symbol_wrt_module("METHOD", NULL, 2)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_PRINT_STREAM = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_PRINT_STREAM", NULL, 0)));
    KWD_JAVA_TRANSLATE_WARN = ((Keyword*)(intern_rigid_symbol_wrt_module("WARN", NULL, 2)));
    KWD_JAVA_TRANSLATE_ERROR = ((Keyword*)(intern_rigid_symbol_wrt_module("ERROR", NULL, 2)));
    KWD_JAVA_TRANSLATE_CONTINUABLE_ERROR = ((Keyword*)(intern_rigid_symbol_wrt_module("CONTINUABLE-ERROR", NULL, 2)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_STANDARD_ERROR = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_STANDARD_ERROR", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_STANDARD_OUT = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_STANDARD_OUT", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_PRINT_NATIVE_STREAM = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_PRINT_NATIVE_STREAM", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_EOL = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_EOL", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_EOL = ((Symbol*)(intern_rigid_symbol_wrt_module("EOL", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_LITERAL = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_LITERAL", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_NULL = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_METHOD_VARIABLE_ARGUMENTSp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-VARIABLE-ARGUMENTS?", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_GET_SYM = ((Symbol*)(intern_rigid_symbol_wrt_module("GET-SYM", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_SYMBOL = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_SYMBOL", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_GET_KWD = ((Symbol*)(intern_rigid_symbol_wrt_module("GET-KWD", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_GET_SGT = ((Symbol*)(intern_rigid_symbol_wrt_module("GET-SGT", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_NULL_INTEGER = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL-INTEGER", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_NULL_FLOAT = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL-FLOAT", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_NULL_SINGLE_FLOAT = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL-SINGLE-FLOAT", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_NULL_CHARACTER = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL-CHARACTER", NULL, 0)));
    SGT_JAVA_TRANSLATE_STELLA_METHOD_SLOT = ((Surrogate*)(intern_rigid_symbol_wrt_module("METHOD-SLOT", NULL, 1)));
    SYM_JAVA_TRANSLATE_STELLA_DEFINEDp = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFINED?", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_NULLp = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL?", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_CONCATENATE = ((Symbol*)(intern_rigid_symbol_wrt_module("CONCATENATE", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_AREF = ((Symbol*)(intern_rigid_symbol_wrt_module("AREF", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_AREF_SETTER = ((Symbol*)(intern_rigid_symbol_wrt_module("AREF-SETTER", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_UNARY_OP = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_UNARY_OP", NULL, 0)));
    SYM_JAVA_TRANSLATE_STELLA_JAVA_BINARY_OP = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_BINARY_OP", NULL, 0)));
  }
}

void help_startup_java_translate5() {
  {
    define_function_object("JAVA-YIELD-FLOTSAM-CLASS-NAME", "(DEFUN (JAVA-YIELD-FLOTSAM-CLASS-NAME STRING) ((MOD MODULE)))", ((cpp_function_code)(&java_yield_flotsam_class_name)), NULL);
    define_function_object("JAVA-YIELD-FULL-NATIVE-CLASS-NAME", "(DEFUN (JAVA-YIELD-FULL-NATIVE-CLASS-NAME STRING) ())", ((cpp_function_code)(&java_yield_full_native_class_name)), NULL);
    define_function_object("JAVA-TRANSLATE-UNIT", "(DEFUN (JAVA-TRANSLATE-UNIT CONS) ((UNIT TRANSLATION-UNIT)))", ((cpp_function_code)(&java_translate_unit)), NULL);
    define_function_object("JAVA-TRANSLATE-DEFINE-GLOBAL-VARIABLE-UNIT", "(DEFUN (JAVA-TRANSLATE-DEFINE-GLOBAL-VARIABLE-UNIT CONS) ((UNIT TRANSLATION-UNIT)))", ((cpp_function_code)(&java_translate_define_global_variable_unit)), NULL);
    define_function_object("WRAP-METHOD-BODY-WITH-JAVA-AUXILIARY-DECLARATIONS", "(DEFUN (WRAP-METHOD-BODY-WITH-JAVA-AUXILIARY-DECLARATIONS CONS) ((METHODBODY CONS) (DECLARATIONS CONS)))", ((cpp_function_code)(&wrap_method_body_with_java_auxiliary_declarations)), NULL);
    define_function_object("JAVA-SYNCHRONIZED-METHOD?", "(DEFUN (JAVA-SYNCHRONIZED-METHOD? BOOLEAN) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&java_synchronized_methodP)), NULL);
    define_function_object("JAVA-TRANSLATE-DEFINE-METHOD-UNIT", "(DEFUN (JAVA-TRANSLATE-DEFINE-METHOD-UNIT CONS) ((UNIT TRANSLATION-UNIT)))", ((cpp_function_code)(&java_translate_define_method_unit)), NULL);
    define_function_object("JAVA-TRANSLATE-DEFPRINT-UNIT", "(DEFUN (JAVA-TRANSLATE-DEFPRINT-UNIT CONS) ((UNIT TRANSLATION-UNIT)))", ((cpp_function_code)(&java_translate_defprint_unit)), NULL);
    define_function_object("JAVA-TRANSLATE-LIST-OF-TREES", "(DEFUN (JAVA-TRANSLATE-LIST-OF-TREES CONS) ((TREES CONS)))", ((cpp_function_code)(&java_translate_list_of_trees)), NULL);
    define_function_object("JAVA-TRANSLATE-A-TREE", "(DEFUN (JAVA-TRANSLATE-A-TREE OBJECT) ((TREE OBJECT)))", ((cpp_function_code)(&java_translate_a_tree)), NULL);
    define_function_object("JAVA-LOOKUP-OPERATOR-TABLE", "(DEFUN (JAVA-LOOKUP-OPERATOR-TABLE CONS) ((OPERATOR SYMBOL)))", ((cpp_function_code)(&java_lookup_operator_table)), NULL);
    define_function_object("JAVA-CREATE-CHARACTER-SUBSTITUTION-TABLE", "(DEFUN (JAVA-CREATE-CHARACTER-SUBSTITUTION-TABLE STRING) ())", ((cpp_function_code)(&java_create_character_substitution_table)), NULL);
    define_function_object("JAVA-SUBSTITUTE-FORBIDDEN-CHARACTERS", "(DEFUN (JAVA-SUBSTITUTE-FORBIDDEN-CHARACTERS STRING) ((NAME STRING) (CASECONVENTION KEYWORD)))", ((cpp_function_code)(&java_substitute_forbidden_characters)), NULL);
    define_function_object("JAVA-CHANGE-CASE", "(DEFUN (JAVA-CHANGE-CASE STRING) ((SYMBOL GENERALIZED-SYMBOL) (CASECONVENTION KEYWORD)))", ((cpp_function_code)(&java_change_case)), NULL);
    define_function_object("JAVA-LITERAL?", "(DEFUN (JAVA-LITERAL? BOOLEAN) ((TYPESPEC TYPE-SPEC)))", ((cpp_function_code)(&java_literalP)), NULL);
    define_function_object("JAVA-NATIVE-LITERAL-WRAPPER-NAMES", "(DEFUN (JAVA-NATIVE-LITERAL-WRAPPER-NAMES STRING-WRAPPER STRING-WRAPPER) ((TYPESPEC TYPE-SPEC)))", ((cpp_function_code)(&java_native_literal_wrapper_names)), NULL);
    define_function_object("JAVA-SPECIAL-SETTER-NAME", "(DEFUN (JAVA-SPECIAL-SETTER-NAME STRING-WRAPPER) ((TYPESPEC TYPE-SPEC)))", ((cpp_function_code)(&java_special_setter_name)), NULL);
    define_function_object("JAVA-TRANSLATE-SYMBOL-CONSTANT-NAME", "(DEFUN (JAVA-TRANSLATE-SYMBOL-CONSTANT-NAME STRING-WRAPPER) ((SYMBOL GENERALIZED-SYMBOL)))", ((cpp_function_code)(&java_translate_symbol_constant_name)), NULL);
    define_function_object("JAVA-TRANSLATE-SYMBOL-CONSTANT-ID", "(DEFUN (JAVA-TRANSLATE-SYMBOL-CONSTANT-ID STRING-WRAPPER) ((SYMBOL GENERALIZED-SYMBOL)))", ((cpp_function_code)(&java_translate_symbol_constant_id)), NULL);
    define_function_object("JAVA-TRANSLATE-SYMBOL-NAME", "(DEFUN (JAVA-TRANSLATE-SYMBOL-NAME STRING-WRAPPER) ((SYMBOL SYMBOL)))", ((cpp_function_code)(&java_translate_symbol_name)), NULL);
    define_function_object("JAVA-TRANSLATE-SYMBOL-ID", "(DEFUN (JAVA-TRANSLATE-SYMBOL-ID STRING-WRAPPER) ((SYMBOL SYMBOL)))", ((cpp_function_code)(&java_translate_symbol_id)), NULL);
    define_function_object("JAVA-TRANSLATE-SURROGATE-NAME", "(DEFUN (JAVA-TRANSLATE-SURROGATE-NAME STRING-WRAPPER) ((SURROGATE SURROGATE)))", ((cpp_function_code)(&java_translate_surrogate_name)), NULL);
    define_function_object("JAVA-TRANSLATE-SURROGATE-ID", "(DEFUN (JAVA-TRANSLATE-SURROGATE-ID STRING-WRAPPER) ((SURROGATE SURROGATE)))", ((cpp_function_code)(&java_translate_surrogate_id)), NULL);
    define_function_object("JAVA-TRANSLATE-KEYWORD-NAME", "(DEFUN (JAVA-TRANSLATE-KEYWORD-NAME STRING-WRAPPER) ((KEYWORD KEYWORD)))", ((cpp_function_code)(&java_translate_keyword_name)), NULL);
    define_function_object("JAVA-TRANSLATE-KEYWORD-ID", "(DEFUN (JAVA-TRANSLATE-KEYWORD-ID STRING-WRAPPER) ((KEYWORD KEYWORD)))", ((cpp_function_code)(&java_translate_keyword_id)), NULL);
    define_function_object("JAVA-TRANSLATE-CLASS-NAMESTRING", "(DEFUN (JAVA-TRANSLATE-CLASS-NAMESTRING STRING-WRAPPER) ((NAMESTRING STRING-WRAPPER)))", ((cpp_function_code)(&java_translate_class_namestring)), NULL);
    define_function_object("JAVA-TRANSLATE-CLASS-NAME", "(DEFUN (JAVA-TRANSLATE-CLASS-NAME STRING-WRAPPER) ((NAMESYMBOL GENERALIZED-SYMBOL)))", ((cpp_function_code)(&java_translate_class_name)), NULL);
    define_function_object("IN-CURRENT-JAVA-PACKAGE?", "(DEFUN (IN-CURRENT-JAVA-PACKAGE? BOOLEAN) ((MODULE MODULE)))", ((cpp_function_code)(&in_current_java_packageP)), NULL);
    define_function_object("JAVA-TRANSLATE-TYPE-SPEC", "(DEFUN (JAVA-TRANSLATE-TYPE-SPEC STRING-WRAPPER) ((TYPESPEC TYPE-SPEC)))", ((cpp_function_code)(&java_translate_type_spec)), NULL);
    define_function_object("JAVA-TRANSLATE-ARRAY-TYPE", "(DEFUN (JAVA-TRANSLATE-ARRAY-TYPE STRING-WRAPPER) ((ARRAYTYPE PARAMETRIC-TYPE-SPECIFIER)))", ((cpp_function_code)(&java_translate_array_type)), NULL);
    define_function_object("JAVA-TRANSLATE-ARRAY-OF-TYPE-SPEC", "(DEFUN (JAVA-TRANSLATE-ARRAY-OF-TYPE-SPEC STRING-WRAPPER) ((TYPESPEC TYPE-SPEC) (FULLYQUALIFIED? BOOLEAN)))", ((cpp_function_code)(&java_translate_array_of_type_spec)), NULL);
    define_function_object("JAVA-YIELD-RENAMED-RESERVED-WORD", "(DEFUN (JAVA-YIELD-RENAMED-RESERVED-WORD STRING-WRAPPER) ((WORD SYMBOL)))", ((cpp_function_code)(&java_yield_renamed_reserved_word)), NULL);
    define_function_object("JAVA-CREATE-RESERVED-WORD-TABLE", "(DEFUN (JAVA-CREATE-RESERVED-WORD-TABLE STRING-HASH-TABLE) ())", ((cpp_function_code)(&java_create_reserved_word_table)), NULL);
    define_function_object("JAVA-LOOKUP-RESERVED-WORD-TABLE", "(DEFUN (JAVA-LOOKUP-RESERVED-WORD-TABLE STRING) ((WORD STRING)))", ((cpp_function_code)(&java_lookup_reserved_word_table)), NULL);
    define_function_object("JAVA-TRANSLATE-NAMESTRING", "(DEFUN (JAVA-TRANSLATE-NAMESTRING STRING) ((NAMESTRING STRING)))", ((cpp_function_code)(&java_translate_namestring)), NULL);
    define_function_object("JAVA-TRANSLATE-NAME", "(DEFUN (JAVA-TRANSLATE-NAME STRING-WRAPPER) ((NAMESYMBOL SYMBOL)))", ((cpp_function_code)(&java_translate_name)), NULL);
    define_function_object("JAVA-TRANSLATE-SPECIAL-TREE", "(DEFUN (JAVA-TRANSLATE-SPECIAL-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_special_tree)), NULL);
    define_function_object("JAVA-TRANSLATE-VERBATIM-TREE", "(DEFUN (JAVA-TRANSLATE-VERBATIM-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_verbatim_tree)), NULL);
    define_function_object("JAVA-TRANSLATE-INLINE-CALL-TREE", "(DEFUN (JAVA-TRANSLATE-INLINE-CALL-TREE OBJECT) ((TREE CONS)))", ((cpp_function_code)(&java_translate_inline_call_tree)), NULL);
    define_function_object("JAVA-TRANSLATE-CATCH", "(DEFUN (JAVA-TRANSLATE-CATCH OBJECT) ((TREE CONS)))", ((cpp_function_code)(&java_translate_catch)), NULL);
    define_function_object("JAVA-TRANSLATE-THROW", "(DEFUN (JAVA-TRANSLATE-THROW OBJECT) ((TREE CONS)))", ((cpp_function_code)(&java_translate_throw)), NULL);
    define_function_object("JAVA-TRANSLATE-UNWIND-PROTECT", "(DEFUN (JAVA-TRANSLATE-UNWIND-PROTECT CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_unwind_protect)), NULL);
    define_function_object("JAVA-TRANSLATE-SIGNAL", "(DEFUN (JAVA-TRANSLATE-SIGNAL CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_signal)), NULL);
    define_function_object("JAVA-TRANSLATE-HANDLER-CASE", "(DEFUN (JAVA-TRANSLATE-HANDLER-CASE CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_handler_case)), NULL);
    define_function_object("JAVA-TRANSLATE-HANDLE-EXCEPTION", "(DEFUN (JAVA-TRANSLATE-HANDLE-EXCEPTION CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_handle_exception)), NULL);
    define_function_object("JAVA-TRANSLATE-VOID-SYS", "(DEFUN (JAVA-TRANSLATE-VOID-SYS CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_void_sys)), NULL);
    define_function_object("JAVA-TRANSLATE-TYPED-SYS", "(DEFUN (JAVA-TRANSLATE-TYPED-SYS CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_typed_sys)), NULL);
    define_function_object("JAVA-TRANSLATE-BAD-SYS", "(DEFUN (JAVA-TRANSLATE-BAD-SYS OBJECT) ((TREE CONS)))", ((cpp_function_code)(&java_translate_bad_sys)), NULL);
    define_function_object("JAVA-TRANSLATE-SETQ-TREE", "(DEFUN (JAVA-TRANSLATE-SETQ-TREE OBJECT) ((TREE CONS)))", ((cpp_function_code)(&java_translate_setq_tree)), NULL);
    define_function_object("JAVA-TRANSLATE-SYS-REFERENCED-SLOT-VALUE", "(DEFUN (JAVA-TRANSLATE-SYS-REFERENCED-SLOT-VALUE OBJECT) ((TREE CONS)))", ((cpp_function_code)(&java_translate_sys_referenced_slot_value)), NULL);
    define_function_object("JAVA-TRANSLATE-SYS-SLOT-VALUE-SETTER", "(DEFUN (JAVA-TRANSLATE-SYS-SLOT-VALUE-SETTER CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_sys_slot_value_setter)), NULL);
    define_function_object("JAVA-TRANSLATE-SYS-SLOT-VALUE", "(DEFUN (JAVA-TRANSLATE-SYS-SLOT-VALUE CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_sys_slot_value)), NULL);
    define_function_object("JAVA-TRANSLATE-PROGN", "(DEFUN (JAVA-TRANSLATE-PROGN CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_progn)), NULL);
    define_function_object("JAVA-TRANSLATE-WITH-PROCESS-LOCK", "(DEFUN (JAVA-TRANSLATE-WITH-PROCESS-LOCK CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_with_process_lock)), NULL);
    define_function_object("JAVA-TRANSLATE-LOOP", "(DEFUN (JAVA-TRANSLATE-LOOP CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_loop)), NULL);
    define_function_object("JAVA-TRANSLATE-ARGUMENT-LIST-ITERATOR", "(DEFUN (JAVA-TRANSLATE-ARGUMENT-LIST-ITERATOR CONS) ((ITERATORBINDINGS (CONS OF CONS)) (CURSORBINDINGS (CONS OF CONS)) (BODY CONS)))", ((cpp_function_code)(&java_translate_argument_list_iterator)), NULL);
    define_function_object("JAVA-TRANSLATE-FOREACH-TREE", "(DEFUN (JAVA-TRANSLATE-FOREACH-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_foreach_tree)), NULL);
    define_function_object("JAVA-TRANSLATE-LET-TREE", "(DEFUN (JAVA-TRANSLATE-LET-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_let_tree)), NULL);
    define_function_object("JAVA-TRANSLATE-STARTUP-TIME-PROGN", "(DEFUN (JAVA-TRANSLATE-STARTUP-TIME-PROGN OBJECT) ((TREE CONS)))", ((cpp_function_code)(&java_translate_startup_time_progn)), NULL);
    define_function_object("JAVA-TRANSLATE-CAST", "(DEFUN (JAVA-TRANSLATE-CAST CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_cast)), NULL);
  }
}

void help_startup_java_translate6() {
  {
    define_function_object("JAVA-TRANSLATE-RETURN-TREE", "(DEFUN (JAVA-TRANSLATE-RETURN-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_return_tree)), NULL);
    define_function_object("JAVA-TRANSLATE-BREAK", "(DEFUN (JAVA-TRANSLATE-BREAK CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_break)), NULL);
    define_function_object("JAVA-TRANSLATE-CONTINUE", "(DEFUN (JAVA-TRANSLATE-CONTINUE CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_continue)), NULL);
    define_function_object("JAVA-TRANSLATE-IF-TREE", "(DEFUN (JAVA-TRANSLATE-IF-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_if_tree)), NULL);
    define_function_object("JAVA-SYMBOL-CASE?", "(DEFUN (JAVA-SYMBOL-CASE? BOOLEAN) ((KEYFORM OBJECT)))", ((cpp_function_code)(&java_symbol_caseP)), NULL);
    define_function_object("JAVA-YIELD-SYMBOL-ID-FORM", "(DEFUN (JAVA-YIELD-SYMBOL-ID-FORM CONS) ((SYMBOLID INTEGER)))", ((cpp_function_code)(&java_yield_symbol_id_form)), NULL);
    define_function_object("JAVA-TRANSLATE-CONDITION", "(DEFUN (JAVA-TRANSLATE-CONDITION CONS) ((CONDITION CONS) (SYMBOLCASEP BOOLEAN)))", ((cpp_function_code)(&java_translate_condition)), NULL);
    define_function_object("JAVA-TRANSLATE-CASE-TREE", "(DEFUN (JAVA-TRANSLATE-CASE-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_case_tree)), NULL);
    define_function_object("JAVA-TRANSLATE-COND-TREE", "(DEFUN (JAVA-TRANSLATE-COND-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_cond_tree)), NULL);
    define_function_object("JAVA-TRANSLATE-WHILE", "(DEFUN (JAVA-TRANSLATE-WHILE CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_while)), NULL);
    define_function_object("JAVA-TRANSLATE-CHOOSE-TREE", "(DEFUN (JAVA-TRANSLATE-CHOOSE-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_choose_tree)), NULL);
    define_function_object("JAVA-TRANSLATE-WHEN-TREE", "(DEFUN (JAVA-TRANSLATE-WHEN-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_when_tree)), NULL);
    define_function_object("JAVA-TRANSLATE-UNLESS-TREE", "(DEFUN (JAVA-TRANSLATE-UNLESS-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_unless_tree)), NULL);
    define_function_object("JAVA-TRANSLATE-MAKE-TREE", "(DEFUN (JAVA-TRANSLATE-MAKE-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_make_tree)), NULL);
    define_function_object("JAVA-TRANSLATE-SYS-NEW", "(DEFUN (JAVA-TRANSLATE-SYS-NEW CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_sys_new)), NULL);
    define_function_object("JAVA-TRANSLATE-NEW-ARRAY", "(DEFUN (JAVA-TRANSLATE-NEW-ARRAY CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_new_array)), NULL);
    define_function_object("JAVA-TRANSLATE-MV-SETQ", "(DEFUN (JAVA-TRANSLATE-MV-SETQ CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_mv_setq)), NULL);
    define_function_object("JAVA-TRANSLATE-MV-FUNCTION-CALL", "(DEFUN (JAVA-TRANSLATE-MV-FUNCTION-CALL CONS) ((FUNCTIONCALL CONS) (MVVECTORNAME SYMBOL)))", ((cpp_function_code)(&java_translate_mv_function_call)), NULL);
    define_function_object("JAVA-TRANSLATE-CODE-SIGNATURE", "(DEFUN (JAVA-TRANSLATE-CODE-SIGNATURE CONS) ((SIGNATURE CONS)))", ((cpp_function_code)(&java_translate_code_signature)), NULL);
    define_function_object("JAVA-TRANSLATE-WITH-NATIVE-WRAPPER", "(DEFUN (JAVA-TRANSLATE-WITH-NATIVE-WRAPPER CONS) ((TREE OBJECT) (TYPE TYPE-SPEC)))", ((cpp_function_code)(&java_translate_with_native_wrapper)), NULL);
    define_function_object("JAVA-TRANSLATE-ACTUAL-PARAMETERS-WITH-NATIVE-WRAPPERS", "(DEFUN (JAVA-TRANSLATE-ACTUAL-PARAMETERS-WITH-NATIVE-WRAPPERS CONS) ((TREES CONS) (PARAMETERTYPES (CONS OF TYPE-SPEC))))", ((cpp_function_code)(&java_translate_actual_parameters_with_native_wrappers)), NULL);
    define_function_object("JAVA-WRAP-WITH-NATIVE-VALUE-EXTRACTION", "(DEFUN (JAVA-WRAP-WITH-NATIVE-VALUE-EXTRACTION CONS) ((TREE CONS) (PRIMARYRETURNTYPE TYPE-SPEC)))", ((cpp_function_code)(&java_wrap_with_native_value_extraction)), NULL);
    define_function_object("JAVA-TRANSLATE-OBJECT-TO-RETURN-TYPE", "(DEFUN (JAVA-TRANSLATE-OBJECT-TO-RETURN-TYPE CONS) ((TREE CONS) (RETURNTYPE TYPE-SPEC)))", ((cpp_function_code)(&java_translate_object_to_return_type)), NULL);
    define_function_object("JAVA-TRANSLATE-CALL-FUNCTION-CODE", "(DEFUN (JAVA-TRANSLATE-CALL-FUNCTION-CODE CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_call_function_code)), NULL);
    define_function_object("JAVA-TRANSLATE-CALL-METHOD-CODE", "(DEFUN (JAVA-TRANSLATE-CALL-METHOD-CODE CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_call_method_code)), NULL);
    define_function_object("JAVA-PACKAGE-PREFIX", "(DEFUN (JAVA-PACKAGE-PREFIX STRING) ((MODULE MODULE) (SEPARATOR STRING)))", ((cpp_function_code)(&java_package_prefix)), NULL);
    define_function_object("JAVA-YIELD-FULLY-QUALIFIED-TYPE-NAME", "(DEFUN (JAVA-YIELD-FULLY-QUALIFIED-TYPE-NAME STRING-WRAPPER) ((TYPESPEC TYPE-SPEC)))", ((cpp_function_code)(&java_yield_fully_qualified_type_name)), NULL);
    define_function_object("JAVA-YIELD-TRANSLATED-CLASS-AND-FUNCTION-NAMES", "(DEFUN (JAVA-YIELD-TRANSLATED-CLASS-AND-FUNCTION-NAMES STRING-WRAPPER STRING-WRAPPER) ((F FUNCTION)))", ((cpp_function_code)(&java_yield_translated_class_and_function_names)), NULL);
    define_function_object("JAVA-YIELD-TRANSLATED-CLASS-AND-METHOD-NAMES", "(DEFUN (JAVA-YIELD-TRANSLATED-CLASS-AND-METHOD-NAMES STRING STRING) ((OWNER TYPE-SPEC) (F FUNCTION)))", ((cpp_function_code)(&java_yield_translated_class_and_method_names)), NULL);
    define_function_object("JAVA-TRANSLATE-CLASS-OBJECT-LOOKUP", "(DEFUN (JAVA-TRANSLATE-CLASS-OBJECT-LOOKUP CONS) ((CLASSNAME STRING)))", ((cpp_function_code)(&java_translate_class_object_lookup)), NULL);
    define_function_object("JAVA-YIELD-CLASS-OBJECT-ARRAY-EXPRESSION", "(DEFUN (JAVA-YIELD-CLASS-OBJECT-ARRAY-EXPRESSION CONS) ((TYPESPEC TYPE-SPEC) (NDIMENSIONS INTEGER)))", ((cpp_function_code)(&java_yield_class_object_array_expression)), NULL);
    define_function_object("JAVA-YIELD-CLASS-OBJECT-EXPRESSION", "(DEFUN (JAVA-YIELD-CLASS-OBJECT-EXPRESSION CONS) ((TYPESPEC TYPE-SPEC)))", ((cpp_function_code)(&java_yield_class_object_expression)), NULL);
    define_function_object("YIELD-CLASS-OBJECT-LOOKUP-NAME-FOR-NATIVE-TYPE", "(DEFUN (YIELD-CLASS-OBJECT-LOOKUP-NAME-FOR-NATIVE-TYPE STRING) ((NATIVETYPE STRING)))", ((cpp_function_code)(&yield_class_object_lookup_name_for_native_type)), NULL);
    define_function_object("JAVA-TRANSLATE-PARAM-TYPE-SPECIFICATIONS", "(DEFUN (JAVA-TRANSLATE-PARAM-TYPE-SPECIFICATIONS CONS) ((FN METHOD-SLOT) (OWNER TYPE-SPEC)))", ((cpp_function_code)(&java_translate_param_type_specifications)), NULL);
    define_function_object("JAVA-TRANSLATE-THE-CODE", "(DEFUN (JAVA-TRANSLATE-THE-CODE OBJECT) ((TREE CONS)))", ((cpp_function_code)(&java_translate_the_code)), NULL);
    define_function_object("JAVA-TRANSLATE-FUNCTION-NAME", "(DEFUN (JAVA-TRANSLATE-FUNCTION-NAME STRING-WRAPPER) ((FUNCTION METHOD-SLOT)))", ((cpp_function_code)(&java_translate_function_name)), NULL);
    define_function_object("JAVA-TRANSLATE-SYS-POINTER-TO-FUNCTION", "(DEFUN (JAVA-TRANSLATE-SYS-POINTER-TO-FUNCTION OBJECT) ((TREE CONS)))", ((cpp_function_code)(&java_translate_sys_pointer_to_function)), NULL);
    define_function_object("JAVA-TRANSLATE-IGNORE-TREE", "(DEFUN (JAVA-TRANSLATE-IGNORE-TREE OBJECT) ((TREE CONS)))", ((cpp_function_code)(&java_translate_ignore_tree)), NULL);
    define_function_object("JAVA-STREAM-IS-STANDARD-OUTPUT?", "(DEFUN (JAVA-STREAM-IS-STANDARD-OUTPUT? BOOLEAN) ((TREE OBJECT)))", ((cpp_function_code)(&java_stream_is_standard_outputP)), NULL);
    define_function_object("JAVA-TRANSLATE-PRINT-STREAM", "(DEFUN (JAVA-TRANSLATE-PRINT-STREAM CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_print_stream)), NULL);
    define_function_object("JAVA-TRANSLATE-PRINT-NATIVE-STREAM", "(DEFUN (JAVA-TRANSLATE-PRINT-NATIVE-STREAM CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_print_native_stream)), NULL);
    define_function_object("JAVA-TRANSLATE-NULL", "(DEFUN (JAVA-TRANSLATE-NULL CONS) ())", ((cpp_function_code)(&java_translate_null)), NULL);
    define_function_object("JAVA-TRANSLATE-EOL", "(DEFUN (JAVA-TRANSLATE-EOL CONS) ())", ((cpp_function_code)(&java_translate_eol)), NULL);
    define_method_object("(DEFMETHOD (JAVA-TRANSLATE-ATOMIC-TREE CONS) ((TREE OBJECT)))", ((cpp_method_code)(&Object::java_translate_atomic_tree)), ((cpp_method_code)(NULL)));
    define_function_object("JAVA-PREPEND-GLOBAL-CLASS-NAME", "(DEFUN (JAVA-PREPEND-GLOBAL-CLASS-NAME STRING) ((NAMESTRING STRING) (MODULE MODULE)))", ((cpp_function_code)(&java_prepend_global_class_name)), NULL);
    define_function_object("JAVA-TRANSLATE-GLOBAL-NAME", "(DEFUN (JAVA-TRANSLATE-GLOBAL-NAME STRING-WRAPPER) ((NAMESYMBOL SYMBOL) (DEFINITION? BOOLEAN)))", ((cpp_function_code)(&java_translate_global_name)), NULL);
    define_function_object("JAVA-TRANSLATE-GENERAL-SYMBOL", "(DEFUN (JAVA-TRANSLATE-GENERAL-SYMBOL CONS) ((TREE SYMBOL)))", ((cpp_function_code)(&java_translate_general_symbol)), NULL);
    define_method_object("(DEFMETHOD (JAVA-TRANSLATE-ATOMIC-TREE CONS) ((TREE SYMBOL)))", ((cpp_method_code)(&Symbol::java_translate_atomic_tree)), ((cpp_method_code)(NULL)));
    define_function_object("JAVA-TRANSLATE-METHOD-PARAMETER", "(DEFUN (JAVA-TRANSLATE-METHOD-PARAMETER OBJECT) ((NAME SYMBOL) (TYPE TYPE-SPEC) (METHOD METHOD-SLOT)))", ((cpp_function_code)(&java_translate_method_parameter)), NULL);
    define_function_object("JAVA-TRANSLATE-RETURN-PARAMETER", "(DEFUN (JAVA-TRANSLATE-RETURN-PARAMETER CONS) ())", ((cpp_function_code)(&java_translate_return_parameter)), NULL);
    define_function_object("JAVA-TRANSLATE-METHOD-PARAMETERS", "(DEFUN (JAVA-TRANSLATE-METHOD-PARAMETERS CONS) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&java_translate_method_parameters)), NULL);
    define_function_object("ZERO-ARGUMENT-FUNCTION?", "(DEFUN (ZERO-ARGUMENT-FUNCTION? BOOLEAN) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&zero_argument_functionP)), NULL);
    define_function_object("JAVA-METHOD-OBJECT-DEFINED-ON-NATIVE-TYPE?", "(DEFUN (JAVA-METHOD-OBJECT-DEFINED-ON-NATIVE-TYPE? BOOLEAN) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&java_method_object_defined_on_native_typeP)), NULL);
    define_function_object("JAVA-METHOD-OBJECT-IS-OVERLOADED-FUNCTION?", "(DEFUN (JAVA-METHOD-OBJECT-IS-OVERLOADED-FUNCTION? BOOLEAN) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&java_method_object_is_overloaded_functionP)), NULL);
    define_function_object("JAVA-METHOD-OBJECT-IS-FUNCTION?", "(DEFUN (JAVA-METHOD-OBJECT-IS-FUNCTION? BOOLEAN) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&java_method_object_is_functionP)), NULL);
    define_function_object("JAVA-FUNCTION-IN-DIFFERENT-MODULE?", "(DEFUN (JAVA-FUNCTION-IN-DIFFERENT-MODULE? BOOLEAN) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&java_function_in_different_moduleP)), NULL);
    define_function_object("JAVA-CREATE-OVERLOADED-FUNCTION-NAME", "(DEFUN (JAVA-CREATE-OVERLOADED-FUNCTION-NAME SYMBOL) ((FUNCTIONNAME SYMBOL) (CLASSTYPE TYPE)))", ((cpp_function_code)(&java_create_overloaded_function_name)), NULL);
    define_method_object("(DEFMETHOD (JAVA-TRANSLATE-METHOD-NAME STRING-WRAPPER) ((METHOD METHOD-SLOT)))", ((cpp_method_code)(&Method_Slot::java_translate_method_name)), ((cpp_method_code)(NULL)));
    define_function_object("JAVA-DELETE-QUOTED-NULL-STATEMENTS", "(DEFUN (JAVA-DELETE-QUOTED-NULL-STATEMENTS CONS) ((TREES CONS)))", ((cpp_function_code)(&java_delete_quoted_null_statements)), NULL);
    define_function_object("JAVA-WRAP-METHOD-BODY-WITH-VARARG-DECLARATIONS", "(DEFUN (JAVA-WRAP-METHOD-BODY-WITH-VARARG-DECLARATIONS CONS) ((METHODBODY CONS)))", ((cpp_function_code)(&java_wrap_method_body_with_vararg_declarations)), NULL);
  }
}

void startup_java_translate() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      help_startup_java_translate1();
      help_startup_java_translate2();
      help_startup_java_translate3();
      help_startup_java_translate4();
      SYM_JAVA_TRANSLATE_STELLA_JAVA_TERNARY_OP = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_TERNARY_OP", NULL, 0)));
      SGT_JAVA_TRANSLATE_STELLA_ARRAY = ((Surrogate*)(intern_rigid_symbol_wrt_module("ARRAY", NULL, 1)));
      SYM_JAVA_TRANSLATE_STELLA_STARTUP_JAVA_TRANSLATE = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-JAVA-TRANSLATE", NULL, 0)));
    }
    if (current_startup_time_phaseP(4)) {
      oJAVA_OPERATOR_TABLEo = ((Key_Value_List*)(dictionary(SGT_JAVA_TRANSLATE_STELLA_KEY_VALUE_LIST, 34, SYM_JAVA_TRANSLATE_STELLA_ii, cons(wrap_string("++"), NIL), SYM_JAVA_TRANSLATE_STELLA___, cons(wrap_string("--"), NIL), SYM_JAVA_TRANSLATE_STELLA_i, cons(wrap_string("+"), NIL), SYM_JAVA_TRANSLATE_STELLA__, cons(wrap_string("-"), NIL), SYM_JAVA_TRANSLATE_STELLA_o, cons(wrap_string("*"), NIL), SYM_JAVA_TRANSLATE_STELLA_s, cons(wrap_string("/"), NIL), SYM_JAVA_TRANSLATE_STELLA_EQp, cons(wrap_string("=="), NIL), SYM_JAVA_TRANSLATE_STELLA_g, cons(wrap_string(">"), NIL), SYM_JAVA_TRANSLATE_STELLA_ge, cons(wrap_string(">="), NIL), SYM_JAVA_TRANSLATE_STELLA_eg, cons(wrap_string(">="), NIL), SYM_JAVA_TRANSLATE_STELLA_l, cons(wrap_string("<"), NIL), SYM_JAVA_TRANSLATE_STELLA_el, cons(wrap_string("<="), NIL), SYM_JAVA_TRANSLATE_STELLA_le, cons(wrap_string("<="), NIL), SYM_JAVA_TRANSLATE_STELLA_AND, cons(wrap_string("&&"), NIL), SYM_JAVA_TRANSLATE_STELLA_OR, cons(wrap_string("||"), NIL), SYM_JAVA_TRANSLATE_STELLA_NOT, cons(wrap_string("!"), NIL), SYM_JAVA_TRANSLATE_STELLA_CHOOSE, cons(wrap_string("?"), cons(wrap_string(":"), NIL)))));
      oJAVA_CHARACTER_SUBSTITUTION_TABLEo = java_create_character_substitution_table();
      oJAVA_RESERVED_WORD_TABLEo = java_create_reserved_word_table();
      oJAVA_PRIMITIVE_ARRAY_TYPE_NAMESo = ((Key_Value_List*)(dictionary(SGT_JAVA_TRANSLATE_STELLA_KEY_VALUE_LIST, 16, wrap_string("byte"), wrap_string("B"), wrap_string("char"), wrap_string("C"), wrap_string("double"), wrap_string("D"), wrap_string("float"), wrap_string("F"), wrap_string("int"), wrap_string("I"), wrap_string("long"), wrap_string("J"), wrap_string("short"), wrap_string("S"), wrap_string("boolean"), wrap_string("Z"))));
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      help_startup_java_translate5();
      help_startup_java_translate6();
      define_function_object("JAVA-WRAP-METHOD-BODY-WITH-VARARG-VALUE-SETUP", "(DEFUN (JAVA-WRAP-METHOD-BODY-WITH-VARARG-VALUE-SETUP CONS) ((METHODBODY CONS)))", ((cpp_function_code)(&java_wrap_method_body_with_vararg_value_setup)), NULL);
      define_function_object("JAVA-TRANSLATE-METHOD", "(DEFUN (JAVA-TRANSLATE-METHOD CONS) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&java_translate_method)), NULL);
      define_function_object("JAVA-TRANSLATE-VARIABLE-LENGTH-ARG-NAME", "(DEFUN (JAVA-TRANSLATE-VARIABLE-LENGTH-ARG-NAME STRING-WRAPPER) ((NAMESYMBOL SYMBOL) (PARAMETERNUMBER INTEGER)))", ((cpp_function_code)(&java_translate_variable_length_arg_name)), NULL);
      define_function_object("JAVA-TRANSLATE-VARIABLE-LENGTH-ACTUALS", "(DEFUN (JAVA-TRANSLATE-VARIABLE-LENGTH-ACTUALS CONS) ((ACTUALS CONS) (UNUSED-METHOD METHOD-SLOT)))", ((cpp_function_code)(&java_translate_variable_length_actuals)), NULL);
      define_function_object("JAVA-TRANSLATE-ACTUAL-PARAMETERS", "(DEFUN (JAVA-TRANSLATE-ACTUAL-PARAMETERS CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_actual_parameters)), NULL);
      define_function_object("JAVA-YIELD-CLASS-NAME-FOR-FUNCTION", "(DEFUN (JAVA-YIELD-CLASS-NAME-FOR-FUNCTION STRING) ((FUNCTION METHOD-SLOT)))", ((cpp_function_code)(&java_yield_class_name_for_function)), NULL);
      define_function_object("JAVA-TRANSLATE-FUNCTION-CALL", "(DEFUN (JAVA-TRANSLATE-FUNCTION-CALL CONS) ((TREE CONS) (METHOD METHOD-SLOT)))", ((cpp_function_code)(&java_translate_function_call)), NULL);
      define_function_object("JAVA-TRANSLATE-DEFINED-OR-NULL", "(DEFUN (JAVA-TRANSLATE-DEFINED-OR-NULL CONS) ((CLASSTYPE TYPE) (OBJECT OBJECT) (NULL? BOOLEAN)))", ((cpp_function_code)(&java_translate_defined_or_null)), NULL);
      define_function_object("JAVA-TRANSLATE-METHOD-CALL", "(DEFUN (JAVA-TRANSLATE-METHOD-CALL CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_method_call)), NULL);
      define_function_object("JAVA-TRANSLATE-OPERATOR-TREE", "(DEFUN (JAVA-TRANSLATE-OPERATOR-TREE CONS) ((TREE CONS)))", ((cpp_function_code)(&java_translate_operator_tree)), NULL);
      define_function_object("JAVA-TRANSLATE-OPERATOR-CALL", "(DEFUN (JAVA-TRANSLATE-OPERATOR-CALL CONS) ((OPERATORNAMES CONS) (ARGUMENTS CONS) (ARITY INTEGER)))", ((cpp_function_code)(&java_translate_operator_call)), NULL);
      define_function_object("JAVA-TRANSLATE-AREF-METHOD-CALL", "(DEFUN (JAVA-TRANSLATE-AREF-METHOD-CALL CONS) ((OPERATOR SYMBOL) (OWNER TYPE) (ARGUMENTS CONS)))", ((cpp_function_code)(&java_translate_aref_method_call)), NULL);
      define_function_object("STARTUP-JAVA-TRANSLATE", "(DEFUN STARTUP-JAVA-TRANSLATE () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_java_translate)), NULL);
      { Method_Slot* function = lookup_function(SYM_JAVA_TRANSLATE_STELLA_STARTUP_JAVA_TRANSLATE);

        set_dynamic_slot_value(function->dynamic_slots, SYM_JAVA_TRANSLATE_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Java-Translate"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *VARARGSTATEMENTS* (CONS OF CONS) NIL :DOCUMENTATION \"A list of new vectors generated for variable-length argument \n             lists\")");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *VARARGDECLS* (CONS OF CONS) NIL :DOCUMENTATION \"A list of vector-pushes generated for variable-length argument \n             lists\")");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *CURRENTVARARGINDEX* INTEGER 1 :DOCUMENTATION \"The current index of the variable length arguments\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *JAVA-OPERATOR-TABLE* KEY-VALUE-LIST (DICTIONARY @KEY-VALUE-LIST (QUOTE ++) (CONS (WRAP-LITERAL \"++\") NIL) (QUOTE --) (CONS (WRAP-LITERAL \"--\") NIL) (QUOTE +) (CONS (WRAP-LITERAL \"+\") NIL) (QUOTE -) (CONS (WRAP-LITERAL \"-\") NIL) (QUOTE *) (CONS (WRAP-LITERAL \"*\") NIL) (QUOTE /) (CONS (WRAP-LITERAL \"/\") NIL) (QUOTE EQ?) (CONS (WRAP-LITERAL \"==\") NIL) (QUOTE >) (CONS (WRAP-LITERAL \">\") NIL) (QUOTE >=) (CONS (WRAP-LITERAL \">=\") NIL) (QUOTE =>) (CONS (WRAP-LITERAL \">=\") NIL) (QUOTE <) (CONS (WRAP-LITERAL \"<\") NIL) (QUOTE =<) (CONS (WRAP-LITERAL \"<=\") NIL) (QUOTE <=) (CONS (WRAP-LITERAL \"<=\") NIL) (QUOTE AND) (CONS (WRAP-LITERAL \"&&\") NIL) (QUOTE OR) (CONS (WRAP-LITERAL \"||\") NIL) (QUOTE NOT) (CONS (WRAP-LITERAL \"!\") NIL) (QUOTE CHOOSE) (CONS (WRAP-LITERAL \"?\") (CONS (WRAP-LITERAL \":\") NIL))) :DOCUMENTATION \"Mapping from STELLA operators to Java operators\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *JAVA-CHARACTER-SUBSTITUTION-TABLE* STRING (JAVA-CREATE-CHARACTER-SUBSTITUTION-TABLE))");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *JAVA-RESERVED-WORD-TABLE* (STRING-HASH-TABLE OF STRING STRING-WRAPPER) (JAVA-CREATE-RESERVED-WORD-TABLE))");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *JAVA-LOOP-NAME* SYMBOL NULL)");
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *JAVA-LOOP-NAME-USED?* BOOLEAN FALSE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *JAVA-PRIMITIVE-ARRAY-TYPE-NAMES* KEY-VALUE-LIST (DICTIONARY @KEY-VALUE-LIST \"byte\" \"B\" \"char\" \"C\" \"double\" \"D\" \"float\" \"F\" \"int\" \"I\" \"long\" \"J\" \"short\" \"S\" \"boolean\" \"Z\"))");
    }
  }
}

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_FLOTSAM_CLASS = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_GLOBAL_VARIABLE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_CLASS = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_TYPE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_METHOD = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_MACRO = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_PRINT_METHOD = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_STARTUP_TIME_PROGN = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_VERBATIM = NULL;

Keyword* KWD_JAVA_TRANSLATE_UNBOUND_SPECIAL_VARIABLE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_SPECIAL = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_TYPE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_GLOBAL = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_STATEMENTS = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_BLOCK = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_APIp = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_THIS = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_DOCUMENTATION = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_COMMENT = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_FUNCTION = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_METHOD = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SELF = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_CONS = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SPECIAL = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_CATCH = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_THROW = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SYS_UNWIND_PROTECT = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SYS_SIGNAL = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SYS_HANDLER_CASE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SYS_HANDLE_EXCEPTION = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_VOID_SYS = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_TYPED_SYS = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_BAD_SYS = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SETQ = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SYS_SLOT_VALUE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SYS_REFERENCED_SLOT_VALUE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SYS_SLOT_VALUE_SETTER = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SYS_CALL_METHOD = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SYS_CALL_FUNCTION = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_PROGN = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_WITH_PROCESS_LOCK = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_LOOP = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SYS_FOREACH = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_LET = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_CAST = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SAFE_CAST = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_RETURN = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_BREAK = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_CONTINUE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_WHILE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_IF = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_CHOOSE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_WHEN = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_UNLESS = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_AND = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_OR = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_NOT = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_EQp = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_g = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_ge = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_l = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_le = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_ii = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA___ = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_COND = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_CASE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_MAKE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SYS_NEW = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_MV_SETQ = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SYS_CALL_FUNCTION_CODE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SYS_CALL_METHOD_CODE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SYS_INLINE_CALL = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_THE_CODE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SYS_POINTER_TO_FUNCTION = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_INLINE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_IGNORE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_PRINT_STREAM = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_PRINT_NATIVE_STREAM = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_KEY_VALUE_LIST = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_i = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA__ = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_o = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_s = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_eg = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_el = NULL;

Keyword* KWD_JAVA_TRANSLATE_UPPERCASE = NULL;

Keyword* KWD_JAVA_TRANSLATE_LOWERCASE = NULL;

Keyword* KWD_JAVA_TRANSLATE_CAPITALIZED = NULL;

Keyword* KWD_JAVA_TRANSLATE_UNESCAPED = NULL;

Keyword* KWD_JAVA_TRANSLATE_NO = NULL;

Keyword* KWD_JAVA_TRANSLATE_PRESERVE = NULL;

Keyword* KWD_JAVA_TRANSLATE_UPCASE = NULL;

Keyword* KWD_JAVA_TRANSLATE_CLEVER = NULL;

Keyword* KWD_JAVA_TRANSLATE_TITLECASEX = NULL;

Keyword* KWD_JAVA_TRANSLATE_TITLECASE = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_LITERAL = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_STRING = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_MUTABLE_STRING = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_CODE = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_BOOLEAN = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_INTEGER = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_SHORT_INTEGER = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_UNSIGNED_SHORT_INTEGER = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_BYTE = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_OCTET = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_THIRY_TWO_BIT_VECTOR = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_FLOAT = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_DOUBLE = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_CHARACTER = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_LONG_INTEGER = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_SINGLE_FLOAT = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_CLASS_JAVA_NATIVE_TYPE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_ANY_VALUE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_OBJECT = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_ABSTRACT = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_BOOLEAN = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_BYTE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_CHAR = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_CONST = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_DEFAULT = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_DO = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_DOUBLE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_ELSE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_EXTENDS = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_FINAL = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_FINALLY = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_FLOAT = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_FOR = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_GOTO = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_IMPLEMENTS = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_IMPORT = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_INSTANCEOF = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_INT = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_INTERFACE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_LONG = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_NATIVE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_NEW = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_PACKAGE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_PRIVATE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_PROTECTED = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_PUBLIC = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SHORT = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_STATIC = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SUPER = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SWITCH = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SYNCHRONIZED = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_THROWS = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_TRANSIENT = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_TRY = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_VOID = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_VOLATILE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_METHOD_CALL = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_IDENT = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_ACTUALS = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_UNWIND_PROTECT = NULL;

Keyword* KWD_JAVA_TRANSLATE_JAVA = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_VERBATIM = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_SIGNAL = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_HANDLER_CASE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_CATCH = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_FUNCTION_CALL = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_ASSIGN = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_SLOT_VALUE_SETTER = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_SLOT_VALUE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_PROGN = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_WITH_PROCESS_LOCK = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_NAMED_STATEMENT = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_LOOP = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_ARG_ENUM = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_WHILE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_CAST = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_ELEMENTS = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_OBJECT = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_LISP_CONS_ITERATOR = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_FOREACH = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_INTEGER_WRAPPER = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_FLOAT_WRAPPER = NULL;

Keyword* KWD_JAVA_TRANSLATE_WRAP_FUNCTION = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_AREF = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_MV_RETURNARRAY = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_RETURN = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_BREAK = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_CONTINUE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_IF = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_SYMBOL_ID = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_OTHERWISE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_CASE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_COND = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_WHEN = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_UNLESS = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_MAKE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_MAKE_ARRAY = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_NATIVE_OBJECT_POINTER = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_CALLER_MV_RETURNARRAY = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_FUNCTION_SIGNATURE = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_VOID = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_FUNCALL = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_METHOD_CODE_CALL = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_MODULE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_PACKAGE = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_ARGUMENT_LIST = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_METHOD_NATIVEp = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_METHOD_CONSTRUCTORp = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_DOUBLE_FLOAT = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_UNSIGNED_LONG_INTEGER = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_THIRTY_TWO_BIT_VECTOR = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_TICKTOCK = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_NATIVE_DATE_TIME = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_ANONYMOUS_ARRAY = NULL;

Keyword* KWD_JAVA_TRANSLATE_FUNCTION = NULL;

Keyword* KWD_JAVA_TRANSLATE_METHOD = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_PRINT_STREAM = NULL;

Keyword* KWD_JAVA_TRANSLATE_WARN = NULL;

Keyword* KWD_JAVA_TRANSLATE_ERROR = NULL;

Keyword* KWD_JAVA_TRANSLATE_CONTINUABLE_ERROR = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_STANDARD_ERROR = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_STANDARD_OUT = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_PRINT_NATIVE_STREAM = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_EOL = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_EOL = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_LITERAL = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_NULL = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_METHOD_VARIABLE_ARGUMENTSp = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_GET_SYM = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_SYMBOL = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_GET_KWD = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_GET_SGT = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_NULL_INTEGER = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_NULL_FLOAT = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_NULL_SINGLE_FLOAT = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_NULL_CHARACTER = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_METHOD_SLOT = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_DEFINEDp = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_NULLp = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_CONCATENATE = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_AREF = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_AREF_SETTER = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_UNARY_OP = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_BINARY_OP = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_JAVA_TERNARY_OP = NULL;

Surrogate* SGT_JAVA_TRANSLATE_STELLA_ARRAY = NULL;

Symbol* SYM_JAVA_TRANSLATE_STELLA_STARTUP_JAVA_TRANSLATE = NULL;

} // end of namespace stella
